clear all
set more off
capture log close
set matsize 3000

do define_macros
do define_programs

cap program drop runmrs_diffmethods
program runmrs_diffmethods

    args first
	
	syntax, controls(varlist) instruments(string) ivmethod(string)

    cap drop ee ee2 fi
    cap drop e1 e2 e2minusone e3
    cap drop vbar v
    cap drop mrs_*
    cap matrix drop mrs
    cap drop phi theta
    cap drop term1 term2
    cap drop Mc Mh Ml Hc Hh Hl Ic Ih Il
    cap drop y
	 
	if "`ivmethod'"=="fuller" {
		local method "fuller(1)"
	}
	if "`ivmethod'" == "gmm" {
		local method "gmm2s"
	}
	
    qui xi: probit sp_employed h_logearn h_logearnflag `controls' `instruments', difficult iterate(50)

    scalar probitconverged = e(converged)
    if probitconverged==0 {
        di in red "Probit didn't converge"
    }

    predict ee,index
    qui gen ee2=ee*ee
    qui gen fi=1/normprob(ee)
    qui gen e1 = fi/(sqrt(6.2831853)*exp(0.5*ee2))
    qui gen e2 = fi*ee/(sqrt(6.2831853)*exp(0.5*ee2)) + 1
    qui gen e2minusone = e2-1
    qui gen e3 =(2+ee2)*fi/(sqrt(6.2831853)*exp(0.5*ee2))
	
    xi: ivreg2 sp_lw e1 e2minusone e3 `controls' (sp_lleis lc = `instruments') if sp_employed==1, `method' `first'
	
	matrix define mrs_= e(b)
    svmat mrs_, name(matcol)
	
	qui gen theta  =  mrs_sp_lleis[1] +1
	qui gen phi    = - mrs_lc[1]+1

	scalar theta = theta[1]
	scalar phi   = phi[1]

end

cap program drop mrs_boot_diffmethods
program mrs_boot_diffmethods, rclass

    syntax, controls(varlist) instruments(string) ivmethod(string)

    cap drop lnimpwage

    cap drop ee ee2 fi
    cap drop e1 e2 e2minusone e3
    cap drop vbar v
    cap drop mrs_*
    cap matrix drop mrs
    cap drop phi theta
    cap drop term1 term2
    cap drop Mc Mh Ml Hc Hh Hl Ic Ih Il
    cap drop y

	runmrs_diffmethods, controls(`controls') instruments(`instruments') ivmethod(`ivmethod')
	
	foreach var in e1 e2minusone e3 `controls' sp_lleis lc _cons {
        return scalar mrs_`var' = mrs_`var'[1]
    }

    return scalar mrs_theta = theta[1]
    return scalar mrs_phi = phi[1]

    /*This regression is necessary for the bootstrap to work - results are irrelevant*/
    qui reg sp_employed

end

program define genmacros

	syntax, varlist(string)
	
	capture macro drop retlist
	foreach var in `varlist' {
		macro define retlist "$retlist mrs_`var' = r(mrs_`var')"
	}
	macro define retlist "$retlist mrs_theta = r(mrs_theta) mrs_phi = r(mrs_phi)"

end

cap program drop elasticities_cf
program define elasticities_cf

	syntax, controls(string) usedir(string) 

	use "`usedir'", clear
	mkmat _all

	use dataset2, clear

	estimates use "$savedir\mrs_whole_sample"
	
	genmrsvars
	genmarshallandhicks

	foreach var in h l c {

		qui centile M`var', centile(7.5 12.5)
		gen bdecM`var' = inrange(M`var',r(c_1),r(c_2))

		qui centile M`var', centile(22.5 27.5)
		gen lqM`var' = inrange(M`var',r(c_1),r(c_2))

		qui centile M`var', centile(47.5 52.5)
		gen medianM`var' = inrange(M`var',r(c_1),r(c_2))

		qui centile M`var', centile(72.5 77.5)
		gen uqM`var' = inrange(M`var',r(c_1),r(c_2))
		
		qui centile M`var', centile(87.5 92.5)
		gen udecM`var' = inrange(M`var',r(c_1),r(c_2))
	}

	drop Mh Ml Mc Hh Hl Hc term1 term2 Ih Il Ic mrs_* y 
	drop theta
	drop phi

	capture postclose postelasticities_cf
	capture erase "$savedir\elasticities_cf.dta"

	#delimit ;
	postfile postelasticities_cf bdecMc lqMc medianMc uqMc udecMc bdecHc lqHc medianHc uqHc udecHc 
	bdecMh lqMh medianMh uqMh udecMh bdecHh lqHh medianHh uqHh udecHh
	bdecMl lqMl medianMl uqMl udecMl bdecHl lqHl medianHl uqHl udecHl using "$savedir\elasticities_cf";
	#delimit cr

	capture macro drop elastretlist

	foreach var in c h l {
		foreach elast in M H {
			  foreach quant in bdec lq median uq udec {
				macro define elastretlist "$elastretlist (val`quant'`elast'`var')"
			}
		}
	}

	forval i = 1/$reps {

		qui gen theta = mrs_theta[`i',1]
		qui gen phi   = mrs_phi[`i',1]
		
		qui genmarshallandhicks
		
		foreach var in `controls' {
			qui gen mrs_`var' = mrs_`var'[`i',1]
		}
		
		foreach var in c l h {	
			foreach elast in M H {
				foreach quant in bdec lq median uq udec {
					qui su `elast'`var' if `quant'M`var'==1
					scalar val`quant'`elast'`var' = r(mean)
				}
			}
		}

		drop Mh Ml Mc Hh Hl Hc term1 term2 Ih Il Ic mrs_* y 
		drop theta phi
		
		post postelasticities_cf $elastretlist

	}

	postclose postelasticities_cf

end

program define maketab 

	syntax, usedir(string)
	
	capture log close
	use "`usedir'", clear
	merge 1:1 _n using "$savedir\elasticities_cf"
	drop _merge

	des
	local varcount = r(k)

	matrix tabresults = J(`varcount',5,-999)

	estimates use "$savedir\mrs_whole_sample"
	matrix b = e(b)
	matnames b
	local mrsvarlist "`r(c)'"

	rename mrs__cons mrs_constant

	local i = 1
	foreach var in `mrsvarlist' {
		local varname = substr("`var'",2,.)
		if "`varname'"=="_cons" {
			local varname "constant"
		}
		local varname "mrs_`varname'"

		qui su `varname'
		matrix tabresults[`i',1] = r(sd)
		qui centile `varname', centile(2.5 97.5 0.5 99.5)
		matrix tabresults[`i',2] = r(c_1)
		matrix tabresults[`i',3] = r(c_2)
		matrix tabresults[`i',4] = r(c_3)
		matrix tabresults[`i',5] = r(c_4)

		local i = `i' + 1
	}

	#delimit ;
	foreach var of varlist mrs_theta mrs_phi {;
		qui su `var';
		matrix tabresults[`i',1] = r(sd);
		qui centile `var', centile(2.5 97.5 0.5 99.5);
		matrix tabresults[`i',2] = r(c_1);
		matrix tabresults[`i',3] = r(c_2);
		matrix tabresults[`i',4] = r(c_3);
		matrix tabresults[`i',5] = r(c_4);
		local i = `i' + 1;
	};
	#delimit cr

	#delimit ;
	foreach var of varlist bdecMc lqMc medianMc uqMc udecMc bdecHc lqHc medianHc uqHc udecHc 
	bdecMh lqMh medianMh uqMh udecMh bdecHh lqHh medianHh uqHh udecHh 
	bdecMl lqMl medianMl uqMl udecMl bdecHl lqHl medianHl uqHl udecHl 
	 {;
		qui su `var';
		matrix tabresults[`i',1] = r(sd);
		qui centile `var', centile(2.5 97.5 0.5 99.5);
		matrix tabresults[`i',2] = r(c_1);
		matrix tabresults[`i',3] = r(c_2);
		matrix tabresults[`i',4] = r(c_3);
		matrix tabresults[`i',5] = r(c_4);
		local i = `i' + 1;
	};


	matrix rownames tabresults = `mrsvarlist' theta phi 
	bdecMc lqMc medianMc uqMc udecMc bdecHc lqHc medianHc uqHc udecHc 
	bdecMh lqMh medianMh uqMh udecMh bdecHh lqHh medianHh uqHh udecHh
	bdecMl lqMl medianMl uqMl udecMl bdecHl lqHl medianHl uqHl udecHl;
	#delimit cr

	matrix colnames tabresults = se 5percentlb 5percentub 1percentlb 1percentub

	log using "$savedir\mrsbootresults.txt", text replace
	matrix list tabresults
	log close
end



use "dataset2", clear

forval Y = 1979/2013 {
	gen yr`Y' = yr==`Y'
}

forval Y = $startyear/$endyear {
    forval i = 1/4 {
        forval j = 1/4 {
            gen edreg`Y'_`i'`j' = region`i'*sp_educ`j'*yr`Y'
        }
    }
}

global instlist_old "edreg19* edreg20*"
global instlist_BDM "i.instgroup"  

*********************************************************
*	Using BDM instruments and the Fuller estimator		*
*********************************************************

use "dataset2", clear

forval Y = 1979/2013 {
	gen yr`Y' = yr==`Y'
}

forval Y = $startyear/$endyear {
    forval i = 1/4 {
        forval j = 1/4 {
            gen edreg`Y'_`i'`j' = region`i'*sp_educ`j'*yr`Y'
        }
    }
}


global controls "$mrs_cntrl"
global instruments "$instlist_BDM"
global ivmethod "fuller"

global savedir "alternative_specs\BDM"

log using "$savedir\whole_sample_mrs.txt", text replace
runmrs_diffmethods, controls($controls) instruments($instruments) ivmethod($ivmethod)
estimates save "$savedir\mrs_whole_sample", replace
genelasticities
log close

genmacros, varlist(e1 e2minusone e3 $controls sp_lleis lc _cons)

bootstrap $retlist, reps($reps) saving("$savedir\mrs_boot", replace) seed(382543) strata(yr qtr cohort intmo intno) cluster(uniqcuid): mrs_boot_diffmethods, controls($controls) instruments($instruments) ivmethod($ivmethod)

use "$savedir\mrs_boot", clear
elasticities_cf, usedir("$savedir\mrs_boot") controls(e1 e2minusone e3 $controls sp_lleis lc _cons)
maketab, usedir("$savedir\mrs_boot")

*********************************************************
*	Using BDM instruments and the GMM estimator		*
*********************************************************

use "dataset2", clear

forval Y = 1979/2013 {
	gen yr`Y' = yr==`Y'
}

forval Y = $startyear/$endyear {
    forval i = 1/4 {
        forval j = 1/4 {
            gen edreg`Y'_`i'`j' = region`i'*sp_educ`j'*yr`Y'
        }
    }
}


global controls "$mrs_cntrl"
global instruments "$instlist_BDM"
global ivmethod "gmm"

global savedir "alternative_specs\BDM_gmm"

log using "$savedir\whole_sample_mrs.txt", text replace
runmrs_diffmethods, controls($controls) instruments($instruments) ivmethod($ivmethod)
estimates save "$savedir\mrs_whole_sample", replace
genelasticities
log close

genmacros, varlist(e1 e2minusone e3 $controls sp_lleis lc _cons)

bootstrap $retlist, reps($reps) saving("$savedir\mrs_boot_gmm", replace) seed(382543) strata(yr qtr cohort intmo intno) cluster(uniqcuid): mrs_boot_diffmethods, controls($controls) instruments($instruments) ivmethod($ivmethod)

use "$savedir\mrs_boot_gmm", clear
elasticities_cf, usedir("$savedir\mrs_boot_gmm") controls(e1 e2minusone e3 $controls sp_lleis lc _cons)
maketab, usedir("$savedir\mrs_boot_gmm")

