set more off

cap program drop mrs_boot_alternatives
program mrs_boot_alternatives, rclass

    syntax, depvar(varname) indepvars(varlist) method(string)

    cap drop lnimpwage

    runmrs_alternatives, depvar(`depvar') indepvars(`indepvars') method(`method')

    qui reg sp_lw $mrs_cntrl $instlist if sp_employed==1
    qui predict lnimpwage, xb

    foreach var in e1 e2minusone e3 $mrs_cntrl `indepvars' _cons {
        return scalar mrs_`var' = mrs_`var'[1]
    }

    return scalar mrs_theta = theta[1]
    return scalar mrs_phi = phi[1]

    /*This regression is necessary for the bootstrap to work - results are irrelevant*/
    qui reg sp_employed

end

cap program drop genmacros
program genmacros 
	syntax, varlist(string)
	capture macro drop retlist
	foreach var in `varlist' {
		macro define retlist "$retlist mrs_`var' = r(mrs_`var')"
	}
	macro define retlist "$retlist mrs_theta = r(mrs_theta) mrs_phi = r(mrs_phi)"

end

cap program drop maketab
program define maketab 

	syntax, usedir(string)
	capture log close
	use "`usedir'", clear
	merge 1:1 _n using "$savedir\elasticities_cf"
	drop _merge

	des
	local varcount = r(k)

	matrix tabresults = J(`varcount',5,-999)

	estimates use "$savedir\mrs_whole_sample"
	matrix b = e(b)
	matnames b
	local mrsvarlist "`r(c)'"

	rename mrs__cons mrs_constant

	local i = 1
	foreach var in `mrsvarlist' {
		local varname = substr("`var'",2,.)
		if "`varname'"=="_cons" {
			local varname "constant"
		}
		local varname "mrs_`varname'"

		qui su `varname'
		matrix tabresults[`i',1] = r(sd)
		qui centile `varname', centile(2.5 97.5 0.5 99.5)
		matrix tabresults[`i',2] = r(c_1)
		matrix tabresults[`i',3] = r(c_2)
		matrix tabresults[`i',4] = r(c_3)
		matrix tabresults[`i',5] = r(c_4)

		local i = `i' + 1
	}

	#delimit ;
	foreach var of varlist mrs_theta mrs_phi {;
		qui su `var';
		matrix tabresults[`i',1] = r(sd);
		qui centile `var', centile(2.5 97.5 0.5 99.5);
		matrix tabresults[`i',2] = r(c_1);
		matrix tabresults[`i',3] = r(c_2);
		matrix tabresults[`i',4] = r(c_3);
		matrix tabresults[`i',5] = r(c_4);
		local i = `i' + 1;
	};
	#delimit cr

	#delimit ;
	foreach var of varlist bdecMc lqMc medianMc uqMc udecMc bdecHc lqHc medianHc uqHc udecHc 
	bdecMh lqMh medianMh uqMh udecMh bdecHh lqHh medianHh uqHh udecHh 
	bdecMl lqMl medianMl uqMl udecMl bdecHl lqHl medianHl uqHl udecHl 
	 {;
		qui su `var';
		matrix tabresults[`i',1] = r(sd);
		qui centile `var', centile(2.5 97.5 0.5 99.5);
		matrix tabresults[`i',2] = r(c_1);
		matrix tabresults[`i',3] = r(c_2);
		matrix tabresults[`i',4] = r(c_3);
		matrix tabresults[`i',5] = r(c_4);
		local i = `i' + 1;
	};


	matrix rownames tabresults = `mrsvarlist' theta phi 
	bdecMc lqMc medianMc uqMc udecMc bdecHc lqHc medianHc uqHc udecHc 
	bdecMh lqMh medianMh uqMh udecMh bdecHh lqHh medianHh uqHh udecHh
	bdecMl lqMl medianMl uqMl udecMl bdecHl lqHl medianHl uqHl udecHl;
	#delimit cr

	matrix colnames tabresults = se 5percentlb 5percentub 1percentlb 1percentub

	log using "$savedir\mrsbootresults.txt", text replace
	matrix list tabresults
	log close
end

cap program drop elasticities_cf
program define elasticities_cf

	syntax, controls(string) usedir(string) depvar(string)

	use "`usedir'", clear
	mkmat _all

	use dataset2, clear

	estimates use "$savedir\mrs_whole_sample"
	
    matrix define mrs_= e(b)
    svmat mrs_, name(matcol)
	
	if "`depvar'"=="sp_lleis" {
        qui gen theta  = (1/mrs_sp_lw[1]) +1
        qui gen phi    = ((1/mrs_sp_lw[1])*mrs_lc[1])+1
    }

    if "`depvar'"=="sp_lw" {
        qui gen theta  =  mrs_sp_lleis[1] +1
        qui gen phi    = - mrs_lc[1]+1
    }

    if "`depvar'"=="lc" {
        qui gen phi    = -(1/mrs_sp_lw[1]) +1
        qui gen theta  = -(mrs_sp_lleis[1]/mrs_sp_lw[1]) + 1
    }

	genmarshallandhicks

	foreach var in h l c {

		qui centile M`var', centile(7.5 12.5)
		gen bdecM`var' = inrange(M`var',r(c_1),r(c_2))

		qui centile M`var', centile(22.5 27.5)
		gen lqM`var' = inrange(M`var',r(c_1),r(c_2))

		qui centile M`var', centile(47.5 52.5)
		gen medianM`var' = inrange(M`var',r(c_1),r(c_2))

		qui centile M`var', centile(72.5 77.5)
		gen uqM`var' = inrange(M`var',r(c_1),r(c_2))
		
		qui centile M`var', centile(87.5 92.5)
		gen udecM`var' = inrange(M`var',r(c_1),r(c_2))
	}

	drop Mh Ml Mc Hh Hl Hc term1 term2 Ih Il Ic mrs_* y 
	drop theta
	drop phi

	capture postclose postelasticities_cf
	capture erase "$savedir\elasticities_cf.dta"

	#delimit ;
	postfile postelasticities_cf bdecMc lqMc medianMc uqMc udecMc bdecHc lqHc medianHc uqHc udecHc 
	bdecMh lqMh medianMh uqMh udecMh bdecHh lqHh medianHh uqHh udecHh
	bdecMl lqMl medianMl uqMl udecMl bdecHl lqHl medianHl uqHl udecHl using "$savedir\elasticities_cf";
	#delimit cr

	capture macro drop elastretlist

	foreach var in c h l {
		foreach elast in M H {
			  foreach quant in bdec lq median uq udec {
				macro define elastretlist "$elastretlist (val`quant'`elast'`var')"
			}
		}
	}

	forval i = 1/$reps {

		qui gen theta = mrs_theta[`i',1]
		qui gen phi   = mrs_phi[`i',1]
		
		qui genmarshallandhicks
		
		foreach var in `controls' {
			qui gen mrs_`var' = mrs_`var'[`i',1]
		}
		
		foreach var in c l h {	
			foreach elast in M H {
				foreach quant in bdec lq median uq udec {
					qui su `elast'`var' if `quant'M`var'==1
					scalar val`quant'`elast'`var' = r(mean)
				}
			}
		}

		drop Mh Ml Mc Hh Hl Hc term1 term2 Ih Il Ic mrs_* y 
		drop theta phi
		
		post postelasticities_cf $elastretlist

	}

	postclose postelasticities_cf

end

***************************************************************************************
*Bootstrapping GMM with LEISURE as the dependent variable*

use dataset2, clear

global savedir "gmm_results\leisure"

drop *inint5
drop sp_participant
drop sp_grosswage h_grosswage
drop numdepchild

log using "$savedir\whole_sample_mrs.txt", text replace
runmrs_alternatives, depvar(sp_lleis) indepvars(sp_lw lc) method(gmm2s) first
estimates save "$savedir\mrs_whole_sample", replace
genelasticities
log close

genmacros, varlist(e1 e2minusone e3 $mrs_cntrl sp_lw lc _cons)

bootstrap $retlist, reps($reps) saving("$savedir\mrs_boot_gmm_leisure", replace) seed(382543) strata(yr qtr cohort intmo intno) cluster(uniqcuid): mrs_boot_alternatives, depvar(sp_lleis) indepvars(lc sp_lw) method(gmm2s)

use "$savedir\mrs_boot_gmm_leisure", clear

elasticities_cf, controls(e1 e2minusone e3 $mrs_cntrl sp_lw lc _cons) usedir("$savedir\mrs_boot_gmm_leisure") depvar("sp_lleis")
maketab, usedir("$savedir\mrs_boot_gmm_leisure")

***************************************************************************************
*Bootstrapping GMM with WAGES as the dependent variable*

use dataset2, clear

global savedir "gmm_results\wages"

drop *inint5
drop sp_participant
drop sp_grosswage h_grosswage
drop numdepchild

log using "$savedir\whole_sample_mrs.txt", text replace
runmrs_alternatives, depvar(sp_lw) indepvars(lc sp_lleis) method(gmm2s) first
estimates save "$savedir\mrs_whole_sample", replace
genelasticities
log close

genmacros, varlist(e1 e2minusone e3 $mrs_cntrl lc sp_lleis _cons)

bootstrap $retlist, reps($reps) saving("$savedir\mrs_boot_gmm_wages", replace) seed(382543) strata(yr qtr cohort intmo intno) cluster(uniqcuid): mrs_boot_alternatives, depvar(sp_lw) indepvars(lc sp_lleis) method(gmm2s) 

use "$savedir\mrs_boot_gmm_wages", clear

elasticities_cf, controls(e1 e2minusone e3 $mrs_cntrl lc sp_lleis _cons) usedir("$savedir\mrs_boot_gmm_wages") depvar("sp_lw")
maketab, usedir("$savedir\mrs_boot_gmm_wages")

***************************************************************************************
*Bootstrapping GMM with CONSUMPTION as the dependent variable*

use dataset2, clear

global savedir "gmm_results\consumption"

drop *inint5
drop sp_participant
drop sp_grosswage h_grosswage
drop numdepchild

log using "$savedir\whole_sample_mrs.txt", text replace
qui runmrs_alternatives, depvar(lc) indepvars(sp_lw sp_lleis) method(gmm2s) first
estimates save "$savedir\mrs_whole_sample", replace
genelasticities
log close

genmacros, varlist(e1 e2minusone e3 $mrs_cntrl sp_lw sp_lleis _cons)

bootstrap $retlist, reps($reps) saving("$savedir\mrs_boot_gmm_consumption", replace) seed(382543) strata(yr qtr cohort intmo intno) cluster(uniqcuid): mrs_boot_alternatives, depvar(lc) indepvars(sp_lw sp_lleis) method(gmm2s)

use "$savedir\mrs_boot_gmm_consumption", clear

elasticities_cf, controls(e1 e2minusone e3 $mrs_cntrl sp_lw sp_lleis _cons) usedir("$savedir\mrs_boot_gmm_consumption") depvar("lc")
maketab, usedir("$savedir\mrs_boot_gmm_consumption")

***************************************************************************************
*Bootstrapping FULLER with LEISURE as the dependent variable*

use dataset2, clear

global savedir "alternative_specs\leisuredepvar"

drop *inint5
drop sp_participant
drop sp_grosswage h_grosswage
drop numdepchild

log using "$savedir\whole_sample_mrs.txt", text replace
runmrs_alternatives, depvar(sp_lleis) indepvars(sp_lw lc) method(fuller(1)) first
estimates save "$savedir\mrs_whole_sample", replace
genelasticities
log close

genmacros, varlist(e1 e2minusone e3 $mrs_cntrl sp_lw lc _cons)

bootstrap $retlist, reps($reps) saving("$savedir\mrs_boot_leisure", replace) seed(382543) strata(yr qtr cohort intmo intno) cluster(uniqcuid): mrs_boot_alternatives, depvar(sp_lleis) indepvars(lc sp_lw) method(fuller(1))

use "$savedir\mrs_boot_leisure", clear

elasticities_cf, controls(e1 e2minusone e3 $mrs_cntrl sp_lw lc _cons) usedir("$savedir\mrs_boot_leisure") depvar("sp_lleis")
maketab, usedir("$savedir\mrs_boot_leisure")

***************************************************************************************
*Bootstrapping FULLER with CONSUMPTION as the dependent variable*

use dataset2, clear

global savedir "alternative_specs\consdepvar"

drop *inint5
drop sp_participant
drop sp_grosswage h_grosswage
drop numdepchild

log using "$savedir\whole_sample_mrs.txt", text replace
qui runmrs_alternatives, depvar(lc) indepvars(sp_lw sp_lleis) method(fuller(1)) first
estimates save "$savedir\mrs_whole_sample", replace
genelasticities
log close

genmacros, varlist(e1 e2minusone e3 $mrs_cntrl sp_lw sp_lleis _cons)

bootstrap $retlist, reps($reps) saving("$savedir\mrs_boot_consumption", replace) seed(382543) strata(yr qtr cohort intmo intno) cluster(uniqcuid): mrs_boot_alternatives, depvar(lc) indepvars(sp_lw sp_lleis) method(fuller(1))

use "$savedir\mrs_boot_consumption", clear

elasticities_cf, controls(e1 e2minusone e3 $mrs_cntrl sp_lw sp_lleis _cons) usedir("$savedir\mrs_boot_consumption") depvar("lc")
maketab, usedir("$savedir\mrs_boot_consumption")

***************************************************************************************
*Bootstrapping FULLER with ANNUAL WAGES as the dependent variable*

use dataset2, clear

global savedir "alternative_specs\annwagesdepvar"

drop *inint5
drop sp_participant
drop sp_grosswage h_grosswage
drop numdepchild

replace sp_lleis = sp_lannleisure
replace sp_leisure = sp_annleisure

drop sp_lannnetwage sp_lannleisure sp_annnetwage sp_annleisure

log using "$savedir\whole_sample_mrs.txt", text replace
runmrs_alternatives, depvar(sp_lw) indepvars(lc sp_lleis) method(fuller(1)) first
estimates save "$savedir\mrs_whole_sample", replace
genelasticities
log close

genmacros, varlist(e1 e2minusone e3 $mrs_cntrl lc sp_lleis _cons)

bootstrap $retlist, reps($reps) saving("$savedir\mrs_boot_annwages", replace) seed(382543) strata(yr qtr cohort intmo intno) cluster(uniqcuid): mrs_boot_alternatives, depvar(sp_lw) indepvars(lc sp_lleis) method(fuller(1)) 

use "$savedir\mrs_boot_annwages", clear

elasticities_cf, controls(e1 e2minusone e3 $mrs_cntrl lc sp_lleis _cons) usedir("$savedir\mrs_boot_annwages") depvar("sp_lw")
maketab, usedir("$savedir\mrs_boot_annwages")

***************************************************************************************
*Bootstrapping LIML with LEISURE as the dependent variable*

use dataset2, clear

global savedir "alternative_specs\liml"

drop *inint5
drop sp_participant
drop sp_grosswage h_grosswage
drop numdepchild

log using "$savedir\whole_sample_mrs.txt", text replace
runmrs_alternatives, depvar(sp_lleis) indepvars(sp_lw lc) method(liml) first
estimates save "$savedir\mrs_whole_sample", replace
genelasticities
log close

genmacros, varlist(e1 e2minusone e3 $mrs_cntrl sp_lw lc _cons)

bootstrap $retlist, reps($reps) saving("$savedir\mrs_boot_liml_leisure", replace) seed(382543) strata(yr qtr cohort intmo intno) cluster(uniqcuid): mrs_boot_alternatives, depvar(sp_lleis) indepvars(lc sp_lw) method(gmm2s)

use "$savedir\mrs_boot_liml_leisure", clear

elasticities_cf, controls(e1 e2minusone e3 $mrs_cntrl sp_lw lc _cons) usedir("$savedir\mrs_boot_liml_leisure") depvar("sp_lleis")
maketab, usedir("$savedir\mrs_boot_liml_leisure")








