clear all
set more off
capture log close
set matsize 3000

do define_macros
do define_programs

/*Our alternative samples*/
/*
i.	excluding individuals who work exactly 40 hours per week (and controlling for the appropriate selection). The justification of this experiment is that these people may be affected by some kind of friction that does not allow them to adjust their hours worked as desired and so it may be more reasonable to exclude them. 
ii.	excluding individuals working less than 20 hours (with appropriate selection correction). There may be certain frictions that prevent individuals working less than that amount of hours.
iii.	including only cohorts of women with similar labour supply behaviour over the life-cycle.
*/

/*In all cases, theta and phi are estimated using a different sample, while the elasticities are calculated for the whole sample*/

cap program drop runmrs_diffsamples
program runmrs_diffsamples

    args first
	
	syntax, spec(numlist) controls(varlist)

    cap drop ee ee2 fi
    cap drop e1 e2 e2minusone e3
    cap drop vbar v
    cap drop mrs_*
    cap matrix drop mrs
    cap drop phi theta
    cap drop term1 term2
    cap drop Mc Mh Ml Hc Hh Hl Ic Ih Il
    cap drop y
	 
	if "`spec'"=="1" {
		local probitdepvar "sp_employed"
		local mrsif "& sp_hours!=40"
		local probitif "if sp_hours!=40"
	}
	
	if "`spec'"=="2" {
		local probitdepvar "sp_hourslt20"
		local mrsif "& sp_hourslt20==0"
		local probitif ""
	}
	
	if "`spec'"=="3" {
		local probitdepvar "sp_employed"
		local mrsif "& inrange(cohort,2,5)"
		local probitif "if inrange(cohort,2,5)"
	}

	qui xi: probit `probitdepvar' h_logearn h_logearnflag `controls' $instlist `probitif', difficult iterate(50)

    scalar probitconverged = e(converged)
    if probitconverged==0 {
        di in red "Probit didn't converge"
    }

    predict ee,index
    qui gen ee2=ee*ee
    qui gen fi=1/normprob(ee)
    qui gen e1 = fi/(sqrt(6.2831853)*exp(0.5*ee2))
    qui gen e2 = fi*ee/(sqrt(6.2831853)*exp(0.5*ee2)) + 1
    qui gen e2minusone = e2-1
    qui gen e3 =(2+ee2)*fi/(sqrt(6.2831853)*exp(0.5*ee2))

    xi: ivreg2 sp_lw e1 e2minusone e3 `controls' (sp_lleis lc = $instlist) if sp_employed==1 `mrsif', fuller(1)
	
	matrix define mrs_= e(b)
    svmat mrs_, name(matcol)
	
	qui gen theta  =  mrs_sp_lleis[1] +1
	qui gen phi    = - mrs_lc[1]+1

	scalar theta = theta[1]
	scalar phi   = phi[1]

end

cap program drop mrs_boot_diffsamples
program mrs_boot_diffsamples, rclass
 
    syntax, spec(numlist) controls(varlist)

    cap drop lnimpwage

    cap drop ee ee2 fi
    cap drop e1 e2 e2minusone e3
    cap drop vbar v
    cap drop mrs_*
    cap matrix drop mrs
    cap drop phi theta
    cap drop y

	runmrs_diffsamples, spec(`spec') controls(`controls')
	
    foreach var in e1 e2minusone e3 `controls' sp_lleis lc _cons {
        return scalar mrs_`var' = mrs_`var'[1]
    }

    return scalar mrs_theta = theta[1]
    return scalar mrs_phi = phi[1]

    /*This regression is necessary for the bootstrap to work - results are irrelevant*/
    qui reg sp_employed

end

program define genmacros

	syntax, varlist(string)
	
	capture macro drop retlist
	foreach var in `varlist' {
		macro define retlist "$retlist mrs_`var' = r(mrs_`var')"
	}
	macro define retlist "$retlist mrs_theta = r(mrs_theta) mrs_phi = r(mrs_phi)"

end

program define maketab 

	capture log close
	use "$savedir\mrs_boot", clear
	merge 1:1 _n using "$savedir\elasticities_cf"
	drop _merge

	des
	local varcount = r(k)

	matrix tabresults = J(`varcount',5,-999)

	estimates use "$savedir\mrs_whole_sample"
	matrix b = e(b)
	matnames b
	local mrsvarlist "`r(c)'"

	rename mrs__cons mrs_constant

	local i = 1
	foreach var in `mrsvarlist' {
		local varname = substr("`var'",2,.)
		if "`varname'"=="_cons" {
			local varname "constant"
		}
		local varname "mrs_`varname'"

		qui su `varname'
		matrix tabresults[`i',1] = r(sd)
		qui centile `varname', centile(2.5 97.5 0.5 99.5)
		matrix tabresults[`i',2] = r(c_1)
		matrix tabresults[`i',3] = r(c_2)
		matrix tabresults[`i',4] = r(c_3)
		matrix tabresults[`i',5] = r(c_4)

		local i = `i' + 1
	}

	#delimit ;
	foreach var of varlist mrs_theta mrs_phi {;
		qui su `var';
		matrix tabresults[`i',1] = r(sd);
		qui centile `var', centile(2.5 97.5 0.5 99.5);
		matrix tabresults[`i',2] = r(c_1);
		matrix tabresults[`i',3] = r(c_2);
		matrix tabresults[`i',4] = r(c_3);
		matrix tabresults[`i',5] = r(c_4);
		local i = `i' + 1;
	};
	#delimit cr

	#delimit ;
	foreach var of varlist bdecMc lqMc medianMc uqMc udecMc bdecHc lqHc medianHc uqHc udecHc 
	bdecMh lqMh medianMh uqMh udecMh bdecHh lqHh medianHh uqHh udecHh 
	bdecMl lqMl medianMl uqMl udecMl bdecHl lqHl medianHl uqHl udecHl 
	 {;
		qui su `var';
		matrix tabresults[`i',1] = r(sd);
		qui centile `var', centile(2.5 97.5 0.5 99.5);
		matrix tabresults[`i',2] = r(c_1);
		matrix tabresults[`i',3] = r(c_2);
		matrix tabresults[`i',4] = r(c_3);
		matrix tabresults[`i',5] = r(c_4);
		local i = `i' + 1;
	};


	matrix rownames tabresults = `mrsvarlist' theta phi 
	bdecMc lqMc medianMc uqMc udecMc bdecHc lqHc medianHc uqHc udecHc 
	bdecMh lqMh medianMh uqMh udecMh bdecHh lqHh medianHh uqHh udecHh
	bdecMl lqMl medianMl uqMl udecMl bdecHl lqHl medianHl uqHl udecHl;
	#delimit cr

	matrix colnames tabresults = se 5percentlb 5percentub 1percentlb 1percentub

	log using "$savedir\mrsbootresults.txt", text replace
	matrix list tabresults
	log close
end

program define elasticities_cf

	syntax, controls(string)

	use "$savedir\mrs_boot", clear
	mkmat _all

	use dataset2, clear
	estimates use "$savedir\mrs_whole_sample"

	genmrsvars
	genmarshallandhicks

	foreach var in h l c {

		qui centile M`var', centile(7.5 12.5)
		gen bdecM`var' = inrange(M`var',r(c_1),r(c_2))

		qui centile M`var', centile(22.5 27.5)
		gen lqM`var' = inrange(M`var',r(c_1),r(c_2))

		qui centile M`var', centile(47.5 52.5)
		gen medianM`var' = inrange(M`var',r(c_1),r(c_2))

		qui centile M`var', centile(72.5 77.5)
		gen uqM`var' = inrange(M`var',r(c_1),r(c_2))
		
		qui centile M`var', centile(87.5 92.5)
		gen udecM`var' = inrange(M`var',r(c_1),r(c_2))
	}

	drop Mh Ml Mc Hh Hl Hc term1 term2 Ih Il Ic mrs_* y 
	drop theta
	drop phi

	capture postclose postelasticities_cf
	capture erase "$savedir\elasticities_cf.dta"

	#delimit ;
	postfile postelasticities_cf bdecMc lqMc medianMc uqMc udecMc bdecHc lqHc medianHc uqHc udecHc 
	bdecMh lqMh medianMh uqMh udecMh bdecHh lqHh medianHh uqHh udecHh
	bdecMl lqMl medianMl uqMl udecMl bdecHl lqHl medianHl uqHl udecHl using "$savedir\elasticities_cf";
	#delimit cr

	capture macro drop elastretlist

	foreach var in c h l {
		foreach elast in M H {
			  foreach quant in bdec lq median uq udec {
				macro define elastretlist "$elastretlist (val`quant'`elast'`var')"
			}
		}
	}

	forval i = 1/$reps {

		qui gen theta = mrs_theta[`i',1]
		qui gen phi   = mrs_phi[`i',1]
		
		qui genmarshallandhicks
		
		foreach var in `controls' {
			qui gen mrs_`var' = mrs_`var'[`i',1]
		}
		
		foreach var in c l h {	
			foreach elast in M H {
				foreach quant in bdec lq median uq udec {
					qui su `elast'`var' if `quant'M`var'==1
					scalar val`quant'`elast'`var' = r(mean)
				}
			}
		}

		drop Mh Ml Mc Hh Hl Hc term1 term2 Ih Il Ic mrs_* y 
		drop theta phi
		
		post postelasticities_cf $elastretlist

	}

	postclose postelasticities_cf

end

/*EXCLUDING THOSE WORKING EXACTLY 40 HOURS*/

use "dataset2", clear

gen sp_hours40 = sp_hours==40
gen sp_hourslt20 = sp_hours<20

global savedir "alternative_specs\exc40"

log using "$savedir\whole_sample_mrs.txt", text replace
runmrs_diffsamples, spec(1) controls($mrs_cntrl)
estimates save "$savedir\mrs_whole_sample", replace
genelasticities
log close

genmacros, varlist(e1 e2minusone e3 $mrs_cntrl sp_lleis lc _cons)
bootstrap $retlist, reps($reps) saving("$savedir\mrs_boot", replace) seed(382543) strata(yr qtr cohort intmo intno) cluster(uniqcuid): mrs_boot_diffsamples, spec(1) controls($mrs_cntrl)

preserve
use "$savedir\mrs_boot", clear

elasticities_cf, controls(e1 e2minusone e3 $mrs_cntrl sp_lleis lc _cons)
maketab

/*EXCLUDING THOSE WORKING LESS THAN 20 HOURS*/

use "dataset2", clear

gen sp_hours40 = sp_hours==40
gen sp_hourslt20 = sp_hours<20

global savedir "alternative_specs\exclt20"

log using "$savedir\whole_sample_mrs.txt", text replace
runmrs_diffsamples, spec(2) controls($mrs_cntrl)
estimates save "$savedir\mrs_whole_sample", replace
genelasticities
log close

genmacros, varlist(e1 e2minusone e3 $mrs_cntrl sp_lleis lc _cons)

bootstrap $retlist, reps($reps) saving("$savedir\mrs_boot", replace) seed(382543) strata(yr qtr cohort intmo intno) cluster(uniqcuid): mrs_boot_diffsamples, spec(2) controls($mrs_cntrl)

use "$savedir\mrs_boot", clear

elasticities_cf, controls(e1 e2minusone e3 $mrs_cntrl sp_lleis lc _cons)
maketab

/*ONLY THOSE COHORTS WITH SIMILAR BEHAVIOUR OVER THE LIFE-CYCLE*/

use "dataset2", clear

gen sp_hours40 = sp_hours==40
gen sp_hourslt20 = sp_hours<20

global savedir "alternative_specs\selectcohorts"

#delimit ;
global oth_cntrl2
"season1
season2
season3
season4
season5
season6
season7
season8
season9
season10
season11
cohort2_educ1
cohort2_educ2
cohort2_educ3
cohort2_educ4
cohort3_educ1
cohort3_educ2
cohort3_educ3
cohort3_educ4
cohort4_educ1
cohort4_educ2
cohort4_educ3
cohort4_educ4
cohort5_educ1
cohort5_educ2
cohort5_educ3";
#delimit cr

log using "$savedir\whole_sample_mrs.txt", text replace
runmrs_diffsamples, spec(3) controls($dem_cntrl $oth_cntrl2)
estimates save "$savedir\mrs_whole_sample", replace
genelasticities
log close

genmacros, varlist(e1 e2minusone e3 $dem_cntrl $oth_cntrl2 sp_lleis lc _cons)

bootstrap $retlist, reps($reps) saving("$savedir\mrs_boot", replace) seed(382543) strata(yr qtr cohort intmo intno) cluster(uniqcuid): mrs_boot_diffsamples, spec(3) controls($dem_cntrl $oth_cntrl2)

use "$savedir\mrs_boot", clear

elasticities_cf, controls(e1 e2minusone e3 $dem_cntrl $oth_cntrl2 sp_lleis lc _cons)
maketab






























