 
makealpha, wagevar(sp_lw) leisurevar(sp_lleis)
*makealpha_leisure, wagevar(sp_lw) leisurevar(sp_lleis)
prepeuler, groupvars($groupvars)

/*********************************************************************/
/*                      RUN EULER ON WHOLE SAMPLE                    */
/*********************************************************************/

log using "eulerresults.txt", text replace
xi: ivreg2 $depvar $euler_exogvar ($euler_endogvar = $euler_instvar) [aw=Nobs], first fuller(1)

*running reverse regression for the F-stat
xi: ivreg2 $euler_endogvar $euler_exogvar ($depvar  = $euler_instvar) [aw=Nobs], first fuller(1)
log close

/*********************************************************************/
/*        CALCULTING OMEGA (VCOV MATRIX FOR THE RESIDUALS)           */
/*********************************************************************/

sort cohort intmo
qui egen group = group(cohort intmo)
global numgroups = r(max)
qui su group
global maxG = r(max)

*Weight factor for the off-diagonal entries.
mata: lambda1 = 0.25
*Weight for off diagonal covariances within diagonal block of matrices.
mata: lambda2 = 0.25

*find optimal levels of lamda
scalar errorcode = 1
*keep iterating until matrix is positive definite
while errorcode !=0 {
        mata: lambda1 = lambda1-0.01
        mata: lambda2 = lambda2-0.01

        mata: assert(lambda1>0)
        mata: assert(lambda2>0)

        *programme in define_programs
        cap drop e? lage?
        cap drop e?? lage??
        qui makeomega, maxG($maxG)
        *Verifying matrix is positive definite.
        mata: assert(issymmetric(Omega)==1)
        mata: eigvalues = .
        mata: eigvalues=symeigenvalues(Omega)
        capture mata: assert(isrealvalues(eigvalues)==1)
        scalar errorcode1 = _rc
        capture mata: assert(rowmin(eigvalues)>0)
        scalar errorcode2 = _rc
        scalar errorcode = errorcode1 + errorcode2

}
*end of while loop

*now try increasing lambda1
scalar errorcode = 0
*keep iterating until matrix is positive definite
while errorcode==0 {

        mata: lambda1 = lambda1+0.01

        *programme in define_programs
        cap drop e? lage?
        cap drop e?? lage??
        qui makeomega, maxG($maxG)
        *Verifying matrix is positive definite.
        mata: assert(issymmetric(Omega)==1)
        mata: eigvalues = .
        mata: eigvalues=symeigenvalues(Omega)
        capture mata: assert(isrealvalues(eigvalues)==1)
        scalar errorcode1 = _rc
        capture mata: assert(rowmin(eigvalues)>0)
        scalar errorcode2 = _rc
        scalar errorcode = errorcode1 + errorcode2

}

mata: lambda1 = lambda1-0.01

*now try increasing lambda2
scalar errorcode = 0
*keep iterating until matrix is positive definite
while errorcode==0 {

        mata: lambda2 = lambda2+0.01
        *programme in define_programs
        cap drop e? lage?
        cap drop e?? lage??
        qui makeomega, maxG($maxG)
        *Verifying matrix is positive definite.
        mata: assert(issymmetric(Omega)==1)
        mata: eigvalues = .
        mata: eigvalues=symeigenvalues(Omega)
        capture mata: assert(isrealvalues(eigvalues)==1)
        scalar errorcode1 = _rc
        capture mata: assert(rowmin(eigvalues)>0)
        scalar errorcode2 = _rc
        scalar errorcode = errorcode1 + errorcode2
}

mata: lambda2 = lambda2-0.01
cap drop e? lage?
cap drop e?? lage??
makeomega, maxG($maxG)

mata: C = cholesky(Omega)
mata: invC = luinv(C)

sort group yr qtr

count
local count = r(N)
mata: e = J(rows(Omega),1,.)

local i = 1

local maxX = wordcount("euler_endogvar") + 1

forval X = 1/`maxX' {
    local j = `count'+`i'-1
    tomata e`X'
    mata: e[`i'..`j',1] = e`X'
    local i = `j'+1
}

/*********************************************************************/
/*                              BOOTSTRAP                            */
/*********************************************************************/

capture macro drop mrsvarnames eulervarnames mrseulerbootvarlist
foreach var in e1 e2minusone e3 $mrs_cntrl sp_lleis lc _cons $psi_mrs_cntrl {
	global mrsvarnames "$mrsvarnames mrs_`var'"
}

foreach var in $euler_exogvar $euler_endogvar _cons {
	global eulervarnames "$eulervarnames euler_`var'"
}

#delimit ; 
global mrseulerbootvarlist "$mrsvarnames mrs_theta mrs_phi $eulervarnames euler_sigma probitconverged";
#delimit cr

capture macro drop retlist
foreach var in e1 e2minusone e3 $mrs_cntrl sp_lleis lc _cons {
    macro define retlist "$retlist (mrs_`var')"
}

macro define retlist "$retlist (mrs_theta) (mrs_phi)"

foreach var in $euler_exogvar $euler_endogvar _cons {
    macro define retlist "$retlist (euler_`var')"
}

macro define retlist "$retlist (euler_sigma)"

macro define retlist "$retlist (probitconverged)"

capture postclose mrseulerboot
capture erase mrseuler_boot.dta
postfile mrseulerboot $mrseulerbootvarlist using mrseuler_boot

set seed 43890589
global numiterations= $reps
global B = $numiterations
while $B >0 {
    use dataset2, clear

    drop sp_participant
    drop numdepchild
    drop *inint5
    drop sp_grosswage h_grosswage

    di "Bootstrap iteration number:  $B"

    bsample, strata(yr qtr cohort intmo intno) cluster(uniqcuid)

    cap drop lnimpwage
    qui runmrs
    qui genmrsvars

    foreach var in e1 e2minusone e3 $mrs_cntrl sp_lleis lc _cons {
        scalar mrs_`var' = mrs_`var'[1]
    }
	
	scalar mrs_theta = theta[1]
	scalar mrs_phi   = phi[1]
	

    qui makealpha, wagevar(sp_lw) leisurevar(sp_lleis)
    qui prepeuler, groupvars($groupvars)

    qui reg $depvar $euler_exogvar $euler_instvar
    qui keep if e(sample)==1

    mata:  res_thisdraw= rnormal(rows(e),1,0,1)
    mata:  epsilon= C*res_thisdraw
    mata:  epsilon = rowshape(epsilon,`maxX')'

    sort cohort intmo
    egen group = group(cohort intmo)
    sort group yr qtr

    forval X = 1/`maxX' {
        mata:  epsilon`X' = epsilon[,`X']
        qui mata:  st_addvar("double", "epsilon`X'")
        mata: st_store(., "epsilon`X'", epsilon`X')
    }

    local X =1
    foreach var of varlist $euler_endogvar $depvar {
        drop `var'
        estimates use "firststage_`var'"
        qui predict `var'temp, xb
        qui gen `var' = `var'temp + epsilon`X'
        qui drop `var'temp
        local ++X
    }

    qui xi: ivreg2 $depvar $euler_exogvar ($euler_endogvar = $euler_instvar) [aw=Nobs], fuller(1)

    matrix define euler_= e(b)
    svmat euler_, name(matcol)
	
    foreach var in $euler_exogvar $euler_endogvar _cons {
        scalar euler_`var' = euler_`var'[1]
    }
	
	if "$depvar"=="Z" {
        scalar euler_sigma = (1/_b[W]) +1
    }
    if "$depvar"=="W" {
        scalar euler_sigma = _b[Z] +1
    }
	
	post mrseulerboot $retlist

    global B = $B-1
}

postclose mrseulerboot

use mrseuler_boot, clear
mkmat _all

use dataset2, clear

estimates use "mrswholesample"

genmrsvars
genmarshallandhicks

foreach var in h l c {

    qui centile M`var', centile(7.5 12.5)
    gen bdecM`var' = inrange(M`var',r(c_1),r(c_2))

    qui centile M`var', centile(22.5 27.5)
    gen lqM`var' = inrange(M`var',r(c_1),r(c_2))

    qui centile M`var', centile(47.5 52.5)
    gen medianM`var' = inrange(M`var',r(c_1),r(c_2))

    qui centile M`var', centile(72.5 77.5)
    gen uqM`var' = inrange(M`var',r(c_1),r(c_2))
	
	qui centile M`var', centile(87.5 92.5)
    gen udecM`var' = inrange(M`var',r(c_1),r(c_2))
}

drop Mh Ml Mc Hh Hl Hc term1 term2 Ih Il Ic mrs_* y 
drop theta
drop phi

capture postclose postelasticities_cf
capture erase elasticities_cf.dta

#delimit ;
postfile postelasticities_cf bdecMc lqMc medianMc uqMc udecMc bdecHc lqHc medianHc uqHc udecHc bdecFc lqFc medianFc uqFc udecFc
bdecMh lqMh medianMh uqMh udecMh bdecHh lqHh medianHh uqHh udecHh bdecFh lqFh medianFh uqFh udecFh
bdecMl lqMl medianMl uqMl udecMl bdecHl lqHl medianHl uqHl udecHl bdecFl lqFl medianFl uqFl udecFl
bdecFc_IR lqFc_IR medianFc_IR uqFc_IR udecFc_IR
bdecFh_IR lqFh_IR medianFh_IR uqFh_IR udecFh_IR
bdecFl_IR lqFl_IR medianFl_IR uqFl_IR udecFl_IR
bdecFc_add lqFc_add medianFc_add uqFc_add udecFc_add 
bdecFh_add lqFh_add medianFh_add uqFh_add udecFh_add
bdecFl_add lqFl_add medianFl_add uqFl_add udecFl_add
bdecFc_IR_add lqFc_IR_add medianFc_IR_add uqFc_IR_add udecFc_IR_add 
bdecFh_IR_add lqFh_IR_add medianFh_IR_add uqFh_IR_add udecFh_IR_add 
bdecFl_IR_add lqFl_IR_add medianFl_IR_add uqFl_IR_add udecFl_IR_add using elasticities_cf;
#delimit cr

capture macro drop elastretlist

foreach var in c h l {
    foreach elast in M H F {
          foreach quant in bdec lq median uq udec {
            macro define elastretlist "$elastretlist (val`quant'`elast'`var')"
        }
    }
}

foreach var in c h l {
	foreach quant in bdec lq median uq udec {
		  macro define elastretlist "$elastretlist (val`quant'F`var'_IR)"
	}
}

foreach var in c h l {
	foreach quant in bdec lq median uq udec {
		  macro define elastretlist "$elastretlist (val`quant'F`var'_add)"
	}
}

foreach var in c h l {
	foreach quant in bdec lq median uq udec {
		  macro define elastretlist "$elastretlist (val`quant'F`var'_IR_add)"
	}
}

forval i = 1/$reps {

	qui gen theta = mrs_theta[`i',1]
	qui gen phi   = mrs_phi[`i',1]
	
	qui genmarshallandhicks
	
	foreach var in e1 e2minusone e3 $mrs_cntrl sp_lleis lc _cons {
		qui gen mrs_`var' = mrs_`var'[`i',1]
	}
	
	qui makealpha, wagevar(sp_lw) leisurevar(sp_lleis)
	
	local sigmascalar = euler_sigma[`i',1]
	calcfrisch, sigma(`sigmascalar')
	
	foreach var in c l h {	
		foreach elast in M H F {
			foreach quant in bdec lq median uq udec {
				qui su `elast'`var' if `quant'M`var'==1
				scalar val`quant'`elast'`var' = r(mean)
			}
		}
	}
	
	foreach var in c l h {	
			foreach quant in bdec lq median uq udec {
				qui su F`var'_IR if `quant'M`var'==1
				scalar val`quant'F`var'_IR = r(mean)
		}
	}
	
	drop M uc ul ucc ull ucl term Fc Fl Fh Fc_IR Fl_IR Fh_IR riskaversion sigma
	
	calcfrisch, sigma(1)
	foreach var in c l h {	
			foreach quant in bdec lq median uq udec {
			qui su F`var' if `quant'M`var'==1
			scalar val`quant'F`var'_add = r(mean)
		}
	}
	
	foreach var in c l h {	
			foreach quant in bdec lq median uq udec {
			qui su F`var'_IR if `quant'M`var'==1
			scalar val`quant'F`var'_IR_add = r(mean)
		}
	}
	
	drop M uc ul ucc ull ucl term Fc Fl Fh Fc_IR Fl_IR Fh_IR riskaversion sigma
	drop Mh Ml Mc Hh Hl Hc term1 term2 Ih Il Ic mrs_* y 
	drop theta phi
	drop v psi* v_lb vi meanvi_ifemp meanvi_ifnoemp meansp_employed adjfactor newvi aterm alpha
	
	post postelasticities_cf $elastretlist

}

postclose postelasticities_cf

do mrseulerbootresults
