
use dataset2, clear

keep if inrange(sp_age,25,55)
keep if cohort_5yr==8|cohort_5yr==9

*Group-level correlation between wages and hours
preserve
collapse (mean) sp_w sp_hours if sp_employed==1, by(cohort sp_educ int_yr)
corr sp_w sp_hours
restore

log using "calibration50s.txt", text replace

*Weekly hours worked 
su sp_hours if sp_employed==1, det

*Net wages
su sp_w, det

*Participation Rate 
su sp_employed

*Participation Rate of Mothers 0-2 
su sp_employed if childage02>0

*Participation Rate Mothers with Children Aged 3-17
su sp_employed if childage02==0 & numdepchild>0

*Participation Rate Childless Women
su sp_employed if numdepchild==0

*Observed Wage Gender Gap 
egen meanspnetwage = mean(sp_w) if sp_employed==1
egen meanhnetwage  = mean(h_w) if h_employed==1

gen gendernetwageratio  = meanspnetwage/meanhnetwage
su gendernetwageratio 

egen meanspnetwage_byage = mean(sp_w) if sp_employed==1, by(sp_age)
egen meanhnetwage_byage = mean(h_w) if h_employed==1, by(sp_age)
gen gendernetwageratio_byage = meanspnetwage_byage/meanhnetwage_byage
 
*correlation between hours and wages 
preserve
collapse (mean) sp_w sp_hours lc if sp_employed==1, by(cohort sp_educ int_yr)
corr sp_w sp_hours
corr sp_hours lc
restore

*Life-cycle profiles for Figure 2
preserve

gen employment_childage02  = sp_employed if inrange(ageyoungestchild,0,2)
gen employment_childage317 = sp_employed if inrange(ageyoungestchild,3,17)
gen employment_nochildren  = sp_employed if childage03==0 & childage415==0 & childage1617==0

gen employment_youngmother = sp_employed if inrange(agefirstchild,21,25) & haskids==1
gen employment_oldmother = sp_employed if inrange(agefirstchild,26,30) & haskids==1	

replace sp_hours=. if sp_employed==0
replace sp_grosswage=. if sp_employed==0
replace h_logearn =. if h_logearn==0
collapse (mean) sp_hours sp_employed sp_lw h_logearn gendernetwageratio_byage employment_childage02 employment_childage317 employment_nochildren employment_youngmother employment_oldmother (sd) sdsp_lw = sp_lw sdh_logearn = h_logearn, by(sp_age)
gen varsp_lw = sdsp_lw^2
gen varh_logearn = sdh_logearn^2
outfile sp_age sp_hours sp_employed sp_lw h_logearn gendernetwageratio_byage varsp_lw varh_logearn employment_childage02 employment_childage317 employment_nochildren employment_youngmother employment_oldmother using "dataprofiles50s.out", wide replace

*Observed Variance Wage Growth (Women)
gen growthinvariance =varsp_lw-varsp_lw[_n-1]
su growthinvariance

*Observed Initial Variance of Wages (Women) 
su varsp_lw if sp_age==25

gen sp_wg=sp_lw-sp_lw[_n-1]
*Wage Growth (if younger than 40) 
su  sp_wg if sp_age<40
*Wage Growth (if older than 40) 
su  sp_wg if sp_age>=40

restore

log close

