
preserve

log using "changesinhours.txt", text replace

*Look at hours and wage changes using the panel component of the CEX
foreach var in sp_hours sp_w h_hours h_w sp_weeks h_weeks {

    qui gen `var'1temp = `var' if intno==2
    qui egen `var'1 = min(`var'1temp), by(uniqcuid)


    qui gen `var'2temp = `var' if intno==5
    qui egen `var'2 = min(`var'2temp), by(uniqcuid)

    qui gen d`var' = `var'2 - `var'1 if intno==5
    qui gen propchange`var' = `var'2/`var'1 if intno==5
    qui gen percentchange`var' = (`var'2/`var'1)-1 if intno==5

    qui gen change`var' = `var'2-`var'1 if intno==5 & `var'2>0 & `var'1>0

}

foreach var in sp_hours sp_w {

    gen absD`var'is0          = abs(change`var') == 0 if change`var'!=.
    gen absD`var'_05          = abs(change`var') >0 & abs(change`var') <=5  if change`var'!=.
    gen absD`var'_610         = inrange(abs(change`var'),6,10)  if change`var'!=.
    gen absD`var'_1120        = inrange(abs(change`var'),11,20) if change`var'!=.
    gen absD`var'_gt20        = abs(change`var')>20 if change`var'!=.

    gen abspercentD`var'is0          = abs(percentchange`var') == 0 if percentchange`var'!=.
    gen abspercentD`var'_05          = abs(percentchange`var') >0 & abs(percentchange`var') <=0.05  if percentchange`var'!=.
    gen abspercentD`var'_610         = abs(percentchange`var') >0.05 & abs(percentchange`var') <=0.10  if percentchange`var'!=.
    gen abspercentD`var'_1120        = abs(percentchange`var') >0.10 & abs(percentchange`var') <=0.20  if percentchange`var'!=.
    gen abspercentD`var'_gt20        = abs(percentchange`var') >0.20 if percentchange`var'!=.

}

*Table 2
su absDsp_hours* if sp_employed==1
gen nowagediff = abs(percentchangesp_w)<0.05 if percentchangesp_w!=.
gen hnowagediff = abs(percentchangeh_w)<0.05 if percentchangeh_w!=.

*Table 2 
su absDsp_hours* if sp_employed==1 & nowagediff==0
su absDsp_hours* if sp_employed==1 & nowagediff==1

gen sp_annhours = sp_weeks*sp_hours
gen lnsp_annhours = ln(sp_annhours)
gen lnsp_weeks = ln(sp_weeks)
gen lnsp_hours = ln(sp_hours)

*work out contributions of each term (use as in Shorrocks decomposition of inequality half of the covariance as counting towards the contribution).
forval e = 1/4 {

    qui corr lnsp_weeks lnsp_hours lnsp_annhours if intno==2 & sp_educ==`e' & yr==2012, cov
    matrix cov= r(C)
	
	matrix spweeksvar = cov[1,1]
	matrix sphoursvar = cov[2,2]
	matrix spannhoursvar = cov[3,3]
	matrix spcov = cov[1,2]
	
	matrix spvars_e`e' = spweeksvar,sphoursvar,spcov,spannhoursvar
    matrix colnames spvars_e`e' = weeks hours covariance annhours
	
}

forval e= 1/4 {
    matrix spvars = nullmat(spvars)\spvars_e`e'
}

qui corr lnsp_weeks lnsp_hours lnsp_annhours if intno==2 & yr==2012, cov
matrix cov= r(C)
matrix spweeksvar = cov[1,1]
matrix sphoursvar = cov[2,2]
matrix spannhoursvar = cov[3,3]
matrix spcov = cov[1,2]
	
matrix spvars_all = spweeksvar,sphoursvar,spcov,spannhoursvar 
matrix colnames spvars_all = weeks hours covariance annhours

matrix spvars = nullmat(spvars)\spvars_all
matrix rownames spvars = sp_educ1 sp_educ2 sp_educ3 sp_educ4 all

/*VARIANCES CONDITIONAL ON WORKING 52 WEEKS A YEAR*/
gen sp_weeks52 = sp_weeks==52
 
forval e = 1/4 {
    qui corr lnsp_weeks lnsp_hours lnsp_annhours if intno==2 & sp_educ==`e' & yr==2012 & sp_weeks52, cov
    matrix cov= r(C)
	
	matrix spweeksvar = cov[1,1]
	matrix sphoursvar = cov[2,2]
	matrix spannhoursvar = cov[3,3]
	matrix spcov = cov[1,2]
	
	matrix spvars_e`e' = spweeksvar,sphoursvar,spcov,spannhoursvar
    matrix colnames spvars_e`e' = weeks hours covariance annhours

}

forval e= 1/4 {
    matrix spvars52 = nullmat(spvars52)\spvars_e`e'
}

qui corr lnsp_weeks lnsp_hours lnsp_annhours if intno==2 & yr==2012 & sp_weeks52, cov
matrix cov= r(C)
matrix spweeksvar = cov[1,1]
matrix sphoursvar = cov[2,2]
matrix spannhoursvar = cov[3,3]
matrix spcov = cov[1,2]

matrix spvars_all = spweeksvar,sphoursvar,spcov,spannhoursvar 
matrix colnames spvars_all = weeks hours covariance annhours
	
matrix spvars52 = nullmat(spvars52)\spvars_all
matrix rownames spvars52 = sp_educ1 sp_educ2 sp_educ3 sp_educ4 all

/*VARIANCES CONDITIONAL ON WORKING 39-52 WEEKS A YEAR*/

gen sp_weeks3952 = inrange(sp_weeks,39,52)

forval e = 1/4 {
    qui corr lnsp_weeks lnsp_hours lnsp_annhours if intno==2 & sp_educ==`e' & yr==2012 & sp_weeks3952, cov
    matrix cov= r(C)

	matrix spweeksvar = cov[1,1]
	matrix sphoursvar = cov[2,2]
	matrix spannhoursvar = cov[3,3]
	matrix spcov = cov[1,2]
	
	matrix spvars_e`e' = spweeksvar,sphoursvar,spcov,spannhoursvar
    matrix colnames spvars_e`e' = weeks hours covariance annhours
	
}

forval e= 1/4 {
    matrix spvars3952 = nullmat(spvars3952)\spvars_e`e'
}

qui corr lnsp_weeks lnsp_hours lnsp_annhours if intno==2 & yr==2012  & sp_weeks3952, cov
matrix cov= r(C)
matrix spweeksvar = cov[1,1]
matrix sphoursvar = cov[2,2]
matrix spannhoursvar = cov[3,3]
matrix spcov = cov[1,2]
	
matrix spvars_all = spweeksvar,sphoursvar,spcov,spannhoursvar 
matrix colnames spvars_all = weeks hours covariance annhours

matrix spvars3952 = nullmat(spvars3952)\spvars_all
matrix rownames spvars3952 = sp_educ1 sp_educ2 sp_educ3 sp_educ4 all

*Table 1 (First Panel)
matrix list spvars
*Table 1 (Second Panel)
matrix list spvars3952
*Table 1 (Third Panel)
matrix list spvars52

log close

restore
