#delimit ;
set more off;
capture log close;

local appendyear = $startyear+1;
local expvars "newid spend_mo spend_yr int_mo int_yr datayear TOTAL TOTALNHOUSING TOTALNDUR EDUCATION MEDICAL healthin";
local demovars "newid cuid intno membno datayear head sex age bls_urbn reltohoh numadults white depchild numdepchild 
inc_hrsq incweekq int_yr int_mo employstat spouseinhh region salaryx salaryx_t fam_type ischild intearnx
finincx unemplx welfarex fssix secestx usbndx savacctx ckbkactx persot64 numpeople educ_summary earner ageyoungestchild incnonwk grospayx payperd cutenure occucode";

/*
/*define demovars differently according to year state only included from 96 onwards (also below)*/
forval Y = $startyear/$endyear {;

       use M:\CEX\derived_data\interview_survey\demogs\cexdemo`Y'q1, clear;
       keep `demovars';
       forval Q = 2/4 {;
          append using M:\CEX\derived_data\interview_survey\demogs\cexdemo`Y'q`Q', keep(`demovars');
          di "`Y' `Q'";
       };

       save "$datadir/dataset_demo`Y'", replace;
};

use "$datadir/dataset_demo$startyear", clear;

forval Y= `appendyear'/$endyear {;
    append using "$datadir/dataset_demo`Y'", keep(`demovars');
    di "`Y' `Q'";
};

sort datayear newid;
save "$datadir/dataset_demo", replace;

forval Y= $startyear/$endyear{;

        use M:\CEX\derived_data\interview_survey\exp\exp`Y'q1, clear;
        keep `expvars';

        forval Q = 2/4 {;
            append using M:\CEX\derived_data\interview_survey\exp\exp`Y'q`Q', keep(`expvars');
            di "`Y' `Q'";
        };

    save "$datadir/dataset_exp`Y'", replace;
};

use "$datadir/dataset_exp$startyear", clear;
forval Y= `appendyear'/$endyear {;
    append using "$datadir/dataset_exp`Y'";
};

sort datayear newid;
save "$datadir/dataset_exp", replace;
*/
use "$datadir/dataset_exp", clear;

gen intmo1 = int_mo==1|int_mo==4|int_mo==7|int_mo==10;
gen intmo2 = int_mo==2|int_mo==5|int_mo==8|int_mo==11;
gen intmo3 = int_mo==3|int_mo==6|int_mo==9|int_mo==12;

assert intmo1+intmo2+intmo3==1;

gen qtr = 1     if (spend_mo==1|spend_mo==2|spend_mo==3) & intmo1==1;
replace qtr = 2 if (spend_mo==4|spend_mo==5|spend_mo==6) & intmo1==1;
replace qtr = 3 if (spend_mo==7|spend_mo==8|spend_mo==9) & intmo1==1;
replace qtr = 4 if (spend_mo==10|spend_mo==11|spend_mo==12) & intmo1==1;

replace qtr = 1 if (spend_mo==11|spend_mo==12|spend_mo==1) & intmo2==1;
replace qtr = 2 if (spend_mo==2|spend_mo==3|spend_mo==4) & intmo2==1;
replace qtr = 3 if (spend_mo==5|spend_mo==6|spend_mo==7) & intmo2==1;
replace qtr = 4 if (spend_mo==8|spend_mo==9|spend_mo==10) & intmo2==1;

replace qtr = 1 if (spend_mo==12|spend_mo==1|spend_mo==2) & intmo3==1;
replace qtr = 2 if (spend_mo==3|spend_mo==4|spend_mo==5) & intmo3==1;
replace qtr = 3 if (spend_mo==6|spend_mo==7|spend_mo==8) & intmo3==1;
replace qtr = 4 if (spend_mo==9|spend_mo==10|spend_mo==11) & intmo3==1;

gen yr = spend_yr;
replace yr = spend_yr+1 if (spend_mo==11|spend_mo==12) & intmo2==1;
replace yr = spend_yr+1 if spend_mo==12 & intmo3==1;

gen nondurable = TOTALNDUR - MEDICAL -EDUCATION - healthin; /*excludes investment in human capital and medical spending*/
gen totalspend = TOTAL;

merge m:1 spend_yr spend_mo using "$datadir/deflator", keepusing(cpi);
drop if _merge~=3;
replace nondurable  = nondurable/(cpi/100);
replace totalspend  = totalspend /(cpi/100);

assert nondurable>=-0.1;
replace nondurable = 0 if nondurable<0;
egen cons = sum(nondurable), by(datayear newid);

label var cons "nondurable consumption in period covered by interview";

gen yrq = yr*10 + qtr;
sort datayear newid;
egen maxyrq = max(yrq), by(datayear newid);
keep if yrq==maxyrq;

bys datayear newid (spend_mo): gen count = _n;
keep if count==1;
drop count;

/*input minimum wages while we still have spend_mo spend_yr*/
/*
http://www.dol.gov/whd/minwage/chart.htm

Jan 1,  1978 $2.65 for all covered, nonexempt workers
Jan 1,  1979 $2.90 for all covered, nonexempt workers
Jan 1,  1980 $3.10 for all covered, nonexempt workers
Jan 1,  1981 $3.35 for all covered, nonexempt workers
Apr 1,  1990 $3.80 for all covered, nonexempt workers
Apr 1,  1991 $4.25 for all covered, nonexempt workers
Oct 1,  1996 $4.75 for all covered, nonexempt workers
Sep 1,  1997 $5.15 for all covered, nonexempt workers
Jul 24, 2007 $5.85 for all covered, nonexempt workers
Jul 24, 2008 $6.55 for all covered, nonexempt workers
Jul 24, 2009 $7.25 for all covered, nonexempt workers
*/

gen minwage = 2.90 if spend_yr==1979;
replace minwage = 3.10 if spend_yr==1980;
replace minwage = 3.35 if (spend_yr>=1981 & spend_yr<=1989)|(spend_yr==1990 & spend_mo<=3);
replace minwage = 3.80 if (spend_yr==1990 & spend_mo>=4)|(spend_yr==1991 & spend_mo<=3);
replace minwage = 4.25 if (spend_yr==1991 & spend_mo>=4)|(spend_yr>=1992 & spend_yr<=1995)|(spend_yr==1996 & spend_mo<=9);
replace minwage = 4.75 if (spend_yr==1996 & spend_mo>=10)|(spend_yr==1997 & spend_mo<=8);
replace minwage = 5.15 if (spend_yr==1997 & spend_mo>=9)|(spend_yr>=1998 & spend_yr<2007)|(spend_yr==2007 & spend_mo<=7);
replace minwage = 5.85 if (spend_yr==2007 & spend_mo>=8)|(spend_yr==2008 & spend_mo<=7);
replace minwage = 6.55 if (spend_yr==2008 & spend_mo>=8)|(spend_yr==2009 & spend_mo<=7);
replace minwage = 7.25 if (spend_yr==2009 & spend_mo>=8)|(spend_yr>2009);

assert minwage~=.;

keep newid datayear int_mo int_yr intmo1 intmo2 intmo3 qtr yr cons minwage;

merge m:m datayear newid using "$datadir/dataset_demo";

drop if _merge~=3;
drop _merge;

drop if bls_urbn==2;

gen yob = int_yr - age;
label var yob "year of birth";

gen elderly = persot64;
label var elderly "Number of people 65 or over";
assert elderly>=0;
gen fam_sz = numpeople;
label var fam_sz "Family size";
assert fam_sz>=0;

gen headspouse = reltohoh ==1;

egen headspousecount = sum(headspouse), by(datayear newid);
assert headspousecount <= 1;

egen headcount = sum(head), by(datayear newid);
assert headcount == 1;

drop if headspousecount==0;

gen sexheadtemp = sex if head==1;
gen sexspousetemp = sex if headspouse==1;

egen h_sex = min(sexheadtemp), by(datayear newid);
egen sp_sex = min(sexspousetemp), by(datayear newid);

gen swapsex = h_sex==2 & sp_sex==1;
gen newhead= swapsex==1 & headspouse==1;
gen newheadspouse= swapsex==1 & head==1;
replace head = newhead if swapsex==1;
replace headspouse= newheadspouse if swapsex==1;

drop newhead;
drop newheadspouse;

drop headspousecount;
drop headcount;
drop h_sex;
drop sp_sex;
drop sexheadtemp;
drop sexspousetemp;

egen headspousecount = sum(headspouse), by(datayear newid);
assert headspousecount == 1;

egen headcount = sum(head), by(datayear newid);
assert headcount == 1;

gen sexheadtemp = sex if head==1;
gen sexspousetemp = sex if headspouse==1;

egen h_sex = min(sexheadtemp), by(datayear newid);
label var h_sex "Sex of head";

egen sp_sex = min(sexspousetemp), by(datayear newid);
label var sp_sex "Sex of spouse";

/*drop same-sex couples*/
drop if h_sex==sp_sex;

/*check that all heads are men*/
assert h_sex==1;
assert sp_sex==2;

drop sexspousetemp;
drop sexheadtemp;

/*************************************************************************/
/*                     Working out hours and wages                       */
/*************************************************************************/

/*create unique cuid identifier - negative in years if the cuid is repeated*/
gen uniqcuid = cuid;
egen maxyear = max(datayear), by(uniqcuid);
replace uniqcuid = -1*uniqcuid if datayear~=maxyear & (datayear~=maxyear-1);

gen headcount2temp = 1 if head==1;
egen headcount2 = sum(headcount2temp), by(uniqcuid intno);
egen maxheadcount2 = max(headcount2), by(uniqcuid);
assert intno>2 if maxheadcount==2;
drop if maxheadcount2==2;
drop headcount2temp;
drop headcount2;
drop maxheadcount2;

forval i = 2/5 {;
    gen intno`i' = intno==`i';
    egen intcount`i' = sum(intno`i'), by(uniqcuid);
    bys uniqcuid: gen interview`i' = intcount`i'>0;
};


replace incweekq = 0 if incweekq<0|incweekq==.;
replace inc_hrsq = 0 if inc_hrsq<0|inc_hrsq==.;

replace salaryx=. if salaryx<0;
gen qsalary = salaryx*0.25;

gen salaryinint5temp =  qsalary if intno==5 & earner==1;
egen salaryinint5    = min(salaryinint5temp), by(uniqcuid membno);

assert salaryinint5 == . if interview5==0;

gen qhours = (incweekq*inc_hrsq)*0.25;

gen hoursinint5temp = qhours if intno==5 & earner==1;
replace hoursinint5temp = 0 if intno==5 & earner==2|earner==-1;
egen hoursinint5    = min(hoursinint5temp), by(uniqcuid membno);

gen wage = qsalary/qhours;
label var wage "Earnings/hours worked";

gen wageinint5 = salaryinint5/hoursinint5;
label var wageinint5 "Earnings/hours worked in 5th interview";

gen hoursunknown = qhours==0 & earner==1;
gen earnunknown = qsalary==. & earner==1;
gen wageunknown = wage==.;

gen  earnerinint5temp = earner==1 if intno==5;
egen earnerinint5     = min(earnerinint5temp), by(uniqcuid membno);

gen weeklyearn = grospayx/1 if payperd==1;
replace weeklyearn = grospayx/2 if payperd==2;
replace weeklyearn = grospayx/4 if payperd==3;
replace weeklyearn = grospayx/13 if payperd==4;
replace weeklyearn = grospayx/52 if payperd==5;
replace weeklyearn = grospayx/2 if payperd==6;
replace weeklyearn = . if payperd==7;
replace weeklyearn = . if grospayx<0;
label var weeklyearn "Measure of wages calculated using GROSPAYX";

gen weeklyhours = inc_hrsq;
replace weeklyhours =0 if earner==2|earner==-1;
label var weeklyhours "Typical hours worked per week";

gen weeklywage = weeklyearn/inc_hrsq;
label var weeklywage "Wage variable calculated using GROSPAYX";

gen weeklyhoursinint5temp = inc_hrsq if intno==5;
egen weeklyhoursinint5 = min(weeklyhoursinint5temp), by(uniqcuid membno);

gen weeklyearninint5temp = weeklyearn if intno==5;
egen weeklyearninint5 = min(weeklyearninint5temp), by(uniqcuid membno);

gen weeklywageinint5 = weeklyearninint5/weeklyhoursinint5;
label var weeklywageinint5 "Wage variable calculated using GROSPAYX in 5th interview";

assert incnonwk!=.;
assert incnonwk==-1 if earner==1;

/*non-participant is someone who was retired, ill, studying, taking care of home or doing something else as the main reason they
 * did not work over the last 12 months*/
gen nonparticipant = inlist(incnonwk,1,2,3,4,6);
gen participantinint5temp = 1- nonparticipant if intno==5;
egen participantinint5 = min(participantinint5temp), by(uniqcuid membno);
assert participantinint5 == 1 if earnerinint5==1|(earnerinint5==0 & incnonwk==5 & intno==5);

gen incnonwkinint5temp = incnonwk if intno==5;
egen incnonwkinint5 = min(incnonwkinint5temp), by(uniqcuid membno);

gen sp_ageinint5temp = age if headspouse==1 & intno==5;
egen sp_ageinint5 = min(sp_ageinint5), by(uniqcuid);

/*children*/
/*depchild is less than 18 not in full time work (and between 16 and 18) and a child of the household head*/
gen childinint5 = depchild==1 if intno==5;
gen childage1inint5temp = childinint5==1 & age<=2;
egen childage02inint5 = sum(childage1inint5temp), by(uniqcuid);
label var childage02inint5 "# of children under 2 in 5th interview";

gen childage2inint5temp = childinint5==1 & age>=3 & age<=15 & intno==5;
egen childage315inint5 = sum(childage2inint5temp), by(uniqcuid);
label var childage315inint5 "# of children between 3 and 15 in 5th interview";

gen childage3inint5temp = childinint5==1 & age>=16 & intno==5;
egen childage1617inint5 = sum(childage3inint5temp), by(uniqcuid);
label var childage1617inint5 "# of children over 16 in 5th interview";

count;
merge 1:1 datayear newid membno using "U:\peter_l\labour_supply\taxsim\data\taxsimall";
drop if _merge==2;
count;

gen sp_frateinint5temp = frate if headspouse==1 & intno==5;
egen sp_frateinint5 = min(sp_frateinint5temp), by(uniqcuid);
label var sp_frateinint5 "marginal tax rate in 5th interview for the spouse";
drop sp_frateinint5temp;

gen h_frateinint5temp = frate if head==1 & intno==5;
egen h_frateinint5 = min(h_frateinint5temp), by(uniqcuid);
label var h_frateinint5 "marginal tax rate in 5th interview for the head";
drop h_frateinint5temp;

drop frate srate _merge;
drop spend_yr newcu_code;

keep if intno==2|intno==5;

/******************************************************************************************************/

/*employment*/

gen slfemployed = employstat==2 & earner==1;
gen headslfemptemp = slfemployed if head==1;
egen h_slfemploy = min(headslfemptemp), by(datayear newid);
label var h_slfemploy "household head is self-employed";
drop headslfemptemp;

gen spouseslfemptemp = slfemployed if headspouse==1;
egen sp_slfemploy = min(spouseslfemptemp), by(datayear newid);
label var sp_slfemploy "Spouse is self-employed";
drop spouseslfemptemp;

gen employed =  earner==1;
gen heademptemp = employed if head==1;
egen h_employed = min(heademptemp), by(datayear newid);
label var h_employed "household head is currently employed (EARNER)";
drop heademptemp;

gen spouseemptemp = employed if headspouse==1;
egen sp_employed = min(spouseemptemp), by(datayear newid);
label var sp_employed "Spouse is currently employed (EARNER)";
drop spouseemptemp;

gen employedinint5 =  earnerinint5==1;
replace employedinint5 =. if earnerinint5==.;
gen headempinint5temp = employedinint5  if head==1;
egen h_employedinint5  = min(headempinint5 temp), by(datayear newid);
label var h_employedinint5  "household head employed in the 5th interview (EARNER)";
drop headempinint5temp;

gen spouseempinint5temp = employedinint5  if headspouse==1;
egen sp_employedinint5  = min(spouseempinint5temp), by(datayear newid);
label var sp_employedinint5  "Spouse is employed in the 5th interview (EARNER)";
drop spouseempinint5temp;

gen headparticipanttemp = 1-nonparticipant if head==1;
egen h_participant= min(headparticipanttemp), by(datayear newid);
label var h_participant "Household head considered a participant";
drop headparticipanttemp;

gen headparticipantinint5temp = participantinint5 if head==1;
egen h_participantinint5 = min(headparticipantinint5temp), by(datayear newid);
label var h_participantinint5 "household head considered a participant in the fifth interview";
drop headparticipantinint5temp;

gen spouseparticipanttemp = 1- nonparticipant if headspouse==1;
egen sp_participant = min(spouseparticipanttemp), by(datayear newid);
label var sp_participant "Spouse considered a participant";
drop spouseparticipanttemp;

gen spouseparticipantinint5temp = participantinint5 if headspouse==1;
egen sp_participantinint5 = min(spouseparticipantinint5temp), by(datayear newid);
label var sp_participantinint5 "Spouse considered a participant in fifth interview";
drop spouseparticipantinint5temp;

assert sp_participant ==1 if sp_employed==1;
assert sp_participantinint5 == 1 if sp_employedinint5==1;

assert h_participant ==1 if h_employed==1;
assert h_participantinint5 == 1 if h_employedinint5==1;

assert h_participant >= h_employed;
assert sp_participant >= sp_employed;
assert sp_participantinint5 >= sp_employedinint5;
assert h_participantinint5 >= h_employedinint5;

gen spouseincnonwkinint5temp = incnonwkinint5 if headspouse==1;
egen sp_whynowrkinint5 = min(spouseincnonwkinint5temp), by(datayear newid);
label var sp_whynowrkinint5 "Main reason spouse didn't work in last 12 months (from fifth interview)";
label values sp_whynowrkinint5 incnonwk;

gen spouseincnonwktemp = incnonwk if headspouse==1;
egen sp_whynowrk = min(spouseincnonwktemp), by(datayear newid);
label var sp_whynowrk "Main reason spouse didn't work in last 12 months";
label values sp_whynowrk incnonwk;

gen earntc = salaryx_t==1;

gen spouse_wagetemp = wage if headspouse ==1;
egen sp_wage = min(spouse_wagetemp), by(datayear newid);
label var sp_wage "Hourly wage of spouse";
drop spouse_wagetemp;

gen spouse_weeklywagetemp = weeklywage if headspouse ==1;
egen sp_weeklywage = min(spouse_weeklywagetemp), by(datayear newid);
label var sp_weeklywage "Hourly wage of spouse (from GROSPAYX)";
drop spouse_weeklywagetemp;

gen spouseearntctemp = earntc if headspouse==1;
egen sp_earntc = min(spouseearntctemp), by(datayear newid);
drop spouseearntctemp;
label define sp_earntc
0 "not topcoded"
1 "topcoded";
label values sp_earntc sp_earntc;
label var sp_earntc "Spouse's earnings top coded";

gen head_wagetemp = wage if head ==1;
egen h_wage = min(head_wagetemp), by(datayear newid);
label var h_wage "Annual hourly wage of head";
drop head_wagetemp;
drop headcount;

gen head_weeklywagetemp = weeklywage if head ==1;
egen h_weeklywage = min(head_weeklywagetemp), by(datayear newid);
label var h_weeklywage "Hourly wage of head (from GROSPAYX)";
drop head_weeklywagetemp;

gen headearntctemp = earntc if head==1;
egen h_earntc = min(headearntctemp), by(datayear newid);
drop headearntctemp;
label define h_earntc
0 "not topcoded"
1 "topcoded";
label values h_earntc h_earntc;
label var h_earntc "head's earnings top coded";

gen spouse_earntemp = qsalary if headspouse ==1;
replace spouse_earntemp = . if earnunknown==1;
egen sp_earn = min(spouse_earntemp), by(datayear newid);
label var sp_earn "Earnings of spouse";
drop spouse_earntemp;

gen head_earntemp = qsalary if head ==1;
replace head_earntemp = . if earnunknown==1;
egen h_earn = min(head_earntemp), by(datayear newid);
label var h_earn "Earnings of head";
drop head_earntemp;

gen head_earninint5temp = salaryinint5 if head ==1;
replace head_earninint5temp = . if earnunknown==1;
egen h_earninint5 = min(head_earninint5temp), by(datayear newid);
label var h_earninint5 "Earnings of head in 5th interview";
drop head_earninint5temp;

gen spouse_earninint5temp = salaryinint5 if headspouse ==1;
replace spouse_earninint5temp = . if earnunknown==1;
egen sp_earninint5 = min(spouse_earninint5temp), by(datayear newid);
label var sp_earninint5 "Earnings of spouse in 5th interview";
drop spouse_earninint5temp;

gen head_wageinint5temp = wageinint5 if head ==1;
egen h_wageinint5 = min(head_wageinint5temp), by(datayear newid);
label var h_wageinint5 "Hourly wage of head in 5th interview";
drop head_wageinint5temp;

gen spouse_wageinint5temp = wageinint5 if headspouse ==1;
egen sp_wageinint5 = min(spouse_wageinint5temp), by(datayear newid);
label var sp_wageinint5 "Hourly wage of spouse in 5th interview";
drop spouse_wageinint5temp;

gen head_weeklywageinint5temp = weeklywageinint5 if head ==1;
egen h_weeklywageinint5 = min(head_weeklywageinint5temp), by(datayear newid);
label var h_weeklywageinint5 "Hourly wage of head in 5th interview";
drop head_weeklywageinint5temp;

gen spouse_weeklywageinint5temp = weeklywageinint5 if headspouse ==1;
egen sp_weeklywageinint5 = min(spouse_weeklywageinint5temp), by(datayear newid);
label var sp_weeklywageinint5 "Hourly wage of spouse in 5th interview";
drop spouse_weeklywageinint5temp;

gen spouse_hourstemp = qhours if headspouse==1;
egen sp_hours = min(spouse_hourstemp), by(datayear newid);
label var sp_hours "Hours of spouse";
drop spouse_hourstemp;

gen spouse_weekstemp = incweekq if headspouse==1;
egen sp_weeks = min(spouse_weekstemp), by(datayear newid);
label var sp_weeks "Weeks worked of spouse";
drop spouse_weekstemp;

gen spouse_weeklyhourstemp = weeklyhours if headspouse==1;
egen sp_weeklyhours = min(spouse_weeklyhourstemp), by(datayear newid);
label var sp_weeklyhours "Hours of spouse (alt definition)";
drop spouse_weeklyhourstemp;

gen head_hourstemp = qhours if head==1;
egen h_hours = min(head_hourstemp), by(datayear newid);
label var h_hours "Hours of head";
drop head_hourstemp;

gen head_hoursinint5temp = hoursinint5 if head==1;
egen h_hoursinint5 = min(head_hoursinint5temp), by(datayear newid);
label var h_hoursinint5 "Hours of head in 5th interview";
drop head_hoursinint5temp;

gen head_weeklyhourstemp = weeklyhours if head==1;
egen h_weeklyhours = min(head_weeklyhourstemp), by(datayear newid);
label var h_weeklyhours "Hours of head (alt definition)";
drop head_weeklyhourstemp;

gen head_weekstemp = incweekq if head==1;
egen h_weeks = min(head_weekstemp), by(datayear newid);
label var h_weeks "Weeks worked of head";
drop head_weekstemp;

gen head_weeklyhoursinint5temp = weeklyhoursinint5 if head==1;
egen h_weeklyhoursinint5 = min(head_weeklyhoursinint5temp), by(datayear newid);
label var h_weeklyhoursinint5 "Hours of head in 5th interview (weekly measure)";
drop head_weeklyhoursinint5temp;

gen spouse_hoursinint5temp = hoursinint5 if headspouse ==1;
egen sp_hoursinint5 = min(spouse_hoursinint5temp), by(datayear newid);
label var sp_hoursinint5 "Hours of spouse in 5th interview";
drop spouse_hoursinint5temp;

gen spouse_weeklyhoursinint5temp = weeklyhoursinint5 if headspouse ==1;
egen sp_weeklyhoursinint5 = min(spouse_weeklyhoursinint5temp), by(datayear newid);
label var sp_weeklyhoursinint5 "Hours of spouse in 5th interview (weekly measure)";
drop spouse_weeklyhoursinint5temp;

/****************************************************************************************************/

gen ageheadtemp = age if head==1;
egen h_age = min(ageheadtemp), by(datayear newid);
label var h_age "Age of head";
drop ageheadtemp;

gen raceheadtemp = white if head==1;
egen h_race = min(raceheadtemp), by(datayear newid);
label define h_race
0 "Non-white"
1 "White";
label values h_race h_race;
label var h_race h_race;
drop raceheadtemp;

gen agespousetemp = age if headspouse==1;
egen sp_age = min(agespousetemp), by(datayear newid);
label var sp_age "Age of spouse";
drop agespousetemp;
gen sexspousetemp = sex if headspouse==1;

gen racespousetemp = white if headspouse==1;
egen sp_race = min(racespousetemp), by(datayear newid);
label define sp_race
0 "Non-white"
1 "White";
label values sp_race sp_race;
label var sp_race "Race of spouse";
drop racespousetemp;

/*children*/
/*depchild is less than 18 not in full time work (and between 16 and 18) and a child of the household head*/
gen child = depchild==1;
gen childage1temp = child==1 & age<=2;
egen childage02 = sum(childage1temp), by(datayear newid);
label var childage02 "# of children under 2";
drop childage1temp;
gen childage2temp = child==1 & age>=3 & age<=15;
egen childage315 = sum(childage2temp), by(datayear newid);
label var childage315 "# of children between 3 and 15";
drop childage2temp;
gen childage3temp = child==1 & age>=16;
egen childage1617 = sum(childage3temp), by(datayear newid);
label var childage1617 "# of children over 16";
drop childage3temp;
gen childage4temp = child==1 & age<=3;
egen childage03 = sum(childage4temp), by(datayear newid);
label var childage03 "# of children under 3";
drop childage4temp;
gen childage5temp = child==1 & age>=4 & age<=15;
egen childage415 = sum(childage5temp), by(datayear newid);
label var childage415 "# of children between 4 and 15";
drop childage5temp;

assert childage02 + childage315 + childage1617 == numdepchild;
assert childage03 + childage415 + childage1617 == numdepchild;

gen educ1 = educ_summary <=1;
gen educ2 = educ_summary ==2;
gen educ3 = educ_summary ==3;
gen educ4 = educ_summary >=4;

gen h_educ1temp = educ1 if head==1;
gen h_educ2temp = educ2 if head==1;
gen h_educ3temp = educ3 if head==1;
gen h_educ4temp = educ4 if head==1;

egen h_educ1 = min(h_educ1temp), by(datayear newid);
egen h_educ2 = min(h_educ2temp), by(datayear newid);
egen h_educ3 = min(h_educ3temp), by(datayear newid);
egen h_educ4 = min(h_educ4temp), by(datayear newid);

label var h_educ1 "Husband high school dropout";
label var h_educ2 "Husband high school";
label var h_educ3 "Husband some college";
label var h_educ4 "Husband completed college";

gen h_educ = 1 if h_educ1==1;
replace h_educ = 2 if h_educ2==1;
replace h_educ = 3 if h_educ3==1;
replace h_educ = 4 if h_educ4==1;

drop h_educ1temp;
drop h_educ2temp;
drop h_educ3temp;
drop h_educ4temp;

assert h_educ1+h_educ2+h_educ3+h_educ4==1;

gen sp_educ1temp = educ1 if headspouse==1;
gen sp_educ2temp = educ2 if headspouse==1;
gen sp_educ3temp = educ3 if headspouse==1;
gen sp_educ4temp = educ4 if headspouse==1;

egen sp_educ1 = min(sp_educ1temp), by(datayear newid);
egen sp_educ2 = min(sp_educ2temp), by(datayear newid);
egen sp_educ3 = min(sp_educ3temp), by(datayear newid);
egen sp_educ4 = min(sp_educ4temp), by(datayear newid);

label var sp_educ1 "Wife high school dropout";
label var sp_educ2 "Wife high school";
label var sp_educ3 "Wife some college";
label var sp_educ4 "Wife completed college";

gen sp_educ = 1 if sp_educ1==1;
replace sp_educ = 2 if sp_educ2==1;
replace sp_educ = 3 if sp_educ3==1;
replace sp_educ = 4 if sp_educ4==1;

drop sp_educ1temp;
drop sp_educ2temp;
drop sp_educ3temp;
drop sp_educ4temp;

assert sp_educ1+sp_educ2+sp_educ3+sp_educ4==1;

gen temp                  = age if depchild==1;
egen ageoldestdepchild    = max(temp), by(datayear newid);
drop temp;

gen agefirstchild = sp_age-ageoldestdepchild;
replace agefirstchild =. if ageoldestdepchild<0;
label var agefirstchild "Spouse age minus age of oldest child";

local numcohorts = 14;

forval j = 1/`numcohorts' {;
    local c`j' = 1915 + 5*`j';
    };

gen spousecohorttemp = 1 if yob<=`c1'& headspouse==1;

forval i = 2/14 {;
    local x = `i'-1;
    replace spousecohorttemp = `i' if yob>`c`x'' & yob<=`c`i'' & headspouse==1;
};

egen cohort = min(spousecohorttemp), by(datayear newid);
drop spousecohorttemp;

label define cohort
1  "`c1'"
2  "`c1'-`c2'"
3  "`c2'-`c3'"
4  "`c3'-`c4'"
5  "`c4'-`c5'"
6  "`c5'-`c6'"
7  "`c6'-`c7'"
8  "`c7'-`c8'"
9  "`c8'-`c9'"
10 "`c9'-`c10'"
11 "`c10'-`c11'"
12 "`c11'-`c12'"
13 "`c12'-`c13'"
14 "`c13'-`c14'";
label values cohort cohort;
label var cohort "Cohort of spouse";

forval i = 1/`numcohorts' {;
        gen mpcohort`i' = `c`i'' - 3;
};

gen cohortage = -999;
forval i = 1/`numcohorts' {;
    replace cohortage = yr - mpcohort`i' if cohort==`i';
};

drop mpcohort*;

drop if h_age<25;
drop if h_age>65;

compress;

merge 1:1 datayear newid membno using "U:\peter_l\labour_supply\taxsim\data\taxsimall";

tab datayear _merge;
drop if _merge~=3;

/*federal*/
gen h_fratetemp = frate if head==1;
egen h_frate = min(h_fratetemp), by(datayear newid);
label var h_frate "marginal federal income tax of head";
drop h_fratetemp;

gen sp_fratetemp = frate if headspouse==1;
egen sp_frate = min(sp_fratetemp), by(datayear newid);
label var sp_frate "marginal federal income tax of spouse";
drop sp_fratetemp;

keep if headspouse==1; 

keep datayear newid uniqcuid int_mo int_yr cuid intno yr qtr h_sex sp_sex sp_wage h_wage sp_weeklywage h_weeklywage h_employed sp_employed
h_slfemploy sp_slfemploy h_age sp_age h_race sp_race numadults childage02 childage03 childage315 childage415 childage1617 numdepchild region
h_educ h_educ1 h_educ2 h_educ3 h_educ4 sp_educ sp_educ1 sp_educ2 sp_educ3 sp_educ4 state h_earntc sp_earntc h_hours sp_hours
fam_type h_earn sp_earn h_frate sp_frate cohort cohortage cons elderly fam_sz
intmo1 intmo2 intmo3 minwage agefirstchild sp_wageinint5 h_wageinint5 sp_weeklywageinint5 h_weeklywageinint5
sp_earninint5 h_earninint5 h_hoursinint5 sp_hoursinint5 h_weeklyhoursinint5 sp_weeklyhoursinint5
sp_employedinint5 h_employedinint5 sp_ageinint5 childage02inint5 childage315inint5 childage1617inint5 sp_frateinint5 h_frateinint5
sp_participant sp_participantinint5 
sp_weeklyhours h_weeklyhours ageyoungestchild h_weeks sp_weeks sp_whynowrk sp_whynowrkinint5;

/*************************************************************************/
/*                            Merging in prices                          */
/*************************************************************************/

/*we need to merge in different quarterly averages for people interviewed in different months*/
gen yr1 = yr if intmo1==1;
gen yr2 = yr if intmo2==1;
gen yr3 = yr if intmo3==1;

gen qtr1 = qtr if intmo1==1;
gen qtr2 = qtr if intmo2==1;
gen qtr3 = qtr if intmo3==1;

merge m:1 yr1 qtr1 using "$datadir/deflator1", keepusing(cpi_qtr1);
drop if _merge==2;
drop _merge;
merge m:1 yr2 qtr2 using "$datadir/deflator2", keepusing(cpi_qtr2);
drop if _merge==2;
drop _merge;
merge m:1 yr3 qtr3 using "$datadir/deflator3", keepusing(cpi_qtr3);
drop if _merge==2;
drop _merge;

bys datayear newid: gen n = _N;
tab n;
bys datayear newid: keep if _n==1;
tab n;
drop n;

gen cpi = cpi_qtr1 if intmo1==1;
replace cpi = cpi_qtr2 if intmo2==1;
replace cpi = cpi_qtr3 if intmo3==1;

assert cpi~=.;

merge m:1 yr1 qtr1 using "$datadir/deflator1_forint5", keepusing(cpi_qtr1forint5);
drop if _merge==2;
drop _merge;
merge m:1 yr2 qtr2 using "$datadir/deflator2_forint5", keepusing(cpi_qtr2forint5);
drop if _merge==2;
drop _merge;
merge m:1 yr3 qtr3 using "$datadir/deflator3_forint5", keepusing(cpi_qtr3forint5);
drop if _merge==2;
drop _merge;

bys datayear newid: gen n = _N;
tab n;
bys datayear newid: keep if _n==1;
tab n;
drop n;

gen     cpi_forint5 = cpi_qtr1forint5 if intmo1==1;
replace cpi_forint5 = cpi_qtr2forint5 if intmo2==1;
replace cpi_forint5 = cpi_qtr3forint5 if intmo3==1;

assert cpi_forint5~=.;

sort yr qtr;

foreach var in h_wage sp_wage h_weeklywage sp_weeklywage h_earn sp_earn minwage {;
    replace `var' = `var'/(cpi/100);

};

foreach var in h_weeklywageinint5 sp_weeklywageinint5 h_wageinint5 sp_wageinint5 h_earninint5 sp_earninint5 {;
    replace `var' = `var'/(cpi_forint5/100);

};

drop cpi_forint5 cpi_qtr1forint5 cpi_qtr2forint5 cpi_qtr3forint5;

/*************************************************************************/
/*                      Merging in interest rates                        */
/*************************************************************************/

merge m:1 yr1 qtr1 using "$datadir/rr1", keepusing(rr_qtr1 ir_qtr1 pi_qtr1);
drop if _merge==2;
drop _merge;
merge m:1 yr2 qtr2 using "$datadir/rr2", keepusing(rr_qtr2 ir_qtr2 pi_qtr2);
drop if _merge==2;
drop _merge;
merge m:1 yr3 qtr3 using "$datadir/rr3", keepusing(rr_qtr3 ir_qtr3 pi_qtr3);
drop if _merge==2;
drop _merge;

bys datayear newid: gen n = _N;
tab n;
bys datayear newid: keep if _n==1;
tab n;
drop n;

gen rr = rr_qtr1 if intmo1==1 ;
replace rr = rr_qtr2 if intmo2==1;
replace rr = rr_qtr3 if intmo3==1;

assert rr~=.;

gen ir = ir_qtr1 if intmo1==1 ;
replace ir = ir_qtr2 if intmo2==1;
replace ir = ir_qtr3 if intmo3==1;

assert ir~=.;

gen pi = pi_qtr1 if intmo1==1 ;
replace pi = pi_qtr2 if intmo2==1;
replace pi = pi_qtr3 if intmo3==1;

assert pi~=.;

keep datayear newid uniqcuid int_mo int_yr cuid intno yr qtr h_sex sp_sex sp_wage h_wage sp_weeklywage h_weeklywage h_employed sp_employed
h_slfemploy sp_slfemploy h_age sp_age h_race sp_race numadults childage02 childage03 childage315 childage415 childage1617 numdepchild region
h_educ h_educ1 h_educ2 h_educ3 h_educ4 sp_educ sp_educ1 sp_educ2 sp_educ3 sp_educ4 h_earntc sp_earntc h_hours sp_hours
fam_type h_earn sp_earn h_frate sp_frate cohort cohortage cons elderly fam_sz
intmo1 intmo2 intmo3 rr ir pi cpi minwage agefirstchild sp_wageinint5 h_wageinint5 sp_weeklywageinint5 h_weeklywageinint5 sp_earninint5 h_earninint5 h_hoursinint5 sp_hoursinint5
h_weeklyhoursinint5 sp_weeklyhoursinint5 sp_employedinint5 h_employedinint5 sp_ageinint5 childage02inint5 childage315inint5 childage1617inint5 sp_frateinint5 h_frateinint5 sp_participant sp_participantinint5 
sp_weeklyhours h_weeklyhours ageyoungestchild h_weeks sp_weeks sp_whynowrk sp_whynowrkinint5;

drop if sp_age <25|sp_age>60;
drop if sp_slfemploy==1|h_slfemploy==1;
drop if sp_earntc==1;
drop if cohort==.;
bys yr qtr cohort: gen count = _N;
drop count;
drop if sp_wage==. & sp_employed==1;
drop if sp_wage==0 & sp_employed==1;
drop if h_wage==. & h_employed==1;
drop if h_wage==0 & h_employed==1;

drop if cons==0;

gen qtr1 = qtr==1;
gen qtr2 = qtr==2;
gen qtr3 = qtr==3;
gen qtr4 = qtr==4;

gen intmo = 1 if intmo1==1;
replace intmo=2 if intmo2==1;
replace intmo=3 if intmo3==1;

gen season1 = intmo==1 & qtr==1;
gen season2 = intmo==2 & qtr==1;
gen season3 = intmo==3 & qtr==1;
gen season4 = intmo==1 & qtr==2;
gen season5 = intmo==2 & qtr==2;
gen season6 = intmo==3 & qtr==2;
gen season7 = intmo==1 & qtr==3;
gen season8 = intmo==2 & qtr==3;
gen season9 = intmo==3 & qtr==3;
gen season10 = intmo==1 & qtr==4;
gen season11 = intmo==2 & qtr==4;
gen season12 = intmo==3 & qtr==4;

gen lnfam_sz = ln(fam_sz);

gen sp_leisure = (1250-sp_hours); 
gen h_leisure = (1250-h_hours);
gen h_lleis = ln(h_leisure);
gen h_lhours = ln(h_hours);
gen h_lleis2 = (h_lleis)^2;
gen sp_lleis = ln(sp_leisure);
gen sp_lhours = ln(sp_hours);
gen sp_lleis2 = (sp_lleis)^2;
gen h_w  = h_wage*(1-(h_frate/100));
label var h_w "marginal net wage of head";
gen h_lw = ln(h_w);
gen sp_w  = sp_wage*(1-(sp_frate/100));
label var sp_w "marginal net wage of spouse";
gen sp_lw = ln(sp_w);

gen h_weeklynetwage  = h_weeklywage*(1-(h_frate/100));
label var h_weeklynetwage  "marginal net wage of head (alt. definition)";
gen h_lnweeklynetwage = ln(h_weeklynetwage);
gen sp_weeklynetwage  = sp_weeklywage*(1-(sp_frate/100));
label var sp_weeklynetwage "marginal net wage of spouse (alt. definition)";
gen sp_lnweeklynetwage = ln(sp_weeklynetwage);

rename sp_wage sp_grosswage;
rename sp_wageinint5 sp_grosswageinint5;
rename h_wage h_grosswage;
rename h_wageinint5 h_grosswageinint5;
gen sp_netwageinint5 = sp_grosswageinint5*(1-(sp_frateinint5/100));
gen h_netwageinint5  = h_grosswageinint5*(1-(h_frateinint5/100));

gen sp_age2 = (sp_age)^2;
gen sp_age3 = (sp_age)^3;
gen h_age2 = (h_age)^2;
gen h_age3 = (h_age)^3;
gen sp_logearn  = log(sp_earn);
label var sp_logearn "Log gross earnings spouse";
gen h_logearn   = log(h_earn);
label var h_logearn "Log gross earnings husband";

drop h_frate sp_frate;
drop sp_earntc h_earntc;
drop fam_sz;
drop if region==.;

save "dataset", replace;
