set more off

/**************** DEFINE PROGRAMS *************************/


cap program drop runmrs
program runmrs

    args first

    cap drop ee ee2 fi
    cap drop e1 e2 e2minusone e3
    cap drop vbar v
    cap drop mrs_*
    cap matrix drop mrs
    cap drop phi theta
    cap drop term1 term2
    cap drop Mc Mh Ml Hc Hh Hl Ic Ih Il
    cap drop y

    qui xi: probit sp_employed h_logearn h_logearnflag $dem_cntrl $oth_cntrl $instlist $yeardummies, difficult iterate(50)
    if "`first'"=="first" {
        estimates store probit
        esttab probit using "probit_results_tex.tex", replace b(%9.3f) se(%9.3f) tex keep(h_logearn h_logearnflag $dem_cntrl $oth_cntrl) wide
    }

    scalar probitconverged = e(converged)
    if probitconverged==0 {
        di in red "Probit didn't converge"
    }

    predict ee,index
    qui gen ee2=ee*ee
    qui gen fi=1/normprob(ee)
    qui gen e1 = fi/(sqrt(6.2831853)*exp(0.5*ee2))
    qui gen e2 = fi*ee/(sqrt(6.2831853)*exp(0.5*ee2)) + 1
    qui gen e2minusone = e2-1
    qui gen e3 =(2+ee2)*fi/(sqrt(6.2831853)*exp(0.5*ee2))

    xi: ivreg2 sp_lw e1 e2minusone e3 $mrs_cntrl (sp_lleis lc = $instlist) if sp_employed==1, fuller(1) `first'

end

capture program drop genmrsvars
program define genmrsvars

    matrix define mrs_= e(b)
    svmat mrs_, name(matcol)

    qui gen theta  =  mrs_sp_lleis[1] +1
    qui gen phi    = - mrs_lc[1]+1

    scalar theta = theta[1]
    scalar phi   = phi[1]

    qui reg sp_lw $mrs_cntrl $instlist if sp_employed==1
    qui predict lnimpwage, xb
	
end

capture program drop genmarshallandhicks
program define genmarshallandhicks

    qui gen y=cons+sp_w*sp_leisure

    /*marshallian*/
    qui gen term1=theta-1 +(phi-1)*sp_w*sp_leisure/cons
    qui gen Mc=(1/term1)*((theta-1)*sp_w*($T-sp_leisure)/cons  -sp_w*sp_leisure/cons)
    qui gen Ml=(1/term1)*((phi-1)*sp_w*($T-sp_leisure)/cons  + 1)
    qui gen Mh=-Ml*sp_leisure/($T-sp_leisure)

    /*hicksian*/
    qui gen term2=-(phi-1)*sp_w*sp_leisure/cons - (theta-1)
    qui gen Ic=-y*(theta-1)/(cons*term2)
    qui gen Il=-y*(phi-1)/(cons*term2)
    qui gen Ih=-Il*sp_leisure/($T-sp_leisure)
    qui gen Hc=Mc - Ic*($T-sp_leisure)*sp_w/y
    qui gen Hl=Ml - Il*($T-sp_leisure)*sp_w/y
    qui gen Hh=-Hl*sp_leisure/($T-sp_leisure)

end

cap program drop runmrs_alternatives
program runmrs_alternatives

    syntax, depvar(varname) indepvars(varlist) method(string) [first]

    cap drop ee ee2 fi
    cap drop e1 e2 e2minusone e3
    cap drop vbar v
    cap drop mrs_*
    cap matrix drop mrs
    cap drop phi theta
    cap drop term1 term2
    cap drop Mc Mh Ml Hc Hh Hl Ic Ih Il
    cap drop y

    qui xi: probit sp_employed h_logearn h_logearnflag $dem_cntrl $oth_cntrl $instlist $yeardummies, difficult iterate(50)

    scalar probitconverged = e(converged)
    if probitconverged==0 {
        di in red "Probit didn't converge"
    }

    predict ee,index
    qui gen ee2=ee*ee
    qui gen fi=1/normprob(ee)
    qui gen e1 = fi/(sqrt(6.2831853)*exp(0.5*ee2))
    qui gen e2 = fi*ee/(sqrt(6.2831853)*exp(0.5*ee2)) + 1
    qui gen e2minusone = e2-1
    qui gen e3 =(2+ee2)*fi/(sqrt(6.2831853)*exp(0.5*ee2))

    xi: ivreg2 `depvar' e1 e2minusone e3 $mrs_cntrl (`indepvars' = $instlist) if sp_employed==1 , `method' `first'

    matrix define mrs_= e(b)
    svmat mrs_, name(matcol)

    if "`depvar'"=="sp_lleis" {
        qui gen theta  = (1/mrs_sp_lw[1]) +1
        qui gen phi    = ((1/mrs_sp_lw[1])*mrs_lc[1])+1
    }

    if "`depvar'"=="sp_lw" {
        qui gen theta  =  mrs_sp_lleis[1] +1
        qui gen phi    = - mrs_lc[1]+1
    }

    if "`depvar'"=="lc" {
        qui gen phi    = -(1/mrs_sp_lw[1]) +1
        qui gen theta  = -(mrs_sp_lleis[1]/mrs_sp_lw[1]) + 1
    }

end

capture program drop makealpha
program makealpha

    syntax, wagevar(varname) leisurevar(varname)

	confirm variable mrs_sp_lleis
	confirm variable mrs_lc
    
    sort mrs__cons
    qui gen v = `wagevar' if sp_employed==1

    foreach var in $alpha_cntrl _cons {
        qui replace v = v- mrs_`var'[1]*`var' if sp_employed==1
    }

    qui replace v = v - (theta-1)*`leisurevar' - (1-phi)*lc if sp_employed==1


    foreach var in $alpha_cntrl {
        qui gen psi_`var' = mrs_`var'[1]*`var'
    }

    qui gen psi__cons = mrs__cons[1]

    qui gen psiz = 0
    /*sum of psi vars*/
    foreach var in $alpha_cntrl _cons {
        qui replace psiz = psiz + psi_`var'
    }

    qui gen v_lb = lnimpwage - psiz - (theta-1)*`leisurevar' - (1-phi)*lc if sp_employed==0

    qui gen vi = v_lb if sp_employed==0
    qui replace vi = v if sp_employed==1

    qui su vi if sp_employed==1, meanonly
    qui gen meanvi_ifemp = r(mean)
    qui su vi if sp_employed==0, meanonly
    qui gen meanvi_ifnoemp = r(mean)

    qui egen meansp_employed = mean(sp_employed)

    qui gen adjfactor = (meansp_employed*meanvi_ifemp)/(1-meansp_employed) + meanvi_ifnoemp

    qui gen newvi = vi
    qui replace newvi = vi - adjfactor if sp_employed==0

    qui gen aterm = exp(psiz+newvi)
	
    qui gen alpha = 1/(aterm+1)

end

/*make alpha when leisure is the dependent variable*/
capture program drop makealpha_leisure
program makealpha_leisure

    syntax, wagevar(varname) leisurevar(varname)

	confirm variable mrs_sp_lw
	confirm variable mrs_lc
    sort mrs__cons

    qui gen v = `wagevar' if sp_employed
    foreach var in $mrs_cntrl _cons {
        qui replace v = v- (1-theta)*mrs_`var'[1]*`var' if sp_employed==1
    }
    qui replace v = v - (theta-1)*`leisurevar' - (1-phi)*lc if sp_employed==1

    foreach var in $alpha_cntrl {
        qui gen psi_`var' = (1-theta)*mrs_`var'[1]*`var'
    }

    qui gen psi__cons = (1-theta)*mrs__cons[1]

    qui gen psiz = 0

    foreach var in $alpha_cntrl _cons {
        qui replace psiz = psiz + psi_`var'
    }

    qui gen v_lb = lnimpwage - psiz - (theta-1)*`leisurevar' - (1-phi)*lc if sp_employed==0
    qui gen vi = v_lb if sp_employed==0
    qui replace vi = v if sp_employed==1

    qui su vi if sp_employed==1, meanonly
    qui gen meanvi_ifemp = r(mean)
    qui su vi if sp_employed==0, meanonly
    qui gen meanvi_ifnoemp = r(mean)

    qui egen meansp_employed = mean(sp_employed)

    qui gen adjfactor = (meansp_employed*meanvi_ifemp)/(1-meansp_employed) + meanvi_ifnoemp

    qui gen newvi = vi
    qui replace newvi = vi - adjfactor if sp_employed==0

    qui gen aterm = exp(psiz+newvi)
    qui gen alpha = 1/(aterm+1)

end

capture program drop makealpha_consumption
program makealpha_consumption

    syntax, wagevar(varname)
	confirm variable mrs_sp_lw
	confirm variable mrs_sp_lleis
    sort mrs__cons

    qui gen v = `wagevar' if sp_employed==1
    foreach var in $mrs_cntrl _cons {
        qui replace v = v- (phi-1)*mrs_`var'[1]*`var' if sp_employed==1
    }
    qui replace v = v - (theta-1)*sp_lleis - (1-phi)*lc if sp_employed==1

    foreach var in $alpha_cntrl {
        qui gen psi_`var' = (phi-1)*mrs_`var'[1]*`var'
    }

    qui gen psi__cons = (phi-1)*mrs__cons[1]

    qui gen psiz = 0

    foreach var in $alpha_cntrl _cons {
        qui replace psiz = psiz + psi_`var'
    }

    qui gen v_lb = lnimpwage - psiz - (theta-1)*sp_lleis - (1-phi)*lc if sp_employed==0
    qui gen vi = v_lb if sp_employed==0
    qui replace vi = v if sp_employed==1

    qui su vi if sp_employed==1, meanonly
    qui gen meanvi_ifemp = r(mean)
    qui su vi if sp_employed==0, meanonly
    qui gen meanvi_ifnoemp = r(mean)

    qui egen meansp_employed = mean(sp_employed)

    qui gen adjfactor = (meansp_employed*meanvi_ifemp)/(1-meansp_employed) + meanvi_ifnoemp

    qui gen newvi = vi
    qui replace newvi = vi - adjfactor if sp_employed==0

    qui gen aterm = exp(psiz+newvi)

end

capture program drop prepeuler
program prepeuler

    syntax, groupvars(varlist)

    local phi = phi
    local theta = theta

    qui gen la = ln(alpha)

    qui gen mvar = ((cons^phi)-1)/phi + (aterm)*((sp_leisure^theta)-1)/theta
    qui assert mvar > 0

    *adjustmens for when aterm gets very large (otherwise stata cannot cope)
    qui gen lnaterm = psiz+newvi
    qui gen tocorrect = lnaterm>80 & aterm==.
    qui gen tocorrectvlargeaterm = lnaterm>95 & aterm==.
    qui gen mvarsmall = (((cons^phi)-1)/phi)/10000 + exp(psiz+newvi + ln(((sp_leisure^theta)-1)/theta) - ln(10000)) if tocorrect==1
    qui gen mvarsmallvlargeaterm = (((cons^phi)-1)/phi)/100000000 + exp(psiz+newvi + ln(((sp_leisure^theta)-1)/theta) - ln(100000000)) if tocorrectvlargeaterm==1

    qui gen lmvar = ln(mvar)
    qui replace lmvar = ln(mvarsmall) + ln(10000) if tocorrect==1
    qui replace lmvar = ln(mvarsmallvlargeaterm) + ln(100000000) if tocorrectvlargeaterm==1
    capture assert lmvar~=.
    if _rc==9 {
        di "Error in prepeuler: ln M is missing"
        pause
    }

    qui gen lfp_childage02 = (sp_employed)*childage02

    qui gen lc_temp1 = ln(cons)
    qui gen lw_temp1 = ln(sp_w)
    qui gen lhw_temp1 = ln(h_w)
    qui gen ll_temp1 = ln(sp_leisure)
    qui gen lm_temp1 = lmvar
    qui gen lhearn_temp1 = h_logearn
    qui gen lspearn_temp1 = sp_logearn
    qui gen lfp_temp1 = sp_employed
    qui gen lfp_childage02_temp1 = lfp_childage02
    qui gen hlfp_temp1 = h_employed
    qui gen la_temp1 = la

    foreach var in sp_age haskids {
        qui gen `var'_lm_temp1 = `var'*lm_temp1
    }

    qui gen lrr = ln(1+rr)
    qui gen lir = ln(1+ir)
    qui gen lpi = ln(1+pi)

    foreach var in $dem_cntrl sp_educ1 sp_educ2 sp_educ3 h_age h_age2 h_age3 {
        qui gen `var'_temp1 = `var'
    }

    qui gen Nobs = 1

    /*t*/
    preserve
    /*average into cohorts*/
    collapse (mean) *_temp1 lrr lir lpi (sum) Nobs, by(`groupvars')
    rename lc_temp1 lc1
    rename lw_temp1 lw1
    rename lhw_temp1 lhw1
    rename ll_temp1 ll1
    rename lm_temp1 lm1
    rename lrr lrr1
    rename lir lir1
    rename lpi lpi1
    rename lfp_temp1 lfp1
    rename lfp_childage02_temp1 lfp_childage021
    rename hlfp_temp1 hlfp1
    rename lhearn_temp1 lhearn1
    rename lspearn_temp1 lspearn1
    rename la_temp1 la1

    foreach var in sp_age haskids {
        qui rename `var'_lm_temp1  `var'_lm1
    }

    foreach var in $dem_cntrl sp_educ1 sp_educ2 sp_educ3 h_age h_age2 h_age3 {
        rename `var'_temp1 `var'1
    }

    sort `groupvars'
    qui save "temp1", replace
    restore

    /*t-1*/
    qui gen lc_temp2 = ln(cons)
    qui gen lw_temp2 = ln(sp_w)
    qui gen lhw_temp2 = ln(h_w)
    qui gen ll_temp2 = ln(sp_leisure)
    qui gen lm_temp2 = lmvar
    qui gen lhearn_temp2 = h_logearn
    qui gen lspearn_temp2 = sp_logearn
    qui gen lfp_temp2 = sp_employed
    qui gen lfp_childage02_temp2 = lfp_childage02
    qui gen hlfp_temp2 = h_employed
    qui gen la_temp2 = la

    foreach var in sp_age haskids {
        qui gen `var'_lm_temp2 = `var'*lm_temp2
    }


    foreach var in $dem_cntrl sp_educ1 sp_educ2 sp_educ3 h_age h_age2 h_age3 {
        qui gen `var'_temp2 = `var'
    }

    preserve
    collapse (mean) *_temp2 lrr lir lpi , by(`groupvars')

    qui replace qtr = qtr+1
    qui replace yr = yr+1 if qtr>4
    qui replace qtr = qtr-4 if qtr>4

    foreach var in $dem_cntrl sp_educ1 sp_educ2 sp_educ3 h_age h_age2 h_age3 {
        rename `var'_temp2 `var'2
        }
    rename lc_temp2 lc2
    rename lw_temp2 lw2
    rename lhw_temp2 lhw2
    rename ll_temp2 ll2
    rename lm_temp2 lm2
    rename lfp_temp2 lfp2
    rename lfp_childage02_temp2 lfp_childage022
    rename hlfp_temp2 hlfp2
    rename lhearn_temp2 lhearn2
    rename lspearn_temp2 lspearn2
    rename lrr lrr2
    rename lir lir2
    rename lpi lpi2
    rename la_temp2 la2
    foreach var in sp_age haskids {
        qui rename `var'_lm_temp2  `var'_lm2
    }

    sort `groupvars'
    qui save "temp2", replace
    restore

    /*t-2*/
    qui gen lc_temp3 = ln(cons)
    qui gen lw_temp3 = ln(sp_w)
    qui gen lhw_temp3 = ln(h_w)
    qui gen ll_temp3 = ln(sp_leisure)
    qui gen lfp_temp3 = sp_employed
    qui gen lfp_childage02_temp3 = lfp_childage02
    qui gen hlfp_temp3 = h_employed
    qui gen lm_temp3 = lmvar
    qui gen lhearn_temp3 = h_logearn
    qui gen lspearn_temp3 = sp_logearn
    qui gen la_temp3 = la
    foreach var in $dem_cntrl sp_educ1 sp_educ2 sp_educ3 h_age h_age2 h_age3 {
        qui gen `var'_temp3 = `var'
    }
    foreach var in sp_age haskids {
        qui gen `var'_lm_temp3 = `var'*lm_temp3
    }

    preserve
    drop if intno==2
    collapse (mean) *_temp3 lrr lir lpi, by(`groupvars')

    qui replace qtr = qtr+2
    qui replace yr = yr+1 if qtr>4
    qui replace qtr = qtr-4 if qtr>4

    foreach var in $dem_cntrl sp_educ1 sp_educ2 sp_educ3 h_age h_age2 h_age3 {
        rename `var'_temp3 `var'3
    }


    rename lc_temp3 lc3
    rename lw_temp3 lw3
    rename lhw_temp3 lhw3
    rename ll_temp3 ll3
    rename lfp_temp3 lfp3
    rename lfp_childage02_temp3 lfp_childage023
    rename hlfp_temp3 hlfp3
    rename lhearn_temp3 lhearn3
    rename lspearn_temp3 lspearn3
    rename lrr lrr3
    rename lir lir3
    rename lpi lpi3
    rename lm_temp3 lm3
    rename la_temp3 la3
    foreach var in sp_age haskids {
        qui rename `var'_lm_temp3  `var'_lm3
    }

    sort `groupvars'
    qui save "temp3", replace
    restore

    /*t-3*/
    qui gen lc_temp4 = ln(cons)
    qui gen lw_temp4 = ln(sp_w)
    qui gen lhw_temp4 = ln(h_w)
    qui gen ll_temp4 = ln(sp_leisure)
    qui gen lfp_temp4 = sp_employed
    qui gen lfp_childage02_temp4 = lfp_childage02
    qui gen hlfp_temp4 = h_employed
    qui gen lm_temp4 = lmvar
    qui gen lhearn_temp4 = h_logearn
    qui gen lspearn_temp4 = sp_logearn
    qui gen la_temp4 = la
    foreach var in $dem_cntrl sp_educ1 sp_educ2 sp_educ3 h_age h_age2 h_age3 {
        qui gen `var'_temp4 = `var'
    }
    foreach var in sp_age haskids {
        qui gen `var'_lm_temp4 = `var'*lm_temp4
    }

    preserve
    drop if intno==2
    collapse (mean) *_temp4 lrr lir lpi, by(`groupvars')
    qui replace qtr = qtr+3
    qui replace yr = yr+1 if qtr>4
    qui replace qtr = qtr-4 if qtr>4
    foreach var in $dem_cntrl sp_educ1 sp_educ2 sp_educ3 h_age h_age2 h_age3 {
        rename `var'_temp4 `var'4
    }

    rename lc_temp4 lc4
    rename lw_temp4 lw4
    rename lhw_temp4 lhw4
    rename ll_temp4 ll4
    rename lfp_temp4 lfp4
    rename lfp_childage02_temp4 lfp_childage024
    rename hlfp_temp4 hlfp4
    rename lhearn_temp4 lhearn4
    rename lspearn_temp4 lspearn4
    rename lrr lrr4
    rename lir lir4
    rename lpi lpi4
    rename lm_temp4 lm4
    rename la_temp4 la4

    foreach var in sp_age haskids {
        qui rename `var'_lm_temp4  `var'_lm4
    }

    sort `groupvars'
    qui save "temp4", replace
    restore

    /*t-4*/
    qui gen lc_temp5 = ln(cons)
    qui gen lw_temp5 = ln(sp_w)
    qui gen lhw_temp5 = ln(h_w)
    qui gen ll_temp5 = ln(sp_leisure)
    qui gen lfp_temp5 = sp_employed
    qui gen lfp_childage02_temp5 = lfp_childage02
    qui gen hlfp_temp5 = h_employed
    qui gen lm_temp5 = lmvar
    qui gen lhearn_temp5 = h_logearn
    qui gen lspearn_temp5 = sp_logearn
    qui gen la_temp5 = la
    foreach var in $dem_cntrl sp_educ1 sp_educ2 sp_educ3 h_age h_age2 h_age3 {
        qui gen `var'_temp5 = `var'
    }

    foreach var in sp_age haskids {
        qui gen `var'_lm_temp5 = `var'*lm_temp5
    }

    preserve
    drop if intno==2
    collapse (mean) *_temp5 lrr lir lpi, by(`groupvars')
    qui replace yr = yr+1
    foreach var in $dem_cntrl sp_educ1 sp_educ2 sp_educ3 h_age h_age2 h_age3 {
        rename `var'_temp5 `var'5
    }
    rename lc_temp5 lc5
    rename lw_temp5 lw5
    rename lhw_temp5 lhw5
    rename ll_temp5 ll5
    rename lfp_temp5 lfp5
    rename lfp_childage02_temp5 lfp_childage025
    rename hlfp_temp5 hlfp5
    rename lhearn_temp5 lhearn5
    rename lspearn_temp5 lspearn5
    rename lrr lrr5
    rename lir lir5
    rename lpi lpi5
    rename lm_temp5 lm5
    rename la_temp5 la5

    foreach var in sp_age haskids {
        qui rename `var'_lm_temp5  `var'_lm5
    }

    sort `groupvars'
    qui save "temp5", replace
    restore

    use "temp1", clear

    qui merge 1:1 `groupvars' using "temp2",nogen
    qui merge 1:1 `groupvars' using "temp3",nogen
    qui merge 1:1 `groupvars' using "temp4",nogen
    qui merge 1:1 `groupvars' using "temp5",nogen

    erase temp1.dta
    erase temp2.dta
    erase temp3.dta
    erase temp4.dta
    erase temp5.dta

    forval i = 1/4 {
        local j = `i'+1
        qui gen dlc`i' = lc`i'-lc`j'
        qui gen dlw`i' = lw`i'-lw`j'
        qui gen dlfp`i' = lfp`i' - lfp`j'
        qui gen dlfp_childage02`i' = lfp_childage02`i' - lfp_childage02`j'
        qui gen dlrr`i' = lrr`i' - lrr`j'
        qui gen dlm`i' = lm`i' - lm`j'
        qui gen dla`i' = la`i' - la`j'
        qui gen dll`i' = la`i' - la`j'
    }

    foreach var in $dem_cntrl sp_educ1 sp_educ2 sp_educ3 h_age h_age2 h_age3 {
        qui gen d`var'1 = `var'1 - `var'2
    }

    foreach var in sp_age_lm haskids_lm {
        qui gen d`var'1 = `var'1 - `var'2
    }

    qui gen season1 = intmo==1 & qtr==1
    qui gen season2 = intmo==2 & qtr==1
    qui gen season3 = intmo==3 & qtr==1
    qui gen season4 = intmo==1 & qtr==2
    qui gen season5 = intmo==2 & qtr==2
    qui gen season6 = intmo==3 & qtr==2
    qui gen season7 = intmo==1 & qtr==3
    qui gen season8 = intmo==2 & qtr==3
    qui gen season9 = intmo==3 & qtr==3
    qui gen season10 = intmo==1 & qtr==4
    qui gen season11 = intmo==2 & qtr==4
    qui gen _season12 = intmo==3 & qtr==4 

    qui gen Z = dlm1
    qui gen W = (1-`phi')*dlc1 - 0.25*lrr1

end

capture program drop calcfrisch
program calcfrisch

    syntax, sigma(numlist)

    gen sigma = `sigma'

    qui gen M = ((cons^phi)-1)/phi + (aterm)*((sp_leisure^theta)-1)/theta

    qui gen double uc =M^(sigma-1)*(cons^(phi-1))
    qui gen double ul =M^(sigma-1)*(aterm)*(sp_leisure^(theta-1))
    qui gen double ucc=M^(sigma-1)*cons^(phi-2)*((sigma-1)*(cons^phi)/M + phi -1)
    qui gen double ull=M^(sigma-1)*(aterm)*sp_leisure^(theta-2)*((sigma-1)*(aterm)*(sp_leisure^theta)/M + theta -1)
    qui gen double ucl=(sigma-1)*M^(sigma-2)*(cons^(phi-1))*(sp_leisure^(theta-1))*(aterm)

    qui gen double term=ucc*ull-ucl^2

    qui gen double Fc=-ucl*uc*sp_w/(cons*term)

    qui gen double Fl=ucc*uc*sp_w/(sp_leisure*term)
    qui gen double Fh=-Fl*sp_leisure/($T-sp_leisure)

    qui gen double riskaversion = -cons*ucc/uc
	
	qui gen double Fc_IR=(uc*ull-ul*ucl)/(cons*term)
    qui gen double Fl_IR=(ul*ucc-uc*ucl)/(sp_leisure*term)
    qui gen double Fh_IR=-Fl_IR*sp_leisure/($T-sp_leisure)

end

*this is the programme used for calculating Frisches within the bootstrap
capture program drop calcfrisch_withlocals
program calcfrisch_withlocals

    syntax, sigma(numlist max=1) cons(numlist max=1) sp_leisure(numlist max=1) sp_w(numlist max=1)

    gen sigma = `sigma'

    qui gen M = ((`cons'^phi)-1)/phi + (aterm)*((`sp_leisure'^theta)-1)/theta if _n==1

    qui gen double uc =M^(sigma-1)*(`cons'^(phi-1)) if _n==1
    qui gen double ul =M^(sigma-1)*(aterm)*(`sp_leisure'^(theta-1)) if _n==1
    qui gen double ucc=M^(sigma-1)*`cons'^(phi-2)*((sigma-1)*(`cons'^phi)/M + phi -1) if _n==1
    qui gen double ull=M^(sigma-1)*(aterm)*`sp_leisure'^(theta-2)*((sigma-1)*(aterm)*(`sp_leisure'^theta)/M + theta -1) if _n==1
    qui gen double ucl=(sigma-1)*M^(sigma-2)*(`cons'^(phi-1))*(`sp_leisure'^(theta-1))*(aterm) if _n==1

    qui gen double term=ucc*ull-ucl^2 if _n==1

    qui gen double Fc=-ucl*uc*`sp_w'/(`cons'*term) if _n==1

    qui gen double Fl=ucc*uc*`sp_w'/(`sp_leisure'*term) if _n==1
    qui gen double Fh=-Fl*`sp_leisure'/($T-`sp_leisure') if _n==1

    qui gen double riskaversion = -`cons'*ucc/uc
	
	qui gen double Fc_IR=(uc*ull-ul*ucl)/(`cons'*term)
    qui gen double Fl_IR=(ul*ucc-uc*ucl)/(`sp_leisure'*term)
    qui gen double Fh_IR=-Fl_IR*`sp_leisure'/($T-`sp_leisure')

end

capture program drop psicalc
program psicalc

        syntax, matrix(string)

        *Get values for individuals at desired consumption quantiles
        foreach var in $mrs_cntrl sp_lleis lc sp_leisure sp_lw cons rr sp_w {
            matrix `var'  = `matrix'[1,"`var'"]
            global `var' = `var'[1,1]
            matrix drop `var'
        }

        foreach var in $alpha_cntrl {
            qui gen psi_`var' = ${mrscoeff_`var'}*${`var'} if _n==1
        }
        qui gen psi__cons = $mrscoeff__cons

        qui gen psiz = 0 if _n==1

        foreach var in $alpha_cntrl _cons {
            qui replace psiz = psiz + psi_`var' if _n==1
        }

        qui gen v = $sp_lw if _n==1
        foreach var in $mrs_cntrl sp_lleis lc {
            qui replace v = v - ${mrscoeff_`var'}*${`var'} if _n==1
        }
        qui replace v = v - $mrscoeff__cons if _n==1

        qui gen aterm = exp(psiz+v) if _n==1

end

capture program drop psicalc_leisure
program psicalc_leisure

        syntax, matrix(string)

        *Get values for individuals at desired consumption quantiles
        foreach var in $mrs_cntrl sp_lw lc sp_lleis sp_leisure cons rr sp_w  {
            matrix `var'  = `matrix'[1,"`var'"]
            global `var' = `var'[1,1]
            matrix drop `var'
        }

        foreach var in $alpha_cntrl {
            qui gen psi_`var' = (1-theta)*${mrscoeff_`var'}*${`var'} if _n==1
        }
        qui gen psi__cons = (1-theta)*$mrscoeff__cons

        qui gen psiz = 0 if _n==1
        foreach var in $alpha_cntrl _cons {
            qui replace psiz = psiz + psi_`var' if _n==1
        }

        qui gen v = $sp_lw if _n==1
        foreach var in $mrs_cntrl {
            qui replace v = v - (1-theta)*${mrscoeff_`var'}*${`var'} if _n==1
        }

        qui replace v = v- (theta-1)*$sp_lleis - (1-phi)*$lc  if _n==1
        qui replace v = v - (1-theta)*$mrscoeff__cons if _n==1

        qui gen aterm = exp(psiz+v) if _n==1

end

capture program drop makeomega
program makeomega

        syntax, maxG(numlist)

        local X = 0
        foreach var of varlist $euler_endogvar $depvar {
            local ++X
            qui reg `var' $euler_exogvar $euler_instvar [aweight = Nobs]
            estimates save "firststage_`var'", replace

            keep if e(sample)==1
            predict e`X', r

            sort group yr qtr
            gen lage`X' = e`X'[_n-1] if group[_n]==group[_n-1]

            forval g = 1/`maxG'  {
                qui count if group == `g'
                local T`g' = r(N)

                qui corr e`X' lage`X' if group==`g', cov
                local cov =  r(cov_12)
                qui su e`X' if group==`g'
                local var = r(Var)

                local covovervar = `cov'/`var'
                di `covovervar'
                capture mata: assert(lambda2*abs(`covovervar')<0.5)
                if _rc!=0 {
                 di in red "lambda2 is too high - matrix guaranteed not positive definite"
                }


                mata: VarVec = J(`T`g'',1,`var')

                mata: Omega_X`X'X`X'_G`g'G`g' = diag(VarVec)
                local I =`T`g''-1

                forval i = 1/`I' {
                    local j = `i'+1
                    mata: Omega_X`X'X`X'_G`g'G`g'[`i',`j'] = lambda2*`cov'
                    mata: Omega_X`X'X`X'_G`g'G`g'[`j',`i'] = lambda2*`cov'
                }
            }

        }
        local maxX = `X'


        forval X = 1/`maxX' {
            forval Y = 1/`maxX' {
                if `X'==`Y' continue
                forval g = 1/`maxG'  {
     
                qui corr e`X' e`Y' if group==`g', cov
                local cov = r(cov_12)

                mata: covVec = J(`T`g'',1,`cov')
                mata: Omega_X`X'X`Y'_G`g'G`g' = lambda1*diag(covVec)
                }
            }
        }

        forval X = 1/`maxX' {
            forval Y = 1/`maxX' {
                if `X'==`Y' continue
                forval g = 1/`maxG'  {
                    forval j = 1/`maxG'  {
                        if `g'==`j' continue
                        mata: Omega_X`X'X`Y'_G`g'G`j' = J(`T`g'',`T`j'',0)
                    }
                }
            }
        }

         preserve
         egen t = group(yr qtr)
         keep t e1-e`maxX' group
         reshape wide e1-e`maxX', i(t) j(group)

         sort t
         forval X = 1/`maxX' {
            forval g = 1/`maxG' {
                    forval j = 1/`maxG' {
                        if `g'==`j' continue

                        qui count if e`X'`g'!=. & e`X'`j'!=.
                        if r(N)>1 {
                            qui corr e`X'`g' e`X'`j', cov
                            local cov = r(cov_12)
                        }
                        else {
                            local cov = 0
                        }

                    if inlist(`g',1,4,7,10) local intmog = 1
                    if inlist(`g',2,5,8,11) local intmog = 2
                    if inlist(`g',3,6,9,12) local intmog = 3

                    if inlist(`j',1,4,7,10) local intmoj = 1
                    if inlist(`j',2,5,8,11) local intmoj = 2
                    if inlist(`j',3,6,9,12) local intmoj = 3

                    local nextfactor = 0
                    local lastfactor = 0

                    qui su t if e`X'`g'!=.
                    local minobsno = r(min)
                    local maxobsno = r(max)

                    qui su t if e`X'`j'!=.
                    local minobsno = min(r(min),`minobsno')
                    local maxobsno = max(r(max),`maxobsno')

                    local totalperiods = `maxobsno' - `minobsno' + 1
                    local totalperiodsminusone = `totalperiods'-1

                    local minobsnoplusone  = `minobsno'  + 1
                    local maxobsnominusone = `maxobsno'  - 1

                    qui gen covvariable = `cov' if e`X'`g'!=. & e`X'`j'!=.
                    qui replace covvariable = 0 if covvariable==.

                    qui gen nextperiodcov = `nextfactor'*`cov' if e`X'`g'!=. & e`X'`j'[_n+1]!=.
                    qui replace nextperiodcov = 0 if nextperiodcov==.

                    qui gen lastperiodcov = `lastfactor'*`cov' if e`X'`g'!=. & e`X'`j'[_n-1]!=.
                    qui replace lastperiodcov = 0 if lastperiodcov==.

                    qui gen missingind`g' = 1-(e`X'`g'==.)
                    qui gen missingind`j' = 1-(e`X'`j'==.)

                    tomata covvariable    in `minobsno'/`maxobsno'
                    tomata nextperiodcov  in `minobsno'/`maxobsnominusone'
                    tomata lastperiodcov  in `minobsnoplusone'/`maxobsno'

                    tomata missingind`g'  in `minobsno'/`maxobsno'
                    tomata missingind`j'  in `minobsno'/`maxobsno'

                    mata: covmatrix = diag(covvariable)

                    forval i = 1/`totalperiodsminusone' {
                        mata: covmatrix[`i',`i'+1] = nextperiodcov[`i',1]
                    }

                    forval i = 1/`totalperiodsminusone' {
                        mata: covmatrix[`i'+1,`i'] = lastperiodcov[`i',1]
                    }


                    mata: covmatrix = select(covmatrix, missingind`g')

                    mata: missingind`j' = missingind`j''
                    mata: covmatrix = select(covmatrix, missingind`j')


                    mata: assert(rows(covmatrix)==`T`g'')
                    mata: assert(cols(covmatrix)==`T`j'')

                    mata: Omega_X`X'X`X'_G`g'G`j' = lambda1*covmatrix

                    mata: mata drop covmatrix

                    drop covvariable missingind`g' missingind`j' nextperiodcov lastperiodcov
                }
            }
    }

    restore

    *Append matrices into complete Omega matrix

        forval X = 1/`maxX' {
                forval g = 1/`maxG'  {
                    forval Y = 1/`maxX' {
                            forval j = 1/`maxG'  {
                                if `Y'==1 & `j'==1 {
                                    mata: Omega_X`X'_G`g' = Omega_X`X'X1_G`g'G1
                                }
                                else {
                                    mata: Omega_X`X'_G`g' = Omega_X`X'_G`g',Omega_X`X'X`Y'_G`g'G`j'
                                }
                                mata: mata drop Omega_X`X'X`Y'_G`g'G`j'
                    }
                }
            }
        }

        forval X = 1/`maxX' {
                forval g = 1/`maxG'  {
                    if `X'==1 & `g'==1 {
                        mata: Omega =  Omega_X1_G1
                    }
                    else {
                        mata: Omega = Omega\Omega_X`X'_G`g'
                    }
                }
        }
end


************* PROGRAMS FOR COBB DOUGLAS AND CES PREFERENCES ***********

capture program drop runmrs_cobbdouglas
program define runmrs_cobbdouglas

	qui xi: probit sp_employed h_logearn h_logearnflag $dem_cntrl $instlist $oth_cntrl $yeardummies, difficult iterate(50)
	
	scalar probitconverged = e(converged)
    if probitconverged==0 {
        di in red "Probit didn't converge"
    }

	predict ee,index
	qui gen ee2=ee*ee
	qui gen fi=1/normprob(ee)
	qui gen e1 = fi/(sqrt(6.2831853)*exp(0.5*ee2))
	qui gen e2 = fi*ee/(sqrt(6.2831853)*exp(0.5*ee2)) + 1
	qui gen e2minusone = e2-1
	qui gen e3 =(2+ee2)*fi/(sqrt(6.2831853)*exp(0.5*ee2))

	gen newdepvar = sp_lw - lc + sp_lleis
	xi: reg newdepvar e1 e2minusone e3 $mrs_cntrl if sp_employed==1

end

capture program drop runmrs_ces
program define runmrs_ces

	qui xi: probit sp_employed h_logearn h_logearnflag $dem_cntrl $instlist $oth_cntrl $yeardummies, difficult iterate(50)
	
	scalar probitconverged = e(converged)
    if probitconverged==0 {
        di in red "Probit didn't converge"
    }

	predict ee,index
	qui gen ee2=ee*ee
	qui gen fi=1/normprob(ee)
	qui gen e1 = fi/(sqrt(6.2831853)*exp(0.5*ee2))
	qui gen e2 = fi*ee/(sqrt(6.2831853)*exp(0.5*ee2)) + 1
	qui gen e2minusone = e2-1
	qui gen e3 =(2+ee2)*fi/(sqrt(6.2831853)*exp(0.5*ee2))

	qui gen sp_lleisminuslc = sp_lleis - lc

	xi: ivreg2 sp_lw e1 e2minusone e3 $mrs_cntrl (sp_lleisminuslc = $instlist) if sp_employed==1, fuller(1) 

end


capture program drop genmrsvars_cobbdouglas
program define genmrsvars_cobbdouglas

	qui gen theta  = 0
	qui gen phi    = 0

	scalar theta = theta[1]
	scalar phi   = phi[1]

	matrix define mrs_= e(b)
	svmat mrs_, name(matcol)

    qui reg sp_lw $mrs_cntrl $instlist if sp_employed==1
    qui predict lnimpwage, xb
	
end

capture program drop genmrsvars_ces
program define genmrsvars_ces

	qui gen theta  = _b[sp_lleisminuslc]+1
	qui gen phi    = _b[sp_lleisminuslc]+1

	scalar theta = theta[1]
	scalar phi   = phi[1]

	matrix define mrs_= e(b)
	svmat mrs_, name(matcol)

    qui reg sp_lw $mrs_cntrl $instlist if sp_employed==1
    qui predict lnimpwage, xb
	
end

cap program drop genelasticities
program define genelasticities 

	genmarshallandhicks

	foreach var in h l c {

		qui centile M`var', centile(7.5 12.5)
		gen bdecM`var' = inrange(M`var',r(c_1),r(c_2))

		qui centile M`var', centile(22.5 27.5)
		gen lqM`var' = inrange(M`var',r(c_1),r(c_2))

		qui centile M`var', centile(47.5 52.5)
		gen medianM`var' = inrange(M`var',r(c_1),r(c_2))

		qui centile M`var', centile(72.5 77.5)
		gen uqM`var' = inrange(M`var',r(c_1),r(c_2))
		
		qui centile M`var', centile(87.5 92.5)
		gen udecM`var' = inrange(M`var',r(c_1),r(c_2))
	}
	
	foreach var in c l h {	
		foreach elast in M H {
			foreach quant in bdec lq median uq udec {
				qui su `elast'`var' if `quant'M`var'==1
				scalar `quant'`elast'`var' = r(mean)
			}
		}
	}
	
	capture matrix drop tabresults
	#delimit ;
	foreach var in
	bdecMc lqMc medianMc uqMc udecMc bdecHc lqHc medianHc uqHc udecHc 
	bdecMh lqMh medianMh uqMh udecMh bdecHh lqHh medianHh uqHh udecHh 
	bdecMl lqMl medianMl uqMl udecMl bdecHl lqHl medianHl uqHl udecHl  {;
		matrix temp = `var';
		matrix tabresults = (nullmat(tabresults)\temp);
	};
	matrix drop temp;
	matrix rownames tabresults = 
	bdecMc lqMc medianMc uqMc udecMc bdecHc lqHc medianHc uqHc udecHc 
	bdecMh lqMh medianMh uqMh udecMh bdecHh lqHh medianHh uqHh udecHh 
	bdecMl lqMl medianMl uqMl udecMl bdecHl lqHl medianHl uqHl udecHl;
	#delimit cr
	
	matrix list tabresults

end

