

graph drop _all
*plot wages and hours against each other for three different cohorts

preserve

forval e = 1/4 {
    forval i = 1/7 {
        qui gen cohort`i'_educ`e' = (cohort==`i')*(sp_educ==`e')
    }
}

*plot graphs for cohorts
gen     cohort_decade = 1 if cohort_5yr==4|cohort_5yr==5
replace cohort_decade = 2 if cohort_5yr==6|cohort_5yr==7
replace cohort_decade = 3 if cohort_5yr==8|cohort_5yr==9
replace cohort_decade = 4 if cohort_5yr==10|cohort_5yr==11

gen ones= 1

gen cohort_decade_age = int_yr - 1935 if cohort_decade==1
replace cohort_decade_age = int_yr - 1945 if cohort_decade==2
replace cohort_decade_age = int_yr - 1955 if cohort_decade==3
replace cohort_decade_age = int_yr - 1965 if cohort_decade==4

collapse (mean) sp_w sp_hours cons cohort_decade_age  (sum) Nobs = ones if sp_employed==1, by(broadeduc cohort_decade int_yr)
*line graphs over time
drop if cohort_decade_age >62

log using cohortgraphfigures.txt, text replace
tabstat sp_w sp_hours if cohort_decade==3 & broadeduc==0, by(int_yr)
tabstat sp_w sp_hours if cohort_decade==3 & broadeduc==1, by(int_yr)
log close


#delimit ;

twoway (line sp_w int_yr if cohort_decade==2 & broadeduc==0, lcolor(black))
       (line sp_w int_yr if cohort_decade==2 & broadeduc==1, lcolor(black) lpattern(dash)), title("1940s") nodraw name(wage_cohort40s) bgcolor(white) graphregion(color(white) lcolor(white)) 
       ylabel(10(5)25,nogrid) xtitle("") ytitle("Net wages ($)");

twoway (line sp_w int_yr if cohort_decade==3 & broadeduc==0, lcolor(black))
       (line sp_w int_yr if cohort_decade==3 & broadeduc==1, lcolor(black) lpattern(dash)), title("1950s") nodraw name(wage_cohort50s) bgcolor(white) graphregion(color(white) lcolor(white)) 
       ylabel(10(5)25,nogrid) xtitle("") ytitle("");

twoway (line sp_w int_yr if cohort_decade==4 & broadeduc==0, lcolor(black))
       (line sp_w int_yr if cohort_decade==4 & broadeduc==1, lcolor(black) lpattern(dash)), title("1960s") nodraw name(wage_cohort60s) bgcolor(white) graphregion(color(white) lcolor(white)) 
       ylabel(10(5)25,nogrid) xtitle("") ytitle("");

twoway (line sp_hours int_yr if cohort_decade==2 & broadeduc==0, lcolor(black))
       (line sp_hours int_yr if cohort_decade==2 & broadeduc==1, lcolor(black) lpattern(dash)), nodraw name(hours_cohort40s) bgcolor(white) graphregion(color(white) lcolor(white)) 
       ylabel(30(5)45,nogrid) xtitle("") ytitle("Hours per week");

twoway (line sp_hours int_yr if cohort_decade==3 & broadeduc==0, lcolor(black))
       (line sp_hours int_yr if cohort_decade==3 & broadeduc==1, lcolor(black) lpattern(dash)), nodraw name(hours_cohort50s) bgcolor(white) graphregion(color(white) lcolor(white)) 
       ylabel(30(5)45,nogrid) xtitle("") ytitle("");

twoway (line sp_hours int_yr if cohort_decade==4 & broadeduc==0, lcolor(black))
       (line sp_hours int_yr if cohort_decade==4 & broadeduc==1, lcolor(black) lpattern(dash)), nodraw name(hours_cohort60s) bgcolor(white) graphregion(color(white) lcolor(white)) 
       ylabel(30(5)45,nogrid) xtitle("") ytitle("") legend(order(1 "High school or less" 2 "More than high school") region(lcolor(white)));

twoway (line cons int_yr if cohort_decade==2 & broadeduc==0, lcolor(black))
       (line cons int_yr if cohort_decade==2 & broadeduc==1, lcolor(black) lpattern(dash)), nodraw name(cons_cohort40s) bgcolor(white) graphregion(color(white) lcolor(white)) 
       ylabel(400(300)1200,nogrid) xtitle("") ytitle("Consumption per week ($)");

twoway (line cons int_yr if cohort_decade==3 & broadeduc==0, lcolor(black))
       (line cons int_yr if cohort_decade==3 & broadeduc==1, lcolor(black) lpattern(dash)), nodraw name(cons_cohort50s) bgcolor(white) graphregion(color(white) lcolor(white)) 
       ylabel(400(300)1200,nogrid) xtitle("") ytitle("");

twoway (line cons int_yr if cohort_decade==4 & broadeduc==0, lcolor(black))
       (line cons int_yr if cohort_decade==4 & broadeduc==1, lcolor(black) lpattern(dash)), nodraw name(cons_cohort60s) bgcolor(white) graphregion(color(white) lcolor(white)) 
       ylabel(400(300)1200,nogrid) xtitle("") ytitle("") legend(order(1 "More than high school" 2 "High school or less"));

/*FIGURE 1*/
grc1leg wage_cohort40s wage_cohort50s wage_cohort60s hours_cohort40s hours_cohort50s hours_cohort60s, graphregion(fcolor(white) lwidth(thick) lcolor(white)) rows(2) cols(3) legendfrom(hours_cohort60s);
graph export "graphs\cohort_decade_wagesandhours.pdf", as(pdf) replace;

#delimit cr
restore



