
set more off
clear all
capture log close

set matsize 3000

do define_macros

use "dataset", clear
keep if datayear<=2012

gen h_logearnflag = h_logearn==.
replace h_logearn = 0 if h_logearn==.

forval y = $startyear/$endyear {
    gen year`y' = int_yr == `y'
}

gen broadeduc = 0 if sp_educ<=2
replace broadeduc = 1 if sp_educ>=3

qui gen     newcohort = 1 if inrange(cohort,1,2)
qui replace newcohort = 2 if inrange(cohort,3,4)
qui replace newcohort = 3 if inrange(cohort,5,6)
qui replace newcohort = 4 if inrange(cohort,7,8)
qui replace newcohort = 5 if inrange(cohort,9,10)
qui replace newcohort = 6 if inrange(cohort,11,12)
qui replace newcohort = 7 if inrange(cohort,13,14)

assert newcohort!=.

gen cohort_5yr = cohort
label var cohort_5yr "5 year birth cohorts"

replace cohort = newcohort
drop newcohort

/*cohorts*/
local numcohorts = 7

/*define cohort boundaries*/
forval j = 1/`numcohorts' {
    local c`j' = 1915 + 10*`j'
}

label drop cohort
label define cohort ///
1  "`c1'"       ///
2  "`c1'-`c2'"  ///
3  "`c2'-`c3'"  ///
4  "`c3'-`c4'"  ///
5  "`c4'-`c5'"  ///
6  "`c5'-`c6'"  ///
7  "`c6'-`c7'"
label values cohort cohort
label var cohort "Cohort of spouse"

drop sp_sex

gen     ageband = 1 if inrange(sp_age,25,34)
replace ageband = 2 if inrange(sp_age,35,44)
replace ageband = 3 if inrange(sp_age,45,54)

*Quantiles of consumption by has kids and age band
gen haskids = childage02 + childage315 + childage1617>0
replace haskids = . if (childage02 + childage315 + childage1617==.)

drop if sp_employed==1 & sp_lnweeklynetwage==.

*make leisure and consumption weekly
gen sp_weeklyleisure  = ($T-sp_weeklyhours)
gen sp_lnweeklyleisure = ln(sp_weeklyleisure)
gen sp_lnweeklyhours = ln(sp_weeklyhours)

gen h_weeklyleisure  = ($T-h_weeklyhours)
gen h_lnweeklyleisure = ln(h_weeklyleisure)
gen h_lnweeklyhours = ln(h_weeklyhours)

replace cons = cons/13
gen lc = ln(cons)

gen sp_annnetwage = sp_w
gen sp_lannnetwage = sp_lw
gen sp_annleisure = sp_leisure/13
gen sp_lannleisure = ln(sp_annleisure)

replace sp_lw = sp_lnweeklynetwage
replace sp_lleis = sp_lnweeklyleisure
replace sp_lhours = sp_lnweeklyhours

replace sp_leisure = sp_weeklyleisure
replace sp_hours = sp_weeklyhours
replace sp_w = sp_weeklynetwage

replace h_hours = h_weeklyhours
replace h_lhours = h_lnweeklyhours
replace h_w = h_weeklynetwage
replace h_lw = h_lnweeklynetwage

/*trim extreme values*/
centile sp_w if sp_employed==1, centile(99)
scalar define sp_wagemax = r(c_1)
centile cons, centile(99)
scalar define cons_max = r(c_1)
drop if sp_w > sp_wagemax & sp_employed==1
drop if cons > cons_max
drop if sp_hours<5 & sp_employed==1
drop if sp_weeklywage <0.75*minwage & sp_employed==1
*assert sp_hours<100

egen instgroup = group(cohort sp_educ int_yr)

do desc
do changesinhours
do makegraphs

drop sp_weeks h_weeks 
drop numadults

egen stratumgroup = group(yr qtr cohort sp_employed intno intmo)
*drop groups with only one member as otherwise bootstrap doesn't work
bys stratumgroup: gen groupcounter = _N
drop if groupcounter ==1
drop groupcounter

forval i = 1/7 {
    qui gen cohort`i' = cohort==`i'
}

forval e = 1/4 {
    forval i = 1/7 {
        qui gen cohort`i'_educ`e' = (cohort==`i')*(sp_educ==`e')
    }
}

gen sp_age4 = int(sp_age^4/10000)
replace sp_age3= int(sp_age3/1000)

gen t  = 0
gen t2 = 0
gen t3 = 0
gen t4 = 0
gen t5 = 0
forval e = 1/4 {
    forval i = 1/7 {
        qui su int_yr if cohort`i'_educ`e'==1
        qui replace t = int_yr - r(min)
        qui replace t2 = t^2
        qui replace t3 = t^3
        qui replace t4 = t^4
        qui replace t5 = t^5

        forval s= 2/5 {
            qui gen scale`s' = t`s'/t
            qui su scale`s' if cohort`i'_educ`e'==1
            qui replace t`s'= t`s'/r(mean)
        }

        qui gen int c`i'`e'_t = int(cohort`i'_educ`e'*t)
        qui gen int c`i'`e'_t2 = int(cohort`i'_educ`e'*t2)
        qui gen int c`i'`e'_t3 = int(cohort`i'_educ`e'*t3)
        qui gen int c`i'`e'_t4 = int(cohort`i'_educ`e'*t4)
        qui gen int c`i'`e'_t5 = int(cohort`i'_educ`e'*t5)

        drop scale2 scale3 scale4 scale5
    }
}

drop h_lleis2 sp_lleis2
drop minwage
drop t t2 t3 t4 t5

replace ageyoungestchild = . if haskids==0

gen region1 = region==1
gen region2 = region==2
gen region3 = region==3
gen region4 = region==4

save dataset2, replace

do calibrationstatistics

*sort cons, stable

use dataset2, clear

do define_programs

log using "whole_sample_mrs.txt", text replace
*first stage regressions
runmrs first
test e1 e2minus e3
estimates save "mrswholesample", replace

genmrsvars

*obtaining complete set of f-stats for first stage (don't observe the f-stat for wages though this is relevant when using the Fuller)
xi: ivreg2 sp_lleis e1 e2minusone e3 $mrs_cntrl (sp_lw lc = $instlist) if sp_employed==1, fuller(1) first cluster(uniqcuid)
*obtain sargan stat for main
xi: ivreg2 sp_lw e1 e2minusone e3 $mrs_cntrl (sp_lleis lc = $instlist) if sp_employed==1, fuller(1) first cluster(uniqcuid)

*Testing the significance of the selection terms in the first stages
qui reg sp_lw $instlist e1 e2minusone e3 $mrs_cntrl if sp_employed==1, cluster(uniqcuid)
test e1 e2minus e3
qui reg lc $instlist e1 e2minusone e3 $mrs_cntrl if sp_employed==1, cluster(uniqcuid)
test e1 e2minus e3
qui reg sp_lleis $instlist e1 e2minusone e3 $mrs_cntrl if sp_employed==1, cluster(uniqcuid)
test e1 e2minus e3

log close

preserve
makealpha, wagevar(sp_lw) leisurevar(sp_lleis)

log using "chi.txt", text replace
su newvi, det
log close

prepeuler, groupvars($groupvars)

qui xi: ivreg2 $depvar $euler_exogvar ($euler_endogvar = $euler_instvar) [aw=Nobs], fuller(1)
estimates save "ivreg_eulerresults", replace

*this do-file reloads the data
do CalcIntensFrisch

xtile pctile_Mh = Mh if sp_employed==1, nquantiles(100)
xtile pctile_Fh = Fh if sp_employed==1, nquantiles(100)
xtile pctile_spw = sp_w if sp_employed==1, nquantiles(100)
xtile pctile_sphours = sp_hours if sp_employed==1, nquantiles(100)

lpoly Mh pctile_spw if pctile_spw<99, nograph generate(Mh_smooth_spw) at(pctile_spw)
lpoly Hh pctile_spw if pctile_spw<99, nograph generate(Hh_smooth_spw) at(pctile_spw)
lpoly Fh pctile_spw if pctile_spw<99, nograph generate(Fh_smooth_spw) at(pctile_spw)

*plot Frisch Hicksian and Marshallian elasticties for percentiles of wage distribution
sort pctile_spw
twoway (line Mh_smooth_spw pctile_spw, lcolor(black) lpattern(solid) lwidth(thick)) (line Hh_smooth_spw pctile_spw, lcolor(black) lpattern(dash) lwidth(thick)) (line Fh_smooth_spw pctile_spw, lcolor(gray) lpattern(shortdash) lwidth(thick)), graphregion(color(white) lcolor(white)) ylabel(, nogrid labsize(small)) xlabel(, labsize(small)) ytitle("Hours elasticity") xtitle("Percentiles of wage distribution") legend(order(1 "Marshallian" 2 "Hicksian" 3 "Intensive Frisch")) name(elastbywages) nodraw

lpoly Mh sp_hours if pctile_Mh<99, nograph generate(Mh_smoothhours) at(sp_hours)
lpoly Hh sp_hours if pctile_Mh<99, nograph generate(Hh_smoothhours) at(sp_hours)
lpoly Fh sp_hours if pctile_Mh<99, nograph generate(Fh_smoothhours) at(sp_hours)

sort sp_hours
twoway (line Mh_smoothhours sp_hours if pctile_Mh<99 & Fh_smoothhours<=6, lcolor(black) lpattern(solid) lwidth(thick)) (line Hh_smoothhours sp_hours if pctile_Mh<99 & Fh_smoothhours<=6, lcolor(black) lpattern(dash) lwidth(thick)) (line Fh_smoothhours sp_hours if pctile_Mh<99 & Fh_smoothhours<=6, lcolor(gray) lpattern(shortdash) lwidth(thick)), graphregion(color(white) lcolor(white)) ylabel(, nogrid labsize(small)) xlabel(, labsize(small)) ytitle("Hours elasticity") xtitle("Weekly hours") legend(order(1 "Marshallian" 2 "Hicksian" 3 "Intensive Frisch")) name(elastbyhours) nodraw

*FIGURE 3 
grc1leg elastbywages elastbyhours, graphregion(color(white) lcolor(white)) ycommon 
graph export "graphs\ElastbywagesandHours.pdf", as(pdf) replace

drop Mh_smooth_spw Hh_smooth_spw Fh_smooth_spw
drop Mh_smoothhours Hh_smoothhours Fh_smoothhours

restore

*************************************************************************************************
*                               MRS + Euler results for the whole sample                                *
*************************************************************************************************

foreach var in e1 e2minusone e3 $mrs_cntrl sp_lleis lc _cons {
    scalar mrs_`var' = mrs_`var'[1]
}

*Results for Tables 3 and 7
log using "wholesampleresults.txt", replace text

estimates use "mrswholesample"
matrix mrs_b = e(b)
matnames mrs_b
local mrsvarlist "`r(c)'"

estimates use "ivreg_eulerresults"
matrix euler_b = e(b)
matnames euler_b
local eulervarlist "`r(c)'"

if "$depvar" == "Z" {
    matrix sigmavar = euler_b[1,"W"]
    local sigma = (1/sigmavar[1,1])+1
}
if "$depvar" == "W" {
    matrix sigmavar = euler_b[1,"Z"]
    local sigma = (sigmavar[1,1]+1)
}

global psi_mrs_cntrl ""


local i = 1
foreach var in `mrsvarlist' {
    local varname = substr("`var'",2,.)
    if "`varname'"=="_cons" {
        local varname "constant"
    }
    matrix tabresults = (nullmat(tabresults)\mrs_b[1,`i'])
    local i = `i' + 1
}

foreach var in theta phi {
    qui su `var', meanonly
    matrix tabresults = tabresults\(`r(mean)')
}


local i = 1
foreach var in `eulervarlist' {
    local varname = substr("`var'",2,.)
    if "`varname'"=="_cons" {
        local varname "constant"
    }
    matrix tabresults = (nullmat(tabresults)\euler_b[1,`i'])
    local i = `i' + 1
}

matrix tabresults = tabresults\(`sigma')

#delimit ;
foreach var in
bdecMc lqMc medianMc uqMc udecMc bdecHc lqHc medianHc uqHc udecHc bdecFc lqFc medianFc uqFc udecFc
bdecMh lqMh medianMh uqMh udecMh bdecHh lqHh medianHh uqHh udecHh bdecFh lqFh medianFh uqFh udecFh
bdecMl lqMl medianMl uqMl udecMl bdecHl lqHl medianHl uqHl udecHl bdecFl lqFl medianFl uqFl udecFl
bdecFc_IR lqFc_IR medianFc_IR uqFc_IR udecFc_IR
bdecFh_IR lqFh_IR medianFh_IR uqFh_IR udecFh_IR
bdecFl_IR lqFl_IR medianFl_IR uqFl_IR udecFl_IR
bdecFc_add lqFc_add medianFc_add uqFc_add udecFc_add 
bdecFh_add lqFh_add medianFh_add uqFh_add udecFh_add
bdecFl_add lqFl_add medianFl_add uqFl_add udecFl_add
bdecFc_IR_add lqFc_IR_add medianFc_IR_add uqFc_IR_add udecFc_IR_add 
bdecFh_IR_add lqFh_IR_add medianFh_IR_add uqFh_IR_add udecFh_IR_add 
bdecFl_IR_add lqFl_IR_add medianFl_IR_add uqFl_IR_add udecFl_IR_add {;
    matrix temp = `var';
    matrix tabresults = (tabresults\temp);
};
matrix drop temp;

matrix rownames tabresults = `mrsvarlist' $psi_mrs_cntrl theta phi `eulervarlist' sigma
bdecMc lqMc medianMc uqMc udecMc bdecHc lqHc medianHc uqHc udecHc bdecFc lqFc medianFc uqFc udecFc
bdecMh lqMh medianMh uqMh udecMh bdecHh lqHh medianHh uqHh udecHh bdecFh lqFh medianFh uqFh udecFh
bdecMl lqMl medianMl uqMl udecMl bdecHl lqHl medianHl uqHl udecHl bdecFl lqFl medianFl uqFl udecFl
bdecFc_IR lqFc_IR medianFc_IR uqFc_IR udecFc_IR
bdecFh_IR lqFh_IR medianFh_IR uqFh_IR udecFh_IR
bdecFl_IR lqFl_IR medianFl_IR uqFl_IR udecFl_IR
bdecFc_add lqFc_add medianFc_add uqFc_add udecFc_add 
bdecFh_add lqFh_add medianFh_add uqFh_add udecFh_add
bdecFl_add lqFl_add medianFl_add uqFl_add udecFl_add
bdecFc_IR_add lqFc_IR_add medianFc_IR_add uqFc_IR_add udecFc_IR_add 
bdecFh_IR_add lqFh_IR_add medianFh_IR_add uqFh_IR_add udecFh_IR_add 
bdecFl_IR_add lqFl_IR_add medianFl_IR_add uqFl_IR_add udecFl_IR_add;
#delimit cr

matrix list tabresults
log close

drop ageband

do bootstrap_mrsandeuler
do bootstrap_ces

*robustness exercises
do bootstrap_altmethods
do bootstrap_altsamples
do bootstrap_altinstruments
