*Calculates elasticities given CES and Cobb-Douglas assumptions

capture log close
set more off
clear all

do define_macros

use dataset2, clear

do define_programs

qui xi: probit sp_employed h_logearn h_logearnflag $dem_cntrl $instlist $oth_cntrl $yeardummies, difficult iterate(50)

predict ee,index
qui gen ee2=ee*ee
qui gen fi=1/normprob(ee)
qui gen e1 = fi/(sqrt(6.2831853)*exp(0.5*ee2))
qui gen e2 = fi*ee/(sqrt(6.2831853)*exp(0.5*ee2)) + 1
qui gen e2minusone = e2-1
qui gen e3 =(2+ee2)*fi/(sqrt(6.2831853)*exp(0.5*ee2))

***************************************************************************************************
*      Estimating Theta and Phi with a Constant Elasticity of Substitution Utility function       *
***************************************************************************************************

log using "CES_elasticities.txt", text replace

gen sp_lleisminuslc = sp_lleis - lc

xi: ivreg2 sp_lw e1 e2minusone e3 $mrs_cntrl $yeardummies (sp_lleisminuslc = $instlist) if sp_employed==1, fuller(1) 

qui gen theta  = _b[sp_lleisminuslc]+1
qui gen phi    = _b[sp_lleisminuslc]+1

scalar theta = theta[1]
scalar phi   = phi[1]

matrix define mrs_= e(b)
svmat mrs_, name(matcol)

qui reg sp_lw $mrs_cntrl $instlist $yeardummies if sp_employed==1
qui predict lnimpwage, xb

foreach var of varlist $yeardummies {
	qui replace lnimpwage = lnimpwage - _b[`var']*`var'
}


makealpha, wagevar(sp_lw) leisurevar(sp_lleis)

save "temp", replace

prepeuler, groupvars($groupvars)

qui xi: ivreg2 $depvar $euler_exogvar ($euler_endogvar = $euler_instvar) [aw=Nobs], fuller(1)
matrix wholesampleresults = e(b)

if "$depvar" == "Z" {
    matrix sigmavar = wholesampleresults[1,"W"]
    global sigma = (1/sigmavar[1,1])+1
}
if "$depvar" == "W" {
    matrix sigmavar = wholesampleresults[1,"Z"]
    global sigma = (sigmavar[1,1]+1)
}

use "temp", clear
calcfrisch, sigma($sigma)
drop term

genmarshallandhicks

foreach var in h l c {

    qui centile M`var', centile(22.5 27.5)
    gen lqM`var' = inrange(M`var',r(c_1),r(c_2))

    qui centile M`var', centile(47.5 52.5)
    gen medianM`var' = inrange(M`var',r(c_1),r(c_2))

    qui centile M`var', centile(72.5 77.5)
    gen uqM`var' = inrange(M`var',r(c_1),r(c_2))

    foreach quant in lq median uq {
        qui su M`var' if `quant'M`var'==1
        scalar `quant'M`var' = r(mean)
        qui su H`var' if `quant'M`var'==1
        scalar `quant'H`var' = r(mean)
        qui su F`var' if `quant'M`var'==1
        scalar `quant'F`var' = r(mean)
    }

}

capture matrix drop tabresults	
#delimit ;
foreach var in 
lqMc medianMc uqMc lqHc medianHc uqHc lqFc medianFc uqFc
lqMh medianMh uqMh lqHh medianHh uqHh lqFh medianFh uqFh
lqMl medianMl uqMl lqHl medianHl uqHl lqFl medianFl uqFl {;
    matrix temp = `var';
    matrix tabresults = (nullmat(tabresults)\temp);
};
matrix drop temp;

matrix rownames tabresults = 
lqMc medianMc uqMc lqHc medianHc uqHc lqFc medianFc uqFc
lqMh medianMh uqMh lqHh medianHh uqHh lqFh medianFh uqFh
lqMl medianMl uqMl lqHl medianHl uqHl lqFl medianFl uqFl;
#delimit cr

matrix list tabresults
log close


***************************************************************************************************
*                                 Cobb-Douglas Utility Function                                   *
***************************************************************************************************

log using "CobbDouglas_elasticities.txt", text replace

capture program drop prepeuler_cobbdouglas
program prepeuler_cobbdouglas

    syntax, groupvars(varlist)

    local phi = phi
    local theta = theta

    qui gen la = ln(alpha)

    qui gen mvar = lc + (aterm)*sp_lleis
    qui assert mvar > 0

    qui gen lnaterm = psiz+newvi
    qui gen tocorrect = lnaterm>80 & aterm==.
    qui gen tocorrectvlargeaterm = lnaterm>95 & aterm==.
    qui gen mvarsmall = (((cons^phi)-1)/phi)/10000 + exp(psiz+newvi + ln(((sp_leisure^theta)-1)/theta) - ln(10000)) if tocorrect==1
    qui gen mvarsmallvlargeaterm = (((cons^phi)-1)/phi)/100000000 + exp(psiz+newvi + ln(((sp_leisure^theta)-1)/theta) - ln(100000000)) if tocorrectvlargeaterm==1

    qui gen lmvar = ln(mvar)
    qui replace lmvar = ln(mvarsmall) + ln(10000) if tocorrect==1
    qui replace lmvar = ln(mvarsmallvlargeaterm) + ln(100000000) if tocorrectvlargeaterm==1
    capture assert lmvar~=.
    if _rc==9 {
        di "Error in prepeuler: ln M is missing"
        pause
    }

    qui gen lfp_childage02 = (sp_employed)*childage02

    qui gen lc_temp1 = ln(cons)
    qui gen lw_temp1 = ln(sp_w)
    qui gen lhw_temp1 = ln(h_w)
    qui gen ll_temp1 = ln(sp_leisure)
    qui gen lm_temp1 = lmvar
    qui gen lhearn_temp1 = h_logearn
    qui gen lspearn_temp1 = sp_logearn
    qui gen lfp_temp1 = sp_employed
    qui gen lfp_childage02_temp1 = lfp_childage02
    qui gen hlfp_temp1 = h_employed
    qui gen la_temp1 = la

    foreach var in sp_age haskids {
        qui gen `var'_lm_temp1 = `var'*lm_temp1
    }

    qui gen lrr = ln(1+rr)
    qui gen lir = ln(1+ir)
    qui gen lpi = ln(1+pi)

    foreach var in $dem_cntrl sp_educ1 sp_educ2 sp_educ3 h_age h_age2 h_age3 {
        qui gen `var'_temp1 = `var'
    }

    qui gen Nobs = 1

    /*t*/
    preserve
    collapse (mean) *_temp1 lrr lir lpi (sum) Nobs, by(`groupvars')
    rename lc_temp1 lc1
    rename lw_temp1 lw1
    rename lhw_temp1 lhw1
    rename ll_temp1 ll1
    rename lm_temp1 lm1
    rename lrr lrr1
    rename lir lir1
    rename lpi lpi1
    rename lfp_temp1 lfp1
    rename lfp_childage02_temp1 lfp_childage021
    rename hlfp_temp1 hlfp1
    rename lhearn_temp1 lhearn1
    rename lspearn_temp1 lspearn1
    rename la_temp1 la1

    foreach var in sp_age haskids {
        qui rename `var'_lm_temp1  `var'_lm1
    }

    foreach var in $dem_cntrl sp_educ1 sp_educ2 sp_educ3 h_age h_age2 h_age3 {
        rename `var'_temp1 `var'1
    }

    sort `groupvars'
    qui save "temp1", replace
    restore

    /*t-1*/
    qui gen lc_temp2 = ln(cons)
    qui gen lw_temp2 = ln(sp_w)
    qui gen lhw_temp2 = ln(h_w)
    qui gen ll_temp2 = ln(sp_leisure)
    qui gen lm_temp2 = lmvar
    qui gen lhearn_temp2 = h_logearn
    qui gen lspearn_temp2 = sp_logearn
    qui gen lfp_temp2 = sp_employed
    qui gen lfp_childage02_temp2 = lfp_childage02
    qui gen hlfp_temp2 = h_employed
    qui gen la_temp2 = la

    foreach var in sp_age haskids {
        qui gen `var'_lm_temp2 = `var'*lm_temp2
    }


    foreach var in $dem_cntrl sp_educ1 sp_educ2 sp_educ3 h_age h_age2 h_age3 {
        qui gen `var'_temp2 = `var'
    }

    preserve
    collapse (mean) *_temp2 lrr lir lpi , by(`groupvars')
    qui replace qtr = qtr+1
    qui replace yr = yr+1 if qtr>4
    qui replace qtr = qtr-4 if qtr>4
    foreach var in $dem_cntrl sp_educ1 sp_educ2 sp_educ3 h_age h_age2 h_age3 {
        rename `var'_temp2 `var'2
        }
    rename lc_temp2 lc2
    rename lw_temp2 lw2
    rename lhw_temp2 lhw2
    rename ll_temp2 ll2
    rename lm_temp2 lm2
    rename lfp_temp2 lfp2
    rename lfp_childage02_temp2 lfp_childage022
    rename hlfp_temp2 hlfp2
    rename lhearn_temp2 lhearn2
    rename lspearn_temp2 lspearn2
    rename lrr lrr2
    rename lir lir2
    rename lpi lpi2
    rename la_temp2 la2
    foreach var in sp_age haskids {
        qui rename `var'_lm_temp2  `var'_lm2
    }

    sort `groupvars'
    qui save "temp2", replace
    restore

    /*t-2*/
    qui gen lc_temp3 = ln(cons)
    qui gen lw_temp3 = ln(sp_w)
    qui gen lhw_temp3 = ln(h_w)
    qui gen ll_temp3 = ln(sp_leisure)
    qui gen lfp_temp3 = sp_employed
    qui gen lfp_childage02_temp3 = lfp_childage02
    qui gen hlfp_temp3 = h_employed
    qui gen lm_temp3 = lmvar
    qui gen lhearn_temp3 = h_logearn
    qui gen lspearn_temp3 = sp_logearn
    qui gen la_temp3 = la
    foreach var in $dem_cntrl sp_educ1 sp_educ2 sp_educ3 h_age h_age2 h_age3 {
        qui gen `var'_temp3 = `var'
    }
    foreach var in sp_age haskids {
        qui gen `var'_lm_temp3 = `var'*lm_temp3
    }

    preserve
    drop if intno==2
    collapse (mean) *_temp3 lrr lir lpi, by(`groupvars')
    qui replace qtr = qtr+2
    qui replace yr = yr+1 if qtr>4
    qui replace qtr = qtr-4 if qtr>4
    foreach var in $dem_cntrl sp_educ1 sp_educ2 sp_educ3 h_age h_age2 h_age3 {
        rename `var'_temp3 `var'3
    }


    rename lc_temp3 lc3
    rename lw_temp3 lw3
    rename lhw_temp3 lhw3
    rename ll_temp3 ll3
    rename lfp_temp3 lfp3
    rename lfp_childage02_temp3 lfp_childage023
    rename hlfp_temp3 hlfp3
    rename lhearn_temp3 lhearn3
    rename lspearn_temp3 lspearn3
    rename lrr lrr3
    rename lir lir3
    rename lpi lpi3
    rename lm_temp3 lm3
    rename la_temp3 la3
    foreach var in sp_age haskids {
        qui rename `var'_lm_temp3  `var'_lm3
    }

    sort `groupvars'
    qui save "temp3", replace
    restore

    /*t-3*/
    qui gen lc_temp4 = ln(cons)
    qui gen lw_temp4 = ln(sp_w)
    qui gen lhw_temp4 = ln(h_w)
    qui gen ll_temp4 = ln(sp_leisure)
    qui gen lfp_temp4 = sp_employed
    qui gen lfp_childage02_temp4 = lfp_childage02
    qui gen hlfp_temp4 = h_employed
    qui gen lm_temp4 = lmvar
    qui gen lhearn_temp4 = h_logearn
    qui gen lspearn_temp4 = sp_logearn
    qui gen la_temp4 = la
    foreach var in $dem_cntrl sp_educ1 sp_educ2 sp_educ3 h_age h_age2 h_age3 {
        qui gen `var'_temp4 = `var'
    }
    foreach var in sp_age haskids {
        qui gen `var'_lm_temp4 = `var'*lm_temp4
    }



    preserve
    drop if intno==2
    collapse (mean) *_temp4 lrr lir lpi, by(`groupvars')
    qui replace qtr = qtr+3
    qui replace yr = yr+1 if qtr>4
    qui replace qtr = qtr-4 if qtr>4
    foreach var in $dem_cntrl sp_educ1 sp_educ2 sp_educ3 h_age h_age2 h_age3 {
        rename `var'_temp4 `var'4
    }

    rename lc_temp4 lc4
    rename lw_temp4 lw4
    rename lhw_temp4 lhw4
    rename ll_temp4 ll4
    rename lfp_temp4 lfp4
    rename lfp_childage02_temp4 lfp_childage024
    rename hlfp_temp4 hlfp4
    rename lhearn_temp4 lhearn4
    rename lspearn_temp4 lspearn4
    rename lrr lrr4
    rename lir lir4
    rename lpi lpi4
    rename lm_temp4 lm4
    rename la_temp4 la4

    foreach var in sp_age haskids {
        qui rename `var'_lm_temp4  `var'_lm4
    }

    sort `groupvars'
    qui save "temp4", replace
    restore

    /*t-4*/
    qui gen lc_temp5 = ln(cons)
    qui gen lw_temp5 = ln(sp_w)
    qui gen lhw_temp5 = ln(h_w)
    qui gen ll_temp5 = ln(sp_leisure)
    qui gen lfp_temp5 = sp_employed
    qui gen lfp_childage02_temp5 = lfp_childage02
    qui gen hlfp_temp5 = h_employed
    qui gen lm_temp5 = lmvar
    qui gen lhearn_temp5 = h_logearn
    qui gen lspearn_temp5 = sp_logearn
    qui gen la_temp5 = la
    foreach var in $dem_cntrl sp_educ1 sp_educ2 sp_educ3 h_age h_age2 h_age3 {
        qui gen `var'_temp5 = `var'
    }

    foreach var in sp_age haskids {
        qui gen `var'_lm_temp5 = `var'*lm_temp5
    }

    preserve
    drop if intno==2
    collapse (mean) *_temp5 lrr lir lpi, by(`groupvars')
    qui replace yr = yr+1
    foreach var in $dem_cntrl sp_educ1 sp_educ2 sp_educ3 h_age h_age2 h_age3 {
        rename `var'_temp5 `var'5
    }
    rename lc_temp5 lc5
    rename lw_temp5 lw5
    rename lhw_temp5 lhw5
    rename ll_temp5 ll5
    rename lfp_temp5 lfp5
    rename lfp_childage02_temp5 lfp_childage025
    rename hlfp_temp5 hlfp5
    rename lhearn_temp5 lhearn5
    rename lspearn_temp5 lspearn5
    rename lrr lrr5
    rename lir lir5
    rename lpi lpi5
    rename lm_temp5 lm5
    rename la_temp5 la5

    foreach var in sp_age haskids {
        qui rename `var'_lm_temp5  `var'_lm5
    }

    sort `groupvars'
    qui save "temp5", replace
    restore

    use "temp1", clear

    qui merge 1:1 `groupvars' using "temp2",nogen
    qui merge 1:1 `groupvars' using "temp3",nogen
    qui merge 1:1 `groupvars' using "temp4",nogen
    qui merge 1:1 `groupvars' using "temp5",nogen

    erase temp1.dta
    erase temp2.dta
    erase temp3.dta
    erase temp4.dta
    erase temp5.dta

    forval i = 1/4 {
        local j = `i'+1
        qui gen dlc`i' = lc`i'-lc`j'
        qui gen dlw`i' = lw`i'-lw`j'
        qui gen dlfp`i' = lfp`i' - lfp`j'
        qui gen dlfp_childage02`i' = lfp_childage02`i' - lfp_childage02`j'
        qui gen dlrr`i' = lrr`i' - lrr`j'
        qui gen dlm`i' = lm`i' - lm`j'
        qui gen dla`i' = la`i' - la`j'
        qui gen dll`i' = la`i' - la`j'
    }

    foreach var in $dem_cntrl sp_educ1 sp_educ2 sp_educ3 h_age h_age2 h_age3 {
        qui gen d`var'1 = `var'1 - `var'2
    }

    foreach var in sp_age_lm haskids_lm {
        qui gen d`var'1 = `var'1 - `var'2
    }

    qui gen season1 = intmo==1 & qtr==1
    qui gen season2 = intmo==2 & qtr==1
    qui gen season3 = intmo==3 & qtr==1
    qui gen season4 = intmo==1 & qtr==2
    qui gen season5 = intmo==2 & qtr==2
    qui gen season6 = intmo==3 & qtr==2
    qui gen season7 = intmo==1 & qtr==3
    qui gen season8 = intmo==2 & qtr==3
    qui gen season9 = intmo==3 & qtr==3
    qui gen season10 = intmo==1 & qtr==4
    qui gen season11 = intmo==2 & qtr==4
    qui gen _season12 = intmo==3 & qtr==4 

    qui gen Z = dlm1
    qui gen W = (1-`phi')*dlc1 - 0.25*lrr1

end

capture program drop calcfrisch_cobbdouglas
program calcfrisch_cobbdouglas

    syntax, sigma(numlist)

    gen sigma = `sigma'

    qui gen M = lc + (aterm)*sp_lleis

    qui gen double uc =M^(sigma-1)*(cons^-1)
    qui gen double ul =M^(sigma-1)*(aterm)*(sp_leisure^-1)
    qui gen double ucc=M^(sigma-1)*(cons^-2)*((sigma-1)/M -1)
    qui gen double ull=M^(sigma-1)*(aterm)*(sp_leisure^-2)*((sigma-1)*aterm/M -1)
    qui gen double ucl=(sigma-1)*M^(sigma-2)*(cons^-1)*(sp_leisure^-1)*aterm

    qui gen double term=ucc*ull-ucl^2

    qui gen double Fc=-ucl*uc*sp_w/(cons*term)

    qui gen double Fl=ucc*uc*sp_w/(sp_leisure*term)
    qui gen double Fh=-Fl*sp_leisure/($T-sp_leisure)

    qui gen double riskaversion = -cons*ucc/uc

end


use dataset2, clear

qui xi: probit sp_employed h_logearn h_logearnflag $dem_cntrl $instlist $oth_cntrl $yeardummies, difficult iterate(50)

predict ee,index
qui gen ee2=ee*ee
qui gen fi=1/normprob(ee)
qui gen e1 = fi/(sqrt(6.2831853)*exp(0.5*ee2))
qui gen e2 = fi*ee/(sqrt(6.2831853)*exp(0.5*ee2)) + 1
qui gen e2minusone = e2-1
qui gen e3 =(2+ee2)*fi/(sqrt(6.2831853)*exp(0.5*ee2))

gen newdepvar = sp_lw - lc + sp_lleis
xi: reg newdepvar e1 e2minusone e3 $mrs_cntrl $yeardummies if sp_employed==1

qui gen theta  = 0
qui gen phi    = 0

scalar theta = theta[1]
scalar phi   = phi[1]

matrix define mrs_= e(b)
svmat mrs_, name(matcol)

qui reg sp_lw $mrs_cntrl $instlist $yeardummies if sp_employed==1
qui predict lnimpwage, xb

foreach var of varlist $yeardummies {
	qui replace lnimpwage = lnimpwage - _b[`var']*`var'
}

makealpha, wagevar(sp_lw) leisurevar(sp_lleis)

save "temp", replace

prepeuler_cobbdouglas, groupvars($groupvars)

qui xi: ivreg2 $depvar $euler_exogvar ($euler_endogvar = $euler_instvar) [aw=Nobs], fuller(1)
matrix wholesampleresults = e(b)

if "$depvar" == "Z" {
    matrix sigmavar = wholesampleresults[1,"W"]
    global sigma = (1/sigmavar[1,1])+1
}
if "$depvar" == "W" {
    matrix sigmavar = wholesampleresults[1,"Z"]
    global sigma = (sigmavar[1,1]+1)
}

use "temp", clear
calcfrisch_cobbdouglas, sigma($sigma)
drop term

genmarshallandhicks

foreach var in h l c {

    qui centile M`var', centile(22.5 27.5)
    gen lqM`var' = inrange(M`var',r(c_1),r(c_2))

    qui centile M`var', centile(47.5 52.5)
    gen medianM`var' = inrange(M`var',r(c_1),r(c_2))

    qui centile M`var', centile(72.5 77.5)
    gen uqM`var' = inrange(M`var',r(c_1),r(c_2))

    foreach quant in lq median uq {
        qui su M`var' if `quant'M`var'==1
        scalar `quant'M`var' = r(mean)
        qui su H`var' if `quant'M`var'==1
        scalar `quant'H`var' = r(mean)
        qui su F`var' if `quant'M`var'==1
        scalar `quant'F`var' = r(mean)
    }

}

capture matrix drop tabresults
#delimit ;
foreach var in 
lqMc medianMc uqMc lqHc medianHc uqHc lqFc medianFc uqFc
lqMh medianMh uqMh lqHh medianHh uqHh lqFh medianFh uqFh
lqMl medianMl uqMl lqHl medianHl uqHl lqFl medianFl uqFl {;
    matrix temp = `var';
    matrix tabresults = (nullmat(tabresults)\temp);
};
matrix drop temp;

matrix rownames tabresults = 
lqMc medianMc uqMc lqHc medianHc uqHc lqFc medianFc uqFc
lqMh medianMh uqMh lqHh medianHh uqHh lqFh medianFh uqFh
lqMl medianMl uqMl lqHl medianHl uqHl lqFl medianFl uqFl;
#delimit cr

matrix list tabresults

log close




