#delimit ;
set more off;

clear;
/*imports nominal interest rate data (3 month treasury bills from the St Louis Fed)*/
/*saves rr, called by master*/

insheet using "rr.csv", comma;
gen year = substr(yr,7,4);
gen mo = substr(yr,4,2);
destring year, replace;
destring mo, replace;
recode mo (1/3 = 1) (4/6 = 2) (7/9 = 3) (10/12 = 4), generate(qtr);
drop yr;
rename year yr;

/*same as qtr*/
recode mo (1/3 = 1) (4/6 = 2) (7/9 = 3) (10/12 = 4), generate(qtr1);
recode mo (11 12 1 = 1) (2/4 = 2) (5/7 = 3) (8/10 = 4), generate(qtr2);
recode mo (12 1 2 = 1) (3/5 = 2) (6/8 = 3) (9/11 = 4), generate(qtr3);

/*change years to correspond with year of interview*/
gen yr1 = yr;
gen yr2 = yr;
replace yr2 = yr+1 if mo==11|mo==12;
gen yr3 = yr;
replace yr3 = yr+1 if mo==12;

/*merge in PCE deflator to calculate real interest rates*/
merge m:1 yr mo using "$datadir/deflator";
drop if _merge~=3;
drop _merge;
replace ir = ir/100;

/*calculate quarterly inflation rate*/
/*inflation rate between t and t+1*/
sort yr mo;
gen pi = cpi[_n+1]/cpi[_n]-1;
/*real interest rate between two periods*/
gen rr = (1+ir)/(1+pi)-1;
/*drop final observation (missing as no inflation)*/
drop if _n==_N;

/*calculate quarterly averages*/
preserve;
collapse (mean) rr ir pi, by(yr1 qtr1);
rename rr rr_qtr1;
rename ir ir_qtr1;
rename pi pi_qtr1;
save "$datadir/rr1", replace;
restore;

preserve;
collapse (mean) rr ir pi, by(yr2 qtr2);
rename rr rr_qtr2;
rename ir ir_qtr2;
rename pi pi_qtr2;
save "$datadir/rr2", replace;
restore;

preserve;
collapse (mean) rr ir pi, by(yr3 qtr3);
rename rr rr_qtr3;
rename ir ir_qtr3;
rename pi pi_qtr3;
save "$datadir/rr3", replace;
restore;

save "$datadir/rr", replace;








