!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
module equations

!this module calculates the value function given the level of consumption (if unemployed)
!or given the level of within period spending (if employed)

use params
use subrout
use wageproc
use bisection
use maknls
!!USE nag_nlp
!!USE nag_uv_min
use summaryindex
use makutility

implicit none

double precision, private :: ax, bx, cx
integer, private ::  ier, xtonon, xtoemp

integer, parameter:: np=2

double precision, private:: valatmin, valatmineps,valatmax,valatmaxeps

double precision aux, maxcon, mincon, nlin_upper(1) , nlin_lower(1), x_lower(2),x_upper(2), &
     lin_lower(1), lin_upper(1), avals(1,2), axx, cxx

integer iter, num_nlin_con, num_lin_con

character (24) restime

!!!!!for powell routines at end of module
    integer, parameter :: nopars=1, NNMAX=20,ITMAXP=200
    integer, private :: ncom
    double precision, private ::  pcom(NNMAX),xicom(NNMAX)
!!!!!



contains


subroutine valnon(valuenon,connon)	 !solving for optimal consumption given not working

double precision connon, valuenon, valuebit, connon2, valuenon2
	
	!	in the non-work section, withnon = connon (avoids problem with fixed cost and value of leis

	!algorithm for solving for optimal value function at asset etc.
	! try using "golden" minimisation routine - does not require derivatives

if (age.le.nt) then 

	! possible initial consumption levels
	if(age.lt.nt.and.noexp.gt.1)then
		xtonon = max(mapping(age,xvalue,0,0.d0),1)
	else
		xtonon = 1
	endif


		ax = minass/2.d0
		cx = max(partcash + mininc(xtonon,age)-minass*0.25d0,minass)				
		bx = cx*0.6d0
	
	

	!perhaps need to check that these consumption levels are ok

	if (borrowdum.le.1) then  !ie saving (and maybe borrowing) allowed 
		
          valatmin = unempfun(ax)
          valatmineps = unempfun(ax*eps1)
          valatmax = unempfun(cx)
          valatmaxeps = unempfun(cx*eps)

          if (valatmin.lt.valatmineps) then !stay at min point

               valuenon = - valatmin
               connon   = ax
             if(solutiongrid==1)countcornerUmin(ki,age)=countcornerUmin(ki,age)+1

          elseif (valatmax.lt.valatmaxeps) then ! stay at max point

               valuenon = - valatmax
               connon   = cx
               
              if(solutiongrid==1)countcornerUmax(ki,age)=countcornerUmax(ki,age)+1
               
          elseif (cx-ax.lt.1.d-7) then 
          
               valuenon = - valatmaxeps
               connon   = cx*eps              
          
          else

   		 valuenon = golden(ax,bx,cx,unempfun,connon)
	     valuenon = - valuenon !solves for min of negative objective

		conold=connon

		wnodeunold = wnode
		pnodeunold = pnode
     	xvalueunold = xvalue

		     
         endif
		 
	  

	else  !no borrowing or saving

			connon		= partcash
			valuenon	= - unempfun(connon)

	endif


elseif(age.eq.nt+ret)then

	valuenon = utility(partcash, hours,age)	
	connon   = partcash
	
	utoday=valuenon
	utomorrow=0.d0
	
else
    
    if(borrowdum.le.1) then 

	ax = minass/2.d0
	cx = max(partcash - minass*0.25d0,minass)				
	bx = cx*0.6d0

	valuenon = golden(ax,bx,cx,retirefun,connon)

    valuenon = - valuenon 
    
    else
        connon		= partcash
	    valuenon	= - retirefun(connon)
    
    endif



endif	
			
end subroutine valnon

! ************************************************
subroutine valemp(value,withemp, leis)
double precision valuef, valuep, value, withempf, withempp, withemp, leisp, leisf, leis

if(hoursdecision==0)then !extensive margin decision ONLY
    cash = partcash + wage*hoursfix - fixed(age)
    
    if(age.lt.nt.and.noexp.gt.1)then
		xtoemp = max(mapping(age,xvalue,1, max(hours*hoursdecision,hoursfix) ),1)  !assume hours to define minincome (it does not matter)		
	else
		xtoemp = 1
	endif
	
	ax = min(minass*0.5d0, 0.5d0*(cash + mininc(xtoemp,age)))
	cx = cash + mininc(xtoemp,age)
	bx = cx*0.6d0

	if (borrowdum.le.1) then !ie saving (and maybe borrowing) are allowed 

		value = golden(ax,bx,cx,empfunhoursfix,withemp)

		if (withemp.ge.cx) then
			write (*,*) 'prob in optimial c emp'
		elseif (withemp.le.ax) then 
			write (*,*) 'prob in optima c emp too low '
		endif

		value = - value !solves for min of negative objective
		
		leis=hourseis

	else  !saving and borrowing not allowed

		withemp	 = cash
		value = - empfunhoursfix(withemp)
		leis=hourseis

	endif

else  !intensive margin decision

    valuef=-100000.d0
    valuep=-100000.d0

    call valempfull(valuef,withempf,leisf)
    value=valuef
    withemp=withempf
    leis=leisf
    
endif

end subroutine valemp



! ************************************************

subroutine valempfull(value,withemp,leis)
use summaryindex
double precision value, withemp, leis, expend, axx,cxx,bxx, check1,check2, obj_f, & 
     time1, time2, check1a, check2a,  tempc, tempa, &
     leisatmin, leisatmax, leisinstep1, consatmin, consatmax
     
integer rate, cptime, i, corner
logical obj_deriv, con_deriv


if (age.le.nt) then 

	! how much expenditure if working all hours
	
	
	cash = partcash + wage*max(hours*hoursdecision,hoursfix)  - fixed(age)  !potential cash
    

	if(age.lt.nt.and.noexp.gt.1)then
		xtoemp = max(mapping(age,xvalue,1, max(hours*hoursdecision,hoursfix) ),1)  !assume hours to define minincome (it does not matter)		
	else
		xtoemp = 1
	endif


	axx = min(minass*0.5d0, 0.5d0*(cash + mininc(xtoemp,age)))
	cxx = cash + mininc(xtoemp,age)     
	bxx = cxx*0.6d0

	
	if (borrowdum.le.1) then !ie saving (and maybe borrowing) are allowed 
		!is this a corner solution in expenditure? 
		valatmin = optimalexpfull(axx)
		leisatmin = leisin
		consatmin = consu
		valatmineps = optimalexpfull(axx*eps1)

		valatmax = optimalexpfull(cxx)
		leisatmax = leisin
		consatmax = consu
		valatmaxeps = optimalexpfull(cxx*eps)

		if (valatmin.lt.valatmineps) then !stay at min point
   			value = - valatmin
			 leis = leisatmin
			 withemp = consatmin
			 
			 
			  if(solutiongrid==1)countcornerEmin(ki,age)=countcornerEmin(ki,age)+1
			  
		elseif (valatmax.lt.valatmaxeps) then ! stay at max point
			 value = - valatmax
			 leis = leisatmax
			 withemp = consatmax
			 
			
			 if(solutiongrid==1)countcornerEmax(ki,age)=countcornerEmax(ki,age)+1
		else

 			obj_f = golden(axx,bxx,cxx,optimalexpfull,expend)
			
			leisold=leisin
			expendold=expend

			wnodeold = wnode
			pnodeold = pnode
     		xvalueold = xvalue
			
			withemp=consu
			leis=leisin
			
			value = - obj_f  !solves for min of negative objective

		endif !corner solution

	else  !saving and borrowing not allowed

		totexpend = cash
		value = - optimalexpfull(totexpend)  
		
		
		leisold=leisin
		expendold=expend

		wnodeold = wnode
		pnodeold = pnode
 		xvalueold = xvalue

        leis=leisin
		withemp=totexpend - leis*wage
		

	endif  !borrowing constrains
else
	value=-10000.d0
endif

end subroutine valempfull


!*****************************************************
double precision function optimalexpfull(expe)
double precision aux1, aux2, expe, ERRABS, leis, leisguess, lim_inf,lim_sup, val1, val2,sav,con, minleis

val1=-100.d0
val2=-100.d0

totexpend=expe

ERRABS=0.00000001d0

minleis=hours-1250.d0

lim_inf=min(minleis,totexpend/wage-2.d0)
lim_sup=min(hours-minhours,totexpend/wage-minhours)




if(abs(lim_inf-lim_sup).lt.2.d0.or.lim_sup.lt.0.d0)then

    leis=0.5d0*totexpend/wage
	con=totexpend-wage*leis
	 if(con.le.0.d0)then
		write(888,*)'neg cons-1',totexpend,leis,con,wage,lim_inf,lim_sup
	endif
			
	
else
    val1=optimalleis(lim_inf)
    val2=optimalleis(lim_sup)
    if(val1*val2.lt.0.d0)then     !interior solution
		    leis=zbrent(optimalleis,lim_inf,lim_sup, ERRABS )
			con=totexpend-wage*leis

			if(con.le.0.d0)then
		write(888,*)'neg cons-2',totexpend,leis,con,wage,lim_inf,lim_sup
	endif
		
			
	elseif(val2.gt.0.d0)then     !corner solution: at max leis still want to increase leis
		    leis=lim_sup
			con=totexpend-wage*leis
		
if(con.le.0.d0)then
		write(888,*)'neg cons-3',totexpend,leis,con,wage,lim_inf,lim_sup
	endif		
			
	elseif(val1.lt.0.d0)then    !corner solution: at min leis you still want to deacrease leis
		    leis=lim_inf
			con=totexpend-wage*leis

			if(con.le.0.d0)then
		write(888,*)'neg cons-4',totexpend,leis,con,wage,lim_inf,lim_sup
	endif
			
	endif
endif	


optimalexpfull = empfun(leis)
leisin = leis



end function optimalexpfull
!*****************************************************


double precision function optimalleis(x)
double precision sav, con, x, hourswk, experto

    con=totexpend-wage*x


	optimalleis= -wage*margutc(con,x,age) + margutl(con,x,age) 


end function


!*************************************************************************************


double precision function empfun(leis)
use summaryindex
double precision  cashtom, con, consto, estim1, estim4, experto,   expto, &
		intaet, intxet, leis, leisto, probhigh, problow, saving,  & 
		totvalue, utnow, valtobit2, valtom, vallow, valhigh, wagto, hourswk


	integer  mina, nodeasbit, nodebit, nodebit2, node, nodeas, &
		nodebit2un, nodeasun, nxx, p, w, xmin, xtom, xx

	con = totexpend - leis*wage
	
	consu = con

	if(con.le.0.d0)then
		write(*,*)'neg cons',age,cash, totexpend, wage,leis
	endif
	
	if(leis.lt.0.d0)then
		write(*,*)'neg leis',age,cash, totexpend, wage,leis
	endif

	utnow = utility(con, leis,age)
	hourswk = hours - leis

	if(hourswk.lt.0.d0)then
		write(25,*)'neg hourswk'
	endif
	
	saving = detrr*(partcash + wage*hourswk - con - fixed(age))

	totvalue = 0.d0


if(ageto.gt.nt)then

	!now calculate expected value
	part		= pxw(pnode,nt)*ssi  
	cashtom		= saving + part
	w=1
	xtom=1
	p=pnode
	leisto = hours

	call find(ageto, cashtom, 1, mina, intaet, intxet, 0)
	node = sumindex(nmindex2,(/w,pnode,xtom,mina/))
    estim1    = valun(node,ageto)     
	estim4    = valun(node+1, ageto)  
	totvalue   = (intaet*estim1 + intxet*estim4) 

    				
else
	experto = humancap(age, expvalue(xvalue,age), hourswk )
	xmin = mapping(age,xvalue,1, hourswk)

	nxx=2
	if(noexp.ge.2)then	
		if (xmin.eq.0) then 
				xmin=1
				probhigh = 0.d0
				nxx=1
		elseif (xmin.eq.noexp) then
				xmin=noexp-1 
				probhigh = 1.d0
		elseif (xmin.lt.noexp) then
				probhigh = 1.d0 - (expvalue(xmin+1,ageto) - experto) / & 
								(expvalue(xmin+1,ageto) - expvalue(xmin,ageto) ) 
		endif

	else
		xmin=1
		probhigh=0.d0
		nxx=1
	endif

	problow = 1.d0 - probhigh

	call futureval(xmin,nxx,saving,vallow,valhigh)
	totvalue= vallow*problow + valhigh*probhigh

endif !!!if ageto>nt



empfun = - (utnow + totvalue*disc)

utoday=utnow
utomorrow=totvalue




end function empfun
!*************************************************************************************

subroutine futureval(xmin,nxx,saving,vlow,vhigh)

double precision  cashtom, con, consto, estim1, estim4, experto,   expto, &
		intaet, intxet, leis, leisto, probhigh, problow, probQ, saving,  & 
		totvalue, utnow, valtobit2, valtom, wagto, hourswk, minleis, vlow, vhigh


	integer  finishw, finishp, mina, nodeasbit, nodebit, nodebit2, node, nodeas, &
		nodebit2un, nodeasun, nxx, p, startw, startp, trans, w, xmin, xtom, xx

vlow=0.d0  
vhigh=0.d0

do xx = 1, nxx

	expto  = expvalue(min(noexp,xmin + (xx-1)), ageto) 
	xtom   = min(xmin + (xx-1),noexp)		

	!now calculate expected value
	valtom  = 0.d0
	
	do w=1,nw
	
		valtobit2 = 0.d0
		do p=1,pnw

		    probQ=(1-quarter)*prob(w,p,wnode,pnode,ageto)+&
		                    &quarter*0.25d0*prob(w,p,wnode,pnode,ageto)
            
            if (dsolve(w,p,wnode,pnode,ageto).gt.0) then 
						part		= pxw(p,ageto)  
						cashtom		= saving + part
						nodeas = sumindex(nmindex,(/w,p,xtom/))
						if (cashtom.ge.asscut(nodeas,ageto)) then !not working
								leisto = hours
								call find(ageto, cashtom, nodeas, mina, intaet, intxet, 0)
								node = sumindex(nmindex2,(/w,p,xtom,mina/))
		
								estim1    = valun(node,ageto)     
								estim4    = valun(node+1, ageto)  
								valtobit2 = valtobit2 + (intaet*estim1 + intxet*estim4)*probQ 
								
								if(wnode.eq.w.AND.pnode.eq.p)valtobit2 = valtobit2 + (intaet*estim1 + intxet*estim4)*quarter*0.75d0
								
						else				 
								call find(ageto, cashtom, nodeas, mina, intaet, intxet,1)
								node = sumindex(nmindex2,(/w,p,xtom,mina/))

								estim1 = valemploy(node,ageto)     
								estim4 = valemploy(node+1, ageto)  
								valtobit2 = valtobit2 + (intaet*estim1 + intxet*estim4)*probQ  
								
								if(wnode.eq.w.AND.pnode.eq.p)valtobit2 = valtobit2 + (intaet*estim1 + intxet*estim4)*quarter*0.75d0 
	

						endif ! end work staus check
						
					

				endif  !dsolve
				
			enddo !end partner

			valtom = valtom + valtobit2 
			

		enddo !end productivity loop
	

		if (xx.eq.1) vlow  = valtom 
		if (xx.eq.2) vhigh = valtom
	

	enddo !end experience loop
	


end subroutine futureval


!*****************************************************
double precision function unempfun(conun)
use summaryindex

	double precision cashtom, conun, consto, estim1, estim4, &
		experto, expto, intaet,  intxet, leisto, probhigh, problow, saving, & 
		totvalue, utnow, valtobit2, valtom, vallow, valhigh,&
		wagto

	integer mina, node,nodebit, nodebit2, nodeas, nodeasbit, &
				nodebit2un, nodeasun, nxx, p, xmin, xtom, xx, w

	utnow = utility(conun, hours,age)
	
	if(conun.le.0.d0)then
		write(*,*)'neg cons unemp',cash
	endif
		
	saving = detrr*(partcash - conun)

	! if not in final period
	! find next periods asset stock on grid  
	
	
			

	totvalue = 0.d0

if(ageto.gt.nt)then

	!now calculate expected value
	part		= pxw(pnode,nt)*ssi  
	cashtom		= saving + part
	w=1
	xtom=1
	p=pnode
	!nodeas = sumindex(nmindex,(/w,p,xtom/))
	leisto = hours

	call find(ageto, cashtom, 1, mina, intaet, intxet, 0)
	node = sumindex(nmindex2,(/w,pnode,xtom,mina/))
    estim1    = valun(node,ageto)     
	estim4    = valun(node+1, ageto)  
	totvalue   = (intaet*estim1 + intxet*estim4) 
    				
else

	experto = expvalue(xvalue,age) * (1.d0 - deltaperF(expvalue(xvalue,age)))

	xmin = mapping(age,xvalue,0,0.d0)

	nxx=2

	if(noexp.gt.2)then	
		if (xmin.eq.0) then
					xmin=1
					probhigh = 0.d0
					nxx=1
		elseif (xmin.eq.noexp) then 
					xmin=noexp-1
					probhigh = 1.d0
		elseif (xmin.lt.noexp) then
					probhigh = 1.d0 - (expvalue(xmin+1,ageto) - experto) / & 
							(expvalue(xmin+1,ageto) - expvalue(xmin,ageto) ) 
		endif
	else
		xmin=1
		probhigh=0.d0
		nxx=1
	endif

	problow = 1.d0 - probhigh
														 
	
	call futureval(xmin,nxx,saving,vallow,valhigh)

	totvalue= vallow*problow + valhigh*probhigh
	
	
endif !!!if ageto>nt

	unempfun = - (utnow + totvalue*disc)

    utoday=utnow
    utomorrow=totvalue
	
	
end function unempfun

!*****************************************************
double precision function retirefun(conun)
use summaryindex

	double precision cashtom, conun, consto, estim1, estim4, &
		experto, expto, intaet,  intxet, leisto, probhigh, problow, saving, & 
		totvalue, utnow, valtobit2, valtom, &
		wagto

	integer mina, node,nodebit, nodebit2, nodeas, nodeasbit, &
				nodebit2un, nodeasun, p, xmin, xtom, xx, w

	utnow = utility(conun, hours,age)
		
	saving = detrr*(partcash - conun)
	totvalue = 0.d0
	valtom  = 0.d0
    cashtom		= saving + pxw(pnode,nt)*ssi

    leisto = hours

	call find(ageto, cashtom, 1, mina, intaet, intxet, 0)
	w = 1
	xtom = 1
	p = pnode
	node = sumindex(nmindex2,(/w,p,xtom,mina/))
    		
	estim1    = valun(node,ageto)     
	estim4    = valun(node+1, ageto)  
	valtobit2 = (intaet*estim1 + intxet*estim4)    					  				

	retirefun = - (utnow + valtobit2*disc)
	
	utoday=utnow
	utomorrow=valtobit2
	
	
end function retirefun



!*****************************************************

double precision function empfunhoursfix(con)
use summaryindex
double precision  cashtom, con, consto, estim1, estim4, experto,   expto, &
		intaet, intxet, leis, leisto, probhigh, problow, saving,  & 
		totvalue, utnow, valtobit2, valtom, vallow, valhigh, wagto, hourswk


	integer  mina, nodeasbit, nodebit, nodebit2, node, nodeas, &
		nodebit2un, nodeasun, nxx, p, w, xmin, xtom, xx

	

	if(con.le.0.d0)then
		write(*,*)'neg cons',cash, totexpend, wage,leis
	endif
	
	if(leis.lt.0.d0)then
		write(*,*)'neg leis',cash, totexpend, wage,leis
	endif

	utnow = utility(con, hourseis,age)
	hourswk = hoursfix 
	
	if(hourswk.lt.0.d0)then
		write(25,*)'neg hourswk'
	endif
	
	saving = detrr*(partcash + wage*hourswk - con - fixed(age))

	totvalue = 0.d0


if(ageto.gt.nt)then

	!now calculate expected value
	part		= pxw(pnode,nt)*ssi  
	cashtom		= saving + part
	w=1
	xtom=1
	p=pnode
	leisto = hours

	call find(ageto, cashtom, 1, mina, intaet, intxet, 0)
	node = sumindex(nmindex2,(/w,pnode,xtom,mina/))
    estim1    = valun(node,ageto)     
	estim4    = valun(node+1, ageto)  
	totvalue   = (intaet*estim1 + intxet*estim4) 

    				
else
	experto = humancap(age, expvalue(xvalue,age), hourswk )
	xmin = mapping(age,xvalue,1, hourswk)

	nxx=2
	if(noexp.ge.2)then	
		if (xmin.eq.0) then 
				xmin=1
				probhigh = 0.d0
				nxx=1
		elseif (xmin.eq.noexp) then
				xmin=noexp-1 
				probhigh = 1.d0
		elseif (xmin.lt.noexp) then
				probhigh = 1.d0 - (expvalue(xmin+1,ageto) - experto) / & 
								(expvalue(xmin+1,ageto) - expvalue(xmin,ageto) ) 
		endif

	else
		xmin=1
		probhigh=0.d0
		nxx=1
	endif

	problow = 1.d0 - probhigh

	call futureval(xmin,nxx,saving,vallow,valhigh)
	totvalue= vallow*problow + valhigh*probhigh

endif !!!if ageto>nt



empfunhoursfix = - (utnow + totvalue*disc)

utoday=utnow
utomorrow=totvalue


end function empfunhoursfix



end module equations
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
