clear matrix
clear
set more off
capture log close

log using baselineSHORT.log,replace

u sample.dta






********************************************************************************************************
*DISCOUNT RATE: target ratio median wealth to median income of working age population 
********************************************************************************************************

sum asset inc if age<62,d

table age, c (mean stat)

********************************************************************************************************
*INTIAl----EXOGENOUS------ WAGE GENDER GAP
********************************************************************************************************
sum part if t==1, d
sum wage if t==1, d



*****************************************************************************************************
*LABOR MARKET: select individual aged 25 TO 60 (in the revision to Eca we moved to waves 1980 to 2012 
*so we now observe cohort 1950 until age 60 (instead of 52 that we did before)
*****************************************************************************************************

keep if age>=25 & age<=55





********************************************************************************************************************
*STATISTICS
********************************************************************************************************************

*************************employment and hours
sum stat  

sum stat if Imot02==1
sum stat if Imot02==1 & agekid<=17
sum stat if Imot317==1
sum stat if Inomot==1


sum hours if stat==1,d



**************************earnings and wages
sum part, d
sum obswage, d


****************correlation hours and female wage
pwcorr hours wage if stat==1


***************depreciation
sort i t


by i:gen beft=t+1 if stat==1 & stat[_n+1]==0 
by i:egen beft1=min(beft)

by i:gen aftt=t+1 if stat==0 & stat[_n+1]==1  & t>beft1
by i:egen aftt1=min(aftt)


gen aux1=wage if t==beft1-1
gen aux2=wage if t==aftt1


by i:egen befw1=min(aux1)
by i:egen aftw1=min(aux2)

drop aux1 aux2

gen duration=aftt1-beft1
gen depr=1-(aftw1/befw1)^(1/duration)

*only exits until age 32 (t=40), come back before 39 (t=69)
qui sum depr if beft1<=40 & aftt1<=69,d 
gen andepr=1-(1-r(p50))^4

sum andepr 


*****************************************************************************************************
*OUTPUT LIFE-CYCLE PROFILES
*****************************************************************************************************



gen edad=age+(t-(age-22)*4)/4


preserve


collapse (mean) obswage lwagesp wagesp lobswage lofwage stat asset cons part lpart obshours Imot02stat Imot317stat Inomotstat Imotyoungstat Imotoldstat (sd) slpart=lpart slobswage=lobswage, by(age)
*outfile age obswage ofwage stat asset cons part obshours using simprofiles.out,wide replace

gen wggap=obswage/(part/550)

sum wggap

gen vlpart=slpart*slpart
gen vlobswage=slobswage*slobswage

gen part2=part/10000
gen asset2=asset/1000

gen lcons=log(cons)


merge 1:1 age using dataprofiles50s.dta
*sp_age sp_hours sp_employed sp_lw h_logearn gendernetwageratio_byage gendergrosswageratio_byage varsp_lw varh_logearn

keep if age<=55
replace sp_hours=sp_hours*13



****************************************************************************************************
*average annual growth of wages
****************************************************************************************************

sort age
gen obswageg=lobswage-lobswage[_n-1]
gen sp_wg=sp_lw-sp_lw[_n-1]

sum obswageg sp_wg if age<40
sum obswageg sp_wg if age>=40


****************************************************************************************************
*average annual growth of variance of wages
****************************************************************************************************

gen gS=vlobswage-vlobswage[_n-1]
gen gD=varsp_lw-varsp_lw[_n-1]


sum gS gD

****************************************************************************************************
*graphs
****************************************************************************************************


line vlobswage age, lpattern(solid)  lcolor(black) || line varsp_lw age, lpattern(dash) lcolor(red) title("Variance of Log Observed Wages", size(small)) ytitle("") xtitle("") ylabel(0.05 0.15 0.25, labsize(3)) xlab(25 30 35 40 45 50 55 , labsize(3)) legend(off) graphregion( color(white )  )  saving(vlobswage, replace)

line stat age , lpattern(solid)  lcolor(black) || line sp_employed age, lpattern(dash) lcolor(red) title("Participation", size(small)) ytitle("") xtitle("") legend(off) ylab(0.4 0.6 0.8 1.0, labsize(3)) xlab(25 30 35 40 45 50 55 , labsize(3)) graphregion( color(white )  ) saving(stat, replace)

line obshours age  , lpattern(solid)  lcolor(black) || line sp_hours age, lpattern(dash) lcolor(red) title("Hours", size(small))ytitle("") xtitle("") legend(off) ylab(200 300 400 500 600, labsize(3))  xlab(25 30 35 40 45 50 55 , labsize(3)) graphregion( color(white )  ) saving(obshours, replace)

line lobswage age , lpattern(solid)  lcolor(black)  || line sp_lw age , lpattern(dash) lcolor(red)  title("Observed Net Wage", size(small)) ytitle("") xtitle("")  legend(off) xlab(25 30 35 40 45 50 55 , labsize(3)) graphregion( color(white )  ) saving(wage3, replace)
*ylab(12 16 20, labsize(3))




graph combine stat.gph obshours.gph vlobswage.gph wage3.gph
graph export simprofilesNRlabor.pdf,replace






restore


****************************************************************************************************
*distribution of total experience/potential experience at 55
****************************************************************************************************
sort i t
by i:egen totstat=sum(stat)
tab totstat

gen totexp55=totstat/124 if t==124

tab totexp55

log close

*end























