clear matrix
clear
set more off
capture log close
set mem 200m

*`j3': SCENARIO(0-NORMAL TIMES,1-RECESSION OR 2-BOOM)
*`j2': SIZE OF WAGE CHANGE (2 (1%),3 (5%), 4 (10%))
*`j': PERIOD IN WHICH THE FRISCH IS CALCULATED (17-age 27; 57-age 37; 97-age 47)


forval j3=0/0 {



*BASELINE***************************************************************************

infile i agekid hit t expval00 yr00 stat00 asset00 part00 partcash00 wage00 cons00 leis00 ialfa00 alfa00 euler00 mrs00 using sample000`j3'.out

gen hours00=1300-leis00 



gen agekid2=agekid+8

gen     Imot=1 if ((t-agekid<=12 & t-agekid>=0)  | (t-agekid2<=12 & t-agekid2>=0))
replace Imot=2 if (((t-agekid<=68 & t-agekid>12)  | (t-agekid2<=68 & t-agekid2>12)) & Imot==.)
replace Imot=3 if (Imot==. )


sort t
by t:egen p25 = pctile(partcash00), p(25)
by t:egen p50 = pctile(partcash00), p(50)
by t:egen p75 = pctile(partcash00), p(75)

gen percentil=1 if partcash00<p25
replace percentil=2 if partcash00>=p25 & partcash00<p50
replace percentil=3 if partcash00>=p50 & partcash00<p75
replace percentil=4 if partcash00>=p75

drop p25 p50 p75

egen p25 = pctile(wage)      if t==1, p(25)
egen p50 = pctile(wage) if t==1, p(50)
egen p75 = pctile(wage) if t==1, p(75)

gen aux=1 if wage<p25 &  t==1
replace aux=2 if wage>=p25 & wage<p50 &  t==1
replace aux=3 if wage>=p50 & wage<p75 &  t==1
replace aux=4 if wage>=p75 &  t==1


sort i
by i:egen inipercentil=min(aux)


keep i t stat00 hours00 wage00 Imot percentil inipercentil
save dB000`j3'.dta,replace

clear




forval j2=4/4{


forval j=13/136{



*COMPUTE ELASTICITIES***************************************************************************

*i,agekid,hit,t,stat,part,wage,leis,alfa
infile i agekid hit t stat part wage leis alfa using sample`j'`j2'`j3'.out


keep if t==`j'

gen tvar=`j'
gen wagevar=`j2'

gen delta=.
replace delta=0.01 if wagevar==2
replace delta=0.05 if wagevar==3
replace delta=0.10 if wagevar==4


gen ldelta=.
replace ldelta=log(1.01) if wagevar==2
replace ldelta=log(1.05) if wagevar==3
replace ldelta=log(1.10) if wagevar==4

gen hours=1300-leis

merge 1:1 i t  using dB000`j3'.dta

keep if t==`j'

log using f`j'`j2'`j3'.log,replace

gen lhours=log(hours)
gen lhours00=log(hours00)


sum hours00 hours,det
sum hours00 if stat00==1,det
sum hours if stat==1,det




egen awage=mean(wage)
egen awage00=mean(wage00)
gen ldeltaw=log(awage)-log(awage00)
sum ldeltaw

*EXTENSIVE MARGIN ELASTICITY
display "Extensive Margin Elasticity"
sum stat stat00
egen astat=mean(stat)
egen astat00=mean(stat00)
gen extelast=(astat-astat00)/delta




gen agekidind=2 if agekid<=17
replace agekidind=3 if agekid>17
replace agekidind=1 if agekid>100

forval q=1/3{

egen astatk`q'=mean(stat) if agekidind==`q'
egen astat00k`q'=mean(stat00)  if agekidind==`q'
gen extelastk`q'=(astatk`q'-astat00k`q')/delta  if agekidind==`q'

}


forval q=1/3{

egen astatImot`q'=mean(stat) if Imot==`q'
egen astat00Imot`q'=mean(stat00)  if Imot==`q'
gen extelastImot`q'=(astatImot`q'-astat00Imot`q')/delta  if Imot==`q'

}


forval q=1/4{

egen astatpercentil`q'=mean(stat) if percentil==`q'
egen astat00percentil`q'=mean(stat00)  if percentil==`q'
gen extelastpercentil`q'=(astatpercentil`q'-astat00percentil`q')/delta  if percentil==`q'

}


forval q=1/4{

egen astatinipercentil`q'=mean(stat) if inipercentil==`q'
egen astat00inipercentil`q'=mean(stat00)  if inipercentil==`q'
gen extelastinipercentil`q'=(astatinipercentil`q'-astat00inipercentil`q')/delta  if inipercentil==`q'

}


*AGGREGATE CHANGE IN HOURS (INCLUDING NON PARTICIPANTS)
display "Aggregate Change in Hours, including non participants"
egen ahours=mean(hours)
egen ahours00=mean(hours00)
gen macroelast=(ahours/ahours00-1)/delta 




*EFFICIENTY UNITS CHANGE
display "Aggregate Change in Gross Earnings"
gen earn00=hours00*wage00
gen earn=hours*wage00
egen aearn=mean(earn) 
egen aearn00=mean(earn00)
gen aggearn=(aearn/aearn00-1)/delta




forval q=1/3{

egen ahoursImot`q'=mean(hours) if Imot==`q'
egen ahours00Imot`q'=mean(hours00)  if Imot==`q'
gen macroelastImot`q'=(ahoursImot`q'/ahours00Imot`q'-1)/delta  if Imot==`q'

}


forval q=1/4{

egen ahourspercentil`q'=mean(hours) if percentil==`q'
egen ahours00percentil`q'=mean(hours00)  if percentil==`q'
gen macroelastpercentil`q'=(ahourspercentil`q'/ahours00percentil`q'-1)/delta  if percentil==`q'

}

forval q=1/4{

egen ahoursinipercentil`q'=mean(hours) if inipercentil==`q'
egen ahours00inipercentil`q'=mean(hours00)  if inipercentil==`q'
gen macroelastinipercentil`q'=(ahoursinipercentil`q'/ahours00inipercentil`q'-1)/delta  if inipercentil==`q'

}

*INTENSIVE MARGIN (INDIVIDUAL HOURS VARIATION CONDITIONING ON WORKING IN BOTH PERIODS)
display "Intensive Margin Elasticity"
gen diflhoursi=(hours/hours00-1)/delta  if stat==1 & stat00==1 


sum extelast
gen e=r(mean)

sum extelastk1
gen e1=r(mean)

sum extelastk2
gen e2=r(mean)

sum extelastk3
gen e3=r(mean)


sum extelastImot1
gen eImot1=r(mean)

sum extelastImot2
gen eImot2=r(mean)

sum extelastImot3
gen eImot3=r(mean)


sum extelastpercentil1
gen ep1=r(mean)
sum extelastpercentil2
gen ep2=r(mean)
sum extelastpercentil3
gen ep3=r(mean)
sum extelastpercentil4
gen ep4=r(mean)

sum extelastinipercentil1
gen einip1=r(mean)
sum extelastinipercentil2
gen einip2=r(mean)
sum extelastinipercentil3
gen einip3=r(mean)
sum extelastinipercentil4
gen einip4=r(mean)


sum macroelast
gen m=r(mean)


sum aggearn
gen eff=r(mean)

sum macroelastImot1
gen mImot1=r(mean)

sum macroelastImot2
gen mImot2=r(mean)

sum macroelastImot3
gen mImot3=r(mean)


sum diflhoursi if Imot==1,det
gen h50m1=r(p50)
sum diflhoursi if Imot==2,det
gen h50m2=r(p50)
sum diflhoursi if Imot==3,det
gen h50m3=r(p50)


sum macroelastpercentil1
gen mp1=r(mean)
sum macroelastpercentil2
gen mp2=r(mean)
sum macroelastpercentil3
gen mp3=r(mean)
sum macroelastpercentil4
gen mp4=r(mean)

sum macroelastinipercentil1
gen minip1=r(mean)
sum macroelastinipercentil2
gen minip2=r(mean)
sum macroelastinipercentil3
gen minip3=r(mean)
sum macroelastinipercentil4
gen minip4=r(mean)

sum diflhoursi,det
gen h25=r(p25)
gen h50=r(p50)
gen h75=r(p75)



sum diflhoursi if percentil==1,det
gen h50p1=r(p50)
sum diflhoursi if percentil==2,det
gen h50p2=r(p50)
sum diflhoursi if percentil==3,det
gen h50p3=r(p50)
sum diflhoursi if percentil==4,det
gen h50p4=r(p50)


sum diflhoursi if inipercentil==1,det
gen h50inip1=r(p50)
sum diflhoursi if inipercentil==2,det
gen h50inip2=r(p50)
sum diflhoursi if inipercentil==3,det
gen h50inip3=r(p50)
sum diflhoursi if inipercentil==4,det
gen h50inip4=r(p50)



keep if i==1

gen age=int((t-1)/4)+22

keep t age e e1 e2 e3 eImot1 eImot2 eImot3 ep1 ep2 ep3 ep4 einip1 einip2 einip3 einip4 h25 h50 h75 h50p1 h50p2 h50p3 h50p4 h50inip1 h50inip2 h50inip3 h50inip4 h50m1 h50m2 h50m3 m mp1 mp2 mp3 mp4 minip1 minip2 minip3 minip4 mImot1 mImot2 mImot3 eff
save  elastage`j'.dta,replace

clear

log close
}



*******************************************************************************************************************************


u elastage13.dta

forval j=14/136{

append using elastage`j'

}

gen var=`j2'
save elast`j2'`j3',replace
clear
}

}

*********************************************************************************************************************************



u elast40

gen edad=age+(t-(age-22)*4)/4

gen ageg=1 if edad>=25 & edad<30
replace ageg=2 if edad>=30 & edad<35
replace ageg=3 if edad>=35 & edad<40
replace ageg=4 if edad>=40 & edad<45
replace ageg=5 if edad>=45 & edad<50
replace ageg=6 if edad>=50 & edad<55

log using baseline-elast.log,replace


table ageg,c(mean e mean h25 mean h50 mean h75 mean m)

latabstat e h25 h50 h75 m eff, by(ageg) nototal f(%5.4f)

latabstat eImot1 eImot2 eImot3, by(ageg) nototal f(%5.4f)

latabstat mImot1 mImot2 mImot3, by(ageg) nototal f(%5.4f)



table ageg,c(mean ep1 mean ep2 mean ep3 mean ep4)

table ageg,c(mean h50p1 mean h50p2 mean h50p3 mean h50p4)

table ageg,c(mean mp1 mean mp2 mean mp3 mean mp4)


sum ep* h50p* mp*

sum eImot* h50m* mImot*


sum einip* h50inip* minip*



log close

*end














