clear matrix
clear
set more off
capture log close
set mem 200m


*BASELINE***************************************************************************
*i t expval yr stat asset part partcash wage cons leis alfa euler

infile i agekid hit t expval00 yr00 stat00 asset00 part00 partcash00 wage00 cons00 leis00 ialfa00 alfa00 euler00 mrs00  uti00 using sample0000.out


egen p25 = pctile(wage)      if t==1, p(25)
egen p50 = pctile(wage) if t==1, p(50)
egen p75 = pctile(wage) if t==1, p(75)

gen aux=1 if wage<p25 &  t==1
replace aux=2 if wage>=p25 & wage<p50 &  t==1
replace aux=3 if wage>=p50 & wage<p75 &  t==1
replace aux=4 if wage>=p75 &  t==1


sort i
by i:egen percentil=min(aux)

gen hours00=1300-leis00 

keep i t stat00 hours00 wage00 part00 cons00 asset00 leis00 uti00 percentil
save dB0000.dta,replace

clear


*COMPUTE ELASTICITIES***************************************************************************

infile i agekid hit t expval yr stat asset part partcash wage cons leis ialfa alfa euler mrs uti using sample0001.out



gen hours=1300-leis

merge 1:1 i t  using dB0000.dta

gen age=int((t-1)/4)+22



gen lhours=log(hours)
gen lhours00=log(hours00)
gen lwage=log(wage)
gen lwage00=log(wage00)

save mergesample.dta,replace




log using elast-h.log,replace


sum hours00 hours,det
sum hours00 if stat00==1,det
sum hours if stat==1,det


sum part



egen awage=mean(wage)
egen awage00=mean(wage00)
gen ldeltaw=log(awage)-log(awage00)
sum ldeltaw



*EXTENSIVE MARGIN ELASTICITY
display "Extensive Margin Elasticity"
egen astat=mean(stat)  if age>=25 & age<=55
egen astat00=mean(stat00)  if age>=25 & age<=55
*gen extelast=(astat-astat00)/ldeltaw
gen extelast=(astat-astat00)/0.1



*AGGREGATE CHANGE IN HOURS (INCLUDING NON PARTICIPANTS)
display "Aggregate Change in Hours, including non participants"
egen ahours=mean(hours)  if age>=25 & age<=55
egen ahours00=mean(hours00)  if age>=25 & age<=55
*gen macroelast=(log(ahours)-log(ahours00))/ldeltaw 
gen macroelast=(ahours/ahours00-1)/0.1 



*EFFICIENTY UNITS CHANGE
gen earn00=hours00*wage00
gen earn=hours*wage00
egen aearn=mean(earn) if age>=25 & age<=55
egen aearn00=mean(earn00) if age>=25 & age<=55
*gen aggearn=(log(aearn)-log(aearn00))/ldeltaw
gen aggearn=(aearn/aearn00-1)/0.1


*INTENSIVE MARGIN (INDIVIDUAL HOURS VARIATION CONDITIONING ON WORKING IN BOTH PERIODS)
display "Intensive Margin Elasticity"
*gen diflhoursi=(lhours-lhours00)/ldeltaw  if stat==1 & stat00==1 
gen diflhoursi=(hours/hours00-1)/0.1  if stat==1 & stat00==1


sum extelast* macroelast* aggearn
sum diflhoursi if age>=25 & age<=55,det



log close






*end














