!********************************************************************
! MODULE makutility
!********************************************************************
MODULE makutility
implicit none
private
public:: int_div, linearint, hpfilt, quick_sort, heapsort
public:: init_random_seed, init_random_seed2
public:: dist_sort, dist_stats, dist_stats_xp
public:: auto_stats, zbrent, zbrent2, zbrent_m, dist_stats_pro
public:: create_numlabel, create_numlabel_xp

contains

!********************************************************************
! subroutine: create_numlabel
!********************************************************************
SUBROUTINE create_numlabel(x1n,x2n,templabel)
	implicit none
	integer,intent(in):: x1n,x2n
	character(len=7),intent(out),dimension(x1n,x2n):: templabel
	character(len=2):: x1a,x2a
	integer:: x1,x2,digit1,digit2
	character(len=1), dimension(0:9):: tempnumber1,tempnumber2
	tempnumber1=(/'0','1','2','3','4','5','6','7','8','9'/)
	tempnumber2=(/' ','1','2','3','4','5','6','7','8','9'/)
	do x1=1, x1n
		call int_div(x1,10,digit2,digit1)
		x1a=tempnumber2(digit2)//tempnumber1(digit1)
		do x2=1, x2n
			call int_div(x2,10,digit2,digit1)
			x2a=tempnumber2(digit2)//tempnumber1(digit1)
			templabel(x1,x2)='('//x1a//','//x2a//')'
		end do
	end do
END SUBROUTINE create_numlabel

!********************************************************************
! subroutine: create_numlabel_xp
!********************************************************************
SUBROUTINE create_numlabel_xp(x1s,x1n,x2s,x2n,templabel)
	implicit none
	integer, intent(in)::x1n,x2n,x1s,x2s
	character(len=7),intent(out),dimension(x1s:x1n,x2s:x2n):: templabel
	character(len=2):: x1a, x2a
	integer:: x1, x2, digit1, digit2
	character(len=1), dimension(0:9):: tempnumber1,tempnumber2
	tempnumber1=(/'0','1','2','3','4','5','6','7','8','9'/)
	tempnumber2=(/' ','1','2','3','4','5','6','7','8','9'/)

	do x1=x1s, x1n
		call int_div(x1,10,digit2,digit1)
		x1a=tempnumber2(digit2)//tempnumber1(digit1)
		do x2=x2s, x2n
			call int_div(x2,10,digit2,digit1)
			x2a=tempnumber2(digit2)//tempnumber1(digit1)
			templabel(x1,x2)='('//x1a//','//x2a//')'
		end do
	end do

END SUBROUTINE create_numlabel_xp

!********************************************************************
! function: zbrent_m
!********************************************************************
real(8) FUNCTION zbrent_m (func,x1,x2,y1,y2,tol)
	implicit none
	real(8), external:: func
	real(8), intent(in):: x1,x2,y1,y2,tol
	integer, parameter:: itmax=100
	real(8), parameter:: epsz=3.e-8
	integer:: iter
	real(8):: a,b,c,d,e,fa,fb,fc,p,q,r,s,tol1,xm

	a=x1
	b=x2
	fa=y1
	fb=y2
	if(fa*fb>0.) pause 'root must be bracketed for zbrent'
	c=b
	fc=fb
	do iter=1,itmax
		if (fb*fc>0.) then
			c=a
			fc=fa
			d=b-a
			e=d
		end if
		if (abs(fc)<abs(fb)) then
			a=b
			b=c
			c=a
			fa=fb
			fb=fc
			fc=fa
		end if
		tol1=2.*epsz*abs(b)+0.5*tol
		xm=.5*(c-b)
		if (abs(xm)<=tol1 .or. fb==0.) then
			zbrent_m=b
			return
		end if
		if (abs(e)>=tol1 .and. abs(fa)>abs(fb)) then
			s=fb/fa
			if(a==c) then
				p=2.*xm*s
				q=1.-s
			else
				q=fa/fc
				r=fb/fc
				p=s*(2.*xm*q*(q-r)-(b-a)*(r-1.))
				q=(q-1.)*(r-1.)*(s-1.)
			end if
			if(p>0.) q=-q
			p=abs(p)
			if (2.*p<min(3.*xm*q-abs(tol1*q),abs(e*q))) then
				e=d
				d=p/q
			else
				d=xm
				e=d
			end if
		else
			d=xm
			e=d
		end if
		a=b
		fa=fb
		if(abs(d)>tol1) then
			b=b+d
		else
			b=b+sign(tol1,xm)
		end if
		fb=func(b)
	end do
	pause 'zbrent exceeding maximum iterations'
	zbrent_m=b
	return
END FUNCTION zbrent_m

!********************************************************************
! function: zbrent
! from Numerical Recipes in Fortran 77, pp354-355.
!********************************************************************
real(8) FUNCTION zbrent (func,x1,x2,tol)
	implicit none
	real(8), external:: func
	real(8), intent(in):: x1,x2,tol
	integer, parameter:: itmax=100
	real(8), parameter:: epsz=3.e-6
	integer:: iter
	real(8):: a,b,c,d,e,fa,fb,fc,p,q,r,s,tol1,xm

	a=x1
	b=x2
	fa=func(a)
	fb=func(b)
	if(fa*fb>0.) pause 'root must be bracketed for zbrent'
	c=b
	fc=fb
	do iter=1,itmax
		if (fb*fc>0.) then
			c=a
			fc=fa
			d=b-a
			e=d
		end if
		if (abs(fc)<abs(fb)) then
			a=b
			b=c
			c=a
			fa=fb
			fb=fc
			fc=fa
		end if
		tol1=2.*epsz*abs(b)+0.5*tol
		xm=.5*(c-b)
		if (abs(xm)<=tol1 .or. fb==0.) then
			zbrent=b
			return
		end if
		if (abs(e)>=tol1 .and. abs(fa)>abs(fb)) then
			s=fb/fa
			if(a==c) then
				p=2.*xm*s
				q=1.-s
			else
				q=fa/fc
				r=fb/fc
				p=s*(2.*xm*q*(q-r)-(b-a)*(r-1.))
				q=(q-1.)*(r-1.)*(s-1.)
			end if
			if(p>0.) q=-q
			p=abs(p)
			if (2.*p<min(3.*xm*q-abs(tol1*q),abs(e*q))) then
				e=d
				d=p/q
			else
				d=xm
				e=d
			end if
		else
			d=xm
			e=d
		end if
		a=b
		fa=fb
		if(abs(d)>tol1) then
			b=b+d
		else
			b=b+sign(tol1,xm)
		end if
		fb=func(b)
	end do
	pause 'zbrent exceeding maximum iterations'
	zbrent=b
	return
END FUNCTION zbrent

!********************************************************************
! function: zbrent2
! from Numerical Recipes in Fortran 77, pp354-355.
!igual que la anterior, pero con otro nombre
!********************************************************************
real(8) FUNCTION zbrent2 (func,x1,x2,tol)
	implicit none
	real(8), external:: func
	real(8), intent(in):: x1,x2,tol
	integer, parameter:: itmax=100
	real(8), parameter:: epsz=3.e-8
	integer:: iter
	real(8):: a,b,c,d,e,fa,fb,fc,p,q,r,s,tol1,xm

	a=x1
	b=x2
	fa=func(a)
	fb=func(b)
	if(fa*fb>0.) pause 'root must be bracketed for zbrent2'
	if(fa*fb>0.)then
		write(*,*)a,b,fa,fb
	endif
	c=b
	fc=fb
	do iter=1,itmax
		if (fb*fc>0.) then
			c=a
			fc=fa
			d=b-a
			e=d
		end if
		if (abs(fc)<abs(fb)) then
			a=b
			b=c
			c=a
			fa=fb
			fb=fc
			fc=fa
		end if
		tol1=2.*epsz*abs(b)+0.5*tol
		xm=.5*(c-b)
		if (abs(xm)<=tol1 .or. fb==0.) then
			zbrent2=b
			return
		end if
		if (abs(e)>=tol1 .and. abs(fa)>abs(fb)) then
			s=fb/fa
			if(a==c) then
				p=2.*xm*s
				q=1.-s
			else
				q=fa/fc
				r=fb/fc
				p=s*(2.*xm*q*(q-r)-(b-a)*(r-1.))
				q=(q-1.)*(r-1.)*(s-1.)
			end if
			if(p>0.) q=-q
			p=abs(p)
			if (2.*p<min(3.*xm*q-abs(tol1*q),abs(e*q))) then
				e=d
				d=p/q
			else
				d=xm
				e=d
			end if
		else
			d=xm
			e=d
		end if
		a=b
		fa=fb
		if(abs(d)>tol1) then
			b=b+d
		else
			b=b+sign(tol1,xm)
		end if
		fb=func(b)
	end do
	pause 'zbrent2 exceeding maximum iterations'
	zbrent2=b
	return
END FUNCTION zbrent2


!********************************************************************
! subroutine : heapsort
!********************************************************************
SUBROUTINE heapsort(n,ra,order)
	implicit none
	integer, parameter:: prec=8
	integer, intent(in):: n
	real(prec), intent(inout), dimension(n):: ra
	integer,    intent(out), dimension(n):: order
	integer:: i, ir, j, l, rorder
	real(prec):: rra
	if (n<2) return
	do l=1,n
		order(l)=l
	end do
	l=n/2+1
	ir=n
10 continue
	if (l>1) then
		l=l-1
		rra=ra(l); rorder=order(l)
	else
		rra=ra(ir); rorder=order(ir)
		ra(ir)=ra(1); order(ir)=order(1)
		ir=ir-1
		if (ir==1) then
			ra(1)=rra; order(1)=rorder
			return
		end if
	end if
	i=l
	j=l+l
20 continue
	if (j<=ir) then
		if (j<ir) then
			if (ra(j)<ra(j+1)) j=j+1
		end if
		if (rra<ra(j)) then
			ra(i)=ra(j); order(i)=order(j)
			i=j
			j=j+j
		else
			j=ir+1
		end if
		goto 20
	end if
	ra(i)=rra; order(i)=rorder
	goto 10

END SUBROUTINE heapsort

!********************************************************************
! subroutine: init_random_seed
!********************************************************************
SUBROUTINE init_random_seed(seed0)
	implicit none
	integer, intent(in):: seed0
	integer:: nseed
	integer, dimension(:), allocatable:: seed_old, seed_new

	call random_seed ()
	call random_seed (SIZE=nseed)
	allocate (seed_old(nseed), seed_new(nseed))
	call random_seed (GET=seed_old(1:nseed))
	seed_new=seed_old*seed0
	call random_seed (PUT=seed_new(1:nseed))
	deallocate (seed_old, seed_new)

END SUBROUTINE init_random_seed

!********************************************************************
! subroutine: init_random_seed2
!********************************************************************
SUBROUTINE init_random_seed2(seed0)
	implicit none
	integer, intent(in):: seed0
	integer:: nseed
	integer, dimension(:), allocatable:: seed_new

	call random_seed ()
	call random_seed (SIZE=nseed)
	allocate (seed_new(nseed))
	seed_new(:)=seed0
	call random_seed (PUT=seed_new(1:nseed))
	deallocate (seed_new)

END SUBROUTINE init_random_seed2

!********************************************************************
! subroutine: auto_stats
!********************************************************************
SUBROUTINE auto_stats(ndata,datain0,datain1,statout)
	implicit none

	!******* parameters *******
	integer, parameter:: prec=8
	real(prec), parameter:: one=1.0_prec, zero=0.0_prec

	!******* input/output variables *******
	integer,    intent(in):: ndata
	real(prec), intent(in),  dimension(ndata):: datain0,datain1
	real(prec), intent(out), dimension(5) :: statout

	!******* local variables *******
	integer:: data0

	!******* calculate means *******
	statout(:)=zero
	statout(1)=sum(datain0(1:ndata))/real(ndata,prec)
	statout(2)=sum(datain1(1:ndata))/real(ndata,prec)

	!******* calculate variances and autocorrlation *******
	do data0=1,ndata
		statout(3)=statout(3)+(datain0(data0)-statout(1))**2.0_prec
		statout(4)=statout(4)+(datain1(data0)-statout(2))**2.0_prec
		statout(5)=statout(5)+(datain0(data0)-statout(1))*&
		(datain1(data0)-statout(2))
	end do
	statout(5)=statout(5)/SQRT(statout(3))/SQRT(statout(4))
	statout(3)=SQRT(statout(3)/real(ndata,prec))
	statout(4)=SQRT(statout(4)/real(ndata,prec))

END SUBROUTINE auto_stats

!********************************************************************
! subroutine: dist_sort
!********************************************************************
SUBROUTINE dist_sort(nid, idnum, idval, idmea)
	implicit none

	!******* variables *******
	integer, parameter:: prec=8
	integer,    intent(in):: nid
	integer,    intent(out), dimension(nid):: idnum
	real(prec), intent(inout), dimension(nid):: idval
	real(prec), intent(inout), dimension(nid):: idmea
	integer:: id0
	real(prec), dimension(nid):: rtemp

	!******* sort idval *******
	!call quick_sort(idval(1:nid), idnum(1:nid))
	call heapsort(nid, idval(1:nid), idnum(1:nid))

	!******* sort other data *******
	do id0=1, nid
		rtemp(id0)=idmea(idnum(id0))
	end do
	idmea(1:nid)=rtemp(1:nid)

END SUBROUTINE dist_sort

!********************************************************************
! subroutine: dist_stats
!********************************************************************
SUBROUTINE dist_stats(nid, idval, idmea, tempstats)
	implicit none

	!******* parameters *******
	integer, parameter:: prec=8
	real(prec), parameter:: one=1.0_prec, zero=0.0_prec
	real(prec):: eps=1.0E-30_prec

	!******* input/output variables *******
	integer,    intent(in):: nid
	real(prec), intent(in),  dimension(nid):: idval
	real(prec), intent(in),  dimension(nid):: idmea
	real(prec), intent(out), dimension(13)  :: tempstats

	!******* local variables *******
	integer:: pc0, id0
	real(prec):: totsum,gini,leftsum,rightsum,left45,right45,triarea
	real(prec):: idmean, idmedian, idcoefvar, idmeasum
	real(prec), dimension(100):: pcprop, pcmea
	real(prec), dimension(nid):: idtemp

	!******* initializtion *******
	idmeasum=sum(idmea(1:nid))
	if (idmeasum<eps) then
		tempstats(1:13)=zero
		return
	end if
	idtemp(1:nid)=idmea(1:nid)/idmeasum
	pcmea(1:100)=0.010_prec
	pcprop(1:100)=0.0_prec

	!******* calculate mean, median, ratio *******
	idmean=zero
	do id0=1, nid
		idmean=idmean+idval(id0)*idmea(id0)/idmeasum
	end do
	do id0=1, nid
		if (sum(idmea(1:id0))/idmeasum>=0.50_prec) exit
		if (id0==nid) exit
	end do
	idmedian=idval(id0)
	tempstats(1)=idmean
	tempstats(2)=idmedian
	if (idmedian>1.0e-10_prec) then
		tempstats(3)=idmean/idmedian
	else
		tempstats(3)=0.0_prec
	end if

	!******* calculate coefficient of variation *******
	idcoefvar=zero
	do id0=1, nid
		idcoefvar=idcoefvar+(idval(id0)-idmean)**2.0_prec*&
		idmea(id0)/idmeasum
	end do
	idcoefvar=idcoefvar**0.50_prec
	if (idmean>1.0e-10_prec) then
		idcoefvar=idcoefvar/idmean
	else
		idcoefvar=0.0_prec
	end if
	tempstats(4)=idcoefvar

	!******* loop *******
	id0=1
	do pc0=1, 100
		do
			if (id0>nid) exit
			if (idtemp(id0)>=pcmea(pc0)) then
				pcprop(pc0)=pcprop(pc0)+pcmea(pc0)*idval(id0)
				idtemp(id0)=idtemp(id0)-pcmea(pc0)
				pcmea(pc0)=zero
				exit
			else
				pcprop(pc0)=pcprop(pc0)+idtemp(id0)*idval(id0)
				pcmea(pc0)=pcmea(pc0)-idtemp(id0)
				idtemp(id0)=0.0_prec
				id0=id0+1
			end if
		end do
	end do

	!******* calculate statistics *******
	totsum=sum(pcprop(1:100))
	tempstats(5)=sum(pcprop( 1:20))/totsum
	tempstats(6)=sum(pcprop(21:40))/totsum
	tempstats(7)=sum(pcprop(41:60))/totsum
	tempstats(8)=sum(pcprop(61:80))/totsum
	tempstats(9)=sum(pcprop(81:100))/totsum
	tempstats(10)=sum(pcprop(81:95))/totsum
	tempstats(11)=sum(pcprop(96:100))/totsum
	tempstats(12)=sum(pcprop(100:100))/totsum

	!******* calculate GINI *******
	gini=0.0_prec; triarea=0.0_prec
	do pc0=1, 100
		if (pc0==1) then
			leftsum=0.0_prec
		else
			leftsum=sum(pcprop(1:pc0-1))/totsum
		end if
		rightsum=sum(pcprop(1:pc0))/totsum
		left45=real(pc0-1, prec)/100.0_prec
		right45=real(pc0, prec)/100.0_prec
		gini=gini+((left45+right45)-(leftsum+rightsum))&
		/100.0_prec/2.0_prec
		triarea=triarea+(left45+right45)/100.0_prec/2.0_prec
	end do
	tempstats(13)=gini/triarea

END SUBROUTINE dist_stats

!********************************************************************
! subroutine: dist_stats_xp
!********************************************************************
SUBROUTINE dist_stats_xp(nid, idval, idmea, tempstats)
	implicit none

	!******* parameters *******
	integer, parameter:: prec=8
	real(prec), parameter:: one=1.0_prec, zero=0.0_prec
	real(prec):: eps=1.0E-30_prec

	!******* input/output variables *******
	integer,    intent(in):: nid
	real(prec), intent(in),  dimension(nid):: idval
	real(prec), intent(in),  dimension(nid):: idmea
	real(prec), intent(out), dimension(15)  :: tempstats

	!******* local variables *******
	integer:: pc0, id0
	real(prec):: totsum,gini,leftsum,rightsum,left45,right45,triarea
	real(prec):: idmean,idmedian,idcoefvar,idmeasum
	real(prec), dimension(100):: pcprop, pcmea
	real(prec), dimension(nid):: idtemp

	!******* initializtion *******
	idmeasum=sum(idmea(1:nid))
	if (idmeasum<eps) then
		tempstats(1:15)=zero
		return
	end if
	idtemp(1:nid)=idmea(1:nid)/idmeasum
	pcmea(1:100)=0.010_prec
	pcprop(1:100)=0.0_prec

	!******* calculate mean, median, ratio *******
	idmean=zero
	do id0=1, nid
		idmean=idmean+idval(id0)*idmea(id0)/idmeasum
	end do
	do id0=1, nid
		if (sum(idmea(1:id0))/idmeasum>=0.50_prec) exit
		if (id0==nid) exit
	end do
	idmedian=idval(id0)
	tempstats(1)=idmean
	tempstats(2)=idmedian
	if (idmedian>1.0e-10_prec) then
		tempstats(3)=idmean/idmedian
	else
		tempstats(3)=0.0_prec
	end if

	!******* calculate coefficient of variation *******
	idcoefvar=zero
	do id0=1, nid
		idcoefvar=idcoefvar+(idval(id0)-idmean)**2.0_prec&
		*idmea(id0)/idmeasum
	end do
	tempstats(4)=idcoefvar
	idcoefvar=idcoefvar**0.50_prec
	tempstats(5)=idcoefvar
	if (idmean>1.0e-10_prec) then
		idcoefvar=idcoefvar/idmean
	else
		idcoefvar=0.0_prec
	end if
	tempstats(6)=idcoefvar

	!******* loop *******
	id0=1
	do pc0=1, 100
		do
			if (id0>nid) exit
			if (idtemp(id0)>=pcmea(pc0)) then
				pcprop(pc0)=pcprop(pc0)+pcmea(pc0)*idval(id0)
				idtemp(id0)=idtemp(id0)-pcmea(pc0)
				pcmea(pc0)=zero
				exit
			else
				pcprop(pc0)=pcprop(pc0)+idtemp(id0)*idval(id0)
				pcmea(pc0)=pcmea(pc0)-idtemp(id0)
				idtemp(id0)=0.0_prec
				id0=id0+1
			end if
		end do
	end do

	!******* calculate statistics *******
	totsum=sum(pcprop(1:100))
	tempstats(7)=sum(pcprop( 1:20))/totsum
	tempstats(8)=sum(pcprop(21:40))/totsum
	tempstats(9)=sum(pcprop(41:60))/totsum
	tempstats(10)=sum(pcprop(61:80))/totsum
	tempstats(11)=sum(pcprop(81:100))/totsum
	tempstats(12)=sum(pcprop(81:95))/totsum
	tempstats(13)=sum(pcprop(96:100))/totsum
	tempstats(14)=sum(pcprop(100:100))/totsum

	!******* calculate GINI *******
	gini=0.0_prec; triarea=0.0_prec
	do pc0=1, 100
		if (pc0==1) then
			leftsum=0.0_prec
		else
			leftsum=sum(pcprop(1:pc0-1))/totsum
		end if
		rightsum=sum(pcprop(1:pc0))/totsum
		left45=real(pc0-1, prec)/100.0_prec
		right45=real(pc0, prec)/100.0_prec
		gini=gini+((left45+right45)-(leftsum+rightsum))&
		/100.0_prec/2.0_prec
		triarea=triarea+(left45+right45)/100.0_prec/2.0_prec
	end do
	tempstats(15)=gini/triarea

END SUBROUTINE dist_stats_xp

!********************************************************************
! subroutine: dist_stats_pro
!********************************************************************
SUBROUTINE dist_stats_pro(nid, idval, idmea, tempstats)
	implicit none

	!******* parameters *******
	integer, parameter:: prec=8
	real(prec), parameter:: one=1.0_prec, zero=0.0_prec
	real(prec):: eps=1.0E-30_prec

	!******* input/output variables *******
	integer,    intent(in):: nid
	real(prec), intent(in),  dimension(nid):: idval
	real(prec), intent(in),  dimension(nid):: idmea
	real(prec), intent(out), dimension(18)  :: tempstats

	!******* local variables *******
	integer:: pc0, id0
	real(prec):: totsum,gini,leftsum,rightsum,left45,right45,triarea
	real(prec):: idmean,idmedian,idcoefvar,idmeasum
	real(prec), dimension(100):: pcprop, pcmea
	real(prec), dimension(nid):: idtemp

	!******* initializtion *******
	idmeasum=sum(idmea(1:nid))
	if (idmeasum<eps) then
		tempstats(1:15)=zero
		return
	end if
	idtemp(1:nid)=idmea(1:nid)/idmeasum
	pcmea(1:100)=0.010_prec
	pcprop(1:100)=0.0_prec

	!******* calculate mean, median, ratio *******
	idmean=zero
	do id0=1, nid
		idmean=idmean+idval(id0)*idmea(id0)/idmeasum
	end do
	do id0=1, nid
		if (sum(idmea(1:id0))/idmeasum>=0.50_prec) exit
		if (id0==nid) exit
	end do
	idmedian=idval(id0)
	tempstats(1)=idmean
	tempstats(2)=idmedian
	if (idmedian>1.0e-10_prec) then
		tempstats(3)=idmean/idmedian
	else
		tempstats(3)=0.0_prec
	end if

	!******* calculate coefficient of variation *******
	idcoefvar=zero
	do id0=1, nid
		idcoefvar=idcoefvar+(idval(id0)-idmean)**2.0_prec&
		*idmea(id0)/idmeasum
	end do
	tempstats(4)=idcoefvar
	idcoefvar=idcoefvar**0.50_prec
	tempstats(5)=idcoefvar
	if (idmean>1.0e-10_prec) then
		idcoefvar=idcoefvar/idmean
	else
		idcoefvar=0.0_prec
	end if
	tempstats(6)=idcoefvar

	!******* loop *******
	id0=1
	do pc0=1, 100
		do
			if (id0>nid) exit
			if (idtemp(id0)>=pcmea(pc0)) then
				pcprop(pc0)=pcprop(pc0)+pcmea(pc0)*idval(id0)
				idtemp(id0)=idtemp(id0)-pcmea(pc0)
				pcmea(pc0)=zero
				exit
			else
				pcprop(pc0)=pcprop(pc0)+idtemp(id0)*idval(id0)
				pcmea(pc0)=pcmea(pc0)-idtemp(id0)
				idtemp(id0)=0.0_prec
				id0=id0+1
			end if
		end do
	end do

	!******* calculate statistics *******
	totsum=sum(pcprop(1:100))
	tempstats(7)=sum(pcprop( 1:20))/totsum
	tempstats(8)=sum(pcprop(21:40))/totsum
	tempstats(9)=sum(pcprop(41:60))/totsum
	tempstats(10)=sum(pcprop(61:80))/totsum
	tempstats(11)=sum(pcprop(81:100))/totsum
	tempstats(12)=sum(pcprop(81:95))/totsum
	tempstats(13)=sum(pcprop(96:100))/totsum
	tempstats(14)=sum(pcprop(100:100))/totsum

	!******* calculate GINI *******
	gini=0.0_prec; triarea=0.0_prec
	do pc0=1, 100
		if (pc0==1) then
			leftsum=0.0_prec
		else
			leftsum=sum(pcprop(1:pc0-1))/totsum
		end if
		rightsum=sum(pcprop(1:pc0))/totsum
		left45=real(pc0-1, prec)/100.0_prec
		right45=real(pc0, prec)/100.0_prec
		gini=gini+((left45+right45)-(leftsum+rightsum))&
		/100.0_prec/2.0_prec
		triarea=triarea+(left45+right45)/100.0_prec/2.0_prec
	end do
	tempstats(15)=gini/triarea

	!******* calculate proportion with pos, zero, and neg *******
	tempstats(16:18)=zero
	do id0=1, nid
		if (idval(id0)>zero) then
			tempstats(16)=tempstats(16)+idmea(id0)
		else if (idval(id0)==zero) then
			tempstats(17)=tempstats(17)+idmea(id0)
		else
			tempstats(18)=tempstats(18)+idmea(id0)
		end if
	end do
	tempstats(16)=tempstats(16)/max(sum(idmea(1:nid)),eps)
	tempstats(17)=tempstats(17)/max(sum(idmea(1:nid)),eps)
	tempstats(18)=tempstats(18)/max(sum(idmea(1:nid)),eps)

END SUBROUTINE dist_stats_pro

!********************************************************************
! function: int_div
! Division between integers
!********************************************************************
SUBROUTINE int_div(numera,denomi,quotient,remain)
	implicit none
	integer, intent(in):: numera, denomi
	integer, intent(out):: quotient, remain
	integer:: div_flag
	remain=numera; quotient=0
	do
		if (remain<denomi) then
			exit
		else
			remain=remain-denomi
			quotient=quotient+1
		end if
	end do
END SUBROUTINE int_div

!********************************************************************
! function: linearint
!********************************************************************
FUNCTION linearint(xx,yy,xval) result (yval)
	implicit none

	!******* parameters *******
	integer, parameter:: prec=8

	!******* input/output variables *******
	real(prec):: yval
	real(prec), intent(in):: xval
	real(prec), intent(in), dimension(:):: xx, yy

	!******* local variables *******
	integer:: nx, x0

	nx=size(xx)
	if (nx==1) then
		yval=yy(1)
		return
	end if
	do x0=1, nx-1
		if (xx(x0+1)>xval) then
			yval=yy(x0)+(yy(x0+1)-yy(x0))*(xval-xx(x0))/(xx(x0+1)-xx(x0))
			return
		else if (x0==nx-1) then
			yval=yy(nx)
			return
		end if
	end do

END FUNCTION linearint

!********************************************************************
! subroutine: HPFILT
! Kalman smoothing routine for HP filter written by E Prescott.
! Taken from Victor's code vra1e.f.
! y=data series(input), d=deviations from trend(output)
! t=trend(output), n=no obs(input), s=smoothing parameter
! (convention is 1600 for quarterly data and 400 for annual data).
! If iopt=1 and n and s are both the same as for the last call,
! this routine uses the values of v from the last call.
! If this option is used, v must not be used for other purposes
! between last and current call.
!********************************************************************
SUBROUTINE HPFILT(y,t,d,v,n,s,iopt)
	implicit none
	integer, intent(in):: n, iopt
	real(8), intent(in):: s
	real(8), intent(in), dimension(n):: y(n)
	real(8), intent(inout), dimension(n,3):: v(n,3)
	real(8), intent(out), dimension(n):: t(n),d(n)
	real(8):: m1,m2,v11,v12,v22,x,z,b11,b12,b22,det,e1,e2,ss
	integer:: nn,i,i1,ib
	ss=0.0; nn=0

	!******* compute seq of cov matrix for f[x(t),x(t-1)|y(<t)] *******
	if(nn/=n .or. s/=ss .or. iopt/=1) then
		ss=s
		nn=n
		v11=1.d0
		v22=1.d0
		v12=0.d0
		do i=3, n
			x=v11
			z=v12
			v11=1.d0/s+4.d0*(x-z)+v22
			v12=2.d0*x-z
			v22=x
			det=v11*v22-v12*v12
			v(i,1)=v22/det
			v(i,3)=v11/det
			v(i,2)=-v12/det
			x=v11+1.d0
			z=v11
			v11=v11-v11*v11/x
			v22=v22-v12*v12/x
			v12=v12-z*v12/x
		end do
	endif

	!******* this is the forward pass *******
	m1=y(2)
	m2=y(1)
	do i=3, n
		x=m1
		m1=2.d0*m1-m2
		m2=x
		t(i-1)=v(i,1)*m1+v(i,2)*m2
		d(i-1)= v(i,2)*m1+v(i,3)*m2
		det=v(i,1)*v(i,3)-v(i,2)*v(i,2)
		v11=v(i,3)/det
		v12=-v(i,2)/det
		z=(y(i)-m1)/(v11+1.d0)
		m1=m1+v11*z
		m2=m2+v12*z
	end do
	t(n)=m1
	t(n-1)=m2

	!******* this is the backward pass *******
	m1=y(n-1)
	m2=y(n)
	do i=n-2,1,-1
		i1=i+1
		ib=n-i+1
		x=m1
		m1=2.d0*m1 - m2
		m2=x

		!******* combine info for y(<i) with info for y(>i) *******
		if(i>2) then
			e1=v(ib,3)*m2+v(ib,2)*m1+t(i)
			e2=v(ib,2)*m2+v(ib,1)*m1+d(i)
			b11=v(ib,3)+v(i1,1)
			b12=v(ib,2)+v(i1,2)
			b22=v(ib,1)+v(i1,3)
			det=b11*b22-b12*b12
			t(i)=(-b12*e1+b11*e2)/det
		endif

		!******* end of combining *******
		det=v(ib,1)*v(ib,3)-v(ib,2)*v(ib,2)
		v11=v(ib,3)/det
		v12=-v(ib,2)/det
		z=(y(i)-m1)/(v11+1.d0)
		m1=m1+v11*z
		m2=m2+v12*z
	end do
	t(1)=m1
	t(2)=m2
	do i=1, n
		d(i)=y(i)-t(i)
	end do
END SUBROUTINE HPFILT

!********************************************************************
! recursive function: quick_sort
! Quick sort routine from:
! Brainerd, W.S., Goldberg, C.H. & Adams, J.C. (1990)
! "Programmer's Guide to Fortran 90", McGraw-Hill ISBN 0-07-000248-7,
! pages 149-150.
! Modified by Alan Miller to include an associated integer array
! which gives the positions of the elements in the original order.
!********************************************************************
RECURSIVE SUBROUTINE quick_sort(list, order)
IMPLICIT NONE
REAL(8), DIMENSION (:), INTENT(INOUT)  :: list
INTEGER, DIMENSION (:), INTENT(OUT) :: order

! Local variable
INTEGER :: i

DO i = 1, SIZE(list)
  order(i) = i
END DO

CALL quick_sort_1(1, SIZE(list))

CONTAINS

RECURSIVE SUBROUTINE quick_sort_1(left_end, right_end)

INTEGER, INTENT(IN) :: left_end, right_end

!     Local variables
INTEGER             :: i, j, itemp
REAL(8)                :: reference, temp
INTEGER, PARAMETER  :: max_simple_sort_size = 6

IF (right_end < left_end + max_simple_sort_size) THEN
  ! Use interchange sort for small lists
  CALL interchange_sort(left_end, right_end)

ELSE
  ! Use partition ("quick") sort
  reference = list((left_end + right_end)/2)
  i = left_end - 1; j = right_end + 1

  DO
    ! Scan list from left end until element >= reference is found
    DO
      i = i + 1
      IF (list(i) >= reference) EXIT
    END DO
    ! Scan list from right end until element <= reference is found
    DO
      j = j - 1
      IF (list(j) <= reference) EXIT
    END DO


    IF (i < j) THEN
      ! Swap two out-of-order elements
      temp = list(i); list(i) = list(j); list(j) = temp
      itemp = order(i); order(i) = order(j); order(j) = itemp
    ELSE IF (i == j) THEN
      i = i + 1
      EXIT
    ELSE
      EXIT
    END IF
  END DO

  IF (left_end < j) CALL quick_sort_1(left_end, j)
  IF (i < right_end) CALL quick_sort_1(i, right_end)
END IF

END SUBROUTINE quick_sort_1


SUBROUTINE interchange_sort(left_end, right_end)

INTEGER, INTENT(IN) :: left_end, right_end

!     Local variables
INTEGER             :: i, j, itemp
REAL(8)                :: temp

DO i = left_end, right_end - 1
  DO j = i+1, right_end
    IF (list(i) > list(j)) THEN
      temp = list(i); list(i) = list(j); list(j) = temp
      itemp = order(i); order(i) = order(j); order(j) = itemp
    END IF
  END DO
END DO

END SUBROUTINE interchange_sort

END SUBROUTINE quick_sort

END MODULE makutility

!********************************************************************
! MODULE: summaryindex
! Function to summarize multidimentional indeces into one integer
! and its inverse function.
! F90 code.
!********************************************************************
MODULE summaryindex

implicit none

!******* control access to subroutines in this module *******
private
public:: sumindex, inv_sumindex

contains

!********************************************************************
! FUNCTION: sumindex
! Instructions:
! (1) size of nmindex = size of mindex ( = ndim in the code)
!********************************************************************
FUNCTION sumindex(nmindex,mindex)
	implicit none

	!******* input variables *******
	integer, intent(in), dimension(:):: nmindex, mindex

	!******* output variables *******
	integer:: sumindex

	!******* local variables *******
	integer:: ndim, dim0

	!******* check the dimension of index *******
	ndim=size(nmindex)

	!******* one dimension *******
	if (ndim==1) then
		sumindex=mindex(1)
		return
	end if

	!******* compute the summarized index *******
	sumindex=0
	do dim0=1, ndim-1
		sumindex=sumindex+(mindex(dim0)-1)*product(nmindex(dim0+1:ndim))
	end do
	sumindex=sumindex+mindex(ndim)

	!******* end of function *******
	return

END FUNCTION sumindex

!********************************************************************
! FUNCTION: inv_sumindex
! Instructions:
! (1) size of nmindex >= 2 (=ndim in the code)
! (2) sindex >= 1 and <= product(nmindex(1:ndim))
!********************************************************************
FUNCTION inv_sumindex(nmindex,sindex)
	implicit none

	!******* input variables *******
	integer, intent(in), dimension(:):: nmindex
	integer, intent(in):: sindex

	!******* output variable *******
	integer, dimension(size(nmindex)):: inv_sumindex

	!******* local variables *******
	integer:: ndim, dim0, indextemp

	!******* check the dimension of index *******
	ndim=size(nmindex)

	!******* one dimension *******
	if (ndim==1) then
		inv_sumindex(1)=sindex
		return
	end if

	!******* set indextemp *******
	indextemp=sindex

	!******* last dimension is a bit different *******
	inv_sumindex(ndim)=mod(indextemp,nmindex(ndim))
	if (inv_sumindex(ndim)==0) inv_sumindex(ndim)=nmindex(ndim)
	indextemp=(indextemp-inv_sumindex(ndim))/nmindex(ndim)

	!******* between the 2nd and the last dimension *******
	if (ndim>2) then
		do dim0=ndim-1, 2, -1
			inv_sumindex(dim0)=mod(indextemp,nmindex(dim0))+1
			indextemp=(indextemp-(inv_sumindex(dim0)-1))/nmindex(dim0)
		end do
	end if

	!******* 1st dimension is a bit different, too *******
	inv_sumindex(1)=indextemp+1

	!******* end of function *******
	return

END FUNCTION inv_sumindex

END MODULE summaryindex

!********************************************************************
! MODULE DSPSP (version 2.0, October 25, 2003)
! Quadratic shape preserving spline proposed by L. Schumaker (1983).
! The algorithm is from "Numerical Method in Economics" by K.
! Judd (1998) pp231-235. If the data implies monotonicity or
! concavity, these shape is passed on to the interpolated function.
! F90 code.
! Double precision.
! DSPSPLINE outputs the necessary data for interpolation, using
! grids and value of the function at the grdis points as inputs.
! The output is the additional breaking points, one for each
! interval.
! DSPSPLINT evaluates the interpolated function at a given point,
! using the output of DSPSPLINE.
! DSPSPLEASY does both of them in one subroutine.
!********************************************************************
MODULE DSPSP
implicit none
private
public:: dspspline, dspsplint, dspspleasy, dspsplintd, dspspleasyd
public:: dspspline2, dspspleasy2, dspspleasyd2
public:: dspspline3, dspspleasy3, dspspleasyd3

CONTAINS

!********************************************************************
! SUBROUTINE DSPSPLINE
!********************************************************************
SUBROUTINE DSPSPLINE(sp_np, sp_x, sp_y, sp_work, sp_error)
	implicit none

	!******* precision controller *******
	integer, parameter:: sp_prec=8
	!double precision

	!******* input variables *******
	integer, intent(in):: sp_np
	!number of grid points (>2)
	real(sp_prec), intent(in), dimension(1:sp_np):: sp_x, sp_y
	!sp_x(1:sp_np) is the array of grid points
	!sp_y(1:sp_np) is the array of the value of functions

	!******* output variables *******
	real(sp_prec), intent(out), dimension(2*sp_np-1):: sp_work
	integer, intent(out):: sp_error
	!error code

	!******* local variables *******
	real(sp_prec), dimension(1:sp_np-1):: sp_zeta
	!array of additional breaking points
	real(sp_prec), dimension(1:sp_np):: sp_d1
	!sp_d1(1:sp_np) is the array to store 1st derivatives
	real(sp_prec), dimension(1:sp_np-1):: sp_delta
	!internally used

	!******* check inputs *******
	call sp_check_inputs1(sp_np, sp_x, sp_y, sp_error)
	if (sp_error>0) return

	!******* compute d1 and delta *******
	call sp_compt_d1(sp_np, sp_x, sp_y, sp_d1, sp_delta)

	!******* compute additional break points *******
	call sp_compt_zeta(sp_np, sp_x, sp_y, sp_d1, sp_delta, sp_zeta)

	!******* construct output array *******
	sp_work(1:sp_np-1)=sp_zeta(1:sp_np-1)
	sp_work(sp_np:2*sp_np-1)=sp_d1(1:sp_np)

	!******* end  of the subroutine *******
	return

END SUBROUTINE DSPSPLINE

!********************************************************************
! SUBROUTINE DSPSPLINE2
!********************************************************************
SUBROUTINE DSPSPLINE2(sp_np, sp_x, sp_y, flag_1, sp_d1_1, &
flag_n, sp_d1_n, sp_work, sp_error)
	implicit none

	!******* precision controller *******
	integer, parameter:: sp_prec=8
	!double precision

	!******* input variables *******
	integer, intent(in):: sp_np, flag_1, flag_n
	!number of grid points (>2)
	real(sp_prec), intent(in), dimension(1:sp_np):: sp_x, sp_y
	!sp_x(1:sp_np) is the array of grid points
	!sp_y(1:sp_np) is the array of the value of functions

	!******* input variables (special for DSPSPLINE2) *******
	real(sp_prec), intent(in):: sp_d1_1, sp_d1_n
	! sp_d1 is the derivative at point 1
	! sp_dn is the derivative at point sp_np

	!******* output variables *******
	real(sp_prec), intent(out), dimension(2*sp_np-1):: sp_work
	integer, intent(out):: sp_error
	!error code

	!******* local variables *******
	real(sp_prec), dimension(1:sp_np-1):: sp_zeta
	!array of additional breaking points
	real(sp_prec), dimension(1:sp_np):: sp_d1
	!sp_d1(1:sp_np) is the array to store 1st derivatives
	real(sp_prec), dimension(1:sp_np-1):: sp_delta
	!internally used

	!******* check inputs *******
	call sp_check_inputs1(sp_np, sp_x, sp_y, sp_error)
	if (sp_error>0) return

	!******* compute d1 and delta *******
	call sp_compt_d1(sp_np, sp_x, sp_y, sp_d1, sp_delta)

	!******* replace sp_d1(1) and sp_d1(sp_np) *******
	if (flag_1/=0) sp_d1(1)=sp_d1_1
	if (flag_n/=0) sp_d1(sp_np)=sp_d1_n

	!******* compute additional break points *******
	call sp_compt_zeta(sp_np, sp_x, sp_y, sp_d1, sp_delta, sp_zeta)

	!******* construct output array *******
	sp_work(1:sp_np-1)=sp_zeta(1:sp_np-1)
	sp_work(sp_np:2*sp_np-1)=sp_d1(1:sp_np)

	!******* end  of the subroutine *******
	return

END SUBROUTINE DSPSPLINE2

!********************************************************************
! SUBROUTINE DSPSPLINE3
!********************************************************************
SUBROUTINE DSPSPLINE3(sp_np, sp_x, sp_y, sp_d, sp_work, sp_error)
	implicit none

	!******* precision controller *******
	integer, parameter:: sp_prec=8
	!double precision

	!******* input variables *******
	integer, intent(in):: sp_np
	!number of grid points (>2)
	real(sp_prec), intent(in), dimension(1:sp_np):: sp_x, sp_y
	!sp_x(1:sp_np) is the array of grid points
	!sp_y(1:sp_np) is the array of the value of functions

	!******* input variables (special for DSPSPLINE3) *******
	real(sp_prec), intent(in), dimension(1:sp_np):: sp_d
	!sp_d(1:sp_np) is the array of derivatives at grid points

	!******* output variables *******
	real(sp_prec), intent(out), dimension(2*sp_np-1):: sp_work
	integer, intent(out):: sp_error
	!error code

	!******* local variables *******
	real(sp_prec), dimension(1:sp_np-1):: sp_zeta
	!array of additional breaking points
	real(sp_prec), dimension(1:sp_np):: sp_d1
	!sp_d1(1:sp_np) is the array to store 1st derivatives
	real(sp_prec), dimension(1:sp_np-1):: sp_delta
	!internally used

	!******* check inputs *******
	call sp_check_inputs1(sp_np, sp_x, sp_y, sp_error)
	if (sp_error>0) return

	!******* compute d1 and delta *******
	call sp_compt_d1(sp_np, sp_x, sp_y, sp_d1, sp_delta)

	!******* replace sp_d1 with sp_d *******
	sp_d1=sp_d

	!******* compute additional break points *******
	call sp_compt_zeta(sp_np, sp_x, sp_y, sp_d1, sp_delta, sp_zeta)

	!******* construct output array *******
	sp_work(1:sp_np-1)=sp_zeta(1:sp_np-1)
	sp_work(sp_np:2*sp_np-1)=sp_d1(1:sp_np)

	!******* end  of the subroutine *******
	return

END SUBROUTINE DSPSPLINE3

!********************************************************************
! SUBROUTINE SP_CHECK_INPUTS1
!********************************************************************
SUBROUTINE SP_CHECK_INPUTS1(sp_np, sp_x, sp_y, sp_error)
	implicit none

	!******* precision controller *******
	integer, parameter:: sp_prec=8
	!double precision

	!******* input variables *******
	integer, intent(in):: sp_np
	real(sp_prec), intent(in), dimension(1:sp_np):: sp_x, sp_y

	!******* output variables *******
	integer, intent(out):: sp_error

	!******* local variables *******
	integer:: p0

	!******* set default error code *******
	sp_error=0

	!******* check number of sp_np *******
	if (sp_np<3) then
		sp_error=1
		return
	end if

	!******* check order of grids *******
	do p0=1, sp_np-1
		if (sp_x(p0)>=sp_x(p0+1)) then
			sp_error=2
			return
		end if
	end do

	!******* end of subroutine *******
	return

END SUBROUTINE SP_CHECK_INPUTS1

!********************************************************************
! SUBROUTINE SP_COMPT_D1
!********************************************************************
SUBROUTINE SP_COMPT_D1(sp_np, sp_x, sp_y, sp_d1, sp_delta)
	implicit none

	!******* precision controller *******
	integer, parameter:: sp_prec=8
	!double precision

	!******* input variables *******
	integer, intent(in):: sp_np
	real(sp_prec), intent(in), dimension(1:sp_np):: sp_x, sp_y

	!******* output variables *******
	real(sp_prec), intent(out), dimension(1:sp_np):: sp_d1
	real(sp_prec), intent(out), dimension(1:sp_np-1):: sp_delta

	!******* local variables *******
	integer:: p0
	real(sp_prec), dimension(1:sp_np-1):: sp_l

	!******* calculate l and delta *******
	do p0=1, sp_np-1
		sp_l(p0)=((sp_x(p0+1)-sp_x(p0))**2._sp_prec+&
		(sp_y(p0+1)-sp_y(p0))**2._sp_prec)**0.5_sp_prec
		sp_delta(p0)=(sp_y(p0+1)-sp_y(p0))/(sp_x(p0+1)-sp_x(p0))
	end do

	!******* compute sp_d1(2:sp_np-1) *******
	do p0=2, sp_np-1
		if ((sp_delta(p0-1)*sp_delta(p0))<=0._sp_prec) then
			sp_d1(p0)=0._sp_prec
		else
			sp_d1(p0)=((sp_l(p0-1)*sp_delta(p0-1))+(sp_l(p0)*sp_delta(p0)))&
			/(sp_l(p0-1)+sp_l(p0))
		end if
	end do

	!******* compute sp_d1(1) and sp_d1(sp_np) *******
	sp_d1(1)=(3._sp_prec*sp_delta(1)-sp_d1(2))/2._sp_prec
	sp_d1(sp_np)=(3._sp_prec*sp_delta(sp_np-1)-sp_d1(sp_np-1))&
	/2._sp_prec

	!******* end of subroutine *******
	return

END SUBROUTINE SP_COMPT_D1

!********************************************************************
! SUBROUTINE SP_COMPT_ZETA
!********************************************************************
SUBROUTINE sp_compt_zeta&
	(sp_np, sp_x, sp_y, sp_d1, sp_delta, sp_zeta)
	implicit none

	!******* precision controller *******
	integer, parameter:: sp_prec=8
	!double precision

	!******* input variables *******
	integer, intent(in):: sp_np
	real(sp_prec), intent(in), dimension(1:sp_np):: sp_x, sp_y, sp_d1
	real(sp_prec), intent(in), dimension(1:sp_np-1):: sp_delta

	!******* output variables *******
	real(sp_prec), intent(out), dimension(1:sp_np-1):: sp_zeta

	!******* local variables *******
	integer:: p0
	real(sp_prec):: zetatemp, delta1, delta2

	!******* compute sp_zeta *******
	do p0=1, sp_np-1
		delta1=sp_d1(p0)-sp_delta(p0)
		delta2=sp_d1(p0+1)-sp_delta(p0)
		if (delta1*delta2>=0._sp_prec) then
			sp_zeta(p0)=(sp_x(p0)+sp_x(p0+1))/2._sp_prec
		else if (abs(delta1)>abs(delta2)) then
			zetatemp=sp_x(p0)+2._sp_prec*(sp_x(p0+1)-sp_x(p0))*delta2/&
			(sp_d1(p0+1)-sp_d1(p0))
			sp_zeta(p0)=(sp_x(p0)+zetatemp)/2._sp_prec
		else
			zetatemp=sp_x(p0+1)+2._sp_prec*(sp_x(p0+1)-sp_x(p0))*delta1/&
			(sp_d1(p0+1)-sp_d1(p0))
			sp_zeta(p0)=(zetatemp+sp_x(p0+1))/2._sp_prec
		end if
	end do

	!******* end of subroutine *******
	return

END SUBROUTINE SP_COMPT_ZETA

!********************************************************************
! SUBROUTINE DSPSPLINT
!********************************************************************
SUBROUTINE DSPSPLINT(sp_np,sp_x,sp_y,sp_work,sp_xv,sp_yv,sp_error)
	implicit none

	!******* precision controller *******
	integer, parameter:: sp_prec=8
	!double precision

	!******* input variables *******
	integer, intent(in):: sp_np
	!number of grid points
	real(sp_prec), intent(in), dimension(1:sp_np):: sp_x, sp_y
	!sp_x(1:sp_np) is the array of grid points
	!sp_y(1:sp_np) is the array of the value of functions
	real(sp_prec), intent(in), dimension(1:2*sp_np-1):: sp_work
	!array of additional breaking points
	real(sp_prec), intent(in):: sp_xv
	!point where the interpolated function is evaluated

	!******* output variables *******
	real(sp_prec), intent(out):: sp_yv
	!value of the interpolated function at sp_xv
	integer, intent(out):: sp_error
	!error code

	!******* local variables *******
	real(sp_prec), dimension(1:sp_np-1):: sp_zeta
	!array of additional breaking points
	real(sp_prec), dimension(1:sp_np):: sp_d1
	!1st derivatives
	real(sp_prec), dimension(1:sp_np-1):: sp_delta
	!internally used

	!******* construct zeta and d1 *******
	sp_zeta(1:sp_np-1)=sp_work(1:sp_np-1)
	sp_d1(1:sp_np)=sp_work(sp_np:2*sp_np-1)

	!******* check inputs *******
	sp_error=0
	call sp_check_inputs2(sp_np, sp_x, sp_y, sp_zeta, sp_xv, sp_error)
	if (sp_error>0) return

	!******* evaluate the interpolated function *******
	call sp_eval(sp_np, sp_x, sp_y, sp_d1, sp_zeta, sp_xv, sp_yv)

	!******* end  of the subroutine *******
	return

END SUBROUTINE DSPSPLINT

!********************************************************************
! SUBROUTINE DSPSPLINTD
! DSPSPLINT plus calculating derivative
!********************************************************************
SUBROUTINE DSPSPLINTD(sp_np,sp_x,sp_y,sp_work,sp_xv,sp_yv,sp_yd,&
sp_error)
	implicit none

	!******* precision controller *******
	integer, parameter:: sp_prec=8
	!double precision

	!******* input variables *******
	integer, intent(in):: sp_np
	!number of grid points
	real(sp_prec), intent(in), dimension(1:sp_np):: sp_x, sp_y
	!sp_x(1:sp_np) is the array of grid points
	!sp_y(1:sp_np) is the array of the value of functions
	real(sp_prec), intent(in), dimension(1:2*sp_np-1):: sp_work
	!array of additional breaking points
	real(sp_prec), intent(in):: sp_xv
	!point where the interpolated function is evaluated

	!******* output variables *******
	real(sp_prec), intent(out):: sp_yv
	!value of the interpolated function at sp_xv
	real(sp_prec), intent(out):: sp_yd
	!derivative of the interpolated function at sp_xv
	integer, intent(out):: sp_error
	!error code

	!******* local variables *******
	real(sp_prec), dimension(1:sp_np-1):: sp_zeta
	!array of additional breaking points
	real(sp_prec), dimension(1:sp_np):: sp_d1
	!1st derivatives
	real(sp_prec), dimension(1:sp_np-1):: sp_delta
	!internally used

	!******* construct zeta and d1 *******
	sp_zeta(1:sp_np-1)=sp_work(1:sp_np-1)
	sp_d1(1:sp_np)=sp_work(sp_np:2*sp_np-1)

	!******* check inputs *******
	sp_error=0
	!call sp_check_inputs2(sp_np, sp_x, sp_y, sp_zeta, sp_xv, sp_error)
	!if (sp_error>0) return

	!******* evaluate the value and the derivative *******
	call sp_evald(sp_np, sp_x, sp_y, sp_d1, sp_zeta, sp_xv, sp_yv,&
	sp_yd)

	!******* end  of the subroutine *******
	return

END SUBROUTINE DSPSPLINTD

!********************************************************************
! SUBROUTINE SP_CHECK_INPUTS2
!********************************************************************
SUBROUTINE SP_CHECK_INPUTS2&
	(sp_np, sp_x, sp_y, sp_zeta, sp_xv, sp_error)
	implicit none

	!******* precision controller *******
	integer, parameter:: sp_prec=8
	!double precision

	!******* input variables *******
	integer, intent(in):: sp_np
	real(sp_prec), intent(in), dimension(1:sp_np):: sp_x, sp_y
	real(sp_prec), intent(in), dimension(1:sp_np-1):: sp_zeta
	real(sp_prec), intent(in):: sp_xv

	!******* output variables *******
	integer, intent(out):: sp_error

	!******* local variables *******
	integer:: p0

	!******* set default error code *******
	sp_error=0

	!******* check number of sp_np *******
	if (sp_np<3) then
		sp_error=1
		return
	end if

	!******* check order of grids *******
	do p0=1, sp_np-1
		if (sp_x(p0)>=sp_x(p0+1)) then
			sp_error=2
			return
		end if
	end do

	!******* check position of zeta *******
	do p0=1, sp_np-1
		if ((sp_zeta(p0)<sp_x(p0)) .or. (sp_zeta(p0)>sp_x(p0+1))) then
			sp_error=3
			return
		end if
	end do

	!******* check position of sp_xv *******
	if ((sp_xv<sp_x(1)) .or. (sp_xv>sp_x(sp_np))) then
		sp_error=4
		return
	end if

	!******* end of subroutine *******
	return

END SUBROUTINE SP_CHECK_INPUTS2

!********************************************************************
! SUBROUTINE SP_EVAL
!********************************************************************
SUBROUTINE SP_EVAL(sp_np,sp_x,sp_y,sp_d1,sp_zeta,sp_xv,sp_yv)
	implicit none

	!******* precision controller *******
	integer, parameter:: sp_prec=8
	!double precision

	!******* input variables *******
	integer, intent(in):: sp_np
	real(sp_prec), intent(in), dimension(1:sp_np):: sp_x, sp_y, sp_d1
	real(sp_prec), intent(in), dimension(1:sp_np-1):: sp_zeta
	real(sp_prec), intent(in):: sp_xv

	!******* output variables *******
	real(sp_prec):: sp_yv

	!******* local variables *******
	integer:: p0, p00
	real(sp_prec):: a1, a2, b1, b2, c1, c2, d1bar, a0, b0

!	!******* find in which interval: p0 *******
!	do p0=1, sp_np-1
!		if (sp_x(p0+1)>=sp_xv) exit
!	end do
!
!	!******* find whether left or right of zeta *******
!	if (sp_xv>=sp_x(p0) .and. sp_zeta(p0)>=sp_xv) then
!		p00=1
!	else if (sp_xv>sp_zeta(p0) .and. sp_x(p0+1)>=sp_xv) then
!		p00=2
!	else
!		print *,'Something wrong...'
!		print *,sp_np,sp_xv
!		print *,sp_x(:)
!		print *,sp_y(:)
!		print *,sp_zeta(:)
!		stop
!	end if

	!******* find p0 and p00 *******
	!******* updated from above on 09/15/2003 *******
	do p0=1, sp_np-1
		if (sp_zeta(p0)>=sp_xv) then
			p00=1
			exit
		else if (sp_x(p0+1)>=sp_xv) then
			p00=2
			exit
		else if (p0==sp_np-1) then
			if (sp_zeta(p0)>=sp_xv) then
				p00=1
				exit
			else
				p00=2
				exit
			end if
		end if
	end do

	!******* evaluate the function *******
	a1=sp_y(p0)
	b1=sp_d1(p0)
	a0=sp_zeta(p0)-sp_x(p0)
	b0=sp_x(p0+1)-sp_zeta(p0)
	d1bar=(2._sp_prec*(sp_y(p0+1)-sp_y(p0))-&
	(a0*sp_d1(p0)+b0*sp_d1(p0+1)))/(sp_x(p0+1)-sp_x(p0))
	c1=(d1bar-sp_d1(p0))/(2._sp_prec*a0)
	if (p00==1) then
		sp_yv=a1+b1*(sp_xv-sp_x(p0))+c1*(sp_xv-sp_x(p0))**2._sp_prec
	else
		a2=a1+a0*b1+a0**2._sp_prec*c1
		b2=d1bar
		c2=(sp_d1(p0+1)-d1bar)/(2._sp_prec*b0)
		sp_yv=a2+b2*(sp_xv-sp_zeta(p0))+c2*(sp_xv-sp_zeta(p0))**2._sp_prec
	end if

	!******* end of subroutine *******
	return

END SUBROUTINE SP_EVAL

!********************************************************************
! SUBROUTINE SP_EVALD
!********************************************************************
SUBROUTINE SP_EVALD(sp_np,sp_x,sp_y,sp_d1,sp_zeta,sp_xv,sp_yv,sp_yd)
	implicit none

	!******* precision controller *******
	integer, parameter:: sp_prec=8
	!double precision

	!******* input variables *******
	integer, intent(in):: sp_np
	real(sp_prec), intent(in), dimension(1:sp_np):: sp_x, sp_y, sp_d1
	real(sp_prec), intent(in), dimension(1:sp_np-1):: sp_zeta
	real(sp_prec), intent(in):: sp_xv

	!******* output variables *******
	real(sp_prec):: sp_yv, sp_yd

	!******* local variables *******
	integer:: p0, p00
	real(sp_prec):: a1, a2, b1, b2, c1, c2, d1bar, a0, b0

!	!******* find in which interval: p0 *******
!	do p0=1, sp_np-1
!		if (sp_x(p0+1)>=sp_xv) exit
!	end do
!
!	!******* find whether left or right of zeta *******
!	if (sp_xv>=sp_x(p0) .and. sp_zeta(p0)>=sp_xv) then
!		p00=1
!	else if (sp_xv>sp_zeta(p0) .and. sp_x(p0+1)>=sp_xv) then
!		p00=2
!	else
!		print *,'Something wrong...'
!		print *,sp_np,sp_xv
!		print *,sp_x(:)
!		print *,sp_y(:)
!		print *,sp_zeta(:)
!		stop
!	end if

	!******* find p0 and p00 *******
	!******* updated from above on 09/15/2003 *******
	do p0=1, sp_np-1
		if (sp_zeta(p0)>=sp_xv) then
			p00=1
			exit
		else if (sp_x(p0+1)>=sp_xv) then
			p00=2
			exit
		else if (p0==sp_np-1) then
			if (sp_zeta(p0)>=sp_xv) then
				p00=1
				exit
			else
				p00=2
				exit
			end if
		end if
	end do

	!******* evaluate the function *******
	a1=sp_y(p0)
	b1=sp_d1(p0)
	a0=sp_zeta(p0)-sp_x(p0)
	b0=sp_x(p0+1)-sp_zeta(p0)
	d1bar=(2._sp_prec*(sp_y(p0+1)-sp_y(p0))-&
	(a0*sp_d1(p0)+b0*sp_d1(p0+1)))/(sp_x(p0+1)-sp_x(p0))
	c1=(d1bar-sp_d1(p0))/(2._sp_prec*a0)
	if (p00==1) then
		sp_yv=a1+b1*(sp_xv-sp_x(p0))+c1*(sp_xv-sp_x(p0))**2._sp_prec
		sp_yd=b1+2._sp_prec*c1*(sp_xv-sp_x(p0))
	else
		a2=a1+a0*b1+a0**2._sp_prec*c1
		b2=d1bar
		c2=(sp_d1(p0+1)-d1bar)/(2._sp_prec*b0)
		sp_yv=a2+b2*(sp_xv-sp_zeta(p0))+c2*(sp_xv-sp_zeta(p0))**2._sp_prec
		sp_yd=b2+2._sp_prec*c2*(sp_xv-sp_zeta(p0))
	end if

	!******* end of subroutine *******
	return

END SUBROUTINE SP_EVALD

!********************************************************************
! SUBROUTINE DSPSPLEASY
!********************************************************************
SUBROUTINE DSPSPLEASY (sp_np, sp_x, sp_y, sp_xv, sp_yv, sp_error)
	implicit none

	!******* precision controller *******
	integer, parameter:: sp_prec=8

	!******* input variables *******
	integer, intent(in):: sp_np
	real(sp_prec), intent(in), dimension(1:sp_np):: sp_x, sp_y
	real(sp_prec), intent(in):: sp_xv

	!******* output variables *******
	real(sp_prec), intent(out):: sp_yv
	integer, intent(out):: sp_error

	!******* local variables *******
	real(sp_prec), dimension(2*sp_np-1):: sp_work

	!******* construct interpolated function *******
	call dspspline(sp_np, sp_x, sp_y, sp_work, sp_error)

	!******* if error occurs, return with the error code+100 *******
	if (sp_error>0) then
		sp_error=sp_error+100
		return
	end if

	!******* compute the value of interpolated function at xv *******
	call dspsplint(sp_np,sp_x,sp_y,sp_work,sp_xv,sp_yv,sp_error)

	!******* if error occurs, return with the error code+100 *******
	if (sp_error>0) then
		sp_error=sp_error+200
		return
	end if

	!******* end  of the subroutine *******
	return

END SUBROUTINE DSPSPLEASY

!********************************************************************
! SUBROUTINE DSPSPLEASYD
!********************************************************************
SUBROUTINE DSPSPLEASYD(sp_np,sp_x,sp_y,sp_xv,sp_yv,sp_yd,sp_error)
	implicit none

	!******* precision controller *******
	integer,parameter:: sp_prec=8

	!******* input variables *******
	integer, intent(in):: sp_np
	real(sp_prec),intent(in),dimension(1:sp_np):: sp_x,sp_y
	real(sp_prec),intent(in):: sp_xv

	!******* output variables *******
	real(sp_prec),intent(out):: sp_yv,sp_yd
	integer,intent(out):: sp_error

	!******* local variables *******
	real(sp_prec),dimension(2*sp_np-1):: sp_work

	!******* construct interpolated function *******
	call dspspline(sp_np,sp_x,sp_y,sp_work,sp_error)

	!******* if error occurs, return with the error code+100 *******
	if (sp_error>0) then
		sp_error=sp_error+100
		return
	end if

	!******* compute the value of interpolated function at xv *******
	call dspsplintd&
	(sp_np,sp_x,sp_y,sp_work,sp_xv,sp_yv,sp_yd,sp_error)

	!******* if error occurs, return with the error code+100 *******
	if (sp_error>0) then
		sp_error=sp_error+200
		return
	end if

	!******* end  of the subroutine *******
	return

END SUBROUTINE DSPSPLEASYD

!********************************************************************
! SUBROUTINE DSPSPLEASY2
!********************************************************************
SUBROUTINE DSPSPLEASY2 (sp_np, sp_x, sp_y, sp_xv, flag_1, sp_d1_1, &
sp_d1_n, flag_n, sp_yv, sp_error)
	implicit none

	!******* precision controller *******
	integer, parameter:: sp_prec=8

	!******* input variables *******
	integer, intent(in):: sp_np, flag_1, flag_n
	real(sp_prec), intent(in), dimension(1:sp_np):: sp_x, sp_y
	real(sp_prec), intent(in):: sp_xv, sp_d1_1, sp_d1_n

	!******* output variables *******
	real(sp_prec), intent(out):: sp_yv
	integer, intent(out):: sp_error

	!******* local variables *******
	real(sp_prec), dimension(2*sp_np-1):: sp_work

	!******* construct interpolated function *******
	call dspspline2(sp_np, sp_x, sp_y, flag_1, sp_d1_1, flag_n, &
	sp_d1_n, sp_work, sp_error)

	!******* if error occurs, return with the error code+100 *******
	if (sp_error>0) then
		sp_error=sp_error+100
		return
	end if

	!******* compute the value of interpolated function at xv *******
	call dspsplint(sp_np,sp_x,sp_y,sp_work,sp_xv,sp_yv,sp_error)

	!******* if error occurs, return with the error code+100 *******
	if (sp_error>0) then
		sp_error=sp_error+200
		return
	end if

	!******* end  of the subroutine *******
	return

END SUBROUTINE DSPSPLEASY2

!********************************************************************
! SUBROUTINE DSPSPLEASY3
!********************************************************************
SUBROUTINE DSPSPLEASY3 (sp_np, sp_x, sp_y, sp_xv, sp_d, &
sp_yv, sp_error)
	implicit none

	!******* precision controller *******
	integer, parameter:: sp_prec=8

	!******* input variables *******
	integer, intent(in):: sp_np
	real(sp_prec), intent(in), dimension(1:sp_np):: sp_x, sp_y, sp_d
	real(sp_prec), intent(in):: sp_xv

	!******* output variables *******
	real(sp_prec), intent(out):: sp_yv
	integer, intent(out):: sp_error

	!******* local variables *******
	real(sp_prec), dimension(2*sp_np-1):: sp_work

	!******* construct interpolated function *******
	call dspspline3(sp_np, sp_x, sp_y, sp_d, sp_work, sp_error)

	!******* if error occurs, return with the error code+100 *******
	if (sp_error>0) then
		sp_error=sp_error+100
		return
	end if

	!******* compute the value of interpolated function at xv *******
	call dspsplint(sp_np,sp_x,sp_y,sp_work,sp_xv,sp_yv,sp_error)

	!******* if error occurs, return with the error code+100 *******
	if (sp_error>0) then
		sp_error=sp_error+200
		return
	end if

	!******* end  of the subroutine *******
	return

END SUBROUTINE DSPSPLEASY3

!********************************************************************
! SUBROUTINE DSPSPLEASYD2
!********************************************************************
SUBROUTINE DSPSPLEASYD2(sp_np,sp_x,sp_y,sp_xv,flag_1,sp_d1_1,&
flag_n,sp_d1_n,sp_yv,sp_yd,sp_error)
	implicit none

	!******* precision controller *******
	integer,parameter:: sp_prec=8

	!******* input variables *******
	integer, intent(in):: sp_np,flag_1,flag_n
	real(sp_prec),intent(in),dimension(1:sp_np):: sp_x,sp_y
	real(sp_prec),intent(in):: sp_xv,sp_d1_1,sp_d1_n

	!******* output variables *******
	real(sp_prec),intent(out):: sp_yv,sp_yd
	integer,intent(out):: sp_error

	!******* local variables *******
	real(sp_prec),dimension(2*sp_np-1):: sp_work

	!******* construct interpolated function *******
	call dspspline2(sp_np,sp_x,sp_y,flag_1,sp_d1_1,flag_n,sp_d1_n,&
	sp_work,sp_error)

	!******* if error occurs, return with the error code+100 *******
	if (sp_error>0) then
		sp_error=sp_error+100
		return
	end if

	!******* compute the value of interpolated function at xv *******
	call dspsplintd&
	(sp_np,sp_x,sp_y,sp_work,sp_xv,sp_yv,sp_yd,sp_error)

	!******* if error occurs, return with the error code+100 *******
	if (sp_error>0) then
		sp_error=sp_error+200
		return
	end if

	!******* end  of the subroutine *******
	return

END SUBROUTINE DSPSPLEASYD2

!********************************************************************
! SUBROUTINE DSPSPLEASYD3
!********************************************************************
SUBROUTINE DSPSPLEASYD3(sp_np,sp_x,sp_y,sp_xv,sp_d,&
sp_yv,sp_yd,sp_error)
	implicit none

	!******* precision controller *******
	integer,parameter:: sp_prec=8

	!******* input variables *******
	integer, intent(in):: sp_np
	real(sp_prec),intent(in),dimension(1:sp_np):: sp_x,sp_y,sp_d
	real(sp_prec),intent(in):: sp_xv

	!******* output variables *******
	real(sp_prec),intent(out):: sp_yv,sp_yd
	integer,intent(out):: sp_error

	!******* local variables *******
	real(sp_prec),dimension(2*sp_np-1):: sp_work

	!******* construct interpolated function *******
	call dspspline3(sp_np,sp_x,sp_y,sp_d,sp_work,sp_error)

	!******* if error occurs, return with the error code+100 *******
	if (sp_error>0) then
		sp_error=sp_error+100
		return
	end if

	!******* compute the value of interpolated function at xv *******
	call dspsplintd&
	(sp_np,sp_x,sp_y,sp_work,sp_xv,sp_yv,sp_yd,sp_error)

	!******* if error occurs, return with the error code+100 *******
	if (sp_error>0) then
		sp_error=sp_error+200
		return
	end if

	!******* end  of the subroutine *******
	return

END SUBROUTINE DSPSPLEASYD3

END MODULE DSPSP

!********************************************************************
! MODULE: mddspspline (version 1.1, August 28, 2002)
! Function to interpolate a function, where:
! (1) shape preserving quadratic interpolation of Schumaker (1983)
!     is used for one dimension, and
! (2) linear interpolation is used for all the other dimensions.
! F90 code. Double Precision.
! Difference from Version 1.0 (May 12, 2002):
! (1) Linear interpolation part accomodates exterpolation
!     (Schumaker part does not).
!********************************************************************
MODULE mddspspline11

!******* load module *******
use summaryindex
use dspsp

implicit none

!******* control access to subroutines in this module *******
private
public:: mddspsp_linint, mddspsp

contains

!********************************************************************
! SUBROUTINE: mddspsp
!********************************************************************
SUBROUTINE mddspsp(nxgrid,xgrid,ygrid,fgridval,xval,yval,fval,ierror)
	implicit none

	!******* parameters *******
	integer, parameter:: prec=8

	!******* input variables *******
	integer, intent(in), dimension(:):: nxgrid
	real(prec), intent(in), dimension(:,:):: xgrid
	real(prec), intent(in), dimension(:):: ygrid
	real(prec), intent(in), dimension(:,:):: fgridval
	real(prec), intent(in), dimension(:):: xval
	real(prec), intent(in):: yval

	!******* output variables *******
	real(prec), intent(out):: fval
	integer, intent(out):: ierror

	!******* local variables *******
	real(prec), dimension(size(ygrid)):: fygridval
	integer:: nygrid
	nygrid=size(ygrid)

	!******* linear interpolation of x *******
	call mddspsp_linint(nxgrid,xgrid,fgridval,xval,nygrid,fygridval)

	!******* shape preserving interpolation of y *******
	call dspspleasy(nygrid,ygrid,fygridval,yval,fval,ierror)

	!******* end of subroutine *******
	return

END SUBROUTINE mddspsp

!********************************************************************
! SUBROUTINE: mddspsp_linint
!********************************************************************
SUBROUTINE mddspsp_linint(nxgrid,xgrid,fgridval,xval,nygrid,fygridval)
	implicit none

	!******* parameters *******
	integer, parameter:: prec=8
	real(prec), parameter:: one=1.0_prec, zero=0.0_prec

	!******* input variables *******
	integer, intent(in), dimension(:):: nxgrid
	real(prec), intent(in), dimension(:,:):: xgrid
	real(prec), intent(in), dimension(:,:):: fgridval
	real(prec), intent(in), dimension(:):: xval
	integer, intent(in):: nygrid

	!******* output variables *******
	real(prec), intent(out), dimension(nygrid):: fygridval

	!******* local variables *******
	integer:: nxdim, xdim0, g0, y0
	real(prec), dimension(size(nxgrid),0:1):: theta
	integer, dimension(size(nxgrid)):: x0

	integer:: nnx1, sx1
	integer, dimension(size(nxgrid)):: x1, nx1
	real(prec):: theta_product

	!******* set x0 and theta *******
	nxdim=size(nxgrid)
	do xdim0=1, nxdim
		!******* find grid to be used (=g0) *******
		if (xval(xdim0)<xgrid(xdim0,1)) then
			g0=1
			x0(xdim0)=g0
		elseif (xval(xdim0)>xgrid(xdim0,nxgrid(xdim0))) then
			g0=nxgrid(xdim0)-1
			x0(xdim0)=g0
		else
			do g0=1, nxgrid(xdim0)-1
				if (xgrid(xdim0,g0+1)>=xval(xdim0)) then
				x0(xdim0)=g0
				exit
				end if
			end do
		end if
		!******* calculate theta(xdim0,0:1) *******
		theta(xdim0,1)=(xval(xdim0)-xgrid(xdim0,g0))/&
		(xgrid(xdim0,g0+1)-xgrid(xdim0,g0))
		theta(xdim0,0)=one-theta(xdim0,1)
	end do

	!******* compute fygridval(1:nygrid)) *******
	fygridval=zero
	nx1(1:nxdim)=2
	nnx1=product(nx1)
	do y0=1, nygrid
		do sx1=1, nnx1
			x1=inv_sumindex(nx1,sx1)-1
			theta_product=one
			do xdim0=1, nxdim
				theta_product=theta_product*theta(xdim0,x1(xdim0))
			end do
			fygridval(y0)=fygridval(y0)+&
			theta_product*fgridval(y0,sumindex(nxgrid,x0+x1))
		end do
	end do

	!******* end of subroutine *******
	return

END SUBROUTINE mddspsp_linint

END MODULE mddspspline11

!********************************************************************
! MODULE makols
!********************************************************************
MODULE makols
implicit none
private
public:: easyols

contains

!********************************************************************
! SUBROUTINE: easyols
!********************************************************************
SUBROUTINE easyols(ndata,nx,x,y,bhat,stats)
	implicit none

	!******* parameter *******
	integer, parameter:: prec=8

	!******* input/output variables *******
	integer,    intent(in)                       :: ndata, nx
	real(prec), intent(in),  dimension(ndata)    :: y
	real(prec), intent(in),  dimension(ndata, nx):: x
	real(prec), intent(out), dimension(5)        :: stats
	real(prec), intent(out), dimension(1+nx   )  :: bhat

	!******* local variables *******
	real(prec), dimension(ndata, nx+1 ):: x0
	real(prec), dimension(nx+1,  ndata):: xt
	real(prec), dimension(ndata, 1    ):: y0
	real(prec), dimension(nx+1,  1    ):: xy, bhat0
	real(prec), dimension(nx+1,  nx+1 ):: xx, xxinv
	integer                            :: d0

	!******* variables for linpacks package *******
	integer,    dimension(nx+1):: ipvt
	integer                    :: info, lda, n, job
	real(prec), dimension(nx+1):: work
	real(prec), dimension(2)   :: det

	!******* variables for calculating statistics *******
	real(prec), dimension(ndata):: e0, yhat, yt, yr
	real(prec)                  :: sse, ssr, sst, avy, rsq, adj_rsq

	!******* construct x0, y0 *******

	do d0=1, ndata
		x0(d0, 1)=1.0
		x0(d0, 2:nx+1)=x(d0, 1:nx)
		y0(d0, 1)     =y(d0)
	end do

	!******* construct transpose matrix of x *******
	xt=TRANSPOSE(x0)

	!******* compute x'x *******
	xx=MATMUL(xt, x0)

	!******* take inverse of xx *******
	xxinv= xx
	lda  =nx+1
	n    =nx+1
	job  =01
	call dgefa(xxinv, lda, n, ipvt, info)
	call dgedi(xxinv, lda, n, ipvt, det, work, job)

	!******* construct xy *******
	xy=MATMUL(xt, y0)

	!******* construct beta hat *******
	bhat0=MATMUL(xxinv, xy)
	do d0=1, 1+nx
		bhat(d0)=bhat0(d0,1)
	end do

	!******* calculate statistics *******
	avy=sum(y(1:ndata))/real(ndata, prec)
	do d0=1, ndata
		yhat(d0)=DOT_PRODUCT(bhat, x0(d0,:))
		e0(d0)=y(d0)-yhat(d0)
		yt(d0)=y(d0)-avy
		yr(d0)=yhat(d0)-avy
	end do
	!sst=DOT_PRODUCT(y, y)-real(ndata,prec)*avy**2.0
	sst=DOT_PRODUCT(yt,yt)
	ssr=DOT_PRODUCT(yr,yr)
	sse=DOT_PRODUCT(e0,e0)
	rsq=ssr/sst
	adj_rsq=1.0-real(ndata-1, prec)/real(ndata-nx-1, prec)*(1.0-rsq)

	!******* assign values to stats vector *******
	stats(1)=sse
	stats(2)=ssr
	stats(3)=sst
	stats(4)=rsq
	stats(5)=adj_rsq

END SUBROUTINE easyols

!********************************************************************
subroutine daxpy(n,da,dx,incx,dy,incy)
!********************************************************************
! constant times a vector plus a vector.
! uses unrolled loops for increments equal to one.
! jack dongarra, linpack, 3/11/78.
! modified 12/3/93, array(1) declarations changed to array(*)
double precision dx(*),dy(*),da
integer i,incx,incy,ix,iy,m,mp1,n
if (n.le.0) return
if (da .eq. 0.0d0) return
if (incx.eq.1.and.incy.eq.1) go to 20
! code for unequal increments or equal increments
! not equal to 1
ix = 1
iy = 1
if (incx.lt.0) ix = (-n+1)*incx + 1
if (incy.lt.0) iy = (-n+1)*incy + 1
do 10 i = 1,n
  dy(iy) = dy(iy) + da*dx(ix)
  ix = ix + incx
  iy = iy + incy
10 continue
return

! code for both increments equal to 1
! clean-up loop
20 m = mod(n,4)
if( m .eq. 0 ) go to 40
do 30 i = 1,m
  dy(i) = dy(i) + da*dx(i)
   30 continue
if( n .lt. 4 ) return
   40 mp1 = m + 1
do 50 i = mp1,n,4
  dy(i) = dy(i) + da*dx(i)
  dy(i + 1) = dy(i + 1) + da*dx(i + 1)
  dy(i + 2) = dy(i + 2) + da*dx(i + 2)
  dy(i + 3) = dy(i + 3) + da*dx(i + 3)
return
   50 continue
end subroutine daxpy

!********************************************************************
subroutine dscal(n,da,dx,incx)
!********************************************************************
! scales a vector by a constant.
! uses unrolled loops for increment equal to one.
! jack dongarra, linpack, 3/11/78.
! modified 3/93 to return if incx .le. 0.
! modified 12/3/93, array(1) declarations changed to array(*)
double precision da,dx(*)
integer i,incx,m,mp1,n,nincx
if( n.le.0 .or. incx.le.0 )return
if(incx.eq.1)go to 20
!  code for increment not equal to 1
nincx = n*incx
do 10 i = 1,nincx,incx
  dx(i) = da*dx(i)
   10 continue
return
!  code for increment equal to 1
!  clean-up loop
   20 m = mod(n,5)
if( m .eq. 0 ) go to 40
do 30 i = 1,m
  dx(i) = da*dx(i)
   30 continue
if( n .lt. 5 ) return
   40 mp1 = m + 1
do 50 i = mp1,n,5
  dx(i) = da*dx(i)
  dx(i + 1) = da*dx(i + 1)
  dx(i + 2) = da*dx(i + 2)
  dx(i + 3) = da*dx(i + 3)
  dx(i + 4) = da*dx(i + 4)
   50 continue
return
end subroutine dscal

!********************************************************************
subroutine dswap(n,dx,incx,dy,incy)
!********************************************************************
! interchanges two vectors.
! uses unrolled loops for increments equal one.
! jack dongarra, linpack, 3/11/78.
! modified 12/3/93, array(1) declarations changed to array(*)
double precision dx(*),dy(*),dtemp
integer i,incx,incy,ix,iy,m,mp1,n
if(n.le.0)return
if(incx.eq.1.and.incy.eq.1)go to 20
! code for unequal increments or equal increments not equal to 1
ix = 1
iy = 1
if(incx.lt.0)ix = (-n+1)*incx + 1
if(incy.lt.0)iy = (-n+1)*incy + 1
do 10 i = 1,n
  dtemp = dx(ix)
  dx(ix) = dy(iy)
  dy(iy) = dtemp
  ix = ix + incx
  iy = iy + incy
   10 continue
return
! code for both increments equal to 1
! clean-up loop
   20 m = mod(n,3)
if( m .eq. 0 ) go to 40
do 30 i = 1,m
  dtemp = dx(i)
  dx(i) = dy(i)
  dy(i) = dtemp
   30 continue
if( n .lt. 3 ) return
   40 mp1 = m + 1
do 50 i = mp1,n,3
  dtemp = dx(i)
  dx(i) = dy(i)
  dy(i) = dtemp
  dtemp = dx(i + 1)
  dx(i + 1) = dy(i + 1)
  dy(i + 1) = dtemp
  dtemp = dx(i + 2)
  dx(i + 2) = dy(i + 2)
  dy(i + 2) = dtemp
   50 continue
return
end subroutine dswap

!********************************************************************
subroutine dgedi(a,lda,n,ipvt,det,work,job)
!********************************************************************
integer lda,n,ipvt(n),job
double precision a(lda,n),det(2),work(n)
! dgedi computes the determinant and inverse of a matrix
! using the factors computed by dgeco or dgefa.
!
! on entry
!  a    double precision(lda, n)
!       the output from dgeco or dgefa.
!  lda  integer
!       the leading dimension of the array  a .
!  n    integer
!       the order of the matrix  a .
!  ipvt integer(n)
!       the pivot vector from dgeco or dgefa.
!  work double precision(n)
!       work vector.  contents destroyed.
!  job  integer
!       = 11   both determinant and inverse.
!       = 01   inverse only.
!       = 10   determinant only.
!
! on return
!  a    inverse of original matrix if requested.
!       otherwise unchanged.
!  det  double precision(2)
!       determinant of original matrix if requested.
!       otherwise not referenced.
!       determinant = det(1) * 10.0**det(2)
!       with  1.0 .le. dabs(det(1)) .lt. 10.0
!       or  det(1) .eq. 0.0 .
! error condition
!       a division by zero will occur if the input factor contains
!       a zero on the diagonal and the inverse is requested.
!       it will not occur if the subroutines are called correctly
!       and if dgeco has set rcond .gt. 0.0 or dgefa has set
!       info .eq. 0 .
!
! linpack. this version dated 08/14/78 .
! cleve moler, university of new mexico, argonne national lab.
!
! subroutines and functions
!       blas daxpy,dscal,dswap
!       fortran dabs,mod
!
! internal variables
double precision t
double precision ten
integer i,j,k,kb,kp1,l,nm1
! compute determinant
if (job/10 .eq. 0) go to 70
   det(1) = 1.0d0
   det(2) = 0.0d0
   ten = 10.0d0
   do 50 i = 1, n
if (ipvt(i) .ne. i) det(1) = -det(1)
det(1) = a(i,i)*det(1)
!  ...exit
if (det(1) .eq. 0.0d0) go to 60
   10 if (dabs(det(1)) .ge. 1.0d0) go to 20
   det(1) = ten*det(1)
   det(2) = det(2) - 1.0d0
go to 10
   20 continue
   30 if (dabs(det(1)) .lt. ten) go to 40
   det(1) = det(1)/ten
   det(2) = det(2) + 1.0d0
go to 30
   40 continue
   50    continue
   60    continue
   70 continue
! compute inverse(u)
if (mod(job,10) .eq. 0) go to 150
   do 100 k = 1, n
a(k,k) = 1.0d0/a(k,k)
t = -a(k,k)
call dscal(k-1,t,a(1,k),1)
kp1 = k + 1
if (n .lt. kp1) go to 90
do 80 j = kp1, n
   t = a(k,j)
   a(k,j) = 0.0d0
   call daxpy(k,t,a(1,k),1,a(1,j),1)
   80 continue
   90 continue
  100    continue
! form inverse(u)*inverse(l)
   nm1 = n - 1
   if (nm1 .lt. 1) go to 140
   do 130 kb = 1, nm1
k = n - kb
kp1 = k + 1
do 110 i = kp1, n
   work(i) = a(i,k)
   a(i,k) = 0.0d0
  110 continue
do 120 j = kp1, n
   t = work(j)
   call daxpy(n,t,a(1,j),1,a(1,k),1)
  120 continue
l = ipvt(k)
if (l .ne. k) call dswap(n,a(1,k),1,a(1,l),1)
  130    continue
  140    continue
  150 continue
return
end subroutine dgedi

!********************************************************************
subroutine dgeco(a,lda,n,ipvt,rcond,z)
!********************************************************************
integer lda,n,ipvt(n)
double precision a(lda,n),z(n)
double precision rcond
! dgeco factors a double precision matrix by gaussian elimination
! and estimates the condition of the matrix.
!
! if  rcond  is not needed, dgefa is slightly faster.
! to solve  a*x = b , follow dgeco by dgesl.
! to compute  inverse(a)*c , follow dgeco by dgesl.
! to compute  determinant(a) , follow dgeco by dgedi.
! to compute  inverse(a) , follow dgeco by dgedi.
!
! on entry
!    a     double precision(lda, n)
!          the matrix to be factored.
!    lda   integer
!          the leading dimension of the array  a .
!    n     integer
!          the order of the matrix  a .
!
! on return
!    a     an upper triangular matrix and the multipliers
!          which were used to obtain it.
!          the factorization can be written  a = l*u  where
!    l     is a product of permutation and unit lower
!          triangular matrices and  u  is upper triangular.
!    ipvt  integer(n)
!          an integer vector of pivot indices.
!    rcond double precision
!          an estimate of the reciprocal condition of  a .
!          for the system  a*x = b , relative perturbations
!          in  a  and  b  of size  epsilon  may cause
!          relative perturbations in  x  of size  epsilon/rcond .
!          if  rcond  is so small that the logical expression
!          1.0 + rcond .eq. 1.0
!          is true, then  a  may be singular to working
!          precision.  in particular,  rcond  is zero  if
!          exact singularity is detected or the estimate
!          underflows.
!    z     double precision(n)
!          a work vector whose contents are usually unimportant.
!          if  a  is close to a singular matrix, then  z  is
!          an approximate null vector in the sense that
!          norm(a*z) = rcond*norm(a)*norm(z) .
!
! linpack. this version dated 08/14/78 .
! cleve moler, university of new mexico, argonne national lab.
!
! subroutines and functions
!          linpack dgefa
!          blas daxpy,ddot,dscal,dasum
!          fortran dabs,dmax1,dsign
!
! internal variables
double precision ek,t,wk,wkm         !******* ddot  erased *******
double precision anorm,s,sm,ynorm    !******* dasum erased *******
integer info,j,k,kb,kp1,l
! compute 1-norm of a
anorm = 0.0d0
do 10 j = 1, n
   anorm = dmax1(anorm,dasum(n,a(1,j),1))
   10 continue
! factor
call dgefa(a,lda,n,ipvt,info)
! rcond = 1/(norm(a)*(estimate of norm(inverse(a)))) .
! estimate = norm(z)/norm(y) where  a*z = y  and  trans(a)*y = e .
! trans(a)  is the transpose of a .  the components of  e  are
! chosen to cause maximum local growth in the elements of w  where
! trans(u)*w = e .  the vectors are frequently rescaled to avoid
! overflow.
!
! solve trans(u)*w = e
ek = 1.0d0
do 20 j = 1, n
   z(j) = 0.0d0
   20 continue
do 100 k = 1, n
   if (z(k) .ne. 0.0d0) ek = dsign(ek,-z(k))
   if (dabs(ek-z(k)) .le. dabs(a(k,k))) go to 30
s = dabs(a(k,k))/dabs(ek-z(k))
call dscal(n,s,z,1)
ek = s*ek
   30    continue
   wk = ek - z(k)
   wkm = -ek - z(k)
   s = dabs(wk)
   sm = dabs(wkm)
   if (a(k,k) .eq. 0.0d0) go to 40
wk = wk/a(k,k)
wkm = wkm/a(k,k)
   go to 50
   40 continue
wk = 1.0d0
wkm = 1.0d0
   50 continue
   kp1 = k + 1
   if (kp1 .gt. n) go to 90
do 60 j = kp1, n
   sm = sm + dabs(z(j)+wkm*a(k,j))
   z(j) = z(j) + wk*a(k,j)
   s = s + dabs(z(j))
   60 continue
if (s .ge. sm) go to 80
   t = wkm - wk
   wk = wkm
   do 70 j = kp1, n
z(j) = z(j) + t*a(k,j)
   70 continue
   80 continue
   90 continue
   z(k) = wk
  100 continue
s = 1.0d0/dasum(n,z,1)
call dscal(n,s,z,1)

! solve trans(l)*y = w
do 120 kb = 1, n
   k = n + 1 - kb
   if (k .lt. n) z(k) = z(k) + ddot(n-k,a(k+1,k),1,z(k+1),1)
   if (dabs(z(k)) .le. 1.0d0) go to 110
s = 1.0d0/dabs(z(k))
call dscal(n,s,z,1)
  110 continue
   l = ipvt(k)
   t = z(l)
   z(l) = z(k)
   z(k) = t
  120 continue
s = 1.0d0/dasum(n,z,1)
call dscal(n,s,z,1)
ynorm = 1.0d0

! solve l*v = y
do 140 k = 1, n
   l = ipvt(k)
   t = z(l)
   z(l) = z(k)
   z(k) = t
   if (k .lt. n) call daxpy(n-k,t,a(k+1,k),1,z(k+1),1)
   if (dabs(z(k)) .le. 1.0d0) go to 130
s = 1.0d0/dabs(z(k))
call dscal(n,s,z,1)
ynorm = s*ynorm
  130 continue
  140 continue
s = 1.0d0/dasum(n,z,1)
call dscal(n,s,z,1)
ynorm = s*ynorm

! solve  u*z = v
do 160 kb = 1, n
   k = n + 1 - kb
   if (dabs(z(k)) .le. dabs(a(k,k))) go to 150
s = dabs(a(k,k))/dabs(z(k))
call dscal(n,s,z,1)
ynorm = s*ynorm
  150 continue
   if (a(k,k) .ne. 0.0d0) z(k) = z(k)/a(k,k)
   if (a(k,k) .eq. 0.0d0) z(k) = 1.0d0
   t = -z(k)
   call daxpy(k-1,t,a(1,k),1,z(1),1)
  160 continue
! make znorm = 1.0
s = 1.0d0/dasum(n,z,1)
call dscal(n,s,z,1)
ynorm = s*ynorm
if (anorm .ne. 0.0d0) rcond = ynorm/anorm
if (anorm .eq. 0.0d0) rcond = 0.0d0
return
end subroutine dgeco

!********************************************************************
subroutine dgefa(a,lda,n,ipvt,info)
!********************************************************************
integer lda,n,ipvt(n),info
double precision a(lda,n)
! dgefa factors a double precision matrix by gaussian elimination.
! dgefa is usually called by dgeco, but it can be called
! directly with a saving in time if  rcond  is not needed.
! (time for dgeco) = (1 + 9/n)*(time for dgefa).
!
! on entry
!    a    double precision(lda, n)
!         the matrix to be factored.
!    lda  integer
!         the leading dimension of the array  a .
!    n    integer
!         the order of the matrix  a .
! on return
!    a    an upper triangular matrix and the multipliers
!         which were used to obtain it.
!         the factorization can be written  a = l*u  where
!         l is a product of permutation and unit lower
!         triangular matrices and  u  is upper triangular.
!    ipvt integer(n)
!         an integer vector of pivot indices.
!    info integer
!         = 0  normal value.
!         = k  if  u(k,k) .eq. 0.0 .  this is not an error
!         condition for this subroutine, but it does
!         indicate that dgesl or dgedi will divide by zero
!         if called.  use  rcond  in dgeco for a reliable
!         indication of singularity.
!
! linpack. this version dated 08/14/78 .
! cleve moler, university of new mexico, argonne national lab.
!
! subroutines and functions
!         blas daxpy,dscal,idamax
!
! internal variables
double precision t
integer j,k,kp1,l,nm1   !******* idamax erased *******
! gaussian elimination with partial pivoting
info = 0
nm1 = n - 1
if (nm1 .lt. 1) go to 70
do 60 k = 1, nm1
   kp1 = k + 1

! find l = pivot index
   l = idamax(n-k+1,a(k,k),1) + k - 1
   ipvt(k) = l

! zero pivot implies this column already triangularized
   if (a(l,k) .eq. 0.0d0) go to 40

! interchange if necessary
if (l .eq. k) go to 10
   t = a(l,k)
   a(l,k) = a(k,k)
   a(k,k) = t
   10 continue

! compute multipliers
t = -1.0d0/a(k,k)
call dscal(n-k,t,a(k+1,k),1)

! row elimination with column indexing
do 30 j = kp1, n
   t = a(l,j)
   if (l .eq. k) go to 20
a(l,j) = a(k,j)
a(k,j) = t
   20 continue
   call daxpy(n-k,t,a(k+1,k),1,a(k+1,j),1)
   30 continue
   go to 50
   40 continue
   info = k
   50 continue
   60 continue
   70 continue
ipvt(n) = n
if (a(n,n) .eq. 0.0d0) info = n
return
end subroutine dgefa

!********************************************************************
double precision function dasum(n,dx,incx)
!********************************************************************
! takes the sum of the absolute values.
! jack dongarra, linpack, 3/11/78.
! modified 3/93 to return if incx .le. 0.
! modified 12/3/93, array(1) declarations changed to array(*)
double precision dx(*),dtemp
integer i,incx,m,mp1,n,nincx
dasum = 0.0d0
dtemp = 0.0d0
if( n.le.0 .or. incx.le.0 )return
if(incx.eq.1)go to 20
! code for increment not equal to 1
nincx = n*incx
do 10 i = 1,nincx,incx
  dtemp = dtemp + dabs(dx(i))
   10 continue
dasum = dtemp
return
! code for increment equal to 1
! clean-up loop
   20 m = mod(n,6)
if( m .eq. 0 ) go to 40
do 30 i = 1,m
  dtemp = dtemp + dabs(dx(i))
   30 continue
if( n .lt. 6 ) go to 60
   40 mp1 = m + 1
do 50 i = mp1,n,6
  dtemp = dtemp + dabs(dx(i)) + dabs(dx(i + 1)) + dabs(dx(i + 2)) &
     + dabs(dx(i + 3)) + dabs(dx(i + 4)) + dabs(dx(i + 5))
   50 continue
   60 dasum = dtemp
return
end function dasum

!********************************************************************
double precision function ddot(n,dx,incx,dy,incy)
!********************************************************************
! forms the dot product of two vectors.
! uses unrolled loops for increments equal to one.
! jack dongarra, linpack, 3/11/78.
! modified 12/3/93, array(1) declarations changed to array(*)
double precision dx(*),dy(*),dtemp
integer i,incx,incy,ix,iy,m,mp1,n
ddot = 0.0d0
dtemp = 0.0d0
if(n.le.0)return
if(incx.eq.1.and.incy.eq.1)go to 20
! code for unequal increments or equal increments not equal to 1
ix = 1
iy = 1
if(incx.lt.0)ix = (-n+1)*incx + 1
if(incy.lt.0)iy = (-n+1)*incy + 1
do 10 i = 1,n
  dtemp = dtemp + dx(ix)*dy(iy)
  ix = ix + incx
  iy = iy + incy
   10 continue
ddot = dtemp
return
! code for both increments equal to 1
! clean-up loop
   20 m = mod(n,5)
if( m .eq. 0 ) go to 40
do 30 i = 1,m
  dtemp = dtemp + dx(i)*dy(i)
   30 continue
if( n .lt. 5 ) go to 60
   40 mp1 = m + 1
do 50 i = mp1,n,5
  dtemp = dtemp + dx(i)*dy(i) + dx(i + 1)*dy(i + 1) + &
  dx(i + 2)*dy(i + 2) + dx(i + 3)*dy(i + 3) + dx(i + 4)*dy(i + 4)
   50 continue
   60 ddot = dtemp
return
end function ddot

!********************************************************************
integer function idamax(n,dx,incx)
!********************************************************************
! finds the index of element having max. absolute value.
! jack dongarra, linpack, 3/11/78.
! modified 3/93 to return if incx .le. 0.
! modified 12/3/93, array(1) declarations changed to array(*)
double precision dx(*),dmax
integer i,incx,ix,n
idamax = 0
if( n.lt.1 .or. incx.le.0 ) return
idamax = 1
if(n.eq.1)return
if(incx.eq.1)go to 20
! code for increment not equal to 1
ix = 1
dmax = dabs(dx(1))
ix = ix + incx
do 10 i = 2,n
   if(dabs(dx(ix)).le.dmax) go to 5
   idamax = i
   dmax = dabs(dx(ix))
    5    ix = ix + incx
   10 continue
return
! code for increment equal to 1
   20 dmax = dabs(dx(1))
do 30 i = 2,n
   if(dabs(dx(i)).le.dmax) go to 30
   idamax = i
   dmax = dabs(dx(i))
   30 continue
return
end function idamax

END MODULE makols

!********************************************************************
! MODULE maknls
!********************************************************************
MODULE maknls
private
public:: minim, nrsimp, nrsimp2, nrsimp3

contains

!********************************************************************
! subroutine: nrsimp
! Use Nelder-Mead downhill simplex method to solve minimization
! problem with multidimensional inputs. The core routine is from
! 'Numerical Recipes in Fortran 77' but I added a subroutine so
! that the initial simplex is also constructed by the code.
! The arguments are:
! ndim             dimension of input variables (IN)
! inip(1:ndim)     initial values of input vector (INOUT)
! factor           size of steps to construct initial simplex (IN)
! ftol             tolerance for convergence (IN)
! funk             name of objective function: fuc(p(1:ndim)) (IN)
! iter             number of maximum iterations (IN)
! funkval          smallest value of funk
! err              error code (OUT)
!********************************************************************
SUBROUTINE nrsimp(ndim,inip,factor,ftol,funk,iter,funkval,err)
	implicit none
	integer,intent(in):: ndim,iter
	real(8),intent(in):: factor,ftol
	real(8),intent(inout),dimension(1:ndim):: inip
	external funk
	integer,intent(out):: err
	real(8),intent(out):: funkval
	real(8),dimension(1:ndim+1,1:ndim):: psimp
	real(8),dimension(1:ndim+1):: ysimp
	integer:: dim0,dim1

	!******* construct simplex *******
	psimp(1:ndim+1,1:ndim)=0.0
	do dim0=1,ndim+1
		do dim1=1,ndim
			psimp(dim0,dim1)=inip(dim1)
		end do
	end do
	do dim0=1,ndim
		psimp(dim0+1,dim0)=psimp(dim0+1,dim0)+factor*inip(dim0)
	end do

	!******* evaluate func at points of simplex *******
	do dim0=1,ndim+1
		call funk(psimp(dim0,1:ndim),ysimp(dim0))
	end do

	!******* call amoeba *******
	call amoeba(psimp,ysimp,ndim,ftol,funk,iter,err)

	!******* assign value *******
	funkval=ysimp(1)
	do dim0=1,ndim
		inip(dim0)=psimp(1,dim0)
	end do

	!******* end *******
	return

END SUBROUTINE nrsimp


!********************************************************************
! subroutine: nrsimp2
! a version of nrsimp where factor is a vector and thus can be 
! specified for each of the element of inip
! The arguments are:
! ndim             dimension of input variables (IN)
! inip(1:ndim)     initial values of input vector and the optimal
!                  values (INOUT)
! factor(1:ndim)   size of steps to construct initial simplex. You
!                  can specify for each element of inip. The size is
!                  spcified relative to the absolute value of the 
!                  corresponding element of inip (IN)
! ftol             tolerance for convergence (IN)
! funk             name of objective. should take the form:
!                  subroutine funk(p(1:ndim),fval) (IN)
! maxiter          number of maximum iterations (IN)
! funkval          smallest value of funk
! err              error code (OUT)
!********************************************************************
SUBROUTINE nrsimp2(ndim,inip,factor,ftol,funk,maxiter,funkval,err)
	implicit none
	integer,intent(in):: ndim,maxiter
	real(8),intent(in):: ftol
	real(8),dimension(1:ndim):: factor
	real(8),intent(inout),dimension(1:ndim):: inip
	external funk
	integer,intent(out):: err
	real(8),intent(out):: funkval
	real(8),dimension(1:ndim+1,1:ndim):: psimp
	real(8),dimension(1:ndim+1):: ysimp
	integer:: dim0,dim1
	integer,dimension(1):: dummy

	!******* construct simplex *******
	psimp(1:ndim+1,1:ndim)=0.0
	do dim0=1,ndim+1
		do dim1=1,ndim
			psimp(dim0,dim1)=inip(dim1)
		end do
	end do
	do dim0=1,ndim
		psimp(dim0+1,dim0)=psimp(dim0+1,dim0)+factor(dim0)*inip(dim0)
	end do

	!******* evaluate func at points of simplex *******
	do dim0=1,ndim+1
		call funk(psimp(dim0,1:ndim),ysimp(dim0))
	end do

	!******* call amoeba *******
	call amoeba(psimp,ysimp,ndim,ftol,funk,maxiter,err)

	!******* assign value *******
	dummy=minloc(ysimp(1:ndim+1))
	dim1=dummy(1)
	funkval=ysimp(dim1)
	do dim0=1,ndim
		inip(dim0)=psimp(dim1,dim0)
	end do

	!******* end *******
	return

END SUBROUTINE nrsimp2

!********************************************************************
! subroutine: nrsimp3
! a version of nrsimp where the step size can be
! specified using absolute level
! The arguments are:
! ndim             dimension of input variables (IN)
! inip(1:ndim)     initial values of input vector and the optimal
!                  values (INOUT)
! factor(1:ndim)   size of steps to construct initial simplex. You
!                  can specify for each element of inip. The size is
!                  spcified in absolute level (IN)
! ftol             tolerance for convergence (IN)
! funk             name of objective. should take the form:
!                  subroutine funk(p(1:ndim),fval) (IN)
! maxiter          number of maximum iterations (IN)
! funkval          smallest value of funk
! err              error code (OUT)
!********************************************************************
SUBROUTINE nrsimp3(ndim,inip,factor,ftol,funk,maxiter,funkval,err)
	implicit none
	integer,intent(in):: ndim,maxiter
	real(8),intent(in):: ftol
	real(8),dimension(1:ndim):: factor
	real(8),intent(inout),dimension(1:ndim):: inip
	external funk
	integer,intent(out):: err
	real(8),intent(out):: funkval
	real(8),dimension(1:ndim+1,1:ndim):: psimp
	real(8),dimension(1:ndim+1):: ysimp
	integer:: dim0,dim1
	integer,dimension(1):: dummy

	!******* construct simplex *******
	psimp(1:ndim+1,1:ndim)=0.0
	do dim0=1,ndim+1
		do dim1=1,ndim
			psimp(dim0,dim1)=inip(dim1)
		end do
	end do
	do dim0=1,ndim
		psimp(dim0+1,dim0)=psimp(dim0+1,dim0)+factor(dim0)
	end do

	!******* evaluate func at points of simplex *******
	do dim0=1,ndim+1
		call funk(psimp(dim0,1:ndim),ysimp(dim0))
	end do

	!******* call amoeba *******
	call amoeba(psimp,ysimp,ndim,ftol,funk,maxiter,err)

	!******* assign value *******
	dummy=minloc(ysimp(1:ndim+1))
	dim1=dummy(1)
	funkval=ysimp(dim1)
	do dim0=1,ndim
		inip(dim0)=psimp(dim1,dim0)
	end do

	!******* end *******
	return

END SUBROUTINE nrsimp3

!********************************************************************
! subroutine: amoeba
!********************************************************************
SUBROUTINE amoeba(p,y,ndim,ftol,funk,itmax,err)
	implicit none
	integer,intent(in):: itmax,ndim
	real(8),intent(in):: ftol
	external funk
	integer,intent(out):: err
	real(8),intent(inout),dimension(ndim+1,ndim):: p
	real(8),intent(inout),dimension(ndim+1):: y
	real(8),parameter:: TINY=1.e-10
	integer:: i,ihi,ilo,inhi,j,m,n,iter
	real(8):: rtol,amosum,swap,ysave,ytry
	real(8),dimension(ndim):: psum
	iter=0
1	do n=1,ndim
		amosum=0.
		do m=1,ndim+1
			amosum=amosum+p(m,n)
		end do
		psum(n)=amosum
	end do
2	ilo=1
	if (y(1)>y(2)) then
		ihi=1
		inhi=2
	else
		ihi=2
		inhi=1
	end if
	do i=1,ndim+1
		if (y(i)<=y(ilo)) ilo=1
		if (y(i)>y(ihi)) then
			inhi=ihi
			ihi=i
		else if (y(i)>y(inhi)) then
			if (i/=ihi) inhi=i
		end if
	end do
	rtol=2.*ABS(y(ihi)-y(ilo))/(ABS(y(ihi))+ABS(y(ilo))+TINY)
	if (rtol<ftol) then
		swap=y(1)
		y(1)=y(ilo)
		y(ilo)=swap
		do n=1,ndim
			swap=p(1,n)
			p(1,n)=p(ilo,n)
			p(ilo,n)=swap
		end do
		err=1
		return
	end if
	if (iter>=itmax) then
		err=2
		return
	end if
	iter=iter+2
	ytry=amotry(p,y,psum,ndim,funk,ihi,-1.0_8)
	if (ytry<=y(ilo)) then
		ytry=amotry(p,y,psum,ndim,funk,ihi,2.0_8)
	else if (ytry>=y(inhi)) then
		ysave=y(ihi)
		ytry=amotry(p,y,psum,ndim,funk,ihi,0.5_8)
		if (ytry>=ysave) then
			do i=1,ndim+1
				if (i/=ilo) then
					do j=1,ndim
						psum(j)=0.5*(p(i,j)+p(ilo,j))
						p(i,j)=psum(j)
					end do
					call funk(psum(1:ndim),y(i))
				end if
			end do
			iter=iter+ndim
			goto 1
		end if
	else
		iter=iter-1
	end if
	goto 2
	return
END SUBROUTINE amoeba

!********************************************************************
! function: amotry
!********************************************************************
real (8) FUNCTION amotry(p,y,psum,ndim,funk,ihi,fac)
	implicit none
	integer,intent(in):: ihi,ndim
	real(8),intent(in):: fac
	real(8),intent(inout),dimension(ndim+1,ndim):: p
	real(8),intent(inout),dimension(ndim):: psum
	real(8),intent(inout),dimension(ndim+1):: y
	external funk
	integer:: j
	real(8):: fac1,fac2,ytry
	real(8),dimension(ndim):: ptry
	fac1=(1.-fac)/real(ndim,8)
	fac2=fac1-fac
	do j=1,ndim
		ptry(j)=psum(j)*fac1-p(ihi,j)*fac2
	end do
	call funk(ptry(1:ndim),ytry)
	if (ytry<y(ihi)) then
		y(ihi)=ytry
		do j=1,ndim
			psum(j)=psum(j)-p(ihi,j)+ptry(j)
			p(ihi,j)=ptry(j)
		end do
	end if
	amotry=ytry
	return
END FUNCTION amotry

!********************************************************************
! subroutine: minim
!  A PROGRAM FOR FUNCTION MINIMIZATION USING THE SIMPLEX METHOD.
!  FOR DETAILS, SEE NELDER & MEAD, THE COMPUTER JOURNAL, JANUARY 1965
!
!  PROGRAMMED BY D.E.SHAW,
!  CSIRO, DIVISION OF MATHEMATICS & STATISTICS
!  P.O. BOX 218, LINDFIELD, N.S.W. 2070
!
!  WITH AMENDMENTS BY R.W.M.WEDDERBURN
!  ROTHAMSTED EXPERIMENTAL STATION
!  HARPENDEN, HERTFORDSHIRE, ENGLAND
!
!  Further amended by Alan Miller
!  CSIRO Division of Mathematics & Statistics
!  Private Bag 10, CLAYTON, VIC. 3169
!
!  Fortran 90 conversion by Alan Miller, June 1995
!  Latest revision - 14 September 1995
!
!  ARGUMENTS:-
!  P()    = INPUT, STARTING VALUES OF PARAMETERS
!           OUTPUT, FINAL VALUES OF PARAMETERS
!  STEP() = INPUT, INITIAL STEP SIZES
!  NOP    = INPUT, NO. OF PARAMETERS, INCL. ANY TO BE HELD FIXED
!  FUNC   = OUTPUT, THE FUNCTION VALUE CORRESPONDING TO THE FINAL
!           PARAMETER VALUES.
!  maxfn  = INPUT, THE MAXIMUM NO. OF FUNCTION EVALUATIONS ALLOWED.
!           Say, 20 times the number of parameters, NOP.
!  IPRINT = INPUT, PRINT CONTROL PARAMETER
!             < 0 NO PRINTING
!             = 0 PRINTING OF PARAMETER VALUES AND THE FUNCTION
!                 VALUE AFTER INITIAL EVIDENCE OF CONVERGENCE.
!             > 0 AS FOR IPRINT = 0 PLUS PROGRESS REPORTS AFTER
!                 EVERY IPRINT EVALUATIONS, PLUS PRINTING FOR THE
!                 INITIAL SIMPLEX.
!  STOPCR = INPUT, STOPPING CRITERION.
!           The criterion is applied to the standard deviation of
!           the values of FUNC at the points of the simplex.
!  NLOOP  = INPUT, THE STOPPING RULE IS APPLIED AFTER EVERY NLOOP
!           FUNCTION EVALUATIONS.   Normally NLOOP should be slightly
!           greater than NOP, say NLOOP = 2*NOP.
!  IQUAD  = INPUT, = 1 IF FITTING OF A QUADRATIC SURFACE IS REQUIRED
!                  = 0 IF NOT
!           N.B. The fitting of a quadratic surface is strongly
!           recommended, provided that the fitted function is
!           continuous in the vicinity of the minimum.   It is often
!           a good indicator of whether a premature termination of
!           the search has occurred.
!  SIMP   = INPUT, CRITERION FOR EXPANDING THE SIMPLEX TO OVERCOME
!           ROUNDING ERRORS BEFORE FITTING THE QUADRATIC SURFACE.
!           The simplex is expanded so that the function values at
!           the points of the simplex exceed those at the supposed
!           minimum by at least an amount SIMP.
!  VAR()  = OUTPUT, CONTAINS THE DIAGONAL ELEMENTS OF THE INVERSE OF
!            THE INFORMATION MATRIX.
!  FUNCTN = INPUT, NAME OF THE USER'S SUBROUTINE - ARGUMENTS (P,FUNC)
!           WHICH RETURNS THE FUNCTION VALUE FOR A GIVEN SET OF
!           PARAMETER VALUES IN ARRAY P.
!           N.B. FUNCTN MUST BE DECLARED EXTERNAL IN THE CALLING
!           PROGRAM.
!  IFAULT = OUTPUT, = 0 FOR SUCCESSFUL TERMINATION
!             = 1 IF MAXIMUM NO. OF FUNCTION EVALUATIONS EXCEEDED
!             = 2 IF INFORMATION MATRIX IS NOT +VE SEMI-DEFINITE
!             = 3 IF NOP < 1
!             = 4 IF NLOOP < 1
!
!  N.B. P, STEP AND VAR (IF IQUAD = 1) MUST HAVE DIMENSION AT LEAST
!       NOP IN THE CALLING PROGRAM.
!********************************************************************
SUBROUTINE minim(p, step, nop, func, maxfn, iprint, stopcr, nloop, &
iquad, simp, var, functn, ifault)

  IMPLICIT DOUBLE PRECISION (a-h,o-z)
  IMPLICIT INTEGER (i-n)

  INTEGER, INTENT(IN)             :: nop, maxfn, iprint, nloop, iquad
  INTEGER, INTENT(OUT)            :: ifault
  DOUBLE PRECISION, INTENT(IN)    :: stopcr, simp
  DOUBLE PRECISION, INTENT(INOUT) :: p(nop), step(nop)
  DOUBLE PRECISION, INTENT(OUT)   :: var(nop), func
  EXTERNAL functn

  !Local variables

  DOUBLE PRECISION:: g(nop+1,nop), h(nop+1), pbar(nop), pstar(nop),&
       pstst(nop), aval(nop), pmin(nop), temp(nop),&
       bmat(nop*(nop+1)/2), vc(nop*(nop+1)/2),&
       zero = 0.d0, half = 0.5d0, one = 1.d0, two = 2.d0

  !A = REFLECTION COEFFICIENT
  !B = CONTRACTION COEFFICIENT, AND
  !C = EXPANSION COEFFICIENT.
  DOUBLE PRECISION, PARAMETER:: a = 1.d0, b = 0.5d0, c = 2.d0

  !SET LOUT = LOGICAL UNIT NO. FOR OUTPUT
  INTEGER, PARAMETER :: lout = 6

  !IF PROGRESS REPORTS HAVE BEEN REQUESTED, PRINT HEADING
  IF (iprint.GT.0) WRITE (lout,5000) iprint

  !CHECK INPUT ARGUMENTS
  ifault = 0
  IF (nop.LE.0) ifault = 3
  IF (nloop.LE.0) ifault = 4
  IF (ifault.NE.0) RETURN

  !SET NAP = NO. OF PARAMETERS TO BE VARIED, I.E. WITH STEP.NE.0
  nap = COUNT(step.NE.zero)
  neval = 0
  loop = 0
  iflag = 0

  !IF NAP = 0 EVALUATE FUNCTION AT THE STARTING POINT AND RETURN
  IF (nap.LE.0) THEN
     CALL functn(p,func)
     RETURN
  END IF

  !SET UP THE INITIAL SIMPLEX
20 g(1,:) = p
  irow = 2
  DO i = 1, nop
     IF (step(i).NE.zero) THEN
        g(irow,:) = p
        g(irow,i) = p(i) + step(i)
        irow = irow + 1
     END IF
  END DO

  np1 = nap + 1
  DO i = 1, np1
     p = g(i,:)
     CALL functn(p,h(i))
     neval = neval + 1
     IF (iprint.GT.0) THEN
        WRITE (lout,5100) neval, h(i), p
     END IF
  END DO

  !START OF MAIN CYCLE.
  !FIND MAX. & MIN. VALUES FOR CURRENT SIMPLEX (HMAX & HMIN).
  Main_loop: DO
     loop = loop + 1
     imax = 1
     imin = 1
     hmax = h(1)
     hmin = h(1)
     DO i = 2, np1
        IF (h(i).GT.hmax) THEN
           imax = i
           hmax = h(i)
        ELSE
           IF (h(i).LT.hmin) THEN
              imin = i
              hmin = h(i)
           END IF
        END IF
     END DO

     !FIND THE CENTROID OF THE VERTICES OTHER THAN P(IMAX)
     pbar = zero
     DO i = 1, np1
        IF (i.NE.imax) THEN
           pbar = pbar + g(i,:)
        END IF
     END DO
     pbar = pbar / FLOAT(nap)

     !REFLECT MAXIMUM THROUGH PBAR TO PSTAR,
     !HSTA = FUNCTION VALUE AT PSTAR.
     pstar = a * (pbar - g(imax,:)) + pbar
     CALL functn(pstar,hsta)
     neval = neval + 1
     IF (iprint.GT.0) THEN
        IF (MOD(neval,iprint).EQ.0) &
        WRITE (lout,5100) neval, hsta, pstar
     END IF

     !IF HSTA < HMIN, REFLECT PBAR THROUGH PSTAR,
     !HSTST = FUNCTION VALUE AT PSTST.
     IF (hsta.LT.hmin) THEN
        pstst = c * (pstar - pbar) + pbar
        CALL functn(pstst,hstst)
        neval = neval + 1
        IF (iprint.GT.0) THEN
           IF (MOD(neval,iprint).EQ.0) &
           WRITE (lout,5100) neval, hstst, pstst
        END IF

        !IF HSTST < HMIN REPLACE CURRENT MAXIMUM POINT BY PSTST AND
        !HMAX BY HSTST, THEN TEST FOR CONVERGENCE.

        IF (hstst.GE.hmin) THEN
        ! REPLACE MAXIMUM POINT BY PSTAR & H(IMAX) BY HSTA.
           g(imax,:) = pstar
           h(imax) = hsta
        ELSE
           g(imax,:) = pstst
           h(imax) = hstst
        END IF
        GO TO 250
     END IF

     !HSTA IS NOT < HMIN.
     !TEST WHETHER IT IS < FUNCTION VALUE AT SOME POINT OTHER THAN
     !P(IMAX).   IF IT IS REPLACE P(IMAX) BY PSTAR & HMAX BY HSTA.

     DO i = 1, np1
        IF (i.NE.imax) THEN
           IF (hsta.LT.h(i)) THEN
           ! REPLACE MAXIMUM POINT BY PSTAR & H(IMAX) BY HSTA.
              g(imax,:) = pstar
              h(imax) = hsta
              GO TO 250
           END IF
        END IF
     END DO

     !HSTA > ALL FUNCTION VALUES EXCEPT POSSIBLY HMAX.
     !IF HSTAR <= HMAX, REPLACE P(IMAX) BY PSTAR & HMAX BY HSTA.

     IF (hsta.LE.hmax) THEN
        g(imax,:) = pstar
        hmax = hsta
        h(imax) = hsta
     END IF

     !CONTRACTED STEP TO THE POINT PSTST,
     !HSTST = FUNCTION VALUE AT PSTST.

     pstst = b * g(imax,:) + (one-b) * pbar
     CALL functn(pstst,hstst)
     neval = neval + 1
     IF (iprint.GT.0) THEN
        IF (MOD(neval,iprint).EQ.0) &
        WRITE (lout,5100) neval, hstst, pstst
     END IF

     !IF HSTST < HMAX REPLACE P(IMAX) BY PSTST & HMAX BY HSTST.

     IF (hstst.LE.hmax) THEN
        g(imax,:) = pstst
        h(imax) = hstst
        GO TO 250
     END IF

     !HSTST > HMAX.
     !SHRINK THE SIMPLEX BY REPLACING EACH POINT, OTHER THAN THE
     !CURRENT MINIMUM, BY A POINT MID-WAY BETWEEN ITS CURRENT
     !POSITION AND THEMINIMUM.

     DO i = 1, np1
        IF (i.NE.imin) THEN
           DO j = 1, nop
              IF (step(j).NE.zero) g(i,j)=(g(i,j)+g(imin,j))*half
              p(j) = g(i,j)
           END DO
           CALL functn(p,h(i))
           neval = neval + 1
           IF (iprint.GT.0) THEN
              IF (MOD(neval,iprint).EQ.0) &
              WRITE (lout,5100) neval, h(i), p
           END IF
        END IF
     END DO

     !IF LOOP = NLOOP TEST FOR CONVERGENCE
     !OTHERWISE REPEAT MAIN CYCLE.

250  IF (loop.LT.nloop) CYCLE Main_loop

     !CALCULATE MEAN & STANDARD DEVIATION OF FUNCTION VALUES FOR THE
     !CURRENT SIMPLEX.

     hmean = SUM( h(1:np1) ) / FLOAT(np1)
     hstd = SUM( (h(1:np1) - hmean) ** 2 )
     hstd = SQRT(hstd / FLOAT(np1))

     !IF THE RMS > STOPCR, SET IFLAG & LOOP TO ZERO AND GO TO THE
     !START OF THE MAIN CYCLE AGAIN.

     IF (hstd.GT.stopcr .AND. neval.LE.maxfn) THEN
        iflag = 0
        loop = 0
        CYCLE Main_loop
     END IF

     !FIND THE CENTROID OF THE CURRENT SIMPLEX
     !AND THE FUNCTION VALUE THERE.

     DO i = 1, nop
        IF (step(i).NE.zero) THEN
           p(i) = SUM( g(1:np1,i) ) / FLOAT(np1)
        END IF
     END DO
     CALL functn(p,func)
     neval = neval + 1
     IF (iprint.GT.0) THEN
        IF (MOD(neval,iprint).EQ.0) WRITE (lout,5100) neval, func, p
     END IF

     !TEST WHETHER THE NO. OF FUNCTION VALUES ALLOWED, maxfn, HAS
     !BEEN OVERRUN; IF SO, EXIT WITH IFAULT = 1.

     IF (neval.GT.maxfn) THEN
        ifault = 1
        IF (iprint.LT.0) RETURN
        WRITE (lout,5200) maxfn
        WRITE (lout,5300) hstd
        WRITE (lout,5400) p
        WRITE (lout,5500) func
        RETURN
     END IF

     !CONVERGENCE CRITERION SATISFIED.
     !IF IFLAG = 0, SET IFLAG & SAVE HMEAN.
     !IF IFLAG = 1 & CHANGE IN HMEAN<=STOPCR THEN SEARCH IS COMPLETE.

     IF (iprint.GE.0) THEN
        WRITE (lout,5600)
        WRITE (lout,5400) p
        WRITE (lout,5500) func
     END IF

     IF (iflag.EQ.0 .OR. ABS(savemn-hmean).GE.stopcr) THEN
        iflag = 1
        savemn = hmean
        loop = 0
     ELSE
        EXIT Main_loop
     END IF

  END DO Main_loop

  IF (iprint.GE.0) THEN
     WRITE (lout,5700) neval
     WRITE (lout,5800) p
     WRITE (lout,5900) func
  END IF
  IF (iquad.LE.0) RETURN

  !QUADRATIC SURFACE FITTING

  IF (iprint.GE.0) WRITE (lout,6000)

  !EXPAND THE FINAL SIMPLEX, IF NECESSARY, TO OVERCOME ROUNDING
  !ERRORS.

  hmin = func
  nmore = 0
  DO i = 1, np1
     DO
        test = ABS(h(i)-func)
        IF (test.LT.simp) THEN
           DO j = 1, nop
              IF (step(j).NE.zero) g(i,j) = (g(i,j)-p(j)) + g(i,j)
              pstst(j) = g(i,j)
           END DO
           CALL functn(pstst,h(i))
           nmore = nmore + 1
           neval = neval + 1
           IF (h(i).GE.hmin) CYCLE
           hmin = h(i)
           IF (iprint.GE.0) WRITE (lout,5100) neval, hmin, pstst
        ELSE
           EXIT
        END IF
     END DO
  END DO

  !FUNCTION VALUES ARE CALCULATED AT AN ADDITIONAL NAP POINTS.

  DO i = 1, nap
     i1 = i + 1
     pstar = (g(1,:) + g(i1,:)) * half
     CALL functn(pstar,aval(i))
     nmore = nmore + 1
     neval = neval + 1
  END DO

  !THE MATRIX OF ESTIMATED SECOND DERIVATIVES IS CALCULATED AND ITS
  !LOWER TRIANGLE STORED IN BMAT.

  a0 = h(1)
  DO i = 1, nap
     i1 = i - 1
     i2 = i + 1
     DO j = 1, i1
        j1 = j + 1
        pstst = (g(i2,:) + g(j1,:)) * half
        CALL functn(pstst,hstst)
        nmore = nmore + 1
        neval = neval + 1
        l = i * (i-1) / 2 + j
        bmat(l) = two * (hstst + a0 - aval(i) - aval(j))
     END DO
  END DO

  l = 0
  DO i = 1, nap
     i1 = i + 1
     l = l + i
     bmat(l) = two * (h(i1) + a0 - two*aval(i))
  END DO

  !THE VECTOR OF ESTIMATED FIRST DERIVATIVES IS CALCULATED AND
  !STORED IN AVAL.

  DO i = 1, nap
     i1 = i + 1
     aval(i) = two * aval(i) - (h(i1) + 3.d0*a0) * half
  END DO

  !THE MATRIX Q OF NELDER & MEAD IS CALCULATED AND STORED IN G.

  pmin = g(1,:)
  DO i = 1, nap
     i1 = i + 1
     g(i1,:) = g(i1,:) - g(1,:)
  END DO

  DO i = 1, nap
     i1 = i + 1
     g(i,:) = g(i1,:)
  END DO

  !INVERT BMAT

  CALL syminv(bmat, nap, bmat, temp, nullty, ifault, rmax)
  IF (ifault.EQ.0) THEN
     irank = nap - nullty
  ELSE
  !BMAT not +ve definite
     ! Resume search for the minimum
     IF (iprint.GE.0) WRITE (lout,6100)
     ifault = 2
     IF (neval.GT.maxfn) RETURN
     WRITE (lout,6200)
     step = half * step
     GO TO 20
  END IF

  !BMAT*A/2 IS CALCULATED AND STORED IN H.

  DO i = 1, nap
     h(i) = zero
     DO j = 1, nap
        IF (j.LE.i) THEN
           l = i * (i-1) / 2 + j
        ELSE
           l = j * (j-1) / 2 + i
        END IF
        h(i) = h(i) + bmat(l) * aval(j)
     END DO
  END DO

  !FIND THE POSITION, PMIN, & VALUE, YMIN, OF THE MINIMUM OF THE
  !QUADRATIC.

  ymin = DOT_PRODUCT( h(1:nap), aval(1:nap) )
  ymin = a0 - ymin
  DO i = 1, nop
     pstst(i) = DOT_PRODUCT( h(1:nap), g(1:nap,i) )
  END DO
  pmin = pmin - pstst
  IF (iprint.GE.0) THEN
     WRITE (lout,6300) ymin, pmin
     WRITE (lout,6400)
  END IF

  !Q*BMAT*Q'/2 IS CALCULATED & ITS LOWER TRIANGLE STORED IN VC

  DO i = 1, nop
     DO j = 1, nap
        h(j) = zero
        DO k = 1, nap
           IF (k.LE.j) THEN
              l = j * (j-1) / 2 + k
           ELSE
              l = k * (k-1) / 2 + j
           END IF
           h(j) = h(j) + bmat(l) * g(k,i) * half
        END DO
     END DO

     DO j = i, nop
        l = j * (j-1) / 2 + i
        vc(l) = DOT_PRODUCT( h(1:nap), g(1:nap,j) )
     END DO
  END DO

  !THE DIAGONAL ELEMENTS OF VC ARE COPIED INTO VAR.

  j = 0
  DO i = 1, nop
     j = j + i
     var(i) = vc(j)
  END DO
  IF (iprint.LT.0) RETURN
  WRITE (lout,6500) irank
  CALL print_tri_matrix(vc, nop, lout)

  WRITE (lout,6600)
  CALL syminv(vc, nap, bmat, temp, nullty, ifault, rmax)

  !BMAT NOW CONTAINS THE INFORMATION MATRIX

  WRITE (lout,6700)
  CALL print_tri_matrix(bmat, nop, lout)

  ii = 0
  ij = 0
  DO i = 1, nop
     ii = ii + i
     IF (vc(ii).GT.zero) THEN
        vc(ii) = one / SQRT(vc(ii))
     ELSE
        vc(ii) = zero
     END IF
     jj = 0
     DO j = 1, i - 1
        jj = jj + j
        ij = ij + 1
        vc(ij) = vc(ij) * vc(ii) * vc(jj)
     END DO
     ij = ij + 1
  END DO

  WRITE (lout,6800)
  ii = 0
  DO i = 1, nop
     ii = ii + i
     IF (vc(ii).NE.zero) vc(ii) = one
  END DO
  CALL print_tri_matrix(vc, nop, lout)

  !Exit, on successful termination.

650 WRITE (lout,6900) nmore
  RETURN

5000 FORMAT (' Progress Report every',i4,' function evaluations'/, &
       ' EVAL.   FUNC.VALUE.',10X,'PARAMETER VALUES')
5100 FORMAT (/1X,i4,2X,g12.5,2X,5G11.4,3(/21X,5G11.4))
5200 FORMAT (' No. of function evaluations > ',i5)
5300 FORMAT (' RMS of function values of last simplex =',g14.6)
5400 FORMAT (' Centroid of last simplex =',4(/1X,6G13.5))
5500 FORMAT (' Function value at centroid =',g14.6)
5600 FORMAT (/' EVIDENCE OF CONVERGENCE')
5700 FORMAT (/' Minimum found after',i5,' function evaluations')
5800 FORMAT (' Minimum at',4(/1X,6G13.6))
5900 FORMAT (' Function value at minimum =',g14.6)
6000 FORMAT (/' Fitting quadratic surface about supposed minimum'/)
6100 FORMAT (/' MATRIX OF ESTIMATED SECOND DERIVATIVES NOT +VE DEFN.'/&
               ' MINIMUM PROBABLY NOT FOUND'/)
6200 FORMAT (/10X,'Search restarting'/)
6300 FORMAT (' Minimum of quadratic surface =',g14.6,' at',4(/1X,6G13.5))
6400 FORMAT (' IF THIS DIFFERS BY MUCH FROM THE MINIMUM ESTIMATED',1X,&
          'FROM THE MINIMIZATION,'/' THE MINIMUM MAY BE FALSE &/OR THE '&
          'INFORMATION MATRIX MAY BE',1X,'INACCURATE'/)
6500 FORMAT (' Rank of information matrix =',i3/ &
          ' Inverse of information matrix:-')
6600 FORMAT (/' If the function minimized was -LOG(LIKELIHOOD),'/&
          ' this is the covariance matrix of the parameters.'/&
          ' If the function was a sum of squares of residuals,'/&
          ' this matrix must be multiplied by twice the estimated ',&
          'residual variance'/' to obtain the covariance matrix.'/)
6700 FORMAT (' INFORMATION MATRIX:-'/)
6800 FORMAT (/' CORRELATION MATRIX:-')
6900 FORMAT (/' A further',i4,' function evaluations have been used'/)

END SUBROUTINE minim

!********************************************************************
! subroutine: syminv
!  ALGORITHM AS7, APPLIED STATISTICS, VOL.17, 1968.
!  ARGUMENTS:-
!  A()    = INPUT, THE SYMMETRIC MATRIX TO BE INVERTED, STORED IN
!             LOWER TRIANGULAR FORM
!  N      = INPUT, ORDER OF THE MATRIX
!  C()    = OUTPUT, THE INVERSE OF A (A GENERALIZED INVERSE IF C IS
!           SINGULAR), ALSO STORED IN LOWER TRIANGULAR.
!             C AND A MAY OCCUPY THE SAME LOCATIONS.
!  W()    = WORKSPACE, DIMENSION AT LEAST N.
!  NULLTY = OUTPUT, THE RANK DEFICIENCY OF A.
!  IFAULT = OUTPUT, ERROR INDICATOR
!              = 1 IF N < 1
!              = 2 IF A IS NOT +VE SEMI-DEFINITE
!              = 0 OTHERWISE
!  RMAX   = OUTPUT, APPROXIMATE BOUND ON THE ACCURACY OF THE DIAGONAL
!             ELEMENTS OF C.  E.G. IF RMAX = 1.E-04 THEN THE DIAGONAL
!             ELEMENTS OF C WILL BE ACCURATE TO ABOUT 4 DEC. DIGITS.
!  LATEST REVISION - 1 April 1985
!********************************************************************
SUBROUTINE syminv(a, n, c, w, nullty, ifault, rmax)
  IMPLICIT DOUBLE PRECISION (a-h,o-z)
  IMPLICIT INTEGER (i-n)
  DIMENSION a(*), c(*), w(n)
  DOUBLE PRECISION :: zero = 0.d0, one = 1.d0

  nrow = n
  ifault = 1
  IF (nrow.GT.0) THEN
     ifault = 0

     !CHOLESKY FACTORIZATION OF A, RESULT IN C

     CALL chola(a, nrow, c, nullty, ifault, rmax, w)
     IF (ifault.EQ.0) THEN
        !INVERT C & FORM THE PRODUCT (CINV)'*CINV, WHERE CINV IS
        !THE INVERSE OF C, ROW BY ROW STARTING WITH THE LAST ROW.
        !IROW = THE ROW NUMBER, NDIAG = LOCATION OF LAST ELEMENT
        !IN THE ROW.
        nn = nrow * (nrow+1) / 2
        irow = nrow
        ndiag = nn
10      IF (c(ndiag).NE.zero) THEN
           l = ndiag
           DO i = irow, nrow
              w(i) = c(l)
              l = l + i
           END DO
           icol = nrow
           jcol = nn
           mdiag = nn

30         l = jcol
           x = zero
           IF (icol.EQ.irow) x = one / w(irow)
           k = nrow
40         IF (k.NE.irow) THEN
              x = x - w(k) * c(l)
              k = k - 1
              l = l - 1
              IF (l.GT.mdiag) l = l - k + 1
              GO TO 40
           END IF

           c(l) = x / w(irow)
           IF (icol.EQ.irow) GO TO 60
           mdiag = mdiag - icol
           icol = icol - 1
           jcol = jcol - 1
           GO TO 30
        END IF ! (c(ndiag).NE.zero)

        l = ndiag
        DO j = irow, nrow
           c(l) = zero
           l = l + j
        END DO

60      ndiag = ndiag - irow
        irow = irow - 1
        IF (irow.NE.0) GO TO 10
     END IF
  END IF
  RETURN

END SUBROUTINE syminv

!********************************************************************
! subroutine: chola
!   ALGORITHM AS6, APPLIED STATISTICS, VOL.17, 1968, WITH
!   MODIFICATIONS BY A.J.MILLER
!   ARGUMENTS:-
!   A()    = INPUT, A +VE DEFINITE MATRIX STORED IN LOWER-
!            TRIANGULAR FORM.
!   N      = INPUT, THE ORDER OF A
!   U()    = OUTPUT, A LOWER TRIANGULAR MATRIX SUCH THAT U*U' = A.
!            A & U MAY OCCUPY THE SAME LOCATIONS.
!   NULLTY = OUTPUT, THE RANK DEFICIENCY OF A.
!   IFAULT = OUTPUT, ERROR INDICATOR
!               = 1 IF N < 1
!               = 2 IF A IS NOT +VE SEMI-DEFINITE
!               = 0 OTHERWISE
!   RMAX   = OUTPUT, AN ESTIMATE OF THE RELATIVE ACCURACY OF THE
!            DIAGONAL ELEMENTS OF U.
!   R()    = OUTPUT, ARRAY CONTAINING BOUNDS ON THE RELATIVE ACCURACY
!            OF EACH DIAGONAL ELEMENT OF U.
!   LATEST REVISION - 1 April 1985
!********************************************************************
SUBROUTINE chola(a, n, u, nullty, ifault, rmax, r)
  IMPLICIT DOUBLE PRECISION (a-h,o-z)
  IMPLICIT INTEGER (i-n)
  DIMENSION a(*), u(*), r(n)

  !ETA SHOULD BE SET EQUAL TO THE SMALLEST +VE VALUE SUCH THAT
  !1.D0 + ETA IS CALCULATED AS BEING GREATER THAN 1.D0 IN THE
  !ACCURACY BEING USED.
  DOUBLE PRECISION :: eta = 1.d-16, zero = 0.d0

  ifault = 1
  IF (n.GT.0) THEN
     ifault = 2
     nullty = 0
     rmax = eta
     r(1) = eta
     j = 1
     k = 0

     !FACTORIZE COLUMN BY COLUMN, ICOL = COLUMN NO.

     DO 50 icol = 1, n
        l = 0

        !IROW = ROW NUMBER WITHIN COLUMN ICOL

        DO 30 irow = 1, icol
           k = k + 1
           w = a(k)
           IF (irow.EQ.icol) rsq = (w*eta) ** 2
           m = j
           DO 10 i = 1, irow
              l = l + 1
              IF (i.EQ.irow) EXIT
              w = w - u(l) * u(m)
              IF (irow.EQ.icol) rsq = rsq + (u(l)**2*r(i)) ** 2
              m = m + 1
10         CONTINUE

           IF (irow.EQ.icol) GO TO 40
           IF (u(l).NE.zero) THEN
              u(k) = w / u(l)
           ELSE
              u(k) = zero
              IF (ABS(w).GT.ABS(rmax*a(k))) GO TO 60
           END IF
30      CONTINUE

        !END OF ROW, ESTIMATE RELATIVE ACCURACY OF DIAGONAL ELEMENT.

40      rsq = SQRT(rsq)
        IF (ABS(w).GT.5.*rsq) THEN
           IF (w.LT.zero) RETURN
           u(k) = SQRT(w)
           r(i) = rsq / w
           IF (r(i).GT.rmax) rmax = r(i)
        ELSE
           u(k) = zero
           nullty = nullty + 1
        END IF

        j = j + icol
50   CONTINUE
     ifault = zero
  END IF
60 RETURN

END SUBROUTINE chola

!********************************************************************
! subroutine: print_tri_matrix
!********************************************************************
SUBROUTINE print_tri_matrix(a, n, lout)
  IMPLICIT NONE
  INTEGER, INTENT(IN)          :: n, lout
  DOUBLE PRECISION, INTENT(IN) :: a(*)

  !Local variables
  INTEGER  :: i, ii, i1, i2, l

  l = 1
  DO l = 1, n, 6
     ii = l * (l-1) / 2
     DO i = l, n
        i1 = ii + l
        ii = ii + i
        i2 = MIN(ii,i1+5)
        WRITE (lout,'(1X,6G13.5)') a(i1:i2)
     END DO
  END DO
  RETURN

END SUBROUTINE print_tri_matrix


END MODULE maknls

!********************************************************************
! module: macubspline
! Set of subroutines to implement cubic spline. Since natural spline
! is used (second derivatives at end points are set to zero), there
! is no need to supply any derivatives. Just x(:) and y(:) are enough
! to implement spline approximation.
! Taken from Numerical Recipes in Fortran 77, pp109-110
! and converted to F90 format.
!********************************************************************
MODULE macubspline
implicit none
private
public:: cubspline,cubsplint,cubsplintd,cubspleasy,cubspleasyd
contains

!********************************************************************
! subrouitne: cubspline
! Given arrays spx(1:nsp),spy(1:nsp) containing a tabulated function,
! i.e. spy(:)=f(spx(:)), with spx(1)<spx(2)<...<spx(nsp), this
! routine returns an array spy2(1:nsp) which contains the second
! derivatives of the interpolating function at the tabulated points
! spx(1:nsp). This routine impements a natural spline, i.e. the
! second derivatives at the bpundary points being set to zero, so
! there is no need to supply boundary conditions.
!********************************************************************
SUBROUTINE cubspline(nsp,spx,spy,spy2)
	implicit none
	integer,parameter:: spprec=8
	integer,intent(in):: nsp
	real(spprec),intent(in), dimension(nsp):: spx,spy
	real(spprec),intent(out),dimension(nsp):: spy2
	integer:: i,k
	real(spprec):: p,qn,sig,un
	real(spprec),dimension(nsp):: u

	!******* set boundary conditions *******
	spy2(1)=0.0_spprec
	u(1)=0.0_spprec

	!******* calculate spy2(:) *******
	do i=2,nsp-1
		sig=(spx(i)-spx(i-1))/(spx(i+1)-spx(i-1))
		p=sig*spy2(i-1)+2.0_spprec
		spy2(i)=(sig-1.0_spprec)/p
		u(i)=(6.0_spprec*((spy(i+1)-spy(i))/(spx(i+1)-spx(i))-&
		(spy(i)-spy(i-1))/(spx(i)-spx(i-1)))/(spx(i+1)-spx(i-1))-&
		sig*u(i-1))/p
	end do
	qn=0.0_spprec
	un=0.0_spprec
	spy2(nsp)=(un-qn*u(nsp-1))/(qn*spy2(nsp-1)+1.0_spprec)
	do k=nsp-1,1,-1
		spy2(k)=spy2(k)*spy2(k+1)+u(k)
	end do

	!******* end of subroutine *******
	return

END SUBROUTINE cubspline

!********************************************************************
! subrouitne: cubsplint
!********************************************************************
SUBROUTINE cubsplint(nsp,spx,spy,spy2,spx0,spy0)
	implicit none
	integer,parameter:: spprec=8
	integer,intent(in):: nsp
	real(spprec),intent(in), dimension(nsp):: spx,spy,spy2
	real(spprec),intent(in) :: spx0
	real(spprec),intent(out):: spy0
	integer:: k,khi,klo
	real(spprec):: spa,spb,sph

	!******* set initial khi and klo *******
	klo=1
	khi=nsp

	!******* detect khi and klo *******
	do while (khi-klo>1)
		k=(khi+klo)/2
		if (spx(k)>spx0) then
			khi=k
		else
			klo=k
		end if
	end do

	!******* calculate spy0=f(spx0) *******
	sph=spx(khi)-spx(klo)
	spa=(spx(khi)-spx0)/sph
	spb=(spx0-spx(klo))/sph
	spy0=spa*spy(klo)+spb*spy(khi)+((spa**3.0_spprec-spa)*spy(klo)+&
	(spb**3.0_spprec-spb)*spy(khi))*(sph**2.0_spprec)/6.0_spprec

	!******* end of subroutine *******
	return

END SUBROUTINE cubsplint

!********************************************************************
! subrouitne: cubsplintd
! Cubsplint plus output spd0 (derivative at spx0)
!********************************************************************
SUBROUTINE cubsplintd(nsp,spx,spy,spy2,spx0,spy0,spd0)
	implicit none
	integer,parameter:: spprec=8
	integer,intent(in):: nsp
	real(spprec),intent(in), dimension(nsp):: spx,spy,spy2
	real(spprec),intent(in) :: spx0
	real(spprec),intent(out):: spy0,spd0
	integer:: k,khi,klo
	real(spprec):: spa,spb,sph

	!******* set initial khi and klo *******
	klo=1
	khi=nsp

	!******* detect khi and klo *******
	do while (khi-klo>1)
		k=(khi+klo)/2
		if (spx(k)>spx0) then
			khi=k
		else
			klo=k
		end if
	end do

	!******* calculate spy0=f(spx0) *******
	sph=spx(khi)-spx(klo)
	spa=(spx(khi)-spx0)/sph
	spb=(spx0-spx(klo))/sph
	spy0=spa*spy(klo)+spb*spy(khi)+((spa**3.0_spprec-spa)*spy(klo)+&
	(spb**3.0_spprec-spb)*spy(khi))*(sph**2.0_spprec)/6.0_spprec

	!******* calculate spd0=df/fx(spx0) *******
	spd0=-spy(klo)/sph+spy(khi)/sph-spy(klo)/2.0_spprec/sph*&
	(spx(khi)-spx0)**2.0_spprec+spy(klo)*sph/6.0_spprec+spy(khi)/&
	2.0_spprec/sph*(spx0-spx(klo))**2.0_spprec-spy(khi)*sph/6.0_spprec

	!******* end of subroutine *******
	return

END SUBROUTINE cubsplintd

!********************************************************************
! subrouitne: cubspleasy
! Implement both cubspline and cubsplint simultaneously
!********************************************************************
SUBROUTINE cubspleasy(nsp,spx,spy,spx0,spy0)
	implicit none
	integer,parameter:: spprec=8
	integer,intent(in):: nsp
	real(spprec),intent(in), dimension(nsp):: spx,spy
	real(spprec),intent(in) :: spx0
	real(spprec),intent(out):: spy0
	real(spprec),dimension(nsp):: spy2
	call cubspline(nsp,spx,spy,spy2)
	call cubsplint(nsp,spx,spy,spy2,spx0,spy0)
	return
END SUBROUTINE cubspleasy

!********************************************************************
! subrouitne: cubspleasyd
! Implement both cubspline and cubsplintd simultaneously
!********************************************************************
SUBROUTINE cubspleasyd(nsp,spx,spy,spx0,spy0,spd0)
	implicit none
	integer,parameter:: spprec=8
	integer,intent(in):: nsp
	real(spprec),intent(in), dimension(nsp):: spx,spy
	real(spprec),intent(in) :: spx0
	real(spprec),intent(out):: spy0,spd0
	real(spprec),dimension(nsp):: spy2
	call cubspline(nsp,spx,spy,spy2)
	call cubsplintd(nsp,spx,spy,spy2,spx0,spy0,spd0)
	return
END SUBROUTINE cubspleasyd

END MODULE macubspline


!* ======================================================================
!* NIST Guide to Available Math Software.
!* Fullsource for module DNSQ from package SLATEC.
!* Retrieved from CAMSUN on Mon Jul  7 07:54:03 1997.
!* ======================================================================
!*DECK DNSQ
module dnsqrout

public:: dnsq

contains 
 
      SUBROUTINE DNSQ (FCN, JAC, IOPT, N, X, FVEC, FJAC, LDFJAC, XTOL,  &
        MAXFEV, ML, MU, EPSFCN, DIAG, MODE, FACTOR, NPRINT, INFO, NFEV,	&
        NJEV, R, LR, QTF, WA1, WA2, WA3, WA4)
!!!C***BEGIN PROLOGUE  DNSQ
!!!C***PURPOSE  Find a zero of a system of a N nonlinear fun!Ctions in N
!!!C            variables by a modification of the Powell hybrid method.
!!!C***LIBRARY   SLATE!!!C
!!!C***!CATEGORY  F2A
!!!C***TYPE      DOUBLE PRECISION (SNSQ-S, DNSQ-D)
!!C***KEYWORDS  NONLINEAR SQUARE SYSTEM, POWELL HYBRID METHOD, ZEROS
!!!C***AUTHOR  Hiebert, K. L. (SNLA)
!!!C***DESCRIPTION
!!!C
!!!C 1. Purpose.
!!!C
!!!C       The purpose of DNSQ is to find a zero of a system of N nonlinear
!!!C       functions in N variables by a modification of the Powell
!!!C       hybrid method.  The user must provide a subroutine which
!!!C       calculates the functions.  The user has the option of either to
!!!C       provide a subroutine which calculates the Jacobian or to let the
!!!C       code calculate it by a forward-difference approximation.
!!!C       This code is the combination of the MINPACK codes (Argonne)
!!!C       HYBRD and HYBRDJ.
!!!C
!!!C 2. Subroutine and Type Statements.
!!!C
!!!C       SUBROUTINE DNSQ(FCN,JAC,IOPT,N,X,FVEC,FJAC,LDFJAC,XTOL,MAXFEV,
!!!C      *                 ML,MU,EPSFCN,DIAG,MODE,FACTOR,NPRINT,INFO,NFEV,
!!!C      *                 NJEV,R,LR,QTF,WA1,WA2,WA3,WA4)
!!!C       INTEGER IOPT,N,MAXFEV,ML,MU,MODE,NPRINT,INFO,NFEV,LDFJAC,NJEV,LR
!!!C       DOUBLE PRECISION XTOL,EPSFCN,FACTOR
!!!C       DOUBLE PRECISION
!!!C       X(N),FVEC(N),DIAG(N),FJAC(LDFJAC,N),R(LR),QTF(N),
!!!C      *     WA1(N),WA2(N),WA3(N),WA4(N)
!!!C       EXTERNAL FCN,JAC
!!!C
!!!C 3. Parameters.
!!!C
!!!C       Parameters designated as input parameters must be specified on
!!!C       entry to DNSQ and are not changed on exit, while parameters
!!!C       designated as output parameters need not be specified on entry
!!!C       and are set to appropriate values on exit from DNSQ.
!!!C
!!!C       FCN is the name of the user-supplied subroutine which calculates
!!!C         the functions.  FCN must be declared in an EXTERNAL statement
!!!C         in the user calling program, and should be written as follows.
!!!C
!!!C         SUBROUTINE FCN(N,X,FVEC,IFLAG)
!!!C         INTEGER N,IFLAG
!!!C         DOUBLE PRECISION X(N),FVEC(N)
!!!C         ----------
!!!C         CALCULATE THE FUNCTIONS AT X AND
!!!C         RETURN THIS VECTOR IN FVEC.
!!!C         ----------
!!!C         RETURN
!!!C         END
!!!C
!!!C         The value of IFLAG should not be changed by FCN unless the
!!!C         user wants to terminate execution of DNSQ.  In this case set
!!!C         IFLAG to a negative integer.
!!!C
!!!C       JAC is the name of the user-supplied subroutine which calculates
!!!C         the Jacobian.  If IOPT=1, then JAC must be declared in an
!!!C         EXTERNAL statement in the user calling program, and should be
!!!C         written as follows.
!!!C
!!!C         SUBROUTINE JAC(N,X,FVEC,FJAC,LDFJAC,IFLAG)
!!!C         INTEGER N,LDFJAC,IFLAG
!!!C         DOUBLE PRECISION X(N),FVEC(N),FJAC(LDFJAC,N)
!!!C         ----------
!!!C         Calculate the Jacobian at X and return this
!!!C         matrix in FJAC.  FVEC contains the function
!!!C         values at X and should not be altered.
!!!C         ----------
!!!C         RETURN
!!!C         END
!!!C
!!!C         The value of IFLAG should not be changed by JAC unless the
!!!C         user wants to terminate execution of DNSQ.  In this case set
!!!C         IFLAG to a negative integer.
!!!C
!!!C         If IOPT=2, JAC can be ignored (treat it as a dummy argument).
!!!C
!!!C       IOPT is an input variable which specifies how the Jacobian will
!!!C         be calculated.  If IOPT=1, then the user must supply the
!!!C         Jacobian through the subroutine JAC.  If IOPT=2, then the
!!!C         code will approximate the Jacobian by forward-differencing.
!!!C
!!!C       N is a positive integer input variable set to the number of
!!!C         functions and variables.
!!!C
!!!C       X is an array of length N.  On input X must contain an initial
!!!C         estimate of the solution vector.  On output X contains the
!!!C         final estimate of the solution vector.
!!!C
!!!C       FVEC is an output array of length N which contains the functions
!!!C         evaluated at the output X.
!!!C
!!!C       FJAC is an output N by N array which contains the orthogonal
!!!C         matrix Q produced by the QR factorization of the final
!!!C         approximate Jacobian.
!!!C
!!!C       LDFJAC is a positive integer input variable not less than N
!!!C         which specifies the leading dimension of the array FJAC.
!!!C
!!!C       XTOL is a nonnegative input variable.  Termination occurs when
!!!C         the relative error between two consecutive iterates is at most
!!!C         XTOL.  Therefore, XTOL measures the relative error desired in
!!!C         the approximate solution.  Section 4 contains more details
!!!C         about XTOL.
!!!C
!!!C       MAXFEV is a positive integer input variable.  Termination occurs
!!!C         when the number of calls to FCN is at least MAXFEV by the end
!!!C         of an iteration.
!!!C
!!!C       ML is a nonnegative integer input variable which specifies the
!!!C         number of subdiagonals within the band of the Jacobian matrix.
!!!C         If the Jacobian is not banded or IOPT=1, set ML to at
!!!C         least N - 1.
!!!C
!!!C       MU is a nonnegative integer input variable which specifies the
!!!C         number of superdiagonals within the band of the Jacobian
!!!C         matrix.  If the Jacobian is not banded or IOPT=1, set MU to at
!!!C         least N - 1.
!!!C
!!!C       EPSFCN is an input variable used in determining a suitable step
!!!C         for the forward-difference approximation.  This approximation
!!!C         assumes that the relative errors in the functions are of the
!!!C         order of EPSFCN.  If EPSFCN is less than the machine
!!!C         precision, it is assumed that the relative errors in the
!!!C         functions are of the order of the machine precision.  If
!!!C         IOPT=1, then EPSFCN can be ignored (treat it as a dummy
!!!C         argument).
!!!C
!!!C       DIAG is an array of length N.  If MODE = 1 (see below), DIAG is
!!C         internally set.  If MODE = 2, DIAG must contain positive
!!C         entries that serve as implicit (multiplicative) scale factors
!!C         for the variables.
!!C
!!C       MODE is an integer input variable.  If MODE = 1, the variables
!!C         will be scaled internally.  If MODE = 2, the scaling is
!!C         specified by the input DIAG.  Other values of MODE are
!!C         equivalent to MODE = 1.
!!C
!!C       FACTOR is a positive input variable used in determining the
!!C         initial step bound.  This bound is set to the product of
!!C         FACTOR and the Euclidean norm of DIAG*X if nonzero, or else to
!!C         FACTOR itself.  In most cases FACTOR should lie in the
!!C         interval (.1,100.).  100. is a generally recommended value.
!!C
!!C       NPRINT is an integer input variable that enables controlled
!!C         printing of iterates if it is positive.  In this case, FCN is
!!C         called with IFLAG = 0 at the beginning of the first iteration
!!C         and every NPRINT iterations thereafter and immediately prior
!!C         to return, with X and FVEC available for printing. appropriate
!!C         print statements must be added to FCN(see example).  If NPRINT
!!C         is not positive, no special calls of FCN with IFLAG = 0 are
!!C         made.
!!C
!!C       INFO is an integer output variable.  If the user has terminated
!!C         execution, INFO is set to the (negative) value of IFLAG.  See
!!C         description of FCN and JAC. Otherwise, INFO is set as follows.
!!C
!!C         INFO = 0  Improper input parameters.
!!C
!!C         INFO = 1  Relative error between two consecutive iterates is
!!C                   at most XTOL.
!!C
!!C         INFO = 2  Number of calls to FCN has reached or exceeded
!!C                   MAXFEV.
!!C
!!C         INFO = 3  XTOL is too small.  No further improvement in the
!!C                   approximate solution X is possible.
!!C
!!C         INFO = 4  Iteration is not making good progress, as measured
!!C                   by the improvement from the last five Jacobian
!!C                   evaluations.
!!C
!!C         INFO = 5  Iteration is not making good progress, as measured
!!C                   by the improvement from the last ten iterations.
!!C
!!C         Sections 4 and 5 contain more details about INFO.
!!C
!!C       NFEV is an integer output variable set to the number of calls to
!!C         FCN.
!!C
!!C       NJEV is an integer output variable set to the number of calls to
!!C         JAC. (If IOPT=2, then NJEV is set to zero.)
!!C
!!C       R is an output array of length LR which contains the upper
!!C         triangular matrix produced by the QR factorization of the
!!C         final approximate Jacobian, stored rowwise.
!!C
!!C       LR is a positive integer input variable not less than
!!C         (N*(N+1))/2.
!!C
!!C       QTF is an output array of length N which contains the vector
!!C         (Q transpose)*FVEC.
!!C
!!C       WA1, WA2, WA3, and WA4 are work arrays of length N.
!!C
!!C
!!C 4. Successful completion.
!!C
!!C       The accuracy of DNSQ is controlled by the convergence parameter
!!C       XTOL.  This parameter is used in a test which makes a comparison
!!C       between the approximation X and a solution XSOL.  DNSQ
!!C       terminates when the test is satisfied.  If the convergence
!!C       parameter is less than the machine precision (as defined by the
!!C       function D1MACH(4)), then DNSQ only attempts to satisfy the test
!!C       defined by the machine precision.  Further progress is not
!!C       usually possible.
!!C
!!C       The test assumes that the functions are reasonably well behaved,
!!C       and, if the Jacobian is supplied by the user, that the functions
!!C       and the Jacobian are coded consistently.  If these conditions
!!C       are not satisfied, then DNSQ may incorrectly indicate
!!C       convergence.  The coding of the Jacobian can be checked by the
!!C       subroutine DCKDER. If the Jacobian is coded correctly or IOPT=2,
!!C       then the validity of the answer can be checked, for example, by
!!C       rerunning DNSQ with a tighter tolerance.
!!C
!!C       Convergence Test.  If DENORM(Z) denotes the Euclidean norm of a
!!C         vector Z and D is the diagonal matrix whose entries are
!!C         defined by the array DIAG, then this test attempts to
!!C         guarantee that
!!C
!!C               DENORM(D*(X-XSOL)) .LE. XTOL*DENORM(D*XSOL).
!!C
!!C         If this condition is satisfied with XTOL = 10**(-K), then the
!!C         larger components of D*X have K significant decimal digits and
!!C         INFO is set to 1.  There is a danger that the smaller
!!C         components of D*X may have large relative errors, but the fast
!!C         rate of convergence of DNSQ usually avoids this possibility.
!!C         Unless high precision solutions are required, the recommended
!!C         value for XTOL is the square root of the machine precision.
!!C
!!C
!!C 5. Unsuccessful Completion.
!!C
!!C       Unsuccessful termination of DNSQ can be due to improper input
!!C       parameters, arithmetic interrupts, an excessive number of
!!C       function evaluations, or lack of good progress.
!!C
!!C       Improper Input Parameters.  INFO is set to 0 if IOPT .LT .1,
!!C         or IOPT .GT. 2, or N .LE. 0, or LDFJAC .LT. N, or
!!C         XTOL .LT. 0.E0, or MAXFEV .LE. 0, or ML .LT. 0, or MU .LT. 0,
!!C         or FACTOR .LE. 0.E0, or LR .LT. (N*(N+1))/2.
!!C
!!C       Arithmetic Interrupts.  If these interrupts occur in the FCN
!!C         subroutine during an early stage of the computation, they may
!!C         be caused by an unacceptable choice of X by DNSQ.  In this
!!C         case, it may be possible to remedy the situation by rerunning
!!C         DNSQ with a smaller value of FACTOR.
!!C
!!C       Excessive Number of Function Evaluations.  A reasonable value
!!C         for MAXFEV is 100*(N+1) for IOPT=1 and 200*(N+1) for IOPT=2.
!!C         If the number of calls to FCN reaches MAXFEV, then this
!!C         indicates that the routine is converging very slowly as
!!C         measured by the progress of FVEC, and INFO is set to 2. This
!!C         situation should be unusual because, as indicated below, lack
!!C         of good progress is usually diagnosed earlier by DNSQ,
!!C         causing termination with info = 4 or INFO = 5.
!!C
!!C       Lack of Good Progress.  DNSQ searches for a zero of the system
!!C         by minimizing the sum of the squares of the functions.  In so
!!C         doing, it can become trapped in a region where the minimum
!!C         does not correspond to a zero of the system and, in this
!!C         situation, the iteration eventually fails to make good
!!C         progress.  In particular, this will happen if the system does
!!C         not have a zero.  If the system has a zero, rerunning DNSQ
!!C         from a different starting point may be helpful.
!!C
!!C
!!C 6. Characteristics of The Algorithm.
!!C
!!C       DNSQ is a modification of the Powell Hybrid method.  Two of its
!!C       main characteristics involve the choice of the correction as a
!!C       convex combination of the Newton and scaled gradient directions,
!!C       and the updating of the Jacobian by the rank-1 method of
!!C       Broyden.  The choice of the correction guarantees (under
!!C       reasonable conditions) global convergence for starting points
!!C       far from the solution and a fast rate of convergence.  The
!!C       Jacobian is calculated at the starting point by either the
!!C       user-supplied subroutine or a forward-difference approximation,
!!C       but it is not recalculated until the rank-1 method fails to
!!C       produce satisfactory progress.
!!C
!!C       Timing.  The time required by DNSQ to solve a given problem
!!C         depends on N, the behavior of the functions, the accuracy
!!C         requested, and the starting point.  The number of arithmetic
!!C         operations needed by DNSQ is about 11.5*(N**2) to process
!!C         each evaluation of the functions (call to FCN) and 1.3*(N**3)
!!C         to process each evaluation of the Jacobian (call to JAC,
!!C         if IOPT = 1).  Unless FCN and JAC can be evaluated quickly,
!!C         the timing of DNSQ will be strongly influenced by the time
!!C         spent in FCN and JAC.
!!C
!!C       Storage.  DNSQ requires (3*N**2 + 17*N)/2 single precision
!!C         storage locations, in addition to the storage required by the
!!C         program.  There are no internally declared storage arrays.
!!C
!!C *Long Description:
!!C
!!C 7. Example.
!!C
!!C       The problem is to determine the values of X(1), X(2), ..., X(9),
!!C       which solve the system of tridiagonal equations
!!C
!!C       (3-2*X(1))*X(1)           -2*X(2)                   = -1
!!C               -X(I-1) + (3-2*X(I))*X(I)         -2*X(I+1) = -1, I=2-8
!!C                                   -X(8) + (3-2*X(9))*X(9) = -1
!!C !!C     **********
!!C
!!C       PROGRAM TEST
!!C !!C
!!C !!C     Driver for DNSQ example.
!!C !!C
!!C       INTEGER J,IOPT,N,MAXFEV,ML,MU,MODE,NPRINT,INFO,NFEV,LDFJAC,LR,
!!C      *        NWRITE
!!C       DOUBLE PRECISION XTOL,EPSFCN,FACTOR,FNORM
!!C       DOUBLE PRECISION X(9),FVEC(9),DIAG(9),FJAC(9,9),R(45),QTF(9),
!!C      *     WA1(9),WA2(9),WA3(9),WA4(9)
!!C       DOUBLE PRECISION DENORM,D1MACH
!!C       EXTERNAL FCN
!!C       DATA NWRITE /6/
!!C !!C
!!C       IOPT = 2
!!C       N = 9
!!C !!C
!!C !!C     THE FOLLOWING STARTING VALUES PROVIDE A ROUGH SOLUTION.
!!C !!C
!!C       DO 10 J = 1, 9
!!C          X(J) = -1.E0
!!C    10    CONTINUE
!!C !!C
!!C       LDFJAC = 9
!!C       LR = 45
!!C !!C
!!C !!C     SET XTOL TO THE SQUARE ROOT OF THE MACHINE PRECISION.
!!C !!C     UNLESS HIGH PRECISION SOLUTIONS ARE REQUIRED,
!!C !!C     THIS IS THE RECOMMENDED SETTING.
!!C !!C
!!C       XTOL = SQRT(D1MACH(4))
!!C !!C
!!C       MAXFEV = 2000
!!C       ML = 1
!!C       MU = 1
!!C       EPSFCN = 0.E0
!!C       MODE = 2
!!C       DO 20 J = 1, 9
!!C          DIAG(J) = 1.E0
!!C    20    CONTINUE
!!C       FACTOR = 1.E2
!!C       NPRINT = 0
!!C !!C
!!C       CALL DNSQ(FCN,JAC,IOPT,N,X,FVEC,FJAC,LDFJAC,XTOL,MAXFEV,ML,MU,
!!C      *           EPSFCN,DIAG,MODE,FACTOR,NPRINT,INFO,NFEV,NJEV,
!!C      *           R,LR,QTF,WA1,WA2,WA3,WA4)
!!C       FNORM = DENORM(N,FVEC)
!!C       WRITE (NWRITE,1000) FNORM,NFEV,INFO,(X(J),J=1,N)
!!C       STOP
!!C  1000 FORMAT (5X,' FINAL L2 NORM OF THE RESIDUALS',E15.7 //
!!C      *        5X,' NUMBER OF FUNCTION EVALUATIONS',I10 //
!!C      *        5X,' EXIT PARAMETER',16X,I10 //
!!C      *        5X,' FINAL APPROXIMATE SOLUTION' // (5X,3E15.7))
!!C       END
!!C       SUBROUTINE FCN(N,X,FVEC,IFLAG)
!!C       INTEGER N,IFLAG
!!C       DOUBLE PRECISION X(N),FVEC(N)
!!C       INTEGER K
!!C       DOUBLE PRECISION ONE,TEMP,TEMP1,TEMP2,THREE,TWO,ZERO
!!C       DATA ZERO,ONE,TWO,THREE /0.E0,1.E0,2.E0,3.E0/
!!C !!C
!!C       IF ( .NE. 0) GO TO 5
!!C !!C
!!C !!C     INSERT PRINT STATEMENTS HERE WHEN NPRINT IS POSITIVE.
!!C !!C
!!C       RETURN
!!C     5 CONTINUE
!!C       DO 10 K = 1, N
!!C          TEMP = (THREE - TWO*X(K))*X(K)
!!C          TEMP1 = ZERO
!!C          IF (K .NE. 1) TEMP1 = X(K-1)
!!C          TEMP2 = ZERO
!!C          IF (K .NE. N) TEMP2 = X(K+1)
!!C          FVEC(K) = TEMP - TEMP1 - TWO*TEMP2 + ONE
!!C    10    CONTINUE
!!C       RETURN
!!C       END
!!C
!!C       Results obtained with different compilers or machines
!!C       may be slightly different.
!!C
!!C       Final L2 norm of the residuals  0.1192636E-07
!!C
!!C       Number of function evaluations        14
!!C
!!C       Exit parameter                         1
!!C
!!C       Final approximate solution
!!C
!!C       -0.5706545E+00 -0.6816283E+00 -0.7017325E+00
!!C       -0.7042129E+00 -0.7013690E+00 -0.6918656E+00
!!C       -0.6657920E+00 -0.5960342E+00 -0.4164121E+00
!!C
!!C***REFERENCES  M. J. D. Powell, A hybrid method for nonlinear equa-
!!C                 tions. In Numerical Methods for Nonlinear Algebraic
!!C                 Equations, P. Rabinowitz, Editor.  Gordon and Breach,
!!C                 1988.
!!C***ROUTINES CALLED  D1MACH, D1MPYQ, D1UPDT, DDOGLG, DENORM, DFDJC1,
!!C                    DQFORM, DQRFAC, XERMSG
!!C***REVISION HISTORY  (YYMMDD)
!!C   800301  DATE WRITTEN
!!C   890531  Changed all specific intrinsics to generic.  (WRB)
!!C   890831  Modified array declarations.  (WRB)
!!C   890831  REVISION DATE from Version 3.2
!!C   891214  Prologue converted to Version 4.0 format.  (BAB)
!!C   900315  CALLs to XERROR changed to CALLs to XERMSG.  (THJ)
!!C   920501  Reformatted the REFERENCES section.  (WRB)
!!C***END PROLOGUE  DNSQ
   
	    !DOUBLE PRECISION ddd1mach,DENORM
			!real*8 ddd1mach,DENORM
      INTEGER I, IFLAG, INFO, IOPT, ITER, IWA(1), J, JM1, L, LDFJAC, &
          LR, MAXFEV, ML, MODE, MU, N, NCFAIL, NCSUC, NFEV, NJEV,	 &
          NPRINT, NSLOW1, NSLOW2
      DOUBLE PRECISION ACTRED, DELTA, DIAG(*), EPSFCN, EPSMCH, FACTOR, &
          FJAC(LDFJAC,*), FNORM, FNORM1, FVEC(*), ONE, P0001, P001,		 &
          P1, P5, PNORM, PRERED, QTF(*), R(*), RATIO, SUM, TEMP,			 &
          WA1(*), WA2(*), WA3(*), WA4(*), X(*), XNORM, XTOL, ZERO
      EXTERNAL FCN !,JAC
      LOGICAL JEVAL,SING
      SAVE ONE, P1, P5, P001, P0001, ZERO
      DATA ONE,P1,P5,P001,P0001,ZERO																	 &
          /1.0D0,1.0D-1,5.0D-1,1.0D-3,1.0D-4,0.0D0/
!!C
!!C     BEGIN BLOCK PERMITTING ...EXITS TO 320
!!C***FIRST EXECUTABLE STATEMENT  DNSQ
         EPSMCH = ddd1mach(4)
!!C
         INFO = 0
         IFLAG = 0
         NFEV = 0
         NJEV = 0
!!C
!!C        CHECK THE INPUT PARAMETERS FOR ERRORS.
!!C
!!C     ...EXIT
         IF (IOPT .LT. 1 .OR. IOPT .GT. 2 .OR. N .LE. 0								&
            .OR. XTOL .LT. ZERO .OR. MAXFEV .LE. 0 .OR. ML .LT. 0		&
            .OR. MU .LT. 0 .OR. FACTOR .LE. ZERO .OR. LDFJAC .LT. N	&
            .OR. LR .LT. (N*(N + 1))/2) GO TO 320
         IF (MODE .NE. 2) GO TO 20
            DO 10 J = 1, N
!!C     .........EXIT
               IF (DIAG(J) .LE. ZERO) GO TO 320
   10       CONTINUE
   20    CONTINUE
!!C
!!C        EVALUATE THE FUNCTION AT THE STARTING POINT
!!C        AND CALCULATE ITS NORM.
!!C
         IFLAG = 1
         CALL FCN(N,X,FVEC,IFLAG)
         NFEV = 1
!!C     ...EXIT
         IF (IFLAG .LT. 0) GO TO 320
         FNORM = DENORM(N,FVEC)
!!C
!!C        INITIALIZE ITERATION COUNTER AND MONITORS.
!!C
         ITER = 1
         NCSUC = 0
         NCFAIL = 0
         NSLOW1 = 0
         NSLOW2 = 0
!!C
!!C        BEGINNING OF THE OUTER LOOP.
!!C
   30    CONTINUE
!!C           BEGIN BLOCK PERMITTING ...EXITS TO 90
               JEVAL = .TRUE.
!!C
!!C              CALCULATE THE JACOBIAN MATRIX.
!!C
!!C
               IF (IOPT .EQ. 2) GO TO 40
!!C
!!C                 USER SUPPLIES JACOBIAN
!!C
                  CALL JAC(N,X,FVEC,FJAC,LDFJAC,IFLAG)
                  NJEV = NJEV + 1
               GO TO 50
   40          CONTINUE
!!C
!!C                 CODE APPROXIMATES THE JACOBIAN
!!C
                  IFLAG = 2
                  CALL DFDJC1(FCN,N,X,FVEC,FJAC,LDFJAC,IFLAG,ML,MU,  &
                             EPSFCN,WA1,WA2)
                  NFEV = NFEV + MIN(ML+MU+1,N)
   50          CONTINUE
!!C
!!C     .........EXIT
               IF (IFLAG .LT. 0) GO TO 320
!!C
!!C              COMPUTE THE QR FACTORIZATION OF THE JACOBIAN.
!!C
               CALL DQRFAC(N,N,FJAC,LDFJAC,.FALSE.,IWA,1,WA1,WA2,WA3)
!!C
!!C              ON THE FIRST ITERATION AND IF MODE IS 1, SCALE ACCORDING
!!C              TO THE NORMS OF THE COLUMNS OF THE INITIAL JACOBIAN.
!!C
!!C           ...EXIT
               IF (ITER .NE. 1) GO TO 90
               IF (MODE .EQ. 2) GO TO 70
                  DO 60 J = 1, N
                     DIAG(J) = WA2(J)
                     IF (WA2(J) .EQ. ZERO) DIAG(J) = ONE
   60             CONTINUE
   70          CONTINUE
!!C
!!C              ON THE FIRST ITERATION, CALCULATE THE NORM OF THE SCALED
!!C              X AND INITIALIZE THE STEP BOUND DELTA.
!!C
               DO 80 J = 1, N
                  WA3(J) = DIAG(J)*X(J)
   80          CONTINUE
               XNORM = DENORM(N,WA3)
               DELTA = FACTOR*XNORM
               IF (DELTA .EQ. ZERO) DELTA = FACTOR
   90       CONTINUE
!!C
!!C           FORM (Q TRANSPOSE)*FVEC AND STORE IN QTF.
!!C
            DO 100 I = 1, N
               QTF(I) = FVEC(I)
  100       CONTINUE
            DO 140 J = 1, N
               IF (FJAC(J,J) .EQ. ZERO) GO TO 130
                  SUM = ZERO
                  DO 110 I = J, N
                     SUM = SUM + FJAC(I,J)*QTF(I)
  110             CONTINUE
                  TEMP = -SUM/FJAC(J,J)
                  DO 120 I = J, N
                     QTF(I) = QTF(I) + FJAC(I,J)*TEMP
  120             CONTINUE
  130          CONTINUE
  140       CONTINUE
!!C
!!C           COPY THE TRIANGULAR FACTOR OF THE QR FACTORIZATION INTO R.
!!C
            SING = .FALSE.
            DO 170 J = 1, N
               L = J
               JM1 = J - 1
               IF (JM1 .LT. 1) GO TO 160
               DO 150 I = 1, JM1
                  R(L) = FJAC(I,J)
                  L = L + N - I
  150          CONTINUE
  160          CONTINUE
               R(L) = WA1(J)
               IF (WA1(J) .EQ. ZERO) SING = .TRUE.
  170       CONTINUE
!!C
!!C           ACCUMULATE THE ORTHOGONAL FACTOR IN FJAC.
!!C
            CALL DQFORM(N,N,FJAC,LDFJAC,WA1)
!!C
!!C           RESCALE IF NECESSARY.
!!C
            IF (MODE .EQ. 2) GO TO 190
               DO 180 J = 1, N
                  DIAG(J) = MAX(DIAG(J),WA2(J))
  180          CONTINUE
  190       CONTINUE
!!C
!!C           BEGINNING OF THE INNER LOOP.
!!C
  200       CONTINUE
!!C
!!C              IF REQUESTED, CALL FCN TO ENABLE PRINTING OF ITERATES.
!!C
               IF (NPRINT .LE. 0) GO TO 210
                  IFLAG = 0
                  IF (MOD(ITER-1,NPRINT) .EQ. 0)    &
                    CALL FCN(N,X,FVEC,IFLAG)
!!C     ............EXIT
                  IF (IFLAG .LT. 0) GO TO 320
  210          CONTINUE
!!C
!!C              DETERMINE THE DIRECTION P.
!!C
               CALL DDOGLG(N,R,LR,DIAG,QTF,DELTA,WA1,WA2,WA3)
!!C
!!C              STORE THE DIRECTION P AND X + P. CALCULATE THE NORM OF P.
!!C
               DO 220 J = 1, N
                  WA1(J) = -WA1(J)
                  WA2(J) = X(J) + WA1(J)
                  WA3(J) = DIAG(J)*WA1(J)
  220          CONTINUE
               PNORM = DENORM(N,WA3)
!!C
!!C              ON THE FIRST ITERATION, ADJUST THE INITIAL STEP BOUND.
!!C
               IF (ITER .EQ. 1) DELTA = MIN(DELTA,PNORM)
!!C
!!C              EVALUATE THE FUNCTION AT X + P AND CALCULATE ITS NORM.
!!C
               IFLAG = 1
               CALL FCN(N,WA2,WA4,IFLAG)
               NFEV = NFEV + 1
!!C     .........EXIT
               IF (IFLAG .LT. 0) GO TO 320
               FNORM1 = DENORM(N,WA4)
!!C
!!C              COMPUTE THE SCALED ACTUAL REDUCTION.
!!C
               ACTRED = -ONE
               IF (FNORM1 .LT. FNORM) ACTRED = ONE - (FNORM1/FNORM)**2
!!C
!!C              COMPUTE THE SCALED PREDICTED REDUCTION.
!!C
               L = 1
               DO 240 I = 1, N
                  SUM = ZERO
                  DO 230 J = I, N
                     SUM = SUM + R(L)*WA1(J)
                     L = L + 1
  230             CONTINUE
                  WA3(I) = QTF(I) + SUM
  240          CONTINUE
               TEMP = DENORM(N,WA3)
               PRERED = ZERO
               IF (TEMP .LT. FNORM) PRERED = ONE - (TEMP/FNORM)**2
!!C
!!C              COMPUTE THE RATIO OF THE ACTUAL TO THE PREDICTED
!!C              REDUCTION.
!!C
               RATIO = ZERO
               IF (PRERED .GT. ZERO) RATIO = ACTRED/PRERED
!!C
!!C              UPDATE THE STEP BOUND.
!!C
               IF (RATIO .GE. P1) GO TO 250
                  NCSUC = 0
                  NCFAIL = NCFAIL + 1
                  DELTA = P5*DELTA
               GO TO 260
  250          CONTINUE
                  NCFAIL = 0
                  NCSUC = NCSUC + 1
                  IF (RATIO .GE. P5 .OR. NCSUC .GT. 1)    &
                    DELTA = MAX(DELTA,PNORM/P5)
                  IF (ABS(RATIO-ONE) .LE. P1) DELTA = PNORM/P5
  260          CONTINUE
!!C
!!C              TEST FOR SUCCESSFUL ITERATION.
!!C
               IF (RATIO .LT. P0001) GO TO 280
!!C
!!C                 SUCCESSFUL ITERATION. UPDATE X, FVEC, AND THEIR NORMS.
!!C
                  DO 270 J = 1, N
                     X(J) = WA2(J)
                     WA2(J) = DIAG(J)*X(J)
                     FVEC(J) = WA4(J)
  270             CONTINUE
                  XNORM = DENORM(N,WA2)
                  FNORM = FNORM1
                  ITER = ITER + 1
  280          CONTINUE
!!C
!!C              DETERMINE THE PROGRESS OF THE ITERATION.
!!C
               NSLOW1 = NSLOW1 + 1
               IF (ACTRED .GE. P001) NSLOW1 = 0
               IF (JEVAL) NSLOW2 = NSLOW2 + 1
               IF (ACTRED .GE. P1) NSLOW2 = 0
!!C
!!C              TEST FOR CONVERGENCE.
!!C
               IF (DELTA .LE. XTOL*XNORM .OR. FNORM .EQ. ZERO) INFO = 1
!!C     .........EXIT
               IF (INFO .NE. 0) GO TO 320
!!C
!!C              TESTS FOR TERMINATION AND STRINGENT TOLERANCES.
!!C
               IF (NFEV .GE. MAXFEV) INFO = 2
               IF (P1*MAX(P1*DELTA,PNORM) .LE. EPSMCH*XNORM) INFO = 3
               IF (NSLOW2 .EQ. 5) INFO = 4
               IF (NSLOW1 .EQ. 10) INFO = 5
!!C     .........EXIT
               IF (INFO .NE. 0) GO TO 320
!!C
!!C              CRITERION FOR RECALCULATING JACOBIAN
!!C
!!C           ...EXIT
               IF (NCFAIL .EQ. 2) GO TO 310
!!C
!!C              CALCULATE THE RANK ONE MODIFICATION TO THE JACOBIAN
!!C              AND UPDATE QTF IF NECESSARY.
!!C
               DO 300 J = 1, N
                  SUM = ZERO
                  DO 290 I = 1, N
                     SUM = SUM + FJAC(I,J)*WA4(I)
  290             CONTINUE
                  WA2(J) = (SUM - WA3(J))/PNORM
                  WA1(J) = DIAG(J)*((DIAG(J)*WA1(J))/PNORM)
                  IF (RATIO .GE. P0001) QTF(J) = SUM
  300          CONTINUE
!!C
!!C              COMPUTE THE QR FACTORIZATION OF THE UPDATED JACOBIAN.
!!C
               CALL D1UPDT(N,N,R,LR,WA1,WA2,WA3,SING)
               CALL D1MPYQ(N,N,FJAC,LDFJAC,WA2,WA3)
               CALL D1MPYQ(1,N,QTF,1,WA2,WA3)
!!C
!!C              END OF THE INNER LOOP.
!!C
               JEVAL = .FALSE.
            GO TO 200
  310       CONTINUE
!!C
!!C           END OF THE OUTER LOOP.
!!C
         GO TO 30
  320 CONTINUE
!!C
!!C     TERMINATION, EITHER NORMAL OR USER IMPOSED.
!!C
      IF (IFLAG .LT. 0) INFO = IFLAG
      IFLAG = 0
      IF (NPRINT .GT. 0) CALL FCN(N,X,FVEC,IFLAG)
      IF (INFO .LT. 0) CALL XERMSG ('SLATEC', 'DNSQ',										&
        'EXECUTION TERMINATED BECAUSE USER SET IFLAG NEGATIVE.', 1, 1)
      IF (INFO .EQ. 0) CALL XERMSG ('SLATEC', 'DNSQ',										&
        'INVALID INPUT PARAMETER.', 2, 1)
      IF (INFO .EQ. 2) CALL XERMSG ('SLATEC', 'DNSQ',										&
        'TOO MANY FUNCTION EVALUATIONS.', 9, 1)
      IF (INFO .EQ. 3) CALL XERMSG ('SLATEC', 'DNSQ',										&
        'XTOL TOO SMALL. NO FURTHER IMPROVEMENT POSSIBLE.', 3, 1)
      IF (INFO .GT. 4) CALL XERMSG ('SLATEC', 'DNSQ',										&
        'ITERATION NOT MAKING GOOD PROGRESS.', 1, 1)
      RETURN
!!C
!!C     LAST CARD OF SUBROUTINE DNSQ.
!!C
      END subroutine dnsq

!______________________________________________________________________________

!*DECK DQFORM

      SUBROUTINE DQFORM (M, N, Q, LDQ, WA)
!!C***BEGIN PROLOGUE  DQFORM
!!C***SUBSIDIARY
!!C***PURPOSE  Subsidiary to DNSQ and DNSQE
!!C***LIBRARY   SLATEC
!!C***TYPE      DOUBLE PRECISION (QFORM-S, DQFORM-D)
!!C***AUTHOR  (UNKNOWN)
!!C***DESCRIPTION
!!C
!!C     This subroutine proceeds from the computed QR factorization of
!!C     an M by N matrix A to accumulate the M by M orthogonal matrix
!!C     Q from its factored form.
!!C
!!C     The subroutine statement is
!!C
!!C       SUBROUTINE DQFORM(M,N,Q,LDQ,WA)
!!C
!!C     where
!!C
!!C       M is a positive integer input variable set to the number
!!C         of rows of A and the order of Q.
!!C
!!C       N is a positive integer input variable set to the number
!!C         of columns of A.
!!C
!!C       Q is an M by M array. On input the full lower trapezoid in
!!C         the first MIN(M,N) columns of Q contains the factored form.
!!C         On output Q has been accumulated into a square matrix.
!!C
!!C       LDQ is a positive integer input variable not less than M
!!C         which specifies the leading dimension of the array Q.
!!C
!!C       WA is a work array of length M.
!!C
!!C***SEE ALSO  DNSQ, DNSQE
!!C***ROUTINES CALLED  (NONE)
!!C***REVISION HISTORY  (YYMMDD)
!!C   800301  DATE WRITTEN
!!C   890531  Changed all specific intrinsics to generic.  (WRB)
!!C   890831  Modified array declarations.  (WRB)
!!C   891214  Prologue converted to Version 4.0 format.  (BAB)
!!C   900326  Removed duplicate information from DESCRIPTION section.
!!C           (WRB)
!!C   900328  Added TYPE section.  (WRB)
!!C***END PROLOGUE  DQFORM
      INTEGER I, J, JM1, K, L, LDQ, M, MINMN, N, NP1
      DOUBLE PRECISION ONE, Q(LDQ,*), SUM, TEMP, WA(*), ZERO
      SAVE ONE, ZERO
      DATA ONE,ZERO /1.0D0,0.0D0/
!!C
!!C     ZERO OUT UPPER TRIANGLE OF Q IN THE FIRST MIN(M,N) COLUMNS.
!!C
!!C***FIRST EXECUTABLE STATEMENT  DQFORM
      MINMN = MIN(M,N)
      IF (MINMN .LT. 2) GO TO 30
      DO 20 J = 2, MINMN
         JM1 = J - 1
         DO 10 I = 1, JM1
            Q(I,J) = ZERO
   10       CONTINUE
   20    CONTINUE
   30 CONTINUE
!!C
!!C     INITIALIZE REMAINING COLUMNS TO THOSE OF THE IDENTITY MATRIX.
!!C
      NP1 = N + 1
      IF (M .LT. NP1) GO TO 60
      DO 50 J = NP1, M
         DO 40 I = 1, M
            Q(I,J) = ZERO
   40       CONTINUE
         Q(J,J) = ONE
   50    CONTINUE
   60 CONTINUE
!!C
!!C     ACCUMULATE Q FROM ITS FACTORED FORM.
!!C
      DO 120 L = 1, MINMN
         K = MINMN - L + 1
         DO 70 I = K, M
            WA(I) = Q(I,K)
            Q(I,K) = ZERO
   70       CONTINUE
         Q(K,K) = ONE
         IF (WA(K) .EQ. ZERO) GO TO 110
         DO 100 J = K, M
            SUM = ZERO
            DO 80 I = K, M
               SUM = SUM + Q(I,J)*WA(I)
   80          CONTINUE
            TEMP = SUM/WA(K)
            DO 90 I = K, M
               Q(I,J) = Q(I,J) - TEMP*WA(I)
   90          CONTINUE
  100       CONTINUE
  110    CONTINUE
  120    CONTINUE
      RETURN
!!C
!!C     LAST CARD OF SUBROUTINE DQFORM.
!!C
      END subroutine dqform


!________________________________________________________________________

!*DECK DQRFAC
      SUBROUTINE DQRFAC (M, N, A, LDA, PIVOT, IPVT, LIPVT, SIGMA,		&
        ACNORM, WA)
!!C***BEGIN PROLOGUE  DQRFAC
!!C***SUBSIDIARY
!!C***PURPOSE  Subsidiary to DNLS1, DNLS1E, DNSQ and DNSQE
!!C***LIBRARY   SLATEC
!!C***TYPE      DOUBLE PRECISION (QRFAC-S, DQRFAC-D)
!!C***AUTHOR  (UNKNOWN)
!!C***DESCRIPTION
!!C
!!C   **** Double Precision version of QRFAC ****
!!C
!!C     This subroutine uses Householder transformations with column
!!C     pivoting (optional) to compute a QR factorization of the
!!C     M by N matrix A. That is, DQRFAC determines an orthogonal
!!C     matrix Q, a permutation matrix P, and an upper trapezoidal
!!C     matrix R with diagonal elements of nonincreasing magnitude,
!!C     such that A*P = Q*R. The Householder transformation for
!!C     column K, K = 1,2,...,MIN(M,N), is of the form
!!C
!!C                           T
!!C           I - (1/U(K))*U*U
!!C
!!C     where U has zeros in the first K-1 positions. The form of
!!C     this transformation and the method of pivoting first
!!C     appeared in the corresponding LINPACK subroutine.
!!C
!!C     The subroutine statement is
!!C
!!C       SUBROUTINE DQRFAC(M,N,A,LDA,PIVOT,IPVT,LIPVT,SIGMA,ACNORM,WA)
!!C
!!C     where
!!C
!!C       M is a positive integer input variable set to the number
!!C         of rows of A.
!!C
!!C       N is a positive integer input variable set to the number
!!C         of columns of A.
!!C
!!C       A is an M by N array. On input A contains the matrix for
!!C         which the QR factorization is to be computed. On output
!!C         the strict upper trapezoidal part of A contains the strict
!!C         upper trapezoidal part of R, and the lower trapezoidal
!!C         part of A contains a factored form of Q (the non-trivial
!!C         elements of the U vectors described above).
!!C
!!C       LDA is a positive integer input variable not less than M
!!C         which specifies the leading dimension of the array A.
!!C
!!C       PIVOT is a logical input variable. If pivot is set .TRUE.,
!!C         then column pivoting is enforced. If pivot is set .FALSE.,
!!C         then no column pivoting is done.
!!C
!!C       IPVT is an integer output array of length LIPVT. IPVT
!!C         defines the permutation matrix P such that A*P = Q*R.
!!C         Column J of P is column IPVT(J) of the identity matrix.
!!C         If pivot is .FALSE., IPVT is not referenced.
!!C
!!C       LIPVT is a positive integer input variable. If PIVOT is
!!C             .FALSE., then LIPVT may be as small as 1. If PIVOT is
!!C             .TRUE., then LIPVT must be at least N.
!!C
!!C       SIGMA is an output array of length N which contains the
!!C         diagonal elements of R.
!!C
!!C       ACNORM is an output array of length N which contains the
!!C         norms of the corresponding columns of the input matrix A.
!!C         If this information is not needed, then ACNORM can coincide
!!C         with SIGMA.
!!C
!!C       WA is a work array of length N. If pivot is .FALSE., then WA
!!C         can coincide with SIGMA.
!!C
!!C***SEE ALSO  DNLS1, DNLS1E, DNSQ, DNSQE
!!C***ROUTINES CALLED  ddd1mach, DENORM
!!C***REVISION HISTORY  (YYMMDD)
!!C   800301  DATE WRITTEN
!!C   890531  Changed all specific intrinsics to generic.  (WRB)
!!C   890831  Modified array declarations.  (WRB)
!!C   891214  Prologue converted to Version 4.0 format.  (BAB)
!!C   900326  Removed duplicate information from DESCRIPTION section.
!!C           (WRB)
!!C   900328  Added TYPE section.  (WRB)
!!C***END PROLOGUE  DQRFAC
      INTEGER M,N,LDA,LIPVT
      INTEGER IPVT(*)
      LOGICAL PIVOT
      SAVE ONE, P05, ZERO
      DOUBLE PRECISION A(LDA,*),SIGMA(*),ACNORM(*),WA(*)
      INTEGER I,J,JP1,K,KMAX,MINMN
      DOUBLE PRECISION AJNORM,EPSMCH,ONE,P05,SUM,TEMP,ZERO
     ! DOUBLE PRECISION ddd1mach,DENORM
      DATA ONE,P05,ZERO /1.0D0,5.0D-2,0.0D0/
!!C***FIRST EXECUTABLE STATEMENT  DQRFAC
      EPSMCH = ddd1mach(4)
!!C
!!C     COMPUTE THE INITIAL COLUMN NORMS AND INITIALIZE SEVERAL ARRAYS.
!!C
      DO 10 J = 1, N
         ACNORM(J) = DENORM(M,A(1,J))
         SIGMA(J) = ACNORM(J)
         WA(J) = SIGMA(J)
         IF (PIVOT) IPVT(J) = J
   10    CONTINUE
!!C
!!C     REDUCE A TO R WITH HOUSEHOLDER TRANSFORMATIONS.
!!C
      MINMN = MIN(M,N)
      DO 110 J = 1, MINMN
         IF (.NOT.PIVOT) GO TO 40
!!C
!!C        BRING THE COLUMN OF LARGEST NORM INTO THE PIVOT POSITION.
!!C
         KMAX = J
         DO 20 K = J, N
            IF (SIGMA(K) .GT. SIGMA(KMAX)) KMAX = K
   20       CONTINUE
         IF (KMAX .EQ. J) GO TO 40
         DO 30 I = 1, M
            TEMP = A(I,J)
            A(I,J) = A(I,KMAX)
            A(I,KMAX) = TEMP
   30       CONTINUE
         SIGMA(KMAX) = SIGMA(J)
         WA(KMAX) = WA(J)
         K = IPVT(J)
         IPVT(J) = IPVT(KMAX)
         IPVT(KMAX) = K
   40    CONTINUE
!!C
!!C        COMPUTE THE HOUSEHOLDER TRANSFORMATION TO REDUCE THE
!!C        J-TH COLUMN OF A TO A MULTIPLE OF THE J-TH UNIT VECTOR.
!!C
         AJNORM = DENORM(M-J+1,A(J,J))
         IF (AJNORM .EQ. ZERO) GO TO 100
         IF (A(J,J) .LT. ZERO) AJNORM = -AJNORM
         DO 50 I = J, M
            A(I,J) = A(I,J)/AJNORM
   50       CONTINUE
         A(J,J) = A(J,J) + ONE
!!C
!!C        APPLY THE TRANSFORMATION TO THE REMAINING COLUMNS
!!C        AND UPDATE THE NORMS.
!!C
         JP1 = J + 1
         IF (N .LT. JP1) GO TO 100
         DO 90 K = JP1, N
            SUM = ZERO
            DO 60 I = J, M
               SUM = SUM + A(I,J)*A(I,K)
   60          CONTINUE
            TEMP = SUM/A(J,J)
            DO 70 I = J, M
               A(I,K) = A(I,K) - TEMP*A(I,J)
   70          CONTINUE
            IF (.NOT.PIVOT .OR. SIGMA(K) .EQ. ZERO) GO TO 80
            TEMP = A(J,K)/SIGMA(K)
            SIGMA(K) = SIGMA(K)*SQRT(MAX(ZERO,ONE-TEMP**2))
            IF (P05*(SIGMA(K)/WA(K))**2 .GT. EPSMCH) GO TO 80
            SIGMA(K) = DENORM(M-J,A(JP1,K))
            WA(K) = SIGMA(K)
   80       CONTINUE
   90       CONTINUE
  100    CONTINUE
         SIGMA(J) = -AJNORM
  110    CONTINUE
      RETURN
!!C
!!C     LAST CARD OF SUBROUTINE DQRFAC.
!!C
      END subroutine dqrfac

!_______________________________________________________________

!*DECK XERMSG
 
      SUBROUTINE XERMSG (LIBRAR, SUBROU, MESSG, NERR, LEVEL)
!!C***BEGIN PROLOGUE  XERMSG
!!C***PURPOSE  Process error messages for SLATEC and other libraries.
!!C***LIBRARY   SLATEC (XERROR)
!!C***CATEGORY  R3C
!!C***TYPE      ALL (XERMSG-A)
!!C***KEYWORDS  ERROR MESSAGE, XERROR
!!C***AUTHOR  Fong, Kirby, (NMFECC at LLNL)
!!C***DESCRIPTION
!!C
!!C   XERMSG processes a diagnostic message in a manner determined by the
!!C   value of LEVEL and the current value of the library error control
!!C   flag, KONTRL.  See subroutine XSETF for details.
!!C
!!C    LIBRAR   A character constant (or character variable) with the name
!!C             of the library.  This will be 'SLATEC' for the SLATEC
!!C             Common Math Library.  The error handling package is
!!C             general enough to be used by many libraries
!!C             simultaneously, so it is desirable for the routine that
!!C             detects and reports an error to identify the library name
!!C             as well as the routine name.
!!C
!!C    SUBROU   A character constant (or character variable) with the name
!!C             of the routine that detected the error.  Usually it is the
!!C             name of the routine that is calling XERMSG.  There are
!!C             some instances where a user callable library routine calls
!!C             lower level subsidiary routines where the error is
!!C             detected.  In such cases it may be more informative to
!!C             supply the name of the routine the user called rather than
!!C             the name of the subsidiary routine that detected the
!!C             error.
!!C
!!C    MESSG    A character constant (or character variable) with the text
!!C             of the error or warning message.  In the example below,
!!C             the message is a character constant that contains a
!!C             generic message.
!!C
!!C                   CALL XERMSG ('SLATEC', 'MMPY',
!!C                  *'THE ORDER OF THE MATRIX EXCEEDS THE ROW DIMENSION',
!!C                  *3, 1)
!!C
!!C             It is possible (and is sometimes desirable) to generate a
!!C             specific message--e.g., one that contains actual numeric
!!C             values.  Specific numeric values can be converted into
!!C             character strings using formatted WRITE statements into
!!C             character variables.  This is called standard Fortran
!!C             internal file I/O and is exemplified in the first three
!!C             lines of the following example.  You can also catenate
!!C             substrings of characters to construct the error message.
!!C             Here is an example showing the use of both writing to
!!C             an internal file and catenating character strings.
!!C
!!C                   CHARACTER*5 CHARN, CHARL
!!C                   WRITE (CHARN,10) N
!!C                   WRITE (CHARL,10) LDA
!!C                10 FORMAT(I5)
!!C                   CALL XERMSG ('SLATEC', 'MMPY', 'THE ORDER'//CHARN//
!!C                  *   ' OF THE MATRIX EXCEEDS ITS ROW DIMENSION OF'//
!!C                  *   CHARL, 3, 1)
!!C
!!C             There are two subtleties worth mentioning.  One is that
!!C             the // for character catenation is used to construct the
!!C             error message so that no single character constant is
!!C             continued to the next line.  This avoids confusion as to
!!C             whether there are trailing blanks at the end of the line.
!!C             The second is that by catenating the parts of the message
!!C             as an actual argument rather than encoding the entire
!!C             message into one large character variable, we avoid
!!C             having to know how long the message will be in order to
!!C             declare an adequate length for that large character
!!C             variable.  XERMSG calls XERPRN to print the message using
!!C             multiple lines if necessary.  If the message is very long,
!!C             XERPRN will break it into pieces of 72 characters (as
!!C             requested by XERMSG) for printing on multiple lines.
!!C             Also, XERMSG asks XERPRN to prefix each line with ' *  '
!!C             so that the total line length could be 76 characters.
!!C             Note also that XERPRN scans the error message backwards
!!C             to ignore trailing blanks.  Another feature is that
!!C             the substring '$$' is treated as a new line sentinel
!!C             by XERPRN.  If you want to construct a multiline
!!C             message without having to count out multiples of 72
!!C             characters, just use '$$' as a separator.  '$$'
!!C             obviously must occur within 72 characters of the
!!C             start of each line to have its intended effect since
!!C             XERPRN is asked to wrap around at 72 characters in
!!C             addition to looking for '$$'.
!!C
!!C    NERR     An integer value that is chosen by the library routine's
!!C             author.  It must be in the range -99 to 999 (three
!!C             printable digits).  Each distinct error should have its
!!C             own error number.  These error numbers should be described
!!C             in the machine readable documentation for the routine.
!!C             The error numbers need be unique only within each routine,
!!C             so it is reasonable for each routine to start enumerating
!!C             errors from 1 and proceeding to the next integer.
!!C
!!C    LEVEL    An integer value in the range 0 to 2 that indicates the
!!C             level (severity) of the error.  Their meanings are
!!C
!!C            -1  A warning message.  This is used if it is not clear
!!C                that there really is an error, but the user's attention
!!C                may be needed.  An attempt is made to only print this
!!C                message once.
!!C
!!C             0  A warning message.  This is used if it is not clear
!!C                that there really is an error, but the user's attention
!!C                may be needed.
!!C
!!C             1  A recoverable error.  This is used even if the error is
!!C                so serious that the routine cannot return any useful
!!C                answer.  If the user has told the error package to
!!C                return after recoverable errors, then XERMSG will
!!C                return to the Library routine which can then return to
!!C                the user's routine.  The user may also permit the error
!!C                package to terminate the program upon encountering a
!!C                recoverable error.
!!C
!!C             2  A fatal error.  XERMSG will not return to its caller
!!C                after it receives a fatal error.  This level should
!!C                hardly ever be used; it is much better to allow the
!!C                user a chance to recover.  An example of one of the few
!!C                cases in which it is permissible to declare a level 2
!!C                error is a reverse communication Library routine that
!!C                is likely to be called repeatedly until it integrates
!!C                across some interval.  If there is a serious error in
!!C                the input such that another step cannot be taken and
!!C                the Library routine is called again without the input
!!C                error having been corrected by the caller, the Library
!!C                routine will probably be called forever with improper
!!C                input.  In this case, it is reasonable to declare the
!!C                error to be fatal.
!!C
!!C    Each of the arguments to XERMSG is input; none will be modified by
!!C    XERMSG.  A routine may make multiple calls to XERMSG with warning
!!C    level messages; however, after a call to XERMSG with a recoverable
!!C    error, the routine should return to the user.  Do not try to call
!!C    XERMSG with a second recoverable error after the first recoverable
!!C    error because the error package saves the error number.  The user
!!C    can retrieve this error number by calling another entry point in
!!C    the error handling package and then clear the error number when
!!C    recovering from the error.  Calling XERMSG in succession causes the
!!C    old error number to be overwritten by the latest error number.
!!C    This is considered harmless for error numbers associated with
!!C    warning messages but must not be done for error numbers of serious
!!C    errors.  After a call to XERMSG with a recoverable error, the user
!!C    must be given a chance to call NUMXER or XERCLR to retrieve or
!!C    clear the error number.
!!C***REFERENCES  R. E. Jones and D. K. Kahaner, XERROR, the SLATEC
!!C                 Error-handling Package, SAND82-0800, Sandia
!!C                 Laboratories, 1982.
!!C***ROUTINES CALLED  FDUMP, J4SAVE, XERCNT, XERHLT, XERPRN, XERSVE
!!C***REVISION HISTORY  (YYMMDD)
!!C   880101  DATE WRITTEN
!!C   880621  REVISED AS DIRECTED AT SLATEC CML MEETING OF FEBRUARY 1988.
!!C           THERE ARE TWO BASIC CHANGES.
!!C           1.  A NEW ROUTINE, XERPRN, IS USED INSTEAD OF XERPRT TO
!!C               PRINT MESSAGES.  THIS ROUTINE WILL BREAK LONG MESSAGES
!!C               INTO PIECES FOR PRINTING ON MULTIPLE LINES.  '$$' IS
!!C               ACCEPTED AS A NEW LINE SENTINEL.  A PREFIX CAN BE
!!C               ADDED TO EACH LINE TO BE PRINTED.  XERMSG USES EITHER
!!C               ' ***' OR ' *  ' AND LONG MESSAGES ARE BROKEN EVERY
!!C               72 CHARACTERS (AT MOST) SO THAT THE MAXIMUM LINE
!!C               LENGTH OUTPUT CAN NOW BE AS GREAT AS 76.
!!C           2.  THE TEXT OF ALL MESSAGES IS NOW IN UPPER CASE SINCE THE
!!C               FORTRAN STANDARD DOCUMENT DOES NOT ADMIT THE EXISTENCE
!!C               OF LOWER CASE.
!!C   880708  REVISED AFTER THE SLATEC CML MEETING OF JUNE 29 AND 30.
!!C           THE PRINCIPAL CHANGES ARE
!!C           1.  CLARIFY COMMENTS IN THE PROLOGUES
!!C           2.  RENAME XRPRNT TO XERPRN
!!C           3.  REWORK HANDLING OF '$$' IN XERPRN TO HANDLE BLANK LINES
!!C               SIMILAR TO THE WAY FORMAT STATEMENTS HANDLE THE /
!!C               CHARACTER FOR NEW RECORDS.
!!C   890706  REVISED WITH THE HELP OF FRED FRITSCH AND REG CLEMENS TO
!!C           CLEAN UP THE CODING.
!!C   890721  REVISED TO USE NEW FEATURE IN XERPRN TO COUNT CHARACTERS IN
!!C           PREFIX.
!!C   891013  REVISED TO CORRECT COMMENTS.
!!C   891214  Prologue converted to Version 4.0 format.  (WRB)
!!C   900510  Changed test on NERR to be -9999999 < NERR < 99999999, but
!!C           NERR .ne. 0, and on LEVEL to be -2 < LEVEL < 3.  Added
!!C           LEVEL=-1 logic, changed calls to XERSAV to XERSVE, and
!!C           XERCTL to XERCNT.  (RWC)
!!C   920501  Reformatted the REFERENCES section.  (WRB)
!!C***END PROLOGUE  XERMSG
      CHARACTER*(*) LIBRAR, SUBROU, MESSG
      CHARACTER*8 XLIBR, XSUBR
      CHARACTER*72  TEMP
      CHARACTER*20  LFIRST
!!C***FIRST EXECUTABLE STATEMENT  XERMSG
      LKNTRL = J4SAVE (2, 0, .FALSE.)
      MAXMES = J4SAVE (4, 0, .FALSE.)
!!C
!!C       LKNTRL IS A LOCAL COPY OF THE CONTROL FLAG KONTRL.
!!C       MAXMES IS THE MAXIMUM NUMBER OF TIMES ANY PARTICULAR MESSAGE
!!C          SHOULD BE PRINTED.
!!C
!!C       WE PRINT A FATAL ERROR MESSAGE AND TERMINATE FOR AN ERROR IN
!!C          CALLING XERMSG.  THE ERROR NUMBER SHOULD BE POSITIVE,
!!C          AND THE LEVEL SHOULD BE BETWEEN 0 AND 2.
!!C
      IF (NERR.LT.-9999999 .OR. NERR.GT.99999999 .OR. NERR.EQ.0 .OR.	&
        LEVEL.LT.-1 .OR. LEVEL.GT.2) THEN
         CALL XERPRN (' ***', -1, 'FATAL ERROR IN...$$ ' //					&
           'XERMSG -- INVALID ERROR NUMBER OR LEVEL$$ '//					&
           'JOB ABORT DUE TO FATAL ERROR.', 72)
         CALL XERSVE (' ', ' ', ' ', 0, 0, 0, KDUMMY)
         CALL XERHLT (' ***XERMSG -- INVALID INPUT')
         RETURN
      ENDIF
!!C
!!C       RECORD THE MESSAGE.
!!C
      I = J4SAVE (1, NERR, .TRUE.)
      CALL XERSVE (LIBRAR, SUBROU, MESSG, 1, NERR, LEVEL, KOUNT)
!!C
!!C       HANDLE PRINT-ONCE WARNING MESSAGES.
!!C
      IF (LEVEL.EQ.-1 .AND. KOUNT.GT.1) RETURN
!!C
!!C       ALLOW TEMPORARY USER OVERRIDE OF THE CONTROL FLAG.
!!C
      XLIBR  = LIBRAR
      XSUBR  = SUBROU
      LFIRST = MESSG
      LERR   = NERR
      LLEVEL = LEVEL
      CALL XERCNT (XLIBR, XSUBR, LFIRST, LERR, LLEVEL, LKNTRL)
!!C
      LKNTRL = MAX(-2, MIN(2,LKNTRL))
      MKNTRL = ABS(LKNTRL)
!!C
!!C       SKIP PRINTING IF THE CONTROL FLAG VALUE AS RESET IN XERCNT IS
!!C       ZERO AND THE ERROR IS NOT FATAL.
!!C
      IF (LEVEL.LT.2 .AND. LKNTRL.EQ.0) GO TO 30
      IF (LEVEL.EQ.0 .AND. KOUNT.GT.MAXMES) GO TO 30
      IF (LEVEL.EQ.1 .AND. KOUNT.GT.MAXMES .AND. MKNTRL.EQ.1) GO TO 30
      IF (LEVEL.EQ.2 .AND. KOUNT.GT.MAX(1,MAXMES)) GO TO 30
!!C
!!C       ANNOUNCE THE NAMES OF THE LIBRARY AND SUBROUTINE BY BUILDING A
!!C       MESSAGE IN CHARACTER VARIABLE TEMP (NOT EXCEEDING 66 CHARACTERS)
!!C       AND SENDING IT OUT VIA XERPRN.  PRINT ONLY IF CONTROL FLAG
!!C       IS NOT ZERO.
!!C
      IF (LKNTRL .NE. 0) THEN
         TEMP(1:21) = 'MESSAGE FROM ROUTINE '
         I = MIN(LEN(SUBROU), 16)
         TEMP(22:21+I) = SUBROU(1:I)
         TEMP(22+I:33+I) = ' IN LIBRARY '
         LTEMP = 33 + I
         I = MIN(LEN(LIBRAR), 16)
         TEMP(LTEMP+1:LTEMP+I) = LIBRAR (1:I)
         TEMP(LTEMP+I+1:LTEMP+I+1) = '.'
         LTEMP = LTEMP + I + 1
         CALL XERPRN (' ***', -1, TEMP(1:LTEMP), 72)
      ENDIF
!!C
!!C       IF LKNTRL IS POSITIVE, PRINT AN INTRODUCTORY LINE BEFORE
!!C       PRINTING THE MESSAGE.  THE INTRODUCTORY LINE TELLS THE CHOICE
!!C       FROM EACH OF THE FOLLOWING THREE OPTIONS.
!!C       1.  LEVEL OF THE MESSAGE
!!C              'INFORMATIVE MESSAGE'
!!C              'POTENTIALLY RECOVERABLE ERROR'
!!C              'FATAL ERROR'
!!C       2.  WHETHER CONTROL FLAG WILL ALLOW PROGRAM TO CONTINUE
!!C              'PROG CONTINUES'
!!C              'PROG ABORTED'
!!C       3.  WHETHER OR NOT A TRACEBACK WAS REQUESTED.  (THE TRACEBACK
!!C           MAY NOT BE IMPLEMENTED AT SOME SITES, SO THIS ONLY TELLS
!!C           WHAT WAS REQUESTED, NOT WHAT WAS DELIVERED.)
!!C              'TRACEBACK REQUESTED'
!!C              'TRACEBACK NOT REQUESTED'
!!C       NOTICE THAT THE LINE INCLUDING FOUR PREFIX CHARACTERS WILL NOT
!!C       EXCEED 74 CHARACTERS.
!!C       WE SKIP THE NEXT BLOCK IF THE INTRODUCTORY LINE IS NOT NEEDED.
!!C
      IF (LKNTRL .GT. 0) THEN
!!C
!!C       THE FIRST PART OF THE MESSAGE TELLS ABOUT THE LEVEL.
!!C
         IF (LEVEL .LE. 0) THEN
            TEMP(1:20) = 'INFORMATIVE MESSAGE,'
            LTEMP = 20
         ELSEIF (LEVEL .EQ. 1) THEN
            TEMP(1:30) = 'POTENTIALLY RECOVERABLE ERROR,'
            LTEMP = 30
         ELSE
            TEMP(1:12) = 'FATAL ERROR,'
            LTEMP = 12
         ENDIF
!!C
!!C       THEN WHETHER THE PROGRAM WILL CONTINUE.
!!C
         IF ((MKNTRL.EQ.2 .AND. LEVEL.GE.1) .OR.        &
            (MKNTRL.EQ.1 .AND. LEVEL.EQ.2)) THEN
            TEMP(LTEMP+1:LTEMP+14) = ' PROG ABORTED,'
            LTEMP = LTEMP + 14
         ELSE
            TEMP(LTEMP+1:LTEMP+16) = ' PROG CONTINUES,'
            LTEMP = LTEMP + 16
         ENDIF
!!C
!!C       FINALLY TELL WHETHER THERE SHOULD BE A TRACEBACK.
!!C
         IF (LKNTRL .GT. 0) THEN
            TEMP(LTEMP+1:LTEMP+20) = ' TRACEBACK REQUESTED'
            LTEMP = LTEMP + 20
         ELSE
            TEMP(LTEMP+1:LTEMP+24) = ' TRACEBACK NOT REQUESTED'
            LTEMP = LTEMP + 24
         ENDIF
         CALL XERPRN (' ***', -1, TEMP(1:LTEMP), 72)
      ENDIF
!!C
!!C       NOW SEND OUT THE MESSAGE.
!!C
      CALL XERPRN (' *  ', -1, MESSG, 72)
!!C
!!C       IF LKNTRL IS POSITIVE, WRITE THE ERROR NUMBER AND REQUEST A
!!C          TRACEBACK.
!!C
      IF (LKNTRL .GT. 0) THEN
         WRITE (TEMP, '(''ERROR NUMBER = '', I8)') NERR
         DO 10 I=16,22
            IF (TEMP(I:I) .NE. ' ') GO TO 20
   10    CONTINUE
!!C
   20    CALL XERPRN (' *  ', -1, TEMP(1:15) // TEMP(I:23), 72)
         CALL FDUMP
      ENDIF
!!C
!!C       IF LKNTRL IS NOT ZERO, PRINT A BLANK LINE AND AN END OF MESSAGE.
!!C
      IF (LKNTRL .NE. 0) THEN
         CALL XERPRN (' *  ', -1, ' ', 72)
         CALL XERPRN (' ***', -1, 'END OF MESSAGE', 72)
         CALL XERPRN ('    ',  0, ' ', 72)
      ENDIF
!!C
!!C       IF THE ERROR IS NOT FATAL OR THE ERROR IS RECOVERABLE AND THE
!!C       CONTROL FLAG IS SET FOR RECOVERY, THEN RETURN.
!!C
   30 IF (LEVEL.LE.0 .OR. (LEVEL.EQ.1 .AND. MKNTRL.LE.1)) RETURN
!!C
!!C       THE PROGRAM WILL BE STOPPED DUE TO AN UNRECOVERED ERROR OR A
!!C       FATAL ERROR.  PRINT THE REASON FOR THE ABORT AND THE ERROR
!!C       SUMMARY IF THE CONTROL FLAG AND THE MAXIMUM ERROR COUNT PERMIT.
!!C
      IF (LKNTRL.GT.0 .AND. KOUNT.LT.MAX(1,MAXMES)) THEN
         IF (LEVEL .EQ. 1) THEN
            CALL XERPRN									&
              (' ***', -1, 'JOB ABORT DUE TO UNRECOVERED ERROR.', 72)
         ELSE
            CALL XERPRN(' ***', -1, 'JOB ABORT DUE TO FATAL ERROR.', 72)
         ENDIF
         CALL XERSVE (' ', ' ', ' ', -1, 0, 0, KDUMMY)
         CALL XERHLT (' ')
      ELSE
         CALL XERHLT (MESSG)
      ENDIF
      RETURN

      END subroutine XERMSG

!__________________________________________________________________________---

!*DECK XERPRN
      SUBROUTINE XERPRN (PREFIX, NPREF, MESSG, NWRAP)
!!C***BEGIN PROLOGUE  XERPRN
!!C***SUBSIDIARY
!!C***PURPOSE  Print error messages processed by XERMSG.
!!C***LIBRARY   SLATEC (XERROR)
!!C***CATEGORY  R3C
!!C***TYPE      ALL (XERPRN-A)
!!C***KEYWORDS  ERROR MESSAGES, PRINTING, XERROR
!!C***AUTHOR  Fong, Kirby, (NMFECC at LLNL)
!!C***DESCRIPTION
!!C
!!C This routine sends one or more lines to each of the (up to five)
!!C logical units to which error messages are to be sent.  This routine
!!C is called several times by XERMSG, sometimes with a single line to
!!C print and sometimes with a (potentially very long) message that may
!!C wrap around into multiple lines.
!!C
!!C PREFIX  Input argument of type CHARACTER.  This argument contains
!!C         characters to be put at the beginning of each line before
!!C         the body of the message.  No more than 16 characters of
!!C         PREFIX will be used.
!!C
!!C NPREF   Input argument of type INTEGER.  This argument is the number
!!C         of characters to use from PREFIX.  If it is negative, the
!!C         intrinsic function LEN is used to determine its length.  If
!!C         it is zero, PREFIX is not used.  If it exceeds 16 or if
!!C         LEN(PREFIX) exceeds 16, only the first 16 characters will be
!!C         used.  If NPREF is positive and the length of PREFIX is less
!!C         than NPREF, a copy of PREFIX extended with blanks to length
!!C         NPREF will be used.
!!C
!!C MESSG   Input argument of type CHARACTER.  This is the text of a
!!C         message to be printed.  If it is a long message, it will be
!!C         broken into pieces for printing on multiple lines.  Each line
!!C         will start with the appropriate prefix and be followed by a
!!C         piece of the message.  NWRAP is the number of characters per
!!C         piece; that is, after each NWRAP characters, we break and
!!C         start a new line.  In addition the characters '$$' embedded
!!C         in MESSG are a sentinel for a new line.  The counting of
!!C         characters up to NWRAP starts over for each new line.  The
!!C         value of NWRAP typically used by XERMSG is 72 since many
!!C         older error messages in the SLATEC Library are laid out to
!!C         rely on wrap-around every 72 characters.
!!C
!!C NWRAP   Input argument of type INTEGER.  This gives the maximum size
!!C         piece into which to break MESSG for printing on multiple
!!C         lines.  An embedded '$$' ends a line, and the count restarts
!!C         at the following character.  If a line break does not occur
!!C         on a blank (it would split a word) that word is moved to the
!!C         next line.  Values of NWRAP less than 16 will be treated as
!!C         16.  Values of NWRAP greater than 132 will be treated as 132.
!!C         The actual line length will be NPREF + NWRAP after NPREF has
!!C         been adjusted to fall between 0 and 16 and NWRAP has been
!!C         adjusted to fall between 16 and 132.
!!C
!!C***REFERENCES  R. E. Jones and D. K. Kahaner, XERROR, the SLATEC
!!C                 Error-handling Package, SAND82-0800, Sandia
!!C                 Laboratories, 1982.
!!C***ROUTINES CALLED  I1MACH, XGETUA
!!C***REVISION HISTORY  (YYMMDD)
!!C   880621  DATE WRITTEN
!!C   880708  REVISED AFTER THE SLATEC CML SUBCOMMITTEE MEETING OF
!!C           JUNE 29 AND 30 TO CHANGE THE NAME TO XERPRN AND TO REWORK
!!C           THE HANDLING OF THE NEW LINE SENTINEL TO BEHAVE LIKE THE
!!C           SLASH CHARACTER IN FORMAT STATEMENTS.
!!C   890706  REVISED WITH THE HELP OF FRED FRITSCH AND REG CLEMENS TO
!!C           STREAMLINE THE CODING AND FIX A BUG THAT CAUSED EXTRA BLANK
!!C           LINES TO BE PRINTED.
!!C   890721  REVISED TO ADD A NEW FEATURE.  A NEGATIVE VALUE OF NPREF
!!C           CAUSES LEN(PREFIX) TO BE USED AS THE LENGTH.
!!C   891013  REVISED TO CORRECT ERROR IN CALCULATING PREFIX LENGTH.
!!C   891214  Prologue converted to Version 4.0 format.  (WRB)
!!C   900510  Added code to break messages between words.  (RWC)
!!C   920501  Reformatted the REFERENCES section.  (WRB)
!!C***END PROLOGUE  XERPRN
      CHARACTER*(*) PREFIX, MESSG
      INTEGER NPREF, NWRAP
      CHARACTER*148 CBUFF
      INTEGER IU(5), NUNIT
      CHARACTER*2 NEWLIN
      PARAMETER (NEWLIN = '$$')
!!C***FIRST EXECUTABLE STATEMENT  XERPRN
      CALL XGETUA(IU,NUNIT)
!!C
!!C       A ZERO VALUE FOR A LOGICAL UNIT NUMBER MEANS TO USE THE STANDARD
!!C       ERROR MESSAGE UNIT INSTEAD.  I1MACH(4) RETRIEVES THE STANDARD
!!C       ERROR MESSAGE UNIT.
!!C
      N = I1MACH(4)
      DO 10 I=1,NUNIT
         IF (IU(I) .EQ. 0) IU(I) = N
   10 CONTINUE
!!C
!!C       LPREF IS THE LENGTH OF THE PREFIX.  THE PREFIX IS PLACED AT THE
!!C       BEGINNING OF CBUFF, THE CHARACTER BUFFER, AND KEPT THERE DURING
!!C       THE REST OF THIS ROUTINE.
!!C
      IF ( NPREF .LT. 0 ) THEN
         LPREF = LEN(PREFIX)
      ELSE
         LPREF = NPREF
      ENDIF
      LPREF = MIN(16, LPREF)
      IF (LPREF .NE. 0) CBUFF(1:LPREF) = PREFIX
!!C
!!C       LWRAP IS THE MAXIMUM NUMBER OF CHARACTERS WE WANT TO TAKE AT ONE
!!C       TIME FROM MESSG TO PRINT ON ONE LINE.
!!C
      LWRAP = MAX(16, MIN(132, NWRAP))
!!C
!!C       SET LENMSG TO THE LENGTH OF MESSG, IGNORE ANY TRAILING BLANKS.
!!C
      LENMSG = LEN(MESSG)
      N = LENMSG
      DO 20 I=1,N
         IF (MESSG(LENMSG:LENMSG) .NE. ' ') GO TO 30
         LENMSG = LENMSG - 1
   20 CONTINUE
   30 CONTINUE
!!C
!!C       IF THE MESSAGE IS ALL BLANKS, THEN PRINT ONE BLANK LINE.
!!C
      IF (LENMSG .EQ. 0) THEN
         CBUFF(LPREF+1:LPREF+1) = ' '
         DO 40 I=1,NUNIT
            WRITE(IU(I), '(A)') CBUFF(1:LPREF+1)
   40    CONTINUE
         RETURN
      ENDIF
!!C
!!C       SET NEXTC TO THE POSITION IN MESSG WHERE THE NEXT SUBSTRING
!!C       STARTS.  FROM THIS POSITION WE SCAN FOR THE NEW LINE SENTINEL.
!!C       WHEN NEXTC EXCEEDS LENMSG, THERE IS NO MORE TO PRINT.
!!C       WE LOOP BACK TO LABEL 50 UNTIL ALL PIECES HAVE BEEN PRINTED.
!!C
!!C       WE LOOK FOR THE NEXT OCCURRENCE OF THE NEW LINE SENTINEL.  THE
!!C       INDEX INTRINSIC FUNCTION RETURNS ZERO IF THERE IS NO OCCURRENCE
!!C       OR IF THE LENGTH OF THE FIRST ARGUMENT IS LESS THAN THE LENGTH
!!C       OF THE SECOND ARGUMENT.
!!C
!!C       THERE ARE SEVERAL CASES WHICH SHOULD BE CHECKED FOR IN THE
!!C       FOLLOWING ORDER.  WE ARE ATTEMPTING TO SET LPIECE TO THE NUMBER
!!C       OF CHARACTERS THAT SHOULD BE TAKEN FROM MESSG STARTING AT
!!C       POSITION NEXTC.
!!C
!!C       LPIECE .EQ. 0   THE NEW LINE SENTINEL DOES NOT OCCUR IN THE
!!C                       REMAINDER OF THE CHARACTER STRING.  LPIECE
!!C                       SHOULD BE SET TO LWRAP OR LENMSG+1-NEXTC,
!!C                       WHICHEVER IS LESS.
!!C
!!C       LPIECE .EQ. 1   THE NEW LINE SENTINEL STARTS AT MESSG(NEXTC:
!!C                       NEXTC).  LPIECE IS EFFECTIVELY ZERO, AND WE
!!C                       PRINT NOTHING TO AVOID PRODUCING UNNECESSARY
!!C                       BLANK LINES.  THIS TAKES CARE OF THE SITUATION
!!C                       WHERE THE LIBRARY ROUTINE HAS A MESSAGE OF
!!C                       EXACTLY 72 CHARACTERS FOLLOWED BY A NEW LINE
!!C                       SENTINEL FOLLOWED BY MORE CHARACTERS.  NEXTC
!!C                       SHOULD BE INCREMENTED BY 2.
!!C
!!C       LPIECE .GT. LWRAP+1  REDUCE LPIECE TO LWRAP.
!!C
!!C       ELSE            THIS LAST CASE MEANS 2 .LE. LPIECE .LE. LWRAP+1
!!C                       RESET LPIECE = LPIECE-1.  NOTE THAT THIS
!!C                       PROPERLY HANDLES THE END CASE WHERE LPIECE .EQ.
!!C                       LWRAP+1.  THAT IS, THE SENTINEL FALLS EXACTLY
!!C                       AT THE END OF A LINE.
!!C
      NEXTC = 1
   50 LPIECE = INDEX(MESSG(NEXTC:LENMSG), NEWLIN)
      IF (LPIECE .EQ. 0) THEN
!!C
!!C       THERE WAS NO NEW LINE SENTINEL FOUND.
!!C
         IDELTA = 0
         LPIECE = MIN(LWRAP, LENMSG+1-NEXTC)
         IF (LPIECE .LT. LENMSG+1-NEXTC) THEN
            DO 52 I=LPIECE+1,2,-1
               IF (MESSG(NEXTC+I-1:NEXTC+I-1) .EQ. ' ') THEN
                  LPIECE = I-1
                  IDELTA = 1
                  GOTO 54
               ENDIF
   52       CONTINUE
         ENDIF
   54    CBUFF(LPREF+1:LPREF+LPIECE) = MESSG(NEXTC:NEXTC+LPIECE-1)
         NEXTC = NEXTC + LPIECE + IDELTA
      ELSEIF (LPIECE .EQ. 1) THEN
!!C
!!C       WE HAVE A NEW LINE SENTINEL AT MESSG(NEXTC:NEXTC+1).
!!C       DON'T PRINT A BLANK LINE.
!!C
         NEXTC = NEXTC + 2
         GO TO 50
      ELSEIF (LPIECE .GT. LWRAP+1) THEN
!!C
!!C       LPIECE SHOULD BE SET DOWN TO LWRAP.
!!C
         IDELTA = 0
         LPIECE = LWRAP
         DO 56 I=LPIECE+1,2,-1
            IF (MESSG(NEXTC+I-1:NEXTC+I-1) .EQ. ' ') THEN
               LPIECE = I-1
               IDELTA = 1
               GOTO 58
            ENDIF
   56    CONTINUE
   58    CBUFF(LPREF+1:LPREF+LPIECE) = MESSG(NEXTC:NEXTC+LPIECE-1)
         NEXTC = NEXTC + LPIECE + IDELTA
      ELSE
!!C
!!C       IF WE ARRIVE HERE, IT MEANS 2 .LE. LPIECE .LE. LWRAP+1.
!!C       WE SHOULD DECREMENT LPIECE BY ONE.
!!C
         LPIECE = LPIECE - 1
         CBUFF(LPREF+1:LPREF+LPIECE) = MESSG(NEXTC:NEXTC+LPIECE-1)
         NEXTC  = NEXTC + LPIECE + 2
      ENDIF
!!C
!!C       PRINT
!!C
      DO 60 I=1,NUNIT
         WRITE(IU(I), '(A)') CBUFF(1:LPREF+LPIECE)
   60 CONTINUE
!!C
      IF (NEXTC .LE. LENMSG) GO TO 50
      RETURN
      END subroutine XERPRN 

!________________________________________________________________________
														
!*DECK XERSVE
      SUBROUTINE XERSVE (LIBRAR, SUBROU, MESSG, KFLAG, NERR, LEVEL,   &
        ICOUNT)
!!C***BEGIN PROLOGUE  XERSVE
!!C***SUBSIDIARY
!!C***PURPOSE  Record that an error has occurred.
!!C***LIBRARY   SLATEC (XERROR)
!!C***CATEGORY  R3
!!C***TYPE      ALL (XERSVE-A)
!!C***KEYWORDS  ERROR, XERROR
!!C***AUTHOR  Jones, R. E., (SNLA)
!!C***DESCRIPTION
!!C
!!C *Usage:
!!C
!!C        INTEGER  KFLAG, NERR, LEVEL, ICOUNT
!!C        CHARACTER * (len) LIBRAR, SUBROU, MESSG
!!C
!!C        CALL XERSVE (LIBRAR, SUBROU, MESSG, KFLAG, NERR, LEVEL, ICOUNT)
!!C
!!C *Arguments:
!!C
!!C        LIBRAR :IN    is the library that the message is from.
!!C        SUBROU :IN    is the subroutine that the message is from.
!!C        MESSG  :IN    is the message to be saved.
!!C        KFLAG  :IN    indicates the action to be performed.
!!C                      when KFLAG > 0, the message in MESSG is saved.
!!C                      when KFLAG=0 the tables will be dumped and
!!C                      cleared.
!!C                      when KFLAG < 0, the tables will be dumped and
!!C                      not cleared.
!!C        NERR   :IN    is the error number.
!!C        LEVEL  :IN    is the error severity.
!!C        ICOUNT :OUT   the number of times this message has been seen,
!!C                      or zero if the table has overflowed and does not
!!C                      contain this message specifically.  When KFLAG=0,
!!C                      ICOUNT will not be altered.
!!C
!!C *Description:
!!C
!!C   Record that this error occurred and possibly dump and clear the
!!C   tables.
!!C
!!C***REFERENCES  R. E. Jones and D. K. Kahaner, XERROR, the SLATEC
!!C                 Error-handling Package, SAND82-0800, Sandia
!!C                 Laboratories, 1982.
!!C***ROUTINES CALLED  I1MACH, XGETUA
!!C***REVISION HISTORY  (YYMMDD)
!!C   800319  DATE WRITTEN
!!C   861211  REVISION DATE from Version 3.2
!!C   891214  Prologue converted to Version 4.0 format.  (BAB)
!!C   900413  Routine modified to remove reference to KFLAG.  (WRB)
!!C   900510  Changed to add LIBRARY NAME and SUBROUTINE to calling
!!C           sequence, use IF-THEN-ELSE, make number of saved entries
!!C           easily changeable, changed routine name from XERSAV to
!!C           XERSVE.  (RWC)
!!C   910626  Added LIBTAB and SUBTAB to SAVE statement.  (BKS)
!!C   920501  Reformatted the REFERENCES section.  (WRB)
!!C***END PROLOGUE  XERSVE
      PARAMETER (LENTAB=10)
      INTEGER LUN(5),I
      CHARACTER*(*) LIBRAR, SUBROU, MESSG
      CHARACTER*8  LIBTAB(LENTAB), SUBTAB(LENTAB), LIB, SUB
      CHARACTER*20 MESTAB(LENTAB), MES
      DIMENSION NERTAB(LENTAB), LEVTAB(LENTAB), KOUNT(LENTAB)
      SAVE LIBTAB, SUBTAB, MESTAB, NERTAB, LEVTAB, KOUNT, KOUNTX, NMSG
      DATA KOUNTX/0/, NMSG/0/
!!C***FIRST EXECUTABLE STATEMENT  XERSVE
!!C
      IF (KFLAG.LE.0) THEN
!!C
!!C        Dump the table.
!!C
         IF (NMSG.EQ.0) RETURN
!!C
!!C        Print to each unit.
!!C
         CALL XGETUA (LUN, NUNIT)
         DO 20 KUNIT = 1,NUNIT
            IUNIT = LUN(KUNIT)
            IF (IUNIT.EQ.0) IUNIT = I1MACH(4)
!!C
!!C           Print the table header.
!!C
            WRITE (IUNIT,9000)
!!C
!!C           Print body of table.
!!C
            DO 10 I = 1,NMSG
               WRITE (IUNIT,9010) LIBTAB(I), SUBTAB(I), MESTAB(I),   &
                 NERTAB(I),LEVTAB(I),KOUNT(I)
   10       CONTINUE
!!C
!!C           Print number of other errors.
!!C
            IF (KOUNTX.NE.0) WRITE (IUNIT,9020) KOUNTX
            WRITE (IUNIT,9030)
   20    CONTINUE
!!C
!!C        Clear the error tables.
!!C
         IF (KFLAG.EQ.0) THEN
            NMSG = 0
            KOUNTX = 0
         ENDIF
      ELSE
!!C
!!C        PROCESS A MESSAGE...
!!C        SEARCH FOR THIS MESSG, OR ELSE AN EMPTY SLOT FOR THIS MESSG,
!!C        OR ELSE DETERMINE THAT THE ERROR TABLE IS FULL.
!!C
         LIB = LIBRAR
         SUB = SUBROU
         MES = MESSG
         DO 30 I = 1,NMSG
            IF (LIB.EQ.LIBTAB(I) .AND. SUB.EQ.SUBTAB(I) .AND.    &
              MES.EQ.MESTAB(I) .AND. NERR.EQ.NERTAB(I) .AND.		 &
              LEVEL.EQ.LEVTAB(I)) THEN
                  KOUNT(I) = KOUNT(I) + 1
                  ICOUNT = KOUNT(I)
                  RETURN
            ENDIF
   30    CONTINUE
!!C
         IF (NMSG.LT.LENTAB) THEN
!!C
!!C           Empty slot found for new message.
!!C
            NMSG = NMSG + 1
            LIBTAB(I) = LIB
            SUBTAB(I) = SUB
            MESTAB(I) = MES
            NERTAB(I) = NERR
            LEVTAB(I) = LEVEL
            KOUNT (I) = 1
            ICOUNT    = 1
         ELSE
!!C
!!C           Table is full.
!!C
            KOUNTX = KOUNTX+1
            ICOUNT = 0
         ENDIF
      ENDIF
      RETURN
!!C
!!C     Formats.
!!C
 9000 FORMAT ('0          ERROR MESSAGE SUMMARY' /&
        &' LIBRARY    SUBROUTINE MESSAGE START             NERR',&
        &'     LEVEL     COUNT')
 9010 FORMAT (1X,A,3X,A,3X,A,3I10)
 9020 FORMAT ('0OTHER ERRORS NOT INDIVIDUALLY TABULATED = ', I10)
 9030 FORMAT (1X)
      END subroutine XERSVE 

!__________________________________________________________________

!*DECK XGETUA
      SUBROUTINE XGETUA (IUNITA, N)
!!C***BEGIN PROLOGUE  XGETUA
!!C***PURPOSE  Return unit number(s) to which error messages are being
!!C            sent.
!!C***LIBRARY   SLATEC (XERROR)
!!C***CATEGORY  R3C
!!C***TYPE      ALL (XGETUA-A)
!!C***KEYWORDS  ERROR, XERROR
!!C***AUTHOR  Jones, R. E., (SNLA)
!!C***DESCRIPTION
!!C
!!C     Abstract
!!C        XGETUA may be called to determine the unit number or numbers
!!C        to which error messages are being sent.
!!C        These unit numbers may have been set by a call to XSETUN,
!!C        or a call to XSETUA, or may be a default value.
!!C
!!C     Description of Parameters
!!C      --Output--
!!C        IUNIT - an array of one to five unit numbers, depending
!!C                on the value of N.  A value of zero refers to the
!!C                default unit, as defined by the I1MACH machine
!!C                constant routine.  Only IUNIT(1),...,IUNIT(N) are
!!C                defined by XGETUA.  The values of IUNIT(N+1),...,
!!C                IUNIT(5) are not defined (for N .LT. 5) or altered
!!C                in any way by XGETUA.
!!C        N     - the number of units to which copies of the
!!C                error messages are being sent.  N will be in the
!!C                range from 1 to 5.
!!C
!!C***REFERENCES  R. E. Jones and D. K. Kahaner, XERROR, the SLATEC
!!C                 Error-handling Package, SAND82-0800, Sandia
!!C                 Laboratories, 1982.
!!C***ROUTINES CALLED  J4SAVE
!!C***REVISION HISTORY  (YYMMDD)
!!C   790801  DATE WRITTEN
!!C   861211  REVISION DATE from Version 3.2
!!C   891214  Prologue converted to Version 4.0 format.  (BAB)
!!C   920501  Reformatted the REFERENCES section.  (WRB)
!!C***END PROLOGUE  XGETUA
      DIMENSION IUNITA(5)
!!C***FIRST EXECUTABLE STATEMENT  XGETUA
      N = J4SAVE(5,0,.FALSE.)
      DO 30 I=1,N
         INDEX = I+4
         IF (I.EQ.1) INDEX = 3
         IUNITA(I) = J4SAVE(INDEX,0,.FALSE.)
   30 CONTINUE
      RETURN

      END subroutine XGETUA 

!______________________________________________________________


!*DECK D1MACH

      DOUBLE PRECISION FUNCTION D1MACH(I)

!!C***BEGIN PROLOGUE  D1MACH
!!C***PURPOSE  Return floating point machine dependent constants.
!!C***LIBRARY   SLATEC
!!C***CATEGORY  R1
!!C***TYPE      DOUBLE PRECISION (R1MACH-S, D1MACH-D)
!!C***KEYWORDS  MACHINE CONSTANTS
!!C***AUTHOR  Fox, P. A., (Bell Labs)
!!C           Hall, A. D., (Bell Labs)
!!C           Schryer, N. L., (Bell Labs)
!!C***DESCRIPTION
!!C
!!C   D1MACH can be used to obtain machine-dependent parameters for the
!!C   local machine environment.  It is a function subprogram with one
!!C   (input) argument, and can be referenced as follows:
!!C
!!C        D = D1MACH(I)
!!C
!!C   where I=1,...,5.  The (output) value of D above is determined by
!!C   the (input) value of I.  The results for various values of I are
!!C   discussed below.
!!C
!!C   D1MACH( 1) = B**(EMIN-1), the smallest positive magnitude.
!!C   D1MACH( 2) = B**EMAX*(1 - B**(-T)), the largest magnitude.
!!C   D1MACH( 3) = B**(-T), the smallest relative spacing.
!!C   D1MACH( 4) = B**(1-T), the largest relative spacing.
!!C   D1MACH( 5) = LOG10(B)
!!C
!!C   Assume double precision numbers are represented in the T-digit,
!!C   base-B form
!!C
!!C              sign (B**E)*( (X(1)/B) + ... + (X(T)/B**T) )
!!C
!!C   where 0 .LE. X(I) .LT. B for I=1,...,T, 0 .LT. X(1), and
!!C   EMIN .LE. E .LE. EMAX.
!!C
!!C   The values of B, T, EMIN and EMAX are provided in I1MACH as
!!C   follows:
!!C   I1MACH(10) = B, the base.
!!C   I1MACH(14) = T, the number of base-B digits.
!!C   I1MACH(15) = EMIN, the smallest exponent E.
!!C   I1MACH(16) = EMAX, the largest exponent E.
!!C
!!C   To alter this function for a particular environment, the desired
!!C   set of DATA statements should be activated by removing the !!C from
!!C   column 1.  Also, the values of D1MACH(1) - D1MACH(4) should be
!!C   checked for consistency with the local operating system.
!!C
!!C***REFERENCES  P. A. Fox, A. D. Hall and N. L. Schryer, Framework for
!!C                 a portable library, ACM Transactions on Mathematical
!!C                 Software 4, 2 (June 1978), pp. 177-188.
!!C***ROUTINES CALLED  XERMSG
!!C***REVISION HISTORY  (YYMMDD)
!!C   750101  DATE WRITTEN
!!C   890213  REVISION DATE from Version 3.2
!!C   891214  Prologue converted to Version 4.0 format.  (BAB)
!!C   900315  CALLs to XERROR changed to CALLs to XERMSG.  (THJ)
!!C   900618  Added DEC RISC constants.  (WRB)
!!C   900723  Added IBM RS 6000 constants.  (WRB)
!!C   900911  Added SUN 386i constants.  (WRB)
!!C   910710  Added HP 730 constants.  (SMR)
!!C   911114  Added Convex IEEE constants.  (WRB)
!!C   920121  Added SUN -r8 compiler option constants.  (WRB)
!!C   920229  Added Touchstone Delta i860 constants.  (WRB)
!!C   920501  Reformatted the REFERENCES section.  (WRB)
!!C   920625  Added CONVEX -p8 and -pd8 compiler option constants.
!!C           (BKS, WRB)
!!C   930201  Added DEC Alpha and SGI constants.  (RWC and WRB)
!!C***END PROLOGUE  D1MACH
!!C
      INTEGER SMALL(4)
      INTEGER LARGE(4)
      INTEGER RIGHT(4)
      INTEGER DIVER(4)
      INTEGER LOG10(4)
!!C
      DOUBLE PRECISION DMACH(5)
      SAVE DMACH
!!C
      DMACH( 1) =  2.23D-308
      DMACH( 2) = 1.79D+308
      DMACH( 3) = 1.11D-16
      DMACH( 4) = 2.22D-16
      DMACH( 5) = 0.301029995663981195D0

!!C      EQUIVALENCE (DMACH(1),SMALL(1))
!!C      EQUIVALENCE (DMACH(2),LARGE(1))
!!C      EQUIVALENCE (DMACH(3),RIGHT(1))
!!C      EQUIVALENCE (DMACH(4),DIVER(1))
!!C      EQUIVALENCE (DMACH(5),LOG10(1))

!!C     MACHINE CONSTANTS FOR THE CRAY
!!C
!!C     DATA SMALL(1) / 201354000000000000000B /
!!C     DATA SMALL(2) / 000000000000000000000B /
!!C     DATA LARGE(1) / 577767777777777777777B /
!!C     DATA LARGE(2) / 000007777777777777774B /
!!C     DATA RIGHT(1) / 376434000000000000000B /
!!C     DATA RIGHT(2) / 000000000000000000000B /
!!C     DATA DIVER(1) / 376444000000000000000B /
!!C     DATA DIVER(2) / 000000000000000000000B /
!!C     DATA LOG10(1) / 377774642023241175717B /
!!C     DATA LOG10(2) / 000007571421742254654B /
!!C
!!C     MACHINE CONSTANTS FOR THE IBM 360/370 SERIES,
!!C     THE XEROX SIGMA 5/7/9, THE SEL SYSTEMS 85/86, AND
!!C     THE PERKIN ELMER (INTERDATA) 7/32.
!!C
!!C     DATA SMALL(1), SMALL(2) / Z00100000, Z00000000 /
!!C     DATA LARGE(1), LARGE(2) / Z7FFFFFFF, ZFFFFFFFF /
!!C     DATA RIGHT(1), RIGHT(2) / Z33100000, Z00000000 /
!!C     DATA DIVER(1), DIVER(2) / Z34100000, Z00000000 /
!!C     DATA LOG10(1), LOG10(2) / Z41134413, Z509F79FF /
!!C
!!C     MACHINE CONSTANTS FOR THE IBM PC
!!C     ASSUMES THAT ALL ARITHMETIC IS DONE IN DOUBLE PRECISION
!!C     ON 8088, I.E., NOT IN 80 BIT FORM FOR THE 8087.
!!C
      DATA SMALL(1) / 2.23D-308  /
!!C      DATA LARGE(1) / 1.79D+308  /
      DATA LARGE(1) / 1790000.  /
      DATA RIGHT(1) / 1.11D-16   /
      DATA DIVER(1) / 2.22D-16   /
      DATA LOG10(1) / 0.301029995663981195D0 /
!!C
!!C     MACHINE CONSTANTS FOR THE IBM RS 6000
!!C
!!C     DATA DMACH(1) / Z'0010000000000000' /
!!C     DATA DMACH(2) / Z'7FEFFFFFFFFFFFFF' /
!!C     DATA DMACH(3) / Z'3CA0000000000000' /
!!C     DATA DMACH(4) / Z'3CB0000000000000' /
!!C     DATA DMACH(5) / Z'3FD34413509F79FF' /
!!C
!!C     MACHINE CONSTANTS FOR THE SILICON GRAPHICS
!!C
!!C     DATA SMALL(1), SMALL(2) / Z'00100000', Z'00000000' /
!!C     DATA LARGE(1), LARGE(2) / Z'7FEFFFFF', Z'FFFFFFFF' /
!!C     DATA RIGHT(1), RIGHT(2) / Z'3CA00000', Z'00000000' /
!!C     DATA DIVER(1), DIVER(2) / Z'3CB00000', Z'00000000' /
!!C     DATA LOG10(1), LOG10(2) / Z'3FD34413', Z'509F79FF' /
!!C
!!C
!!C***FIRST EXECUTABLE STATEMENT  D1MACH
      IF (I .LT. 1 .OR. I .GT. 5) CALL XERMSG ('SLATEC', 'D1MACH',  &
        'I OUT OF BOUNDS', 1, 2)
!!C
      D1MACH = DMACH(I)

      RETURN
      END function D1MACH 

!_____________________________________________________________

!*DECK D1MACH

      DOUBLE PRECISION FUNCTION ddD1MACH(I)

!!C***BEGIN PROLOGUE  D1MACH
!!C***PURPOSE  Return floating point machine dependent constants.
!!C***LIBRARY   SLATEC
!!C***CATEGORY  R1
!!C***TYPE      DOUBLE PRECISION (R1MACH-S, D1MACH-D)
!!C***KEYWORDS  MACHINE CONSTANTS
!!C***AUTHOR  Fox, P. A., (Bell Labs)
!!C           Hall, A. D., (Bell Labs)
!!C           Schryer, N. L., (Bell Labs)
!!C***DESCRIPTION
!!C
!!C   D1MACH can be used to obtain machine-dependent parameters for the
!!C   local machine environment.  It is a function subprogram with one
!!C   (input) argument, and can be referenced as follows:
!!C
!!C        D = D1MACH(I)
!!C
!!C   where I=1,...,5.  The (output) value of D above is determined by
!!C   the (input) value of I.  The results for various values of I are
!!C   discussed below.
!!C
!!C   D1MACH( 1) = B**(EMIN-1), the smallest positive magnitude.
!!C   D1MACH( 2) = B**EMAX*(1 - B**(-T)), the largest magnitude.
!!C   D1MACH( 3) = B**(-T), the smallest relative spacing.
!!C   D1MACH( 4) = B**(1-T), the largest relative spacing.
!!C   D1MACH( 5) = LOG10(B)
!!C
!!C   Assume double precision numbers are represented in the T-digit,
!!C   base-B form
!!C
!!C              sign (B**E)*( (X(1)/B) + ... + (X(T)/B**T) )
!!C
!!C   where 0 .LE. X(I) .LT. B for I=1,...,T, 0 .LT. X(1), and
!!C   EMIN .LE. E .LE. EMAX.
!!C
!!C   The values of B, T, EMIN and EMAX are provided in I1MACH as
!!C   follows:
!!C   I1MACH(10) = B, the base.
!!C   I1MACH(14) = T, the number of base-B digits.
!!C   I1MACH(15) = EMIN, the smallest exponent E.
!!C   I1MACH(16) = EMAX, the largest exponent E.
!!C
!!C   To alter this function for a particular environment, the desired
!!C   set of DATA statements should be activated by removing the !!C from
!!C   column 1.  Also, the values of D1MACH(1) - D1MACH(4) should be
!!C   checked for consistency with the local operating system.
!!C
!!C***REFERENCES  P. A. Fox, A. D. Hall and N. L. Schryer, Framework for
!!C                 a portable library, ACM Transactions on Mathematical
!!C                 Software 4, 2 (June 1978), pp. 177-188.
!!C***ROUTINES CALLED  XERMSG
!!C***REVISION HISTORY  (YYMMDD)
!!C   750101  DATE WRITTEN
!!C   890213  REVISION DATE from Version 3.2
!!C   891214  Prologue converted to Version 4.0 format.  (BAB)
!!C   900315  CALLs to XERROR changed to CALLs to XERMSG.  (THJ)
!!C   900618  Added DEC RISC constants.  (WRB)
!!C   900723  Added IBM RS 6000 constants.  (WRB)
!!C   900911  Added SUN 386i constants.  (WRB)
!!C   910710  Added HP 730 constants.  (SMR)
!!C   911114  Added Convex IEEE constants.  (WRB)
!!C   920121  Added SUN -r8 compiler option constants.  (WRB)
!!C   920229  Added Touchstone Delta i860 constants.  (WRB)
!!C   920501  Reformatted the REFERENCES section.  (WRB)
!!C   920625  Added CONVEX -p8 and -pd8 compiler option constants.
!!C           (BKS, WRB)
!!C   930201  Added DEC Alpha and SGI constants.  (RWC and WRB)
!!C***END PROLOGUE  D1MACH
!!C
      INTEGER SMALL(4)
      INTEGER LARGE(4)
      INTEGER RIGHT(4)
      INTEGER DIVER(4)
      INTEGER LOG10(4)
!!C
      DOUBLE PRECISION DMACH(5)
      SAVE DMACH
!!C
      DMACH( 1) =  2.23D-308
      DMACH( 2) = 1.79D+308
      DMACH( 3) = 1.11D-16
      DMACH( 4) = 2.22D-16
      DMACH( 5) = 0.301029995663981195D0

!!C      EQUIVALENCE (DMACH(1),SMALL(1))
!!C      EQUIVALENCE (DMACH(2),LARGE(1))
!!C      EQUIVALENCE (DMACH(3),RIGHT(1))
!!C      EQUIVALENCE (DMACH(4),DIVER(1))
!!C      EQUIVALENCE (DMACH(5),LOG10(1))

!!C     MACHINE CONSTANTS FOR THE CRAY
!!C
!!C     DATA SMALL(1) / 201354000000000000000B /
!!C     DATA SMALL(2) / 000000000000000000000B /
!!C     DATA LARGE(1) / 577767777777777777777B /
!!C     DATA LARGE(2) / 000007777777777777774B /
!!C     DATA RIGHT(1) / 376434000000000000000B /
!!C     DATA RIGHT(2) / 000000000000000000000B /
!!C     DATA DIVER(1) / 376444000000000000000B /
!!C     DATA DIVER(2) / 000000000000000000000B /
!!C     DATA LOG10(1) / 377774642023241175717B /
!!C     DATA LOG10(2) / 000007571421742254654B /
!!C
!!C     MACHINE CONSTANTS FOR THE IBM 360/370 SERIES,
!!C     THE XEROX SIGMA 5/7/9, THE SEL SYSTEMS 85/86, AND
!!C     THE PERKIN ELMER (INTERDATA) 7/32.
!!C
!!C     DATA SMALL(1), SMALL(2) / Z00100000, Z00000000 /
!!C     DATA LARGE(1), LARGE(2) / Z7FFFFFFF, ZFFFFFFFF /
!!C     DATA RIGHT(1), RIGHT(2) / Z33100000, Z00000000 /
!!C     DATA DIVER(1), DIVER(2) / Z34100000, Z00000000 /
!!C     DATA LOG10(1), LOG10(2) / Z41134413, Z509F79FF /
!!C
!!C     MACHINE CONSTANTS FOR THE IBM PC
!!C     ASSUMES THAT ALL ARITHMETIC IS DONE IN DOUBLE PRECISION
!!C     ON 8088, I.E., NOT IN 80 BIT FORM FOR THE 8087.
!!C
      DATA SMALL(1) / 2.23D-308  /
!!C      DATA LARGE(1) / 1.79D+308  /
      DATA LARGE(1) / 1790000.  /
      DATA RIGHT(1) / 1.11D-16   /
      DATA DIVER(1) / 2.22D-16   /
      DATA LOG10(1) / 0.301029995663981195D0 /
!!C
!!C     MACHINE CONSTANTS FOR THE IBM RS 6000
!!C
!!C     DATA DMACH(1) / Z'0010000000000000' /
!!C     DATA DMACH(2) / Z'7FEFFFFFFFFFFFFF' /
!!C     DATA DMACH(3) / Z'3CA0000000000000' /
!!C     DATA DMACH(4) / Z'3CB0000000000000' /
!!C     DATA DMACH(5) / Z'3FD34413509F79FF' /
!!C
!!C     MACHINE CONSTANTS FOR THE SILICON GRAPHICS
!!C
!!C     DATA SMALL(1), SMALL(2) / Z'00100000', Z'00000000' /
!!C     DATA LARGE(1), LARGE(2) / Z'7FEFFFFF', Z'FFFFFFFF' /
!!C     DATA RIGHT(1), RIGHT(2) / Z'3CA00000', Z'00000000' /
!!C     DATA DIVER(1), DIVER(2) / Z'3CB00000', Z'00000000' /
!!C     DATA LOG10(1), LOG10(2) / Z'3FD34413', Z'509F79FF' /
!!C
!!C
!!C***FIRST EXECUTABLE STATEMENT  D1MACH
      IF (I .LT. 1 .OR. I .GT. 5) CALL XERMSG ('SLATEC', 'ddD1MACH',  &
        'I OUT OF BOUNDS', 1, 2)
!!C
      ddD1MACH = DMACH(I)

      RETURN
      END function ddD1MACH 




!__________________________________________________________________

!*DECK I1MACH
      INTEGER FUNCTION I1MACH (I)
!!C***BEGIN PROLOGUE  I1MACH
!!C***PURPOSE  Return integer machine dependent constants.
!!C***LIBRARY   SLATEC
!!C***CATEGORY  R1
!!C***TYPE      INTEGER (I1MACH-I)
!!C***KEYWORDS  MACHINE CONSTANTS
!!C***AUTHOR  Fox, P. A., (Bell Labs)
!!C           Hall, A. D., (Bell Labs)
!!C           Schryer, N. L., (Bell Labs)
!!C***DESCRIPTION
!!C
!!C   I1MACH can be used to obtain machine-dependent parameters for the
!!C   local machine environment.  It is a function subprogram with one
!!C   (input) argument and can be referenced as follows:
!!C
!!C        K = I1MACH(I)
!!C
!!C   where I=1,...,16.  The (output) value of K above is determined by
!!C   the (input) value of I.  The results for various values of I are
!!C   discussed below.
!!C
!!C   I/O unit numbers:
!!C     I1MACH( 1) = the standard input unit.
!!C     I1MACH( 2) = the standard output unit.
!!C     I1MACH( 3) = the standard punch unit.
!!C     I1MACH( 4) = the standard error message unit.
!!C
!!C   Words:
!!C     I1MACH( 5) = the number of bits per integer storage unit.
!!C     I1MACH( 6) = the number of characters per integer storage unit.
!!C
!!C   Integers:
!!C     assume integers are represented in the S-digit, base-A form
!!C
!!C                sign ( X(S-1)*A**(S-1) + ... + X(1)*A + X(0) )
!!C
!!C                where 0 .LE. X(I) .LT. A for I=0,...,S-1.
!!C     I1MACH( 7) = A, the base.
!!C     I1MACH( 8) = S, the number of base-A digits.
!!C     I1MACH( 9) = A**S - 1, the largest magnitude.
!!C
!!C   Floating-Point Numbers:
!!C     Assume floating-point numbers are represented in the T-digit,
!!C     base-B form
!!C                sign (B**E)*( (X(1)/B) + ... + (X(T)/B**T) )
!!C
!!C                where 0 .LE. X(I) .LT. B for I=1,...,T,
!!C                0 .LT. X(1), and EMIN .LE. E .LE. EMAX.
!!C     I1MACH(10) = B, the base.
!!C
!!C   Single-Precision:
!!C     I1MACH(11) = T, the number of base-B digits.
!!C     I1MACH(12) = EMIN, the smallest exponent E.
!!C     I1MACH(13) = EMAX, the largest exponent E.
!!C
!!C   Double-Precision:
!!C     I1MACH(14) = T, the number of base-B digits.
!!C     I1MACH(15) = EMIN, the smallest exponent E.
!!C     I1MACH(16) = EMAX, the largest exponent E.
!!C
!!C   To alter this function for a particular environment, the desired
!!C   set of DATA statements should be activated by removing the !!C from
!!C   column 1.  Also, the values of I1MACH(1) - I1MACH(4) should be
!!C   checked for consistency with the local operating system.
!!C
!!C***REFERENCES  P. A. Fox, A. D. Hall and N. L. Schryer, Framework for
!!C                 a portable library, ACM Transactions on Mathematical
!!C                 Software 4, 2 (June 1978), pp. 177-188.
!!C***ROUTINES CALLED  (NONE)
!!C***REVISION HISTORY  (YYMMDD)
!!C   750101  DATE WRITTEN
!!C   891012  Added VAX G-floating constants.  (WRB)
!!C   891012  REVISION DATE from Version 3.2
!!C   891214  Prologue converted to Version 4.0 format.  (BAB)
!!C   900618  Added DEC RISC constants.  (WRB)
!!C   900723  Added IBM RS 6000 constants.  (WRB)
!!C   901009  Correct I1MACH(7) for IBM Mainframes. Should be 2 not 16.
!!C           (RWC)
!!C   910710  Added HP 730 constants.  (SMR)
!!C   911114  Added Convex IEEE constants.  (WRB)
!!C   920121  Added SUN -r8 compiler option constants.  (WRB)
!!C   920229  Added Touchstone Delta i860 constants.  (WRB)
!!C   920501  Reformatted the REFERENCES section.  (WRB)
!!C   920625  Added Convex -p8 and -pd8 compiler option constants.
!!C           (BKS, WRB)
!!C   930201  Added DEC Alpha and SGI constants.  (RWC and WRB)
!!C   930618  Corrected I1MACH(5) for Convex -p8 and -pd8 compiler
!!C           options.  (DWL, RWC and WRB).
!!C***END PROLOGUE  I1MACH
!!C
      INTEGER IMACH(16),OUTPUT
      SAVE IMACH
      EQUIVALENCE (IMACH(4),OUTPUT)
!!C     MACHINE CONSTANTS FOR THE CRAY
!!C     USING THE 46 BIT INTEGER COMPILER OPTION
!!C
!!C     DATA IMACH( 1) /        100 /
!!C     DATA IMACH( 2) /        101 /
!!C     DATA IMACH( 3) /        102 /
!!C     DATA IMACH( 4) /        101 /
!!C     DATA IMACH( 5) /         64 /
!!C     DATA IMACH( 6) /          8 /
!!C     DATA IMACH( 7) /          2 /
!!C     DATA IMACH( 8) /         46 /
!!C     DATA IMACH( 9) / 1777777777777777B /
!!C     DATA IMACH(10) /          2 /
!!C     DATA IMACH(11) /         47 /
!!C     DATA IMACH(12) /      -8189 /
!!C     DATA IMACH(13) /       8190 /
!!C     DATA IMACH(14) /         94 /
!!C     DATA IMACH(15) /      -8099 /
!!C     DATA IMACH(16) /       8190 /
!!C
!!C     MACHINE CONSTANTS FOR THE CRAY
!!C     USING THE 64 BIT INTEGER COMPILER OPTION
!!C
!!C     DATA IMACH( 1) /        100 /
!!C     DATA IMACH( 2) /        101 /
!!C     DATA IMACH( 3) /        102 /
!!C     DATA IMACH( 4) /        101 /
!!C     DATA IMACH( 5) /         64 /
!!C     DATA IMACH( 6) /          8 /
!!C     DATA IMACH( 7) /          2 /
!!C     DATA IMACH( 8) /         63 /
!!C     DATA IMACH( 9) / 777777777777777777777B /
!!C     DATA IMACH(10) /          2 /
!!C     DATA IMACH(11) /         47 /
!!C     DATA IMACH(12) /      -8189 /
!!C     DATA IMACH(13) /       8190 /
!!C     DATA IMACH(14) /         94 /
!!C     DATA IMACH(15) /      -8099 /
!!C     DATA IMACH(16) /       8190 /
!!C
!!C
!!C     MACHINE CONSTANTS FOR THE IBM PC

      DATA IMACH( 1) /          5 /
      DATA IMACH( 2) /          6 /
      DATA IMACH( 3) /          0 /
      DATA IMACH( 4) /          0 /
      DATA IMACH( 5) /         32 /
      DATA IMACH( 6) /          4 /
      DATA IMACH( 7) /          2 /
      DATA IMACH( 8) /         31 /
      DATA IMACH( 9) / 2147483647 /
      DATA IMACH(10) /          2 /
      DATA IMACH(11) /         24 /
      DATA IMACH(12) /       -125 /
      DATA IMACH(13) /        127 /
      DATA IMACH(14) /         53 /
      DATA IMACH(15) /      -1021 /
      DATA IMACH(16) /       1023 /
!!C
!!C     MACHINE CONSTANTS FOR THE IBM RS 6000
!!C
!!C     DATA IMACH( 1) /          5 /
!!C     DATA IMACH( 2) /          6 /
!!C     DATA IMACH( 3) /          6 /
!!C     DATA IMACH( 4) /          0 /
!!C     DATA IMACH( 5) /         32 /
!!C     DATA IMACH( 6) /          4 /
!!C     DATA IMACH( 7) /          2 /
!!C     DATA IMACH( 8) /         31 /
!!C     DATA IMACH( 9) / 2147483647 /
!!C     DATA IMACH(10) /          2 /
!!C     DATA IMACH(11) /         24 /
!!C     DATA IMACH(12) /       -125 /
!!C     DATA IMACH(13) /        128 /
!!C     DATA IMACH(14) /         53 /
!!C     DATA IMACH(15) /      -1021 /
!!C     DATA IMACH(16) /       1024 /
!!C
!!C
!!C
!!C***FIRST EXECUTABLE STATEMENT  I1MACH
      IF (I .LT. 1  .OR.  I .GT. 16) GO TO 10
!!C
      I1MACH = IMACH(I)
      RETURN
!!C
 10   CONTINUE
      WRITE (UNIT = OUTPUT, FMT = 9000)
 9000 FORMAT ('1ERROR    1 IN I1MACH - I OUT OF BOUNDS')
!!C
!!C     CALL FDUMP
!!C
      STOP
      END function I1MACH

!______________________________________________________________

!*DECK D1MPYQ
      SUBROUTINE D1MPYQ (M, N, A, LDA, V, W)
!!C***BEGIN PROLOGUE  D1MPYQ
!!C***SUBSIDIARY
!!C***PURPOSE  Subsidiary to DNSQ and DNSQE
!!C***LIBRARY   SLATEC
!!C***TYPE      DOUBLE PRECISION (R1MPYQ-S, D1MPYQ-D)
!!C***AUTHOR  (UNKNOWN)
!!C***DESCRIPTION
!!C
!!C     Given an M by N matrix A, this subroutine computes A*Q where
!!C     Q is the product of 2*(N - 1) transformations
!!C
!!C           GV(N-1)*...*GV(1)*GW(1)*...*GW(N-1)
!!C
!!C     and GV(I), GW(I) are Givens rotations in the (I,N) plane which
!!C     eliminate elements in the I-th and N-th planes, respectively.
!!C     Q itself is not given, rather the information to recover the
!!C     GV, GW rotations is supplied.
!!C
!!C     The SUBROUTINE statement is
!!C
!!C       SUBROUTINE D1MPYQ(M,N,A,LDA,V,W)
!!C
!!C     where
!!C
!!C       M is a positive integer input variable set to the number
!!C         of rows of A.
!!C
!!C       N IS a positive integer input variable set to the number
!!C         of columns of A.
!!C
!!C       A is an M by N array. On input A must contain the matrix
!!C         to be postmultiplied by the orthogonal matrix Q
!!C         described above. On output A*Q has replaced A.
!!C
!!C       LDA is a positive integer input variable not less than M
!!C         which specifies the leading dimension of the array A.
!!C
!!C       V is an input array of length N. V(I) must contain the
!!C         information necessary to recover the Givens rotation GV(I)
!!C         described above.
!!C
!!C       W is an input array of length N. W(I) must contain the
!!C         information necessary to recover the Givens rotation GW(I)
!!C         described above.
!!C
!!C***SEE ALSO  DNSQ, DNSQE
!!C***ROUTINES CALLED  (NONE)
!!C***REVISION HISTORY  (YYMMDD)
!!C   800301  DATE WRITTEN
!!C   890531  Changed all specific intrinsics to generic.  (WRB)
!!C   890831  Modified array declarations.  (WRB)
!!C   891214  Prologue converted to Version 4.0 format.  (BAB)
!!C   900326  Removed duplicate information from DESCRIPTION section.
!!C           (WRB)
!!C   900328  Added TYPE section.  (WRB)
!!C***END PROLOGUE  D1MPYQ
      INTEGER I, J, LDA, M, N, NM1, NMJ
      DOUBLE PRECISION A(LDA,*), COS, ONE, SIN, TEMP, V(*), W(*)
      SAVE ONE
      DATA ONE /1.0D0/
!!C
!!C     APPLY THE FIRST SET OF GIVENS ROTATIONS TO A.
!!C
!!C***FIRST EXECUTABLE STATEMENT  D1MPYQ
      NM1 = N - 1
      IF (NM1 .LT. 1) GO TO 50
      DO 20 NMJ = 1, NM1
         J = N - NMJ
         IF (ABS(V(J)) .GT. ONE) COS = ONE/V(J)
         IF (ABS(V(J)) .GT. ONE) SIN = SQRT(ONE-COS**2)
         IF (ABS(V(J)) .LE. ONE) SIN = V(J)
         IF (ABS(V(J)) .LE. ONE) COS = SQRT(ONE-SIN**2)
         DO 10 I = 1, M
            TEMP = COS*A(I,J) - SIN*A(I,N)
            A(I,N) = SIN*A(I,J) + COS*A(I,N)
            A(I,J) = TEMP
   10       CONTINUE
   20    CONTINUE
!!C
!!C     APPLY THE SECOND SET OF GIVENS ROTATIONS TO A.
!!C
      DO 40 J = 1, NM1
         IF (ABS(W(J)) .GT. ONE) COS = ONE/W(J)
         IF (ABS(W(J)) .GT. ONE) SIN = SQRT(ONE-COS**2)
         IF (ABS(W(J)) .LE. ONE) SIN = W(J)
         IF (ABS(W(J)) .LE. ONE) COS = SQRT(ONE-SIN**2)
         DO 30 I = 1, M
            TEMP = COS*A(I,J) + SIN*A(I,N)
            A(I,N) = -SIN*A(I,J) + COS*A(I,N)
            A(I,J) = TEMP
   30       CONTINUE
   40    CONTINUE
   50 CONTINUE
      RETURN
!!C
!!C     LAST CARD OF SUBROUTINE D1MPYQ.
!!C
      END subroutine D1MPYQ 

!__________________________________________________________

!*DECK D1UPDT
      SUBROUTINE D1UPDT (M, N, S, LS, U, V, W, SING)
!!C***BEGIN PROLOGUE  D1UPDT
!!C***SUBSIDIARY
!!C***PURPOSE  Subsidiary to DNSQ and DNSQE
!!C***LIBRARY   SLATEC
!!C***TYPE      DOUBLE PRECISION (R1UPDT-S, D1UPDT-D)
!!C***AUTHOR  (UNKNOWN)
!!C***DESCRIPTION
!!C
!!C     Given an M by N lower trapezoidal matrix S, an M-vector U,
!!C     and an N-vector V, the problem is to determine an
!!C     orthogonal matrix Q such that
!!C
!!C                   t
!!C           (S + U*V )*Q
!!C
!!C     is again lower trapezoidal.
!!C
!!C     This subroutine determines Q as the product of 2*(N - 1)
!!C     transformations
!!C
!!C           GV(N-1)*...*GV(1)*GW(1)*...*GW(N-1)
!!C
!!C     where GV(I), GW(I) are Givens rotations in the (I,N) plane
!!C     which eliminate elements in the I-th and N-th planes,
!!C     respectively. Q itself is not accumulated, rather the
!!C     information to recover the GV, GW rotations is returned.
!!C
!!C     The SUBROUTINE statement is
!!C
!!C       SUBROUTINE D1UPDT(M,N,S,LS,U,V,W,SING)
!!C
!!C     where
!!C
!!C       M is a positive integer input variable set to the number
!!C         of rows of S.
!!C
!!C       N is a positive integer input variable set to the number
!!C         of columns of S. N must not exceed M.
!!C
!!C       S is an array of length LS. On input S must contain the lower
!!C         trapezoidal matrix S stored by columns. On output S contains
!!C         the lower trapezoidal matrix produced as described above.
!!C
!!C       LS is a positive integer input variable not less than
!!C         (N*(2*M-N+1))/2.
!!C
!!C       U is an input array of length M which must contain the
!!C         vector U.
!!C
!!C       V is an array of length N. On input V must contain the vector
!!C         V. On output V(I) contains the information necessary to
!!C         recover the Givens rotation GV(I) described above.
!!C
!!C       W is an output array of length M. W(I) contains information
!!C         necessary to recover the Givens rotation GW(I) described
!!C         above.
!!C
!!C       SING is a LOGICAL output variable. SING is set TRUE if any
!!C         of the diagonal elements of the output S are zero. Otherwise
!!C         SING is set FALSE.
!!C
!!C***SEE ALSO  DNSQ, DNSQE
!!C***ROUTINES CALLED  D1MACH
!!C***REVISION HISTORY  (YYMMDD)
!!C   800301  DATE WRITTEN
!!C   890531  Changed all specific intrinsics to generic.  (WRB)
!!C   890831  Modified array declarations.  (WRB)
!!C   891214  Prologue converted to Version 4.0 format.  (BAB)
!!C   900326  Removed duplicate information from DESCRIPTION section.
!!C           (WRB)
!!C   900328  Added TYPE section.  (WRB)
!!C***END PROLOGUE  D1UPDT
!      DOUBLE PRECISION ddd1mach
      INTEGER I, J, JJ, L, LS, M, N, NM1, NMJ
      DOUBLE PRECISION COS, COTAN, GIANT, ONE, P25, P5, S(*),  &
          SIN, TAN, TAU, TEMP, U(*), V(*), W(*), ZERO
      LOGICAL SING
      SAVE ONE, P5, P25, ZERO
      DATA ONE,P5,P25,ZERO /1.0D0,5.0D-1,2.5D-1,0.0D0/
!!C
!!C     GIANT IS THE LARGEST MAGNITUDE.
!!C
!!C***FIRST EXECUTABLE STATEMENT  D1UPDT
      GIANT = ddd1mach(2)
!!C
!!C     INITIALIZE THE DIAGONAL ELEMENT POINTER.
!!C
      JJ = (N*(2*M - N + 1))/2 - (M - N)
!!C
!!C     MOVE THE NONTRIVIAL PART OF THE LAST COLUMN OF S INTO W.
!!C
      L = JJ
      DO 10 I = N, M
         W(I) = S(L)
         L = L + 1
   10    CONTINUE
!!C
!!C     ROTATE THE VECTOR V INTO A MULTIPLE OF THE N-TH UNIT VECTOR
!!C     IN SUCH A WAY THAT A SPIKE IS INTRODUCED INTO W.
!!C
      NM1 = N - 1
      IF (NM1 .LT. 1) GO TO 70
      DO 60 NMJ = 1, NM1
         J = N - NMJ
         JJ = JJ - (M - J + 1)
         W(J) = ZERO
         IF (V(J) .EQ. ZERO) GO TO 50
!!C
!!C        DETERMINE A GIVENS ROTATION WHICH ELIMINATES THE
!!C        J-TH ELEMENT OF V.
!!C
         IF (ABS(V(N)) .GE. ABS(V(J))) GO TO 20
            COTAN = V(N)/V(J)
            SIN = P5/SQRT(P25+P25*COTAN**2)
            COS = SIN*COTAN
            TAU = ONE
            IF (ABS(COS)*GIANT .GT. ONE) TAU = ONE/COS
            GO TO 30
   20    CONTINUE
            TAN = V(J)/V(N)
            COS = P5/SQRT(P25+P25*TAN**2)
            SIN = COS*TAN
            TAU = SIN
   30    CONTINUE
!!C
!!C        APPLY THE TRANSFORMATION TO V AND STORE THE INFORMATION
!!C        NECESSARY TO RECOVER THE GIVENS ROTATION.
!!C
         V(N) = SIN*V(J) + COS*V(N)
         V(J) = TAU
!!C
!!C        APPLY THE TRANSFORMATION TO S AND EXTEND THE SPIKE IN W.
!!C
         L = JJ
         DO 40 I = J, M
            TEMP = COS*S(L) - SIN*W(I)
            W(I) = SIN*S(L) + COS*W(I)
            S(L) = TEMP
            L = L + 1
   40       CONTINUE
   50    CONTINUE
   60    CONTINUE
   70 CONTINUE
!!C
!!C     ADD THE SPIKE FROM THE RANK 1 UPDATE TO W.
!!C
      DO 80 I = 1, M
         W(I) = W(I) + V(N)*U(I)
   80    CONTINUE
!!C
!!C     ELIMINATE THE SPIKE.
!!C
      SING = .FALSE.
      IF (NM1 .LT. 1) GO TO 140
      DO 130 J = 1, NM1
         IF (W(J) .EQ. ZERO) GO TO 120
!!C
!!C        DETERMINE A GIVENS ROTATION WHICH ELIMINATES THE
!!C        J-TH ELEMENT OF THE SPIKE.
!!C
         IF (ABS(S(JJ)) .GE. ABS(W(J))) GO TO 90
            COTAN = S(JJ)/W(J)
            SIN = P5/SQRT(P25+P25*COTAN**2)
            COS = SIN*COTAN
            TAU = ONE
            IF (ABS(COS)*GIANT .GT. ONE) TAU = ONE/COS
            GO TO 100
   90    CONTINUE
            TAN = W(J)/S(JJ)
            COS = P5/SQRT(P25+P25*TAN**2)
            SIN = COS*TAN
            TAU = SIN
  100    CONTINUE
!!C
!!C        APPLY THE TRANSFORMATION TO S AND REDUCE THE SPIKE IN W.
!!C
         L = JJ
         DO 110 I = J, M
            TEMP = COS*S(L) + SIN*W(I)
            W(I) = -SIN*S(L) + COS*W(I)
            S(L) = TEMP
            L = L + 1
  110       CONTINUE
!!C
!!C        STORE THE INFORMATION NECESSARY TO RECOVER THE
!!C        GIVENS ROTATION.
!!C
         W(J) = TAU
  120    CONTINUE
!!C
!!C        TEST FOR ZERO DIAGONAL ELEMENTS IN THE OUTPUT S.
!!C
         IF (S(JJ) .EQ. ZERO) SING = .TRUE.
         JJ = JJ + (M - J + 1)
  130    CONTINUE
  140 CONTINUE
!!C
!!C     MOVE W BACK INTO THE LAST COLUMN OF THE OUTPUT S.
!!C
      L = JJ
      DO 150 I = N, M
         S(L) = W(I)
         L = L + 1
  150    CONTINUE
      IF (S(JJ) .EQ. ZERO) SING = .TRUE.
      RETURN
!!C
!!C     LAST CARD OF SUBROUTINE D1UPDT.
!!C
      END subroutine D1UPDT 

!_________________________________________________________________

!*DECK DDOGLG
      SUBROUTINE DDOGLG (N, R, LR, DIAG, QTB, DELTA, X, WA1, WA2)
!!C***BEGIN PROLOGUE  DDOGLG
!!C***SUBSIDIARY
!!C***PURPOSE  Subsidiary to DNSQ and DNSQE
!!C***LIBRARY   SLATEC
!!C***TYPE      DOUBLE PRECISION (DOGLEG-S, DDOGLG-D)
!!C***AUTHOR  (UNKNOWN)
!!C***DESCRIPTION
!!C
!!C     Given an M by N matrix A, an N by N nonsingular diagonal
!!C     matrix D, an M-vector B, and a positive number DELTA, the
!!C     problem is to determine the convex combination X of the
!!C     Gauss-Newton and scaled gradient directions that minimizes
!!C     (A*X - B) in the least squares sense, subject to the
!!C     restriction that the Euclidean norm of D*X be at most DELTA.
!!C
!!C     This subroutine completes the solution of the problem
!!C     if it is provided with the necessary information from the
!!C     QR factorization of A. That is, if A = Q*R, where Q has
!!C     orthogonal columns and R is an upper triangular matrix,
!!C     then DDOGLG expects the full upper triangle of R and
!!C     the first N components of (Q transpose)*B.
!!C
!!C     The subroutine statement is
!!C
!!C       SUBROUTINE DDOGLG(N,R,LR,DIAG,QTB,DELTA,X,WA1,WA2)
!!C
!!C     where
!!C
!!C       N is a positive integer input variable set to the order of R.
!!C
!!C       R is an input array of length LR which must contain the upper
!!C         triangular matrix R stored by rows.
!!C
!!C       LR is a positive integer input variable not less than
!!C         (N*(N+1))/2.
!!C
!!C       DIAG is an input array of length N which must contain the
!!C         diagonal elements of the matrix D.
!!C
!!C       QTB is an input array of length N which must contain the first
!!C         N elements of the vector (Q transpose)*B.
!!C
!!C       DELTA is a positive input variable which specifies an upper
!!C         bound on the Euclidean norm of D*X.
!!C
!!C       X is an output array of length N which contains the desired
!!C         convex combination of the Gauss-Newton direction and the
!!C         scaled gradient direction.
!!C
!!C       WA1 and WA2 are work arrays of length N.
!!C
!!C***SEE ALSO  DNSQ, DNSQE
!!C***ROUTINES CALLED  ddd1mach, DENORM
!!C***REVISION HISTORY  (YYMMDD)
!!C   800301  DATE WRITTEN
!!C   890531  Changed all specific intrinsics to generic.  (WRB)
!!C   890831  Modified array declarations.  (WRB)
!!C   891214  Prologue converted to Version 4.0 format.  (BAB)
!!C   900326  Removed duplicate information from DESCRIPTION section.
!!C           (WRB)
!!C   900328  Added TYPE section.  (WRB)
!!C***END PROLOGUE  DDOGLG
      !DOUBLE PRECISION ddd1mach,DENORM
      INTEGER I, J, JJ, JP1, K, L, LR, N
      DOUBLE PRECISION ALPHA, BNORM, DELTA, DIAG(*), EPSMCH, GNORM,  &
          ONE, QNORM, QTB(*), R(*), SGNORM, SUM, TEMP, WA1(*),			 &
          WA2(*), X(*), ZERO
      SAVE ONE, ZERO
      DATA ONE,ZERO /1.0D0,0.0D0/
!!C
!!C     EPSMCH IS THE MACHINE PRECISION.
!!C
!!C***FIRST EXECUTABLE STATEMENT  DDOGLG
      EPSMCH = ddd1mach(4)
!!C
!!C     FIRST, CALCULATE THE GAUSS-NEWTON DIRECTION.
!!C
      JJ = (N*(N + 1))/2 + 1
      DO 50 K = 1, N
         J = N - K + 1
         JP1 = J + 1
         JJ = JJ - K
         L = JJ + 1
         SUM = ZERO
         IF (N .LT. JP1) GO TO 20
         DO 10 I = JP1, N
            SUM = SUM + R(L)*X(I)
            L = L + 1
   10       CONTINUE
   20    CONTINUE
         TEMP = R(JJ)
         IF (TEMP .NE. ZERO) GO TO 40
         L = J
         DO 30 I = 1, J
            TEMP = MAX(TEMP,ABS(R(L)))
            L = L + N - I
   30       CONTINUE
         TEMP = EPSMCH*TEMP
         IF (TEMP .EQ. ZERO) TEMP = EPSMCH
   40    CONTINUE
         X(J) = (QTB(J) - SUM)/TEMP
   50    CONTINUE
!!C
!!C     TEST WHETHER THE GAUSS-NEWTON DIRECTION IS ACCEPTABLE.
!!C
      DO 60 J = 1, N
         WA1(J) = ZERO
         WA2(J) = DIAG(J)*X(J)
   60    CONTINUE
      QNORM = DENORM(N,WA2)
      IF (QNORM .LE. DELTA) GO TO 140
!!C
!!C     THE GAUSS-NEWTON DIRECTION IS NOT ACCEPTABLE.
!!C     NEXT, CALCULATE THE SCALED GRADIENT DIRECTION.
!!C
      L = 1
      DO 80 J = 1, N
         TEMP = QTB(J)
         DO 70 I = J, N
            WA1(I) = WA1(I) + R(L)*TEMP
            L = L + 1
   70       CONTINUE
         WA1(J) = WA1(J)/DIAG(J)
   80    CONTINUE
!!C
!!C     CALCULATE THE NORM OF THE SCALED GRADIENT AND TEST FOR
!!C     THE SPECIAL CASE IN WHICH THE SCALED GRADIENT IS ZERO.
!!C
      GNORM = DENORM(N,WA1)
      SGNORM = ZERO
      ALPHA = DELTA/QNORM
      IF (GNORM .EQ. ZERO) GO TO 120
!!C
!!C     CALCULATE THE POINT ALONG THE SCALED GRADIENT
!!C     AT WHICH THE QUADRATIC IS MINIMIZED.
!!C
      DO 90 J = 1, N
         WA1(J) = (WA1(J)/GNORM)/DIAG(J)
   90    CONTINUE
      L = 1
      DO 110 J = 1, N
         SUM = ZERO
         DO 100 I = J, N
            SUM = SUM + R(L)*WA1(I)
            L = L + 1
  100       CONTINUE
         WA2(J) = SUM
  110    CONTINUE
      TEMP = DENORM(N,WA2)
      SGNORM = (GNORM/TEMP)/TEMP
!!C
!!C     TEST WHETHER THE SCALED GRADIENT DIRECTION IS ACCEPTABLE.
!!C
      ALPHA = ZERO
      IF (SGNORM .GE. DELTA) GO TO 120
!!C
!!C     THE SCALED GRADIENT DIRECTION IS NOT ACCEPTABLE.
!!C     FINALLY, CALCULATE THE POINT ALONG THE DOGLEG
!!C     AT WHICH THE QUADRATIC IS MINIMIZED.
!!C
      BNORM = DENORM(N,QTB)
      TEMP = (BNORM/GNORM)*(BNORM/QNORM)*(SGNORM/DELTA)
      TEMP = TEMP - (DELTA/QNORM)*(SGNORM/DELTA)**2        &
            + SQRT((TEMP-(DELTA/QNORM))**2								 &
                    +(ONE-(DELTA/QNORM)**2)*(ONE-(SGNORM/DELTA)**2))
      ALPHA = ((DELTA/QNORM)*(ONE - (SGNORM/DELTA)**2))/TEMP
  120 CONTINUE
!!C
!!C     FORM APPROPRIATE CONVEX COMBINATION OF THE GAUSS-NEWTON
!!C     DIRECTION AND THE SCALED GRADIENT DIRECTION.
!!C
      TEMP = (ONE - ALPHA)*MIN(SGNORM,DELTA)
      DO 130 J = 1, N
         X(J) = TEMP*WA1(J) + ALPHA*X(J)
  130    CONTINUE
  140 CONTINUE
      RETURN
!!C
!!C     LAST CARD OF SUBROUTINE DDOGLG.
!!C
      END subroutine DDOGLG

!___________________________________________________________

!*DECK DENORM
      DOUBLE PRECISION FUNCTION DENORM (N, X)
!!C***BEGIN PROLOGUE  DENORM
!!C***SUBSIDIARY
!!C***PURPOSE  Subsidiary to DNSQ and DNSQE
!!C***LIBRARY   SLATEC
!!C***TYPE      DOUBLE PRECISION (ENORM-S, DENORM-D)
!!C***AUTHOR  (UNKNOWN)
!!C***DESCRIPTION
!!C
!!C     Given an N-vector X, this function calculates the
!!C     Euclidean norm of X.
!!C
!!C     The Euclidean norm is computed by accumulating the sum of
!!C     squares in three different sums. The sums of squares for the
!!C     small and large components are scaled so that no overflows
!!C     occur. Non-destructive underflows are permitted. Underflows
!!C     and overflows do not occur in the computation of the unscaled
!!C     sum of squares for the intermediate components.
!!C     The definitions of small, intermediate and large components
!!C     depend on two constants, RDWARF and RGIANT. The main
!!C     restrictions on these constants are that RDWARF**2 not
!!C     underflow and RGIANT**2 not overflow. The constants
!!C     given here are suitable for every known computer.
!!C
!!C     The function statement is
!!C
!!C       DOUBLE PRECISION FUNCTION DENORM(N,X)
!!C
!!C     where
!!C
!!C       N is a positive integer input variable.
!!C
!!C       X is an input array of length N.
!!C
!!C***SEE ALSO  DNSQ, DNSQE
!!C***ROUTINES CALLED  (NONE)
!!C***REVISION HISTORY  (YYMMDD)
!!C   800301  DATE WRITTEN
!!C   890531  Changed all specific intrinsics to generic.  (WRB)
!!C   890831  Modified array declarations.  (WRB)
!!C   891214  Prologue converted to Version 4.0 format.  (BAB)
!!C   900326  Removed duplicate information from DESCRIPTION section.
!!C           (WRB)
!!C   900328  Added TYPE section.  (WRB)
!!C***END PROLOGUE  DENORM
      INTEGER I, N
      DOUBLE PRECISION AGIANT, FLOATN, ONE, RDWARF, RGIANT, S1, S2, S3,  &
          X(*), X1MAX, X3MAX, XABS, ZERO
      SAVE ONE, ZERO, RDWARF, RGIANT
      DATA ONE,ZERO,RDWARF,RGIANT /1.0D0,0.0D0,3.834D-20,1.304D19/
!!C***FIRST EXECUTABLE STATEMENT  DENORM
      S1 = ZERO
      S2 = ZERO
      S3 = ZERO
      X1MAX = ZERO
      X3MAX = ZERO
      FLOATN = N
      AGIANT = RGIANT/FLOATN
      DO 90 I = 1, N
         XABS = ABS(X(I))
         IF (XABS .GT. RDWARF .AND. XABS .LT. AGIANT) GO TO 70
            IF (XABS .LE. RDWARF) GO TO 30
!!C
!!C              SUM FOR LARGE COMPONENTS.
!!C
               IF (XABS .LE. X1MAX) GO TO 10
                  S1 = ONE + S1*(X1MAX/XABS)**2
                  X1MAX = XABS
                  GO TO 20
   10          CONTINUE
                  S1 = S1 + (XABS/X1MAX)**2
   20          CONTINUE
               GO TO 60
   30       CONTINUE
!!C
!!C              SUM FOR SMALL COMPONENTS.
!!C
               IF (XABS .LE. X3MAX) GO TO 40
                  S3 = ONE + S3*(X3MAX/XABS)**2
                  X3MAX = XABS
                  GO TO 50
   40          CONTINUE
                  IF (XABS .NE. ZERO) S3 = S3 + (XABS/X3MAX)**2
   50          CONTINUE
   60       CONTINUE
            GO TO 80
   70    CONTINUE
!!C
!!C           SUM FOR INTERMEDIATE COMPONENTS.
!!C
            S2 = S2 + XABS**2
   80    CONTINUE
   90    CONTINUE
!!C
!!C     CALCULATION OF NORM.
!!C
      IF (S1 .EQ. ZERO) GO TO 100
         DENORM = X1MAX*SQRT(S1+(S2/X1MAX)/X1MAX)
         GO TO 130
  100 CONTINUE
         IF (S2 .EQ. ZERO) GO TO 110
            IF (S2 .GE. X3MAX)																&
              DENORM = SQRT(S2*(ONE+(X3MAX/S2)*(X3MAX*S3)))
            IF (S2 .LT. X3MAX)																&
              DENORM = SQRT(X3MAX*((S2/X3MAX)+(X3MAX*S3)))
            GO TO 120
  110    CONTINUE
            DENORM = X3MAX*SQRT(S3)
  120    CONTINUE
  130 CONTINUE
      RETURN
!!C
!!C     LAST CARD OF FUNCTION DENORM.
!!C
      END function DENORM 

!______________________________________________________________________

!*DECK DFDJC1
      SUBROUTINE DFDJC1 (FCN, N, X, FVEC, FJAC, LDFJAC, IFLAG, ML, MU, &
        EPSFCN, WA1, WA2)
!!C***BEGIN PROLOGUE  DFDJC1
!!C***SUBSIDIARY
!!C***PURPOSE  Subsidiary to DNSQ and DNSQE
!!C***LIBRARY   SLATEC
!!C***TYPE      DOUBLE PRECISION (FDJAC1-S, DFDJC1-D)
!!C***AUTHOR  (UNKNOWN)
!!C***DESCRIPTION
!!C
!!C     This subroutine computes a forward-difference approximation
!!C     to the N by N Jacobian matrix associated with a specified
!!C     problem of N functions in N variables. If the Jacobian has
!!C     a banded form, then function evaluations are saved by only
!!C     approximating the nonzero terms.
!!C
!!C     The subroutine statement is
!!C
!!C       SUBROUTINE DFDJC1(FCN,N,X,FVEC,FJAC,LDFJAC,IFLAG,ML,MU,EPSFCN,
!!C                         WA1,WA2)
!!C
!!C     where
!!C
!!C       FCN is the name of the user-supplied subroutine which
!!C         calculates the functions. FCN must be declared
!!C         in an EXTERNAL statement in the user calling
!!C         program, and should be written as follows.
!!C
!!C         SUBROUTINE FCN(N,X,FVEC,IFLAG)
!!C         INTEGER N,IFLAG
!!C         DOUBLE PRECISION X(N),FVEC(N)
!!C         ----------
!!C         Calculate the functions at X and
!!C         return this vector in FVEC.
!!C         ----------
!!C         RETURN
!!C
!!C         The value of IFLAG should not be changed by FCN unless
!!C         the user wants to terminate execution of DFDJC1.
!!C         In this case set IFLAG to a negative integer.
!!C
!!C       N is a positive integer input variable set to the number
!!C         of functions and variables.
!!C
!!C       X is an input array of length N.
!!C
!!C       FVEC is an input array of length N which must contain the
!!C         functions evaluated at X.
!!C
!!C       FJAC is an output N by N array which contains the
!!C         approximation to the Jacobian matrix evaluated at X.
!!C
!!C       LDFJAC is a positive integer input variable not less than N
!!C         which specifies the leading dimension of the array FJAC.
!!C
!!C       IFLAG is an integer variable which can be used to terminate
!!C         the execution of DFDJC1. See description of FCN.
!!C
!!C       ML is a nonnegative integer input variable which specifies
!!C         the number of subdiagonals within the band of the
!!C         Jacobian matrix. If the Jacobian is not banded, set
!!C         ML to at least N - 1.
!!C
!!C       EPSFCN is an input variable used in determining a suitable
!!C         step length for the forward-difference approximation. This
!!C         approximation assumes that the relative errors in the
!!C         functions are of the order of EPSFCN. If EPSFCN is less
!!C         than the machine precision, it is assumed that the relative
!!C         errors in the functions are of the order of the machine
!!C         precision.
!!C
!!C       MU is a nonnegative integer input variable which specifies
!!C         the number of superdiagonals within the band of the
!!C         Jacobian matrix. If the Jacobian is not banded, set
!!C         MU to at least N - 1.
!!C
!!C       WA1 and WA2 are work arrays of length N. If ML + MU + 1 is at
!!C         least N, then the Jacobian is considered dense, and WA2 is
!!C         not referenced.
!!C
!!C***SEE ALSO  DNSQ, DNSQE
!!C***ROUTINES CALLED  ddd1mach
!!C***REVISION HISTORY  (YYMMDD)
!!C   800301  DATE WRITTEN
!!C   890531  Changed all specific intrinsics to generic.  (WRB)
!!C   890831  Modified array declarations.  (WRB)
!!C   891214  Prologue converted to Version 4.0 format.  (BAB)
!!C   900326  Removed duplicate information from DESCRIPTION section.
!!C           (WRB)
!!C   900328  Added TYPE section.  (WRB)
!!C***END PROLOGUE  DFDJC1
      !DOUBLE PRECISION ddd1mach
      INTEGER dum, I, IFLAG, J, K, LDFJAC, ML, MSUM, MU, N
      DOUBLE PRECISION EPS, EPSFCN, EPSMCH, FJAC(LDFJAC,*),    &
          FVEC(*), H, TEMP, WA1(*), WA2(*), X(*), ZERO
	  EXTERNAL FCN
      SAVE ZERO
      DATA ZERO /0.0D0/
!!C
!!C     EPSMCH IS THE MACHINE PRECISION.
!!C
!!C***FIRST EXECUTABLE STATEMENT  DFDJC1
			dum = 4
      EPSMCH = ddd1mach(dum)
!!C
      EPS = SQRT(MAX(EPSFCN,EPSMCH))
      MSUM = ML + MU + 1
      IF (MSUM .LT. N) GO TO 40
!!C
!!C        COMPUTATION OF DENSE APPROXIMATE JACOBIAN.
!!C


         DO 20 J = 1, N
            TEMP = X(J)
            H = EPS*ABS(TEMP)
            IF (H .EQ. ZERO) H = EPS
            X(J) = TEMP + H
            CALL FCN(N,X,WA1,IFLAG)
            IF (IFLAG .LT. 0) GO TO 30
            X(J) = TEMP
            DO 10 I = 1, N
               FJAC(I,J) = (WA1(I) - FVEC(I))/H
   10          CONTINUE
   20       CONTINUE
   30    CONTINUE
         GO TO 110
   40 CONTINUE
!!C
!!C        COMPUTATION OF BANDED APPROXIMATE JACOBIAN.
!!C
         DO 90 K = 1, MSUM
            DO 60 J = K, N, MSUM
               WA2(J) = X(J)
               H = EPS*ABS(WA2(J))
               IF (H .EQ. ZERO) H = EPS
               X(J) = WA2(J) + H
   60          CONTINUE
            CALL FCN(N,X,WA1,IFLAG)
            IF (IFLAG .LT. 0) GO TO 100
            DO 80 J = K, N, MSUM
               X(J) = WA2(J)
               H = EPS*ABS(WA2(J))
               IF (H .EQ. ZERO) H = EPS
               DO 70 I = 1, N
                  FJAC(I,J) = ZERO
                  IF (I .GE. J - MU .AND. I .LE. J + ML)			&
                    FJAC(I,J) = (WA1(I) - FVEC(I))/H
   70             CONTINUE
   80          CONTINUE
   90       CONTINUE
  100    CONTINUE
  110 CONTINUE
      RETURN
!!C
!!C     LAST CARD OF SUBROUTINE DFDJC1.
!!C
      END subroutine DFDJC1 

!________________________________________________________________

!*DECK FDUMP
      SUBROUTINE FDUMP
!!C***BEGIN PROLOGUE  FDUMP
!!C***PURPOSE  Symbolic dump (should be locally written).
!!C***LIBRARY   SLATEC (XERROR)
!!C***CATEGORY  R3
!!C***TYPE      ALL (FDUMP-A)
!!C***KEYWORDS  ERROR, XERMSG
!!C***AUTHOR  Jones, R. E., (SNLA)
!!C***DESCRIPTION
!!C
!!C        ***Note*** Machine Dependent Routine
!!C        FDUMP is intended to be replaced by a locally written
!!C        version which produces a symbolic dump.  Failing this,
!!C        it should be replaced by a version which prints the
!!C        subprogram nesting list.  Note that this dump must be
!!C        printed on each of up to five files, as indicated by the
!!C        XGETUA routine.  See XSETUA and XGETUA for details.
!!C
!!C     Written by Ron Jones, with SLATEC Common Math Library Subcommittee
!!C
!!C***REFERENCES  (NONE)
!!C***ROUTINES CALLED  (NONE)
!!C***REVISION HISTORY  (YYMMDD)
!!C   790801  DATE WRITTEN
!!C   861211  REVISION DATE from Version 3.2
!!C   891214  Prologue converted to Version 4.0 format.  (BAB)
!!C***END PROLOGUE  FDUMP
!!C***FIRST EXECUTABLE STATEMENT  FDUMP
      RETURN
      END subroutine fdump

!*DECK J4SAVE
      double precision FUNCTION J4SAVE (IWHICH, IVALUE, ISET)
!!C***BEGIN PROLOGUE  J4SAVE
!!C***SUBSIDIARY
!!C***PURPOSE  Save or recall global variables needed by error
!!C            handling routines.
!!C***LIBRARY   SLATEC (XERROR)
!!C***TYPE      INTEGER (J4SAVE-I)
!!C***KEYWORDS  ERROR MESSAGES, ERROR NUMBER, RECALL, SAVE, XERROR
!!C***AUTHOR  Jones, R. E., (SNLA)
!!C***DESCRIPTION
!!C
!!C     Abstract
!!C        J4SAVE saves and recalls several global variables needed
!!C        by the library error handling routines.
!!C
!!C     Description of Parameters
!!C      --Input--
!!C        IWHICH - Index of item desired.
!!C                = 1 Refers to current error number.
!!C                = 2 Refers to current error control flag.
!!C                = 3 Refers to current unit number to which error
!!C                    messages are to be sent.  (0 means use standard.)
!!C                = 4 Refers to the maximum number of times any
!!C                     message is to be printed (as set by XERMAX).
!!C                = 5 Refers to the total number of units to which
!!C                     each error message is to be written.
!!C                = 6 Refers to the 2nd unit for error messages
!!C                = 7 Refers to the 3rd unit for error messages
!!C                = 8 Refers to the 4th unit for error messages
!!C                = 9 Refers to the 5th unit for error messages
!!C        IVALUE - The value to be set for the IWHICH-th parameter,
!!C                 if ISET is .TRUE. .
!!C        ISET   - If ISET=.TRUE., the IWHICH-th parameter will BE
!!C                 given the value, IVALUE.  If ISET=.FALSE., the
!!C                 IWHICH-th parameter will be unchanged, and IVALUE
!!C                 is a dummy parameter.
!!C      --Output--
!!C        The (old) value of the IWHICH-th parameter will be returned
!!C        in the function value, J4SAVE.
!!C
!!C***SEE ALSO  XERMSG
!!C***REFERENCES  R. E. Jones and D. K. Kahaner, XERROR, the SLATEC
!!C                 Error-handling Package, SAND82-0800, Sandia
!!C                 Laboratories, 1982.
!!C***ROUTINES CALLED  (NONE)
!!C***REVISION HISTORY  (YYMMDD)
!!C   790801  DATE WRITTEN
!!C   891214  Prologue converted to Version 4.0 format.  (BAB)
!!C   900205  Minor modifications to prologue.  (WRB)
!!C   900402  Added TYPE section.  (WRB)
!!C   910411  Added KEYWORDS section.  (WRB)
!!C   920501  Reformatted the REFERENCES section.  (WRB)
!!C***END PROLOGUE  J4SAVE
      LOGICAL ISET
      INTEGER IPARAM(9)
      SAVE IPARAM
      DATA IPARAM(1),IPARAM(2),IPARAM(3),IPARAM(4)/0,2,0,10/
      DATA IPARAM(5)/1/
      DATA IPARAM(6),IPARAM(7),IPARAM(8),IPARAM(9)/0,0,0,0/
!!C***FIRST EXECUTABLE STATEMENT  J4SAVE
      J4SAVE = IPARAM(IWHICH)
      IF (ISET) IPARAM(IWHICH) = IVALUE
      RETURN
      END function j4save

!_____________________________________________________________________

!*DECK XERCNT
      SUBROUTINE XERCNT (LIBRAR, SUBROU, MESSG, NERR, LEVEL, KONTRL)
!!C***BEGIN PROLOGUE  XERCNT
!!C***SUBSIDIARY
!!C***PURPOSE  Allow user control over handling of errors.
!!C***LIBRARY   SLATEC (XERROR)
!!C***CATEGORY  R3C
!!C***TYPE      ALL (XERCNT-A)
!!C***KEYWORDS  ERROR, XERROR
!!C***AUTHOR  Jones, R. E., (SNLA)
!!C***DESCRIPTION
!!C
!!C     Abstract
!!C        Allows user control over handling of individual errors.
!!C        Just after each message is recorded, but before it is
!!C        processed any further (i.e., before it is printed or
!!C        a decision to abort is made), a call is made to XERCNT.
!!C        If the user has provided his own version of XERCNT, he
!!C        can then override the value of KONTROL used in processing
!!C        this message by redefining its value.
!!C        KONTRL may be set to any value from -2 to 2.
!!C        The meanings for KONTRL are the same as in XSETF, except
!!C        that the value of KONTRL changes only for this message.
!!C        If KONTRL is set to a value outside the range from -2 to 2,
!!C        it will be moved back into that range.
!!C
!!C     Description of Parameters
!!C
!!C      --Input--
!!C        LIBRAR - the library that the routine is in.
!!C        SUBROU - the subroutine that XERMSG is being called from
!!C        MESSG  - the first 20 characters of the error message.
!!C        NERR   - same as in the call to XERMSG.
!!C        LEVEL  - same as in the call to XERMSG.
!!C        KONTRL - the current value of the control flag as set
!!C                 by a call to XSETF.
!!C
!!C      --Output--
!!C        KONTRL - the new value of KONTRL.  If KONTRL is not
!!C                 defined, it will remain at its original value.
!!C                 This changed value of control affects only
!!C                 the current occurrence of the current message.
!!C
!!C***REFERENCES  R. E. Jones and D. K. Kahaner, XERROR, the SLATEC
!!C                 Error-handling Package, SAND82-0800, Sandia
!!C                 Laboratories, 1982.
!!C***ROUTINES CALLED  (NONE)
!!C***REVISION HISTORY  (YYMMDD)
!!C   790801  DATE WRITTEN
!!C   861211  REVISION DATE from Version 3.2
!!C   891214  Prologue converted to Version 4.0 format.  (BAB)
!!C   900206  Routine changed from user-callable to subsidiary.  (WRB)
!!C   900510  Changed calling sequence to include LIBRARY and SUBROUTINE
!!C           names, changed routine name from XERCTL to XERCNT.  (RWC)
!!C   920501  Reformatted the REFERENCES section.  (WRB)
!!C***END PROLOGUE  XERCNT
      CHARACTER*(*) LIBRAR, SUBROU, MESSG
!!C***FIRST EXECUTABLE STATEMENT  XERCNT
      RETURN
      END subroutine XERCNT 

!______________________________________________________________________

!*DECK XERHLT
      SUBROUTINE XERHLT (MESSG)
!!C***BEGIN PROLOGUE  XERHLT
!!C***SUBSIDIARY
!!C***PURPOSE  Abort program execution and print error message.
!!C***LIBRARY   SLATEC (XERROR)
!!C***CATEGORY  R3C
!!C***TYPE      ALL (XERHLT-A)
!!C***KEYWORDS  ABORT PROGRAM EXECUTION, ERROR, XERROR
!!C***AUTHOR  Jones, R. E., (SNLA)
!!C***DESCRIPTION
!!C
!!C     Abstract
!!C        ***Note*** machine dependent routine
!!C        XERHLT aborts the execution of the program.
!!C        The error message causing the abort is given in the calling
!!C        sequence, in case one needs it for printing on a dayfile,
!!C        for example.
!!C
!!C     Description of Parameters
!!C        MESSG is as in XERMSG.
!!C
!!C***REFERENCES  R. E. Jones and D. K. Kahaner, XERROR, the SLATEC
!!C                 Error-handling Package, SAND82-0800, Sandia
!!C                 Laboratories, 1982.
!!C***ROUTINES CALLED  (NONE)
!!C***REVISION HISTORY  (YYMMDD)
!!C   790801  DATE WRITTEN
!!C   861211  REVISION DATE from Version 3.2
!!C   891214  Prologue converted to Version 4.0 format.  (BAB)
!!C   900206  Routine changed from user-callable to subsidiary.  (WRB)
!!C   900510  Changed calling sequence to delete length of character
!!C           and changed routine name from XERABT to XERHLT.  (RWC)
!!C   920501  Reformatted the REFERENCES section.  (WRB)
!!C***END PROLOGUE  XERHLT
      CHARACTER*(*) MESSG
!!C***FIRST EXECUTABLE STATEMENT  XERHLT

			return 
			end subroutine XERHLT 

			subroutine jac


			end subroutine jac

      
  END module dnsqrout


